@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2000-2002 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,14 +30,33 @@
 #include <locale.h>
 
 #include "gmime-message.h"
+#include "gmime-multipart.h"
+#include "gmime-part.h"
 #include "gmime-utils.h"
 #include "gmime-stream-mem.h"
 
-static void g_mime_message_destroy (GMimeObject *object);
 
-static GMimeObject object_template = {
-	0, 0, g_mime_message_destroy
-};
+static void g_mime_message_class_init (GMimeMessageClass *klass);
+static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass);
+static void g_mime_message_finalize (GObject *object);
+
+/* GMimeObject class methods */
+static void message_init (GMimeObject *object);
+static void message_add_header (GMimeObject *object, const char *header, const char *value);
+static void message_set_header (GMimeObject *object, const char *header, const char *value);
+static const char *message_get_header (GMimeObject *object, const char *header);
+static void message_remove_header (GMimeObject *object, const char *header);
+static char *message_get_headers (GMimeObject *object);
+static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
+
+static void message_set_sender (GMimeMessage *message, const char *sender);
+static void message_set_reply_to (GMimeMessage *message, const char *reply_to);
+static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
+static void message_set_subject (GMimeMessage *message, const char *subject);
+static void message_set_message_id (GMimeMessage *message, const char *message_id);
+
+static GMimeObjectClass *parent_class = NULL;
+
 
 static char *rfc822_headers[] = {
 	"Return-Path",
@@ -53,6 +72,280 @@ static char *rfc822_headers[] = {
 };
 
 
+GType
+g_mime_message_get_type (void)
+{
+	static GType type = 0;
+	
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (GMimeMessageClass),
+			NULL, /* base_class_init */
+			NULL, /* base_class_finalize */
+			(GClassInitFunc) g_mime_message_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GMimeMessage),
+			0,   /* n_preallocs */
+			(GInstanceInitFunc) g_mime_message_init,
+		};
+		
+		type = g_type_register_static (GMIME_TYPE_OBJECT, "GMimeMessage", &info, 0);
+	}
+	
+	return type;
+}
+
+
+static void
+g_mime_message_class_init (GMimeMessageClass *klass)
+{
+	GMimeObjectClass *object_class = GMIME_OBJECT_CLASS (klass);
+	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	
+	parent_class = g_type_class_ref (GMIME_TYPE_OBJECT);
+	
+	gobject_class->finalize = g_mime_message_finalize;
+	
+	object_class->init = message_init;
+	object_class->add_header = message_add_header;
+	object_class->set_header = message_set_header;
+	object_class->get_header = message_get_header;
+	object_class->remove_header = message_remove_header;
+	object_class->get_headers = message_get_headers;
+	object_class->write_to_stream = message_write_to_stream;
+}
+
+static void
+g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
+{
+	message->from = NULL;
+	message->reply_to = NULL;
+	message->recipients = g_hash_table_new (g_str_hash, g_str_equal);
+	message->subject = NULL;
+	message->date = 0;
+	message->gmt_offset = 0;
+	message->message_id = NULL;
+	message->mime_part = NULL;
+}
+
+static gboolean
+recipients_destroy (gpointer key, gpointer value, gpointer user_data)
+{
+	InternetAddressList *recipients = value;
+	
+	internet_address_list_destroy (recipients);
+	
+	return TRUE;
+}
+
+static void
+g_mime_message_finalize (GObject *object)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	
+	g_free (message->from);
+	g_free (message->reply_to);
+	
+	/* destroy all recipients */
+	g_hash_table_foreach_remove (message->recipients, recipients_destroy, NULL);
+	g_hash_table_destroy (message->recipients);
+	
+	g_free (message->subject);
+	
+	g_free (message->message_id);
+	
+	/* unref child mime part */
+	if (message->mime_part)
+		g_mime_object_unref (GMIME_OBJECT (message->mime_part));
+	
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
+static void
+message_init (GMimeObject *object)
+{
+	/* no-op */
+	GMIME_OBJECT_CLASS (parent_class)->init (object);
+}
+
+
+enum {
+	HEADER_FROM,
+	HEADER_REPLY_TO,
+	HEADER_TO,
+	HEADER_CC,
+	HEADER_BCC,
+	HEADER_SUBJECT,
+	HEADER_DATE,
+	HEADER_MESSAGE_ID,
+	HEADER_UNKNOWN
+};
+
+static char *headers[] = {
+	"From",
+	"Reply-To",
+	"To",
+	"Cc",
+	"Bcc",
+	"Subject",
+	"Date",
+	"Message-Id",
+	NULL
+};
+
+static gboolean
+process_header (GMimeObject *object, const char *header, const char *value)
+{
+	GMimeMessage *message = (GMimeMessage *) object;	
+	int offset, i;
+	time_t date;
+	
+	for (i = 0; headers[i]; i++) {
+		if (!strcasecmp (headers[i], header))
+			break;
+	}
+	
+	switch (i) {
+	case HEADER_FROM:
+		message_set_sender (message, value);
+		break;
+	case HEADER_REPLY_TO:
+		message_set_reply_to (message, value);
+		break;
+	case HEADER_TO:
+		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_TO, value);
+		break;
+	case HEADER_CC:
+		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_CC, value);
+		break;
+	case HEADER_BCC:
+		message_add_recipients_from_string (message, GMIME_RECIPIENT_TYPE_BCC, value);
+		break;
+	case HEADER_SUBJECT:
+		message_set_subject (message, value);
+		break;
+	case HEADER_DATE:
+		if (value) {
+			date = g_mime_utils_header_decode_date (value, &offset);
+			message->date = date;
+			message->gmt_offset = offset;
+		}
+		break;
+	case HEADER_MESSAGE_ID:
+		message_set_message_id (message, value);
+		break;
+	default:
+		return FALSE;
+		break;
+	}
+	
+	return TRUE;
+}
+
+static void
+message_add_header (GMimeObject *object, const char *header, const char *value)
+{
+	if (!strcasecmp ("MIME-Version", header))
+		return;
+	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
+	
+	if (strncasecmp ("Content-", header, 8)) {
+		if (process_header (object, header, value))
+			g_mime_header_add (object->headers, header, value);
+		else
+			GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
+	}
+}
+
+static void
+message_set_header (GMimeObject *object, const char *header, const char *value)
+{
+	if (!strcasecmp ("MIME-Version", header))
+		return;
+	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
+	
+	if (strncasecmp ("Content-", header, 8)) {
+		if (process_header (object, header, value))
+			g_mime_header_set (object->headers, header, value);
+		else
+			GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
+	}
+}
+
+static const char *
+message_get_header (GMimeObject *object, const char *header)
+{
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
+	
+	if (!strcasecmp ("MIME-Version", header))
+		return "1.0";
+	
+	if (strncasecmp ("Content-", header, 8))
+		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
+	else
+		return NULL;
+}
+
+static void
+message_remove_header (GMimeObject *object, const char *header)
+{
+	if (!strcasecmp ("MIME-Version", header))
+		return;
+	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a multipart */
+	
+	if (!strncasecmp ("Content-", header, 8))
+		return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+}
+
+static char *
+message_get_headers (GMimeObject *object)
+{
+	/* FIXME: get mime part headers too? */
+	return GMIME_OBJECT_CLASS (parent_class)->get_headers (object);
+}
+
+static ssize_t
+message_write_to_stream (GMimeObject *object, GMimeStream *stream)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	ssize_t nwritten, total = 0;
+	
+	/* write the content headers */
+	nwritten = g_mime_header_write_to_stream (object->headers, stream);
+	if (nwritten == -1)
+		return -1;
+	
+	total += nwritten;
+	
+	if (message->mime_part) {
+		nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
+		if (nwritten == -1)
+			return -1;
+		
+		total += nwritten;
+		
+		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
+	} else {
+		nwritten = g_mime_stream_write (stream, "\n", 1);
+		if (nwritten == -1)
+			return -1;
+	}
+	
+	total += nwritten;
+	
+	return total;
+}
+
+
 /**
  * g_mime_message_new: Create a new MIME Message object
  * @pretty_headers: make pretty headers 
@@ -68,66 +361,27 @@ GMimeMessage *
 g_mime_message_new (gboolean pretty_headers)
 {
 	GMimeMessage *message;
-	GMimeHeader *headers;
 	int i;
 	
-	message = g_new0 (GMimeMessage, 1);
-	g_mime_object_construct (GMIME_OBJECT (message),
-				 &object_template,
-				 GMIME_MESSAGE_TYPE);
-	
-	message->header = g_new0 (GMimeMessageHeader, 1);
-	
-	message->header->recipients = g_hash_table_new (g_str_hash, g_str_equal);
-	
-	message->header->headers = headers = g_mime_header_new ();
+	message = g_object_new (GMIME_TYPE_MESSAGE, NULL, NULL);
 	
 	if (pretty_headers) {
 		/* Populate with the "standard" rfc822 headers so we can have a standard order */
 		for (i = 0; rfc822_headers[i]; i++) 
-			g_mime_header_set (headers, rfc822_headers[i], NULL);
+			g_mime_object_set_header (GMIME_OBJECT (message), rfc822_headers[i], NULL);
 	}
 	
 	return message;
 }
 
-static gboolean
-recipients_destroy (gpointer key, gpointer value, gpointer user_data)
-{
-	InternetAddressList *recipients = value;
-	
-	internet_address_list_destroy (recipients);
-	
-	return TRUE;
-}
 
 static void
-g_mime_message_destroy (GMimeObject *object)
+message_set_sender (GMimeMessage *message, const char *sender)
 {
-	GMimeMessage *message = (GMimeMessage *) object;
+	if (message->from)
+		g_free (message->from);
 	
-	g_return_if_fail (GMIME_IS_MESSAGE (object));
-	
-	g_free (message->header->from);
-	g_free (message->header->reply_to);
-	
-	/* destroy all recipients */
-	g_hash_table_foreach_remove (message->header->recipients, recipients_destroy, NULL);
-	g_hash_table_destroy (message->header->recipients);
-	
-	g_free (message->header->subject);
-	
-	g_free (message->header->message_id);
-	
-	g_mime_header_destroy (message->header->headers);
-	
-	g_free (message->header);
-	
-	/* unref child mime part */
-	if (message->mime_part)
-		g_mime_object_unref (GMIME_OBJECT (message->mime_part));
-	
-	g_free (message);
+	message->from = g_strstrip (g_strdup (sender));
 }
 
 
@@ -142,17 +396,10 @@ g_mime_message_destroy (GMimeObject *object)
 void
 g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 {
-	GMimeMessageHeader *header;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
-	header = message->header;
-	
-	if (header->from)
-		g_free (header->from);
-	
-	header->from = g_strstrip (g_strdup (sender));
-	g_mime_header_set (header->headers, "From", header->from);
+	message_set_sender (message, sender);
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", message->from);
 }
 
 
@@ -169,7 +416,17 @@ g_mime_message_get_sender (GMimeMessage *message)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	return message->header->from;
+	return message->from;
+}
+
+
+static void
+message_set_reply_to (GMimeMessage *message, const char *reply_to)
+{
+	if (message->reply_to)
+		g_free (message->reply_to);
+	
+	message->reply_to = g_strstrip (g_strdup (reply_to));
 }
 
 
@@ -183,17 +440,10 @@ g_mime_message_get_sender (GMimeMessage *message)
 void
 g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 {
-	GMimeMessageHeader *header;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
-	header = message->header;
-	
-	if (header->reply_to)
-		g_free (header->reply_to);
-	
-	header->reply_to = g_strstrip (g_strdup (reply_to));
-	g_mime_header_set (header->headers, "Reply-To", header->reply_to);
+	message_set_reply_to (message, reply_to);
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
 }
 
 
@@ -210,7 +460,7 @@ g_mime_message_get_reply_to (GMimeMessage *message)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	return message->header->reply_to;
+	return message->reply_to;
 }
 
 
@@ -225,10 +475,10 @@ sync_recipient_header (GMimeMessage *message, const char *type)
 		char *string;
 		
 		string = internet_address_list_to_string (recipients, TRUE);
-		g_mime_header_set (message->header->headers, type, string);
+		g_mime_header_set (GMIME_OBJECT (message)->headers, type, string);
 		g_free (string);
 	} else
-		g_mime_header_set (message->header->headers, type, NULL);
+		g_mime_header_set (GMIME_OBJECT (message)->headers, type, NULL);
 }
 
 
@@ -240,8 +490,8 @@ sync_recipient_header (GMimeMessage *message, const char *type)
  * @address: The recipient's address
  *
  * Add a recipient of a chosen type to the MIME Message. Available
- * recipient types include: GMIME_RECIPIENT_TYPE_TO,
- * GMIME_RECIPIENT_TYPE_CC and GMIME_RECIPIENT_TYPE_BCC.
+ * recipient types include: #GMIME_RECIPIENT_TYPE_TO,
+ * #GMIME_RECIPIENT_TYPE_CC and #GMIME_RECIPIENT_TYPE_BCC.
  **/
 void
 g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *name, const char *address)
@@ -249,19 +499,42 @@ g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *nam
 	InternetAddressList *recipients;
 	InternetAddress *ia;
 	
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (type != NULL);
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (address != NULL);
+	
 	ia = internet_address_new_name (name, address);
 	
-	recipients = g_hash_table_lookup (message->header->recipients, type);
-	g_hash_table_remove (message->header->recipients, type);
+	recipients = g_hash_table_lookup (message->recipients, type);
+	g_hash_table_remove (message->recipients, type);
 	
 	recipients = internet_address_list_append (recipients, ia);
 	internet_address_unref (ia);
 	
-	g_hash_table_insert (message->header->recipients, type, recipients);
+	g_hash_table_insert (message->recipients, type, recipients);
 	sync_recipient_header (message, type);
 }
 
 
+static void
+message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
+{
+	InternetAddressList *recipients, *addrlist;
+	
+	recipients = g_hash_table_lookup (message->recipients, type);
+	g_hash_table_remove (message->recipients, type);
+	
+	addrlist = internet_address_parse_string (string);
+	if (addrlist) {
+		recipients = internet_address_list_concat (recipients, addrlist);
+		internet_address_list_destroy (addrlist);
+	}
+	
+	g_hash_table_insert (message->recipients, type, recipients);
+}
+
+
 /**
  * g_mime_message_add_recipients_from_string:
  * @message: MIME Message
@@ -270,26 +543,18 @@ g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *nam
  *
  * Add a list of recipients of a chosen type to the MIME
  * Message. Available recipient types include:
- * GMIME_RECIPIENT_TYPE_TO, GMIME_RECIPIENT_TYPE_CC and
- * GMIME_RECIPIENT_TYPE_BCC. The string must be in the format
+ * #GMIME_RECIPIENT_TYPE_TO, #GMIME_RECIPIENT_TYPE_CC and
+ * #GMIME_RECIPIENT_TYPE_BCC. The string must be in the format
  * specified in rfc822.
  **/
 void
 g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
 {
-	InternetAddressList *recipients, *addrlist;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (type != NULL);
 	g_return_if_fail (string != NULL);
 	
-	recipients = g_hash_table_lookup (message->header->recipients, type);
-	g_hash_table_remove (message->header->recipients, type);
-	
-	addrlist = internet_address_parse_string (string);
-	if (addrlist)
-		recipients = internet_address_list_concat (recipients, addrlist);
-	
-	g_hash_table_insert (message->header->recipients, type, recipients);
+	message_add_recipients_from_string (message, type, string);
 	
 	sync_recipient_header (message, type);
 }
@@ -300,19 +565,30 @@ g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, co
  * @message: MIME Message
  * @type: Recipient type
  *
- * Gets a list of recipients of type @type from @message.
+ * Gets a list of recipients of type @type from @message. Available
+ * recipient types include: #GMIME_RECIPIENT_TYPE_TO,
+ * #GMIME_RECIPIENT_TYPE_CC and #GMIME_RECIPIENT_TYPE_BCC.
  *
  * Returns a list of recipients of a chosen type from the MIME
- * Message. Available recipient types include:
- * GMIME_RECIPIENT_TYPE_TO, GMIME_RECIPIENT_TYPE_CC and
- * GMIME_RECIPIENT_TYPE_BCC.
+ * Message.
  **/
 InternetAddressList *
 g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	g_return_val_if_fail (type != NULL, NULL);
 	
-	return g_hash_table_lookup (message->header->recipients, type);
+	return g_hash_table_lookup (message->recipients, type);
+}
+
+
+static void
+message_set_subject (GMimeMessage *message, const char *subject)
+{
+	if (message->subject)
+		g_free (message->subject);
+	
+	message->subject = g_strstrip (g_strdup (subject));
 }
 
 
@@ -326,17 +602,10 @@ g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 void
 g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 {
-	GMimeMessageHeader *header;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
-	header = message->header;
-	
-	if (header->subject)
-		g_free (header->subject);
-	
-	header->subject = g_strstrip (g_strdup (subject));
-	g_mime_header_set (header->headers, "Subject", header->subject);
+	message_set_subject (message, subject);
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "Subject", message->subject);
 }
 
 
@@ -353,17 +622,17 @@ g_mime_message_get_subject (GMimeMessage *message)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	return message->header->subject;
+	return message->subject;
 }
 
 
 /**
  * g_mime_message_set_date:
  * @message: MIME Message
- * @date: Sent-date (ex: gotten from time (NULL);)
+ * @date: Sent-date (ex: gotten from time (NULL))
  * @gmt_offset: GMT date offset (in +/- hours)
  * 
- * Set the sent-date on a MIME Message.
+ * Sets the sent-date on a MIME Message.
  **/
 void
 g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
@@ -372,11 +641,11 @@ g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
 	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
-	message->header->date = date;
-	message->header->gmt_offset = gmt_offset;
+	message->date = date;
+	message->gmt_offset = gmt_offset;
 	
 	date_str = g_mime_message_get_date_string (message);
-	g_mime_header_set (message->header->headers, "Date", date_str);
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "Date", date_str);
 	g_free (date_str);
 }
 
@@ -394,9 +663,12 @@ void
 g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (date != NULL);
 	
-	*date = message->header->date;
-	*gmt_offset = message->header->gmt_offset;
+	*date = message->date;
+	
+	if (gmt_offset)
+		*gmt_offset = message->gmt_offset;
 }
 
 
@@ -412,45 +684,41 @@ char *
 g_mime_message_get_date_string (GMimeMessage *message)
 {
 	char *date_str;
-	char *locale;
 	
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	locale = g_strdup (setlocale (LC_TIME, ""));
-	setlocale (LC_TIME, "POSIX");
-	
-	date_str = g_mime_utils_header_format_date (message->header->date,
-						    message->header->gmt_offset);
-	
-	if (locale != NULL)
-		setlocale (LC_TIME, locale);
-	g_free (locale);
+	date_str = g_mime_utils_header_format_date (message->date,
+						    message->gmt_offset);
 	
 	return date_str;
 }
 
 
+static void
+message_set_message_id (GMimeMessage *message, const char *message_id)
+{
+	if (message->message_id)
+		g_free (message->message_id);
+	
+	message->message_id = g_strstrip (g_strdup (message_id));
+}
+
+
 /**
  * g_mime_message_set_message_id: 
  * @message: MIME Message
- * @id: message-id
+ * @message_id: message-id
  *
  * Set the Message-Id on a message.
  **/
 void
-g_mime_message_set_message_id (GMimeMessage *message, const char *id)
+g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 {
-	GMimeMessageHeader *header;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
-	header = message->header;
-	
-	if (header->message_id)
-		g_free (header->message_id);
-	
-	header->message_id = g_strstrip (g_strdup (id));
-	g_mime_header_set (header->headers, "Message-Id", header->message_id);
+	message_set_message_id (message, message_id);
+	g_mime_header_set (GMIME_OBJECT (message)->headers,
+			   "Message-Id", message->message_id);
 }
 
 
@@ -467,7 +735,7 @@ g_mime_message_get_message_id (GMimeMessage *message)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	return message->header->message_id;
+	return message->message_id;
 }
 
 
@@ -486,7 +754,7 @@ g_mime_message_add_header (GMimeMessage *message, const char *header, const char
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (header != NULL);
 	
-	g_mime_header_add (message->header->headers, header, value);
+	g_mime_object_add_header (GMIME_OBJECT (message), header, value);
 }
 
 
@@ -505,7 +773,7 @@ g_mime_message_set_header (GMimeMessage *message, const char *header, const char
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (header != NULL);
 	
-	g_mime_header_set (message->header->headers, header, value);
+	g_mime_object_set_header (GMIME_OBJECT (message), header, value);
 }
 
 
@@ -525,7 +793,7 @@ g_mime_message_get_header (GMimeMessage *message, const char *header)
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	g_return_val_if_fail (header != NULL, NULL);
 	
-	return g_mime_header_get (message->header->headers, header);
+	return g_mime_object_get_header (GMIME_OBJECT (message), header);
 }
 
 
@@ -537,10 +805,9 @@ g_mime_message_get_header (GMimeMessage *message, const char *header)
  * Set the root-level MIME part of the message.
  **/
 void
-g_mime_message_set_mime_part (GMimeMessage *message, GMimePart *mime_part)
+g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (GMIME_IS_PART (mime_part));
 	
 	g_mime_object_ref (GMIME_OBJECT (mime_part));
 	
@@ -556,21 +823,17 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimePart *mime_part)
  * @message: MIME Message
  * @stream: output stream
  *
- * Write the contents of the MIME Message to @string.
+ * Write the contents of the MIME Message to @stream.
+ *
+ * Returns -1 on fail.
  **/
-void
+ssize_t
 g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
 {
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (stream != NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	g_mime_header_write_to_stream (message->header->headers, stream);
-	
-	if (message->mime_part) {
-		g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
-		g_mime_part_write_to_stream (message->mime_part, stream);
-	} else
-		g_mime_stream_write (stream, "\n", 1);
+	return g_mime_object_write_to_stream (GMIME_OBJECT (message), stream);
 }
 
 
@@ -585,22 +848,9 @@ g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
 char *
 g_mime_message_to_string (GMimeMessage *message)
 {
-	GMimeStream *stream;
-	GByteArray *array;
-	char *str;
-	
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	array = g_byte_array_new ();
-	stream = g_mime_stream_mem_new ();
-	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), array);
-	g_mime_message_write_to_stream (message, stream);
-	g_mime_stream_unref (stream);
-	g_byte_array_append (array, "", 1);
-	str = array->data;
-	g_byte_array_free (array, FALSE);
-	
-	return str;
+	return g_mime_object_to_string (GMIME_OBJECT (message));
 }
 
 
@@ -634,27 +884,27 @@ g_mime_message_to_string (GMimeMessage *message)
  * guarenteed to always be correct.
  **/
 static char *
-multipart_get_body (GMimePart *multipart, gboolean want_plain, gboolean *is_html)
+multipart_get_body (GMimeMultipart *multipart, gboolean want_plain, gboolean *is_html)
 {
-	GMimePart *first = NULL;
+	GMimeObject *first = NULL;
 	const char *content;
 	char *body = NULL;
-	GList *child;
+	GList *subpart;
 	size_t len;
 	
-	child = multipart->children;
-	while (child) {
+	subpart = multipart->subparts;
+	while (subpart) {
 		const GMimeContentType *type;
-		GMimePart *mime_part;
+		GMimeObject *mime_part;
 		
-		mime_part = child->data;
-		type = g_mime_part_get_content_type (mime_part);
+		mime_part = subpart->data;
+		type = g_mime_object_get_content_type (mime_part);
 		
 		if (g_mime_content_type_is_type (type, "text", want_plain ? "plain" : "html")) {
 			/* we got what we came for */
 			*is_html = !want_plain;
 			
-			content = g_mime_part_get_content (mime_part, &len);
+			content = g_mime_part_get_content (GMIME_PART (mime_part), &len);
 			g_free (body);
 			body = g_strndup (content, len);
 			break;
@@ -665,21 +915,21 @@ multipart_get_body (GMimePart *multipart, gboolean want_plain, gboolean *is_html
 			body = NULL;
 		} else if (g_mime_content_type_is_type (type, "multipart", "*") && !first && !body) {
 			/* look in the multipart for the body */
-			body = multipart_get_body (mime_part, want_plain, is_html);
+			body = multipart_get_body (GMIME_MULTIPART (mime_part), want_plain, is_html);
 			
 			/* You are probably asking: "why don't we break here?"
 			 * The answer is because the real message body could
 			 * be a part after this multipart */
 		}
 		
-		child = child->next;
+		subpart = subpart->next;
 	}
 	
 	if (!body && first) {
 		/* we didn't get the type we wanted but still got the body */
 		*is_html = want_plain;
 		
-		content = g_mime_part_get_content (first, &len);
+		content = g_mime_part_get_content (GMIME_PART (first), &len);
 		body = g_strndup (content, len);
 	}
 	
@@ -697,8 +947,9 @@ multipart_get_body (GMimePart *multipart, gboolean want_plain, gboolean *is_html
  * specified by @want_plain.
  *
  * Returns the prefered form of the message body. Sets the value of
- * @is_html to TRUE if the part returned is in HTML format, otherwise
- * FALSE.
+ * @is_html to %TRUE if the part returned is in HTML format, otherwise
+ * %FALSE.
+ *
  * Note: This function is NOT guarenteed to always work as it
  * makes some assumptions that are not necessarily true. It is
  * recommended that you traverse the MIME structure yourself.
@@ -710,11 +961,11 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 	const char *content;
 	char *body = NULL;
 	size_t len = 0;
-
-	g_return_val_if_fail (message!=NULL, NULL);
-	g_return_val_if_fail (is_html!=NULL, NULL);
 	
-	type = g_mime_part_get_content_type (message->mime_part);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	g_return_val_if_fail (is_html != NULL, NULL);
+	
+	type = g_mime_object_get_content_type (message->mime_part);
 	if (g_mime_content_type_is_type (type, "text", "*")) {
 		/* this *has* to be the message body */
 		if (g_mime_content_type_is_type (type, "text", want_plain ? "plain" : "html"))
@@ -722,11 +973,11 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 		else
 			*is_html = want_plain;
 		
-		content = g_mime_part_get_content (message->mime_part, &len);
+		content = g_mime_part_get_content (GMIME_PART (message->mime_part), &len);
 		body = g_strndup (content, len);
 	} else if (g_mime_content_type_is_type (type, "multipart", "*")) {
 		/* lets see if we can find a body in the multipart */
-		body = multipart_get_body (message->mime_part, want_plain, is_html);
+		body = multipart_get_body (GMIME_MULTIPART (message->mime_part), want_plain, is_html);
 	}
 	
 	return body;
@@ -746,7 +997,7 @@ g_mime_message_get_headers (GMimeMessage *message)
 {
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	return g_mime_header_to_string (message->header->headers);
+	return g_mime_object_get_headers (GMIME_OBJECT (message));
 }
 
 
@@ -764,5 +1015,6 @@ g_mime_message_foreach_part (GMimeMessage *message, GMimePartFunc callback, gpoi
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (callback != NULL);
 	
-	g_mime_part_foreach (message->mime_part, callback, data);
+	if (GMIME_IS_MULTIPART (message->mime_part))
+		g_mime_multipart_foreach (GMIME_MULTIPART (message->mime_part), callback, data);
 }
