@@ -24,17 +24,24 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+#include <stdio.h>
+#include <ctype.h>
+#include <errno.h>
+
 #include "gmime-param.h"
 #include "gmime-table-private.h"
 #include "gmime-charset.h"
 #include "gmime-utils.h"
-#include <ctype.h>
+#include "gmime-iconv.h"
+#include "gmime-iconv-utils.h"
 
 #include "strlib.h"
 
-#define d(x)
+#define d(x) x
 #define w(x)
 
+
 static unsigned char tohex[16] = {
 	'0', '1', '2', '3', '4', '5', '6', '7',
 	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
@@ -67,18 +74,18 @@ g_mime_param_new (const char *name, const char *value)
 
 #define HEXVAL(c) (isdigit (c) ? (c) - '0' : tolower (c) - 'a' + 10)
 
-static char *
-hex_decode (const char *in, unsigned int len)
+static size_t
+hex_decode (const unsigned char *in, size_t len, unsigned char *out)
 {
-	register unsigned char *inptr, *outptr;
+	register const unsigned char *inptr;
+	register unsigned char *outptr;
 	const unsigned char *inend;
-	char *outbuf;
 	
-	inend = (const unsigned char *) in + len;
+	inptr = in;
+	inend = in + len;
 	
-	outptr = outbuf = g_malloc (len);
+	outptr = out;
 	
-	inptr = (unsigned char *) in;
 	while (inptr < inend) {
 		if (*inptr == '%') {
 			if (isxdigit (inptr[1]) && isxdigit (inptr[2])) {
@@ -92,41 +99,72 @@ hex_decode (const char *in, unsigned int len)
 	
 	*outptr = '\0';
 	
-	return outbuf;
+	return outptr - out;
 }
 
 /* an rfc2184 encoded string looks something like:
  * us-ascii'en'This%20is%20even%20more%20
  */
 static char *
-rfc2184_decode (const char *in, unsigned int len)
+rfc2184_decode (const char *in, size_t len)
 {
 	const char *inptr = in;
 	const char *inend = in + len;
-	/*const char *charset;*/
+	const char *charset = NULL;
 	char *decoded = NULL;
-	/*char *encoding;*/
+	char *charenc;
 	
 	/* skips to the end of the charset / beginning of the locale */
 	inptr = memchr (inptr, '\'', len);
 	if (!inptr)
 		return NULL;
 	
-#if 0
-	/* someday we'll need to do something with the charset... */
-	encoding = g_strndup (in, (unsigned) (inptr - in));
-	charset = g_mime_iconv_charset_name (encoding);
-	g_free (encoding);
-#endif
+	/* save the charset */
+	len = inptr - in;
+	charenc = g_alloca (len + 1);
+	memcpy (charenc, in, len);
+	charenc[len] = '\0';
+	charset = charenc;
 	
 	/* skip to the end of the locale */
-	inptr = memchr (inptr + 1, '\'', (unsigned) (inend - inptr - 1));
+	inptr = memchr (inptr + 1, '\'', (unsigned int) (inend - inptr - 1));
 	if (!inptr)
 		return NULL;
 	
 	inptr++;
-	if (inptr < inend)
-		decoded = hex_decode (inptr, (unsigned) (inend - inptr));
+	if (inptr < inend) {
+		len = inend - inptr;
+		if (strcasecmp (charset, "UTF-8") != 0) {
+			char *udecoded;
+			iconv_t cd;
+			
+			decoded = g_alloca (len + 1);
+			len = hex_decode (inptr, len, decoded);
+			
+			cd = g_mime_iconv_open ("UTF-8", charset);
+			if (cd == (iconv_t) -1) {
+				d(g_warning ("Cannot convert from %s to UTF-8, param display may "
+					     "be corrupt: %s", charset, g_strerror (errno)));
+				charset = g_mime_charset_locale_name ();
+				cd = g_mime_iconv_open ("UTF-8", charset);
+				if (cd == (iconv_t) -1)
+					return NULL;
+			}
+			
+			udecoded = g_mime_iconv_strndup (cd, decoded, len);
+			g_mime_iconv_close (cd);
+			
+			if (!udecoded) {
+				d(g_warning ("Failed to convert \"%.*s\" to UTF-8, display may be "
+					     "corrupt: %s", (int)len, decoded, g_strerror (errno)));
+			}
+			
+			decoded = udecoded;
+		} else {
+			decoded = g_malloc (len + 1);
+			hex_decode (inptr, len, decoded);
+		}
+	}
 	
 	return decoded;
 }
@@ -196,11 +234,11 @@ decode_quoted_string (const char **in)
 		
 		if (*inptr == '"') {
 			start++;
-			out = g_strndup (start, (unsigned) (inptr - start));
+			out = g_strndup (start, (unsigned int) (inptr - start));
 			inptr++;
 		} else {
 			/* string wasn't properly quoted */
-			out = g_strndup (start, (unsigned) (inptr - start));
+			out = g_strndup (start, (unsigned int) (inptr - start));
 		}
 	}
 	
@@ -222,7 +260,7 @@ decode_token (const char **in)
 		inptr++;
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned) (inptr - start));
+		return g_strndup (start, (unsigned int) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -260,7 +298,7 @@ decode_param_token (const char **in)
 		inptr++;
 	if (inptr > start) {
 		*in = inptr;
-		return g_strndup (start, (unsigned) (inptr - start));
+		return g_strndup (start, (unsigned int) (inptr - start));
 	} else {
 		return NULL;
 	}
@@ -366,6 +404,22 @@ decode_param (const char **in, char **paramp, char **valuep, gboolean *is_rfc218
 		}
 	}
 	
+	if (value && !g_utf8_validate (value, -1, NULL)) {
+		/* A (broken) mailer has sent us an unencoded 8bit value.
+		 * Attempt to save it by assuming it's in the user's
+		 * locale and converting to UTF-8 */
+		char *buf;
+		
+		buf = g_mime_iconv_locale_to_utf8 (value);
+		if (buf) {
+			g_free (value);
+			value = buf;
+		} else {
+			d(g_warning ("Failed to convert %s param value (\"%s\") to UTF-8: %s",
+				     param, value, g_strerror (errno)));
+		}
+	}
+	
 	if (param && value) {
 		*paramp = param;
 		*valuep = value;
@@ -393,8 +447,12 @@ decode_param_list (const char **in)
 		char *name, *value;
 		
 		/* invalid format? */
-		if (decode_param (&inptr, &name, &value, &is_rfc2184) != 0)
+		if (decode_param (&inptr, &name, &value, &is_rfc2184) != 0) {
+			if (*inptr == ';') {
+				continue;
+			}
 			break;
+		}
 		
 		if (is_rfc2184 && tail && !strcasecmp (name, tail->name)) {
 			/* rfc2184 allows a parameter to be broken into multiple parts
@@ -565,15 +623,15 @@ g_mime_param_append_param (GMimeParam *params, GMimeParam *param)
 static char *
 encode_param (const unsigned char *in, gboolean *encoded)
 {
-	const unsigned char *inptr;
-	char *outstr, *charset;
-	int encoding;
+	register const unsigned char *inptr;
+	unsigned char *outbuf = NULL;
+	iconv_t cd = (iconv_t) -1;
+	const char *charset = NULL;
+	char *outstr;
 	GString *out;
 	
 	*encoded = FALSE;
 	
-	g_return_val_if_fail (in != NULL, NULL);
-	
 	for (inptr = in; *inptr && inptr - in < GMIME_FOLD_LEN; inptr++)
 		if (*inptr > 127)
 			break;
@@ -581,29 +639,43 @@ encode_param (const unsigned char *in, gboolean *encoded)
 	if (*inptr == '\0')
 		return g_strdup (in);
 	
+	if (*inptr > 127)
+		charset = g_mime_charset_best (in, strlen (in));
+	
+	if (!charset)
+		charset = "iso-8859-1";
+	
+	if (strcasecmp (charset, "UTF-8") != 0)
+		cd = g_mime_iconv_open (charset, "UTF-8");
+	
+	if (cd != (iconv_t) -1) {
+		outbuf = g_mime_iconv_strdup (cd, in);
+		g_mime_iconv_close (cd);
+		inptr = outbuf;
+	} else {
+		charset = "UTF-8";
+		inptr = in;
+	}
+	
+	/* FIXME: set the 'language' as well, assuming we can get that info...? */
 	out = g_string_new ("");
-	inptr = in;
-	encoding = 0;
+	g_string_sprintfa (out, "%s''", charset);
+	
 	while (inptr && *inptr) {
-		unsigned int c = *inptr++ & 0xff;
+		unsigned char c = *inptr++;
 		
-		if (c > 127 && c < 256) {
-			encoding = MAX (encoding, 1);
+		/* FIXME: make sure that '\'', '*', and ';' are also encoded */
+		
+		if (c > 127) {
 			g_string_sprintfa (out, "%%%c%c", tohex[(c >> 4) & 0xf], tohex[c & 0xf]);
 		} else if (is_lwsp (c) || !(gmime_special_table[c] & IS_ESAFE)) {
 			g_string_sprintfa (out, "%%%c%c", tohex[(c >> 4) & 0xf], tohex[c & 0xf]);
 		} else {
-			g_string_append_c (out, (char) c);
+			g_string_append_c (out, c);
 		}
 	}
 	
-	if (encoding) {
-		charset = g_strdup_printf ("%s''", g_mime_charset_locale_name ());
-		g_string_prepend (out, charset);
-		g_free (charset);
-	} else {
-		g_string_prepend (out, "iso-8859-1''");
-	}
+	g_free (outbuf);
 	
 	outstr = out->str;
 	g_string_free (out, FALSE);
@@ -629,17 +701,6 @@ g_string_append_len_quoted (GString *out, const char *in, size_t len)
 	g_string_append_c (out, '"');
 }
 
-static void
-g_string_append_len (GString *out, const char *in, size_t len)
-{
-	char *buf;
-	
-	buf = alloca (len + 1);
-	strlcpy (buf, in, len);
-	
-	g_string_append (out, buf);
-}
-
 static void
 param_list_format (GString *out, GMimeParam *param, gboolean fold)
 {
