@@ -96,6 +96,13 @@ find_matching_sent_article (const Article * article)
 			if (!strcmp (body_1, body_2))
 				retval = compare;
 		}
+		else
+		if (body_1==NULL)
+		{
+			/* article's body is NULL, meaning it hasn't been
+			   read yet. Risk it and cancel this one */
+			retval = compare;
+		}
 		g_free (body_1);
 		g_free (body_2);
 	}
@@ -111,13 +118,49 @@ find_matching_sent_article (const Article * article)
 static gint
 article_cancel_run (TaskFuncPtr* task, gpointer user_data)
 {
-        int status = nntp_cancel (STATUS_ITEM(task),
-	                          ARTICLE(user_data),
-	                          TASK(task)->sock);
+	Article * article = ARTICLE(user_data);
+        int       status = nntp_cancel (STATUS_ITEM(task),
+			                article,
+	                                TASK(task)->sock);
 
 	if (status == TASK_SUCCESS)
 	{
-		/* FIXME: add something to the article to denote that it's been cancelled */
+		gchar        * subject = g_strdup (article_get_subject (article));
+		gchar        * message_id = g_strdup (article_get_message_id (article));
+
+#if 0
+		/* FIXME: this doesn't seem to work: group_get_article_by_message_id */
+		/* doesn't return the article. Not sure why.                         */
+
+		const gchar  * newsgroups = article_get_extra_header (article, HEADER_NEWSGROUPS);
+		gchar       ** groups = g_strsplit (newsgroups, ",", 0);
+		guint          i;
+
+		for (i=0; groups[i]; i++) {
+
+			Group     * group = server_get_named_group (TASK(task)->server, groups[i]);
+			GPtrArray * articles = group_get_article_array (group);
+			guint       n;
+		      
+			group_ref_articles (group, STATUS_ITEM(task));
+
+			article = (Article *) group_get_article_by_message_id (group, message_id);
+			if (article) {
+				group_remove_article (group, article);
+			}
+
+			group_unref_articles (group, STATUS_ITEM(task));
+		}
+
+		g_strfreev (groups);
+
+#endif
+		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) cancelled"),
+			subject, message_id);
+
+		g_free (subject);
+		g_free (message_id);
+
 	}
 
 	return 0;
@@ -125,7 +168,7 @@ article_cancel_run (TaskFuncPtr* task, gpointer user_data)
 static gchar*
 article_cancel_describe (const StatusItem * status)
 {
-	return g_strdup (_("Cancelling article"));
+	return g_strdup (_("Canceling article"));
 }
 static void
 article_cancel_dtor (gpointer data)
@@ -141,14 +184,11 @@ article_cancel (Article * article)
 {
 	Server * server;
 	Article * ours;
-	const gchar * newsgroups;
 
 	/* sanity clause */
 	g_return_if_fail (article != NULL);
 	server = article->group==NULL ? NULL : article->group->server;
 	g_return_if_fail (server != NULL);
-	newsgroups = article_get_header (article, HEADER_NEWSGROUPS);
-	g_return_if_fail (is_nonempty_string (newsgroups));
 
 	ours = find_matching_sent_article (article);
 	if (ours == NULL)
@@ -195,7 +235,7 @@ article_supersede (const Article * article)
 		gchar * message_id = gnksa_generate_message_id_from_email_addr (ours->author_addr);
 		Article * super = article_dup (sendlater, ours);
 		article_set_header (super, HEADER_MESSAGE_ID, message_id, DO_CHUNK);
-		article_set_header (super, HEADER_SUPERCEDES, article_get_message_id(article), DO_CHUNK);
+		article_set_header (super, HEADER_SUPERSEDES, article_get_message_id(article), DO_CHUNK);
 
 		/* edit the copy */
 		message_edit_window (super);
@@ -241,8 +281,7 @@ article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 		group_ref_articles (folder, NULL);
 
 		/* create an array of articles to add. */
-		addme = g_ptr_array_new ();
-		pan_g_ptr_array_reserve (addme, articles->len);
+		addme = g_ptr_array_sized_new (articles->len);
 		for (i=0; i!=articles->len; ++i)
 		{
 			const Article * a_old;
@@ -309,120 +348,56 @@ article_copy_articles_to_folder (Group * folder, Article ** articles, gint qty)
 ***/
 
 static void
-open_article (const Article * a)
-{
-	gchar * file;
-	const gchar * files;
-
-	g_return_if_fail (article_is_valid(a));
-
-	files = article_get_header (a, PAN_HEADER_FILENAME);
-	while ((file = get_next_token_str (files, '\n', &files))) {
-		open_outside_file (file);
-		g_free (file);
-	}
-
-}
-
-static void
-open_decoded_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
-{
-	gint status = GPOINTER_TO_INT(call_arg);
-
-	if (status == TASK_SUCCESS) {
-		TaskSave * task = TASK_SAVE(call_obj);
-		if (task!=NULL && task->articles!=NULL) {
-			const Article * a = ARTICLE(task->articles->data);
-			if (article_is_valid(a))
-				open_article (a);
-		}
-	}
-}
-
-static void
-decode_impl (Article * a,
-             gboolean save_bodies,
-             gboolean save_attachments,
-             gboolean open_attachments)
+save_attachments_impl (Article ** articles, int len)
 {
+	int i;
 	Task * task;
-	GSList * articles;
-	const gchar * files;
+	GSList * tasks = NULL;
 	debug_enter ("decode_impl");
 
-	g_return_if_fail (article_is_valid(a));
-	g_return_if_fail (save_bodies || save_attachments || open_attachments);
+	g_return_if_fail (articles_are_valid ((const Article**)articles, len));
 
-	/* got to save before you can open */
-	if (open_attachments)
-		save_attachments = TRUE;
+	for (i=0; i<len; ++i)
+	{
+		Article * a = articles[i];
 
-	/* if user is opening the files & we already have them... */
-	files = article_get_header (a, PAN_HEADER_FILENAME);
-	if (open_attachments && is_nonempty_string(files)) {
-		open_article (a);
-		debug_exit ("decode_impl");
-		return;
+		/* build an array of this article and all its children. */
+		GSList * articles;
+		articles = g_slist_copy (a->threads);
+		articles = g_slist_prepend (articles, a);
+
+		/* queue for decode */
+		task = TASK(task_save_new (a->group, article_get_subject(a), articles));
+		task_save_set_attachments (TASK_SAVE(task), NULL, NULL);
+		tasks = g_slist_prepend (tasks, task);
+
+		g_slist_free (articles);
 	}
 
-	/* build an array of this article and all its children. */
-	articles = g_slist_copy (a->threads);
-	articles = g_slist_prepend (articles, a);
-
-	/* queue for decode */
-	task = TASK(task_save_new (a->group, article_get_subject(a), articles));
-	if (save_bodies)
-		task_save_set_bodies (TASK_SAVE(task), NULL, NULL);
-	if (save_attachments)
-		task_save_set_attachments (TASK_SAVE(task), NULL, NULL);
-	if (open_attachments)
-		pan_callback_add (task->task_ran_callback, open_decoded_cb, NULL);
-	task_save_validate_and_queue_or_self_destruct (TASK_SAVE(task), -1);
+	if (tasks != NULL)
+	{
+		tasks = g_slist_reverse (tasks);
+		queue_insert_tasks (tasks, -1);
+		tasks = NULL;
+	}
 
 	/* cleanup this loop */
-	g_slist_free (articles);
 	debug_exit ("decode_impl");
 }
 
-static void
-seleted_decode_impl (gboolean save_bodies,
-                     gboolean save_attachments,
-                     gboolean open_attachments)
-{
-	guint i;
-	GPtrArray * sel;
-	debug_enter ("seleted_decode_impl");
-
-	/* decode each selected article */
-	sel = articlelist_get_selected_articles_nolock ();
-	for (i=0; i<sel->len; ++i)
-		decode_impl (ARTICLE(g_ptr_array_index(sel,i)),
-		             save_bodies,
-		             save_attachments,
-		             open_attachments);
-	g_ptr_array_free (sel, TRUE);
-
-	debug_exit ("seleted_decode_impl");
-}
-
-void
-article_action_selected_save (void)
-{
-	seleted_decode_impl (TRUE, TRUE, FALSE);
-}
-
 void
 article_action_selected_save_attachments (void)
-{		
-	seleted_decode_impl (FALSE, TRUE, FALSE);
-}
-
-void
-article_action_selected_open_attachments (void)
 {
-	seleted_decode_impl (FALSE, TRUE, TRUE);
-}
+	GPtrArray * sel;
+	debug_enter ("article_action_selected_save_attachments");
 
+	/* decode each selected article */
+	sel = articlelist_get_selected_articles_nolock ();
+	save_attachments_impl ((Article**)sel->pdata, sel->len);
+	g_ptr_array_free (sel, TRUE);
+
+	debug_exit ("article_action_selected_save_attachments");
+}
 
 void
 article_action_selected_save_as (void)
@@ -584,22 +559,6 @@ delete_articles (GPtrArray * articles)
 	debug_exit ("article_action_delete_articles");
 }
 
-void
-article_action_delete_selected_threads (void)
-{
-	GPtrArray * sel;
-	GPtrArray * all;
-	debug_enter ("article_action_delete_selected_threads");
-
-       	sel = articlelist_get_selected_articles_nolock ();
-       	all = article_get_unique_threads (sel, GET_WHOLE_THREAD);
-	delete_articles (all);
-	g_ptr_array_free (all, TRUE);
-	g_ptr_array_free (sel, TRUE);
-
-	debug_exit ("article_action_delete_selected_threads");
-}
-
 void
 article_action_delete_selected_articles (void)
 {
