@@ -24,12 +24,20 @@
 #include <pan/base/article.h>
 #include <pan/base/group.h>
 
-void      article_cancel                    (Article         * article);
+void      article_cancel                             (Article         * article);
 
-void      article_supersede                 (const Article   * article);
+void      article_supersede                          (const Article   * article);
 
-gboolean  article_copy_articles_to_folder   (Group           * folder,
-                                             Article        ** articles,
-                                             int               qty);
+gboolean  article_copy_articles_to_folder            (Group           * folder,
+                                                      Article        ** articles,
+                                                      gint              qty);
+
+void      article_action_selected_save_attachments   (void);
+
+void      article_action_selected_save_as            (void);
+
+void      article_action_delete_selected_articles    (void);
+
+void      article_action_edit_selected               (void);
 
 #endif
