@@ -21,7 +21,8 @@
  * 
  */
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/article.h>
 #include <pan/base/pan-i18n.h>
@@ -34,6 +35,7 @@
 #include <pan/filters/filter-manager.h>
 
 #include <pan/article-filter-ui.h>
+#include <pan/globals.h>
 #include <pan/util.h>
 
 typedef struct
@@ -150,38 +152,46 @@ add_to_filters (ArticleFilterDialog * d)
 
 
 static gint
-dialog_close_cb (GnomeDialog * dialog, gpointer data)
+dialog_close_cb (GtkDialog * dialog, gpointer data)
 {
 	g_free (data);
 	return FALSE;
 }
 
 static void
-dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
+dialog_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
 	ArticleFilterDialog * d = (ArticleFilterDialog*) data;
 
-	if (index==0)
+	if (response==GTK_RESPONSE_ACCEPT)
+	{
 		add_to_filters (d);
-	if (index==0 || index==1)
-		gnome_dialog_close (dialog);
-	if (index==2) {
-		GtkWidget * l = gnome_message_box_new (_(
-"Addresses and Subjects can be changed before being added\n"
+	}
+	if (response==GTK_RESPONSE_HELP)
+	{
+		GtkWidget * w = gtk_message_dialog_new (GTK_WINDOW(dialog), GTK_DIALOG_DESTROY_WITH_PARENT,
+				 GTK_MESSAGE_INFO,
+				 GTK_BUTTONS_CLOSE,
+				 "%s", 
+		_("Addresses and Subjects can be changed before being added\n"
 "to the Spam/Bozo filters.\n"
 "\n"
 "For example, a subject of `-->MAKE MONEY FAST<--' will\n"
-"likey find more spam if shortened to just `MAKE MONEY FAST'.\n"
+"likely find more spam if shortened to just `MAKE MONEY FAST'.\n"
 "\n"
 "Likewise, \"Dudley Dursley\" <ddursley@spambot.com>' could\n"
 "be shortened to `<ddurseley@spambot.com>' or even `<*@spambot.com>'.\n"
 "\n"
 "Wildcards are supported and comparisons are case-insensitive.\n"
 "For more complex pattern matching, enable the regular expression\n"
-"button.  http://pan.rebelbase.com/regexp.txt for a tutorial.\n"),
-			GNOME_MESSAGE_BOX_INFO, GNOME_STOCK_BUTTON_OK, NULL);
-		gnome_dialog_set_parent (GNOME_DIALOG(l), GTK_WINDOW(d->dialog));
-	        gtk_widget_show_all (l);
+"button.  http://pan.rebelbase.com/regexp.txt for a tutorial.\n"));
+	        gtk_widget_show_all (w);
+		g_signal_connect_swapped (GTK_OBJECT(w), "response", 
+		                          G_CALLBACK (gtk_widget_destroy), GTK_OBJECT(w));
+	}
+	if (response==GTK_RESPONSE_ACCEPT || response==GTK_RESPONSE_CLOSE)
+	{
+		gtk_widget_destroy (GTK_WIDGET(dialog));
 	}
 }
 
@@ -193,7 +203,7 @@ activate_me_cb (GtkWidget * w, gpointer user_data)
 }
 
 GtkWidget*
-article_filter_dialog_new (const Article * a)
+article_filter_dialog_new (GtkWidget * window, const Article * a)
 {
 	gchar * pch;
 	gint row = 0;
@@ -201,26 +211,29 @@ article_filter_dialog_new (const Article * a)
 	GtkWidget * t;
 	GtkWidget * tb;
 	ArticleFilterDialog * d;
+	gchar author[256];
 	const gint atch = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
 	const int wide_enough = 250;
 
 	/* dialog */
 	d = g_new0 (ArticleFilterDialog, 1);
-	w = d->dialog = gnome_dialog_new (_("Pan: Filter Out Articles"),
-	                                  GNOME_STOCK_BUTTON_OK,
-	                                  GNOME_STOCK_BUTTON_CLOSE,
-	                                  GNOME_STOCK_BUTTON_HELP,
-	                                  NULL);
+	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Filter Out Articles"),
+	                                             GTK_WINDOW(window),
+	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                             GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+	                                             GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+	                                             GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+	                                             NULL);
 	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT(w), "close", GTK_SIGNAL_FUNC(dialog_close_cb), d);
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(dialog_clicked_cb), d);
+	g_signal_connect (w, "close", G_CALLBACK(dialog_close_cb), d);
+	g_signal_connect (w, "response", G_CALLBACK(dialog_response_cb), d);
 
 	/* main table */
 	t = gtk_table_new (3, 3, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE(t), GNOME_PAD_SMALL);
-	gtk_table_set_col_spacings (GTK_TABLE(t), GNOME_PAD_SMALL);
-	gtk_container_set_border_width (GTK_CONTAINER(t), GNOME_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(w)->vbox), t, TRUE, TRUE, 0);
+	gtk_table_set_row_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
+	gtk_table_set_col_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(t), GUI_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(w)->vbox), t, TRUE, TRUE, 0);
 
 	/* bozo author */
 	d->bozo_author_tb = tb = w = gtk_check_button_new_with_label (_("Add Author to Bozo Filter"));
@@ -228,14 +241,13 @@ article_filter_dialog_new (const Article * a)
 	w = gtk_entry_new ();
 	gtk_widget_set_usize (w, wide_enough, -1);
 	d->bozo_author_entry = w;
-	pch = article_get_author_str (a);
-	gtk_entry_set_text (GTK_ENTRY(w), pch);
-	g_free (pch);
-	gtk_signal_connect (GTK_OBJECT(w), "changed", activate_me_cb, tb);
+	article_get_author_str (a, author, sizeof(author));
+	pan_gtk_entry_set_text (w, author);
+	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("regexp"));
 	d->bozo_author_regexp_tb = w;
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", activate_me_cb, tb);
+	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
 	++row;
 
@@ -245,14 +257,13 @@ article_filter_dialog_new (const Article * a)
 	w = gtk_entry_new ();
 	gtk_widget_set_usize (w, wide_enough, -1);
 	d->spam_author_entry = w;
-	pch = article_get_author_str (a);
-	gtk_entry_set_text (GTK_ENTRY(w), pch);
-	g_free (pch);
-	gtk_signal_connect (GTK_OBJECT(w), "changed", activate_me_cb, tb);
+	article_get_author_str (a, author, sizeof(author));
+	pan_gtk_entry_set_text (w, author);
+	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("regexp"));
 	d->spam_author_regexp_tb = w;
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", activate_me_cb, tb);
+	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
 	++row;
 
@@ -264,13 +275,13 @@ article_filter_dialog_new (const Article * a)
 	d->spam_subject_entry = w;
 	pch = g_strdup (article_get_subject(a));
 	g_strstrip (pch);
-	gtk_entry_set_text (GTK_ENTRY(w), pch);
+	pan_gtk_entry_set_text (w, pch);
 	g_free (pch);
-	gtk_signal_connect (GTK_OBJECT(w), "changed", activate_me_cb, tb);
+	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
 	w = gtk_toggle_button_new_with_label (_("regexp"));
 	d->spam_subject_regexp_tb = w;
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", activate_me_cb, tb);
+	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
 	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
 
 	return d->dialog;
