@@ -23,10 +23,12 @@
 #include <config.h>
 #include <string.h>
 
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/article.h>
 #include <pan/base/fnmatch.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/article-find.h>
@@ -35,6 +37,12 @@
 #include <pan/gui.h>
 #include <pan/util.h>
 
+enum
+{
+	PAN_RESPONSE_FIND_NEXT = 1,
+	PAN_RESPONSE_FIND_PREV = 2
+};
+
 static GtkWidget *find_dialog;
 
 static void
@@ -54,6 +62,7 @@ typedef struct
 
 	const gchar *author;
 	const gchar *subject;
+	const gchar *message_id;
 } 
 header_compare_data;
 
@@ -83,6 +92,10 @@ compare_article (const Article* article,
 	if (header->subject && fnmatch(header->subject,article_get_subject(article),flags))
 		return -1;
 
+	/* make sure the message-id (if user specified one) matches.. */
+	if (header->message_id && fnmatch(header->message_id,article_get_message_id(article),flags))
+		return -1;
+
 	/* hooray, a match */
 	return 0;
 }
@@ -90,12 +103,14 @@ compare_article (const Article* article,
 static void
 find_article (const gchar* author,
               const gchar* subject,
+              const gchar* message_id,
               gboolean case_sensitive,
               gboolean unread_only,
               gboolean reverse)
 {
 	gchar * author_tmp;
 	gchar * subject_tmp;
+	gchar * message_id_tmp;
 	GtkCTreeNode * sel;
 	GtkCTreeNode * tmp;
 	GtkCTree* tree = GTK_CTREE(Pan.article_ctree);
@@ -120,12 +135,21 @@ find_article (const gchar* author,
 			author_tmp = g_strdup_printf ("*%s*", author);
 	}
 
+	message_id_tmp = NULL;
+	if (message_id != NULL) {
+		if (strchr(message_id,'*'))
+			message_id_tmp = g_strdup (message_id);
+		else
+			message_id_tmp = g_strdup_printf ("*%s*", message_id);
+	}
+
 	/* sanity checks */
 	g_return_if_fail (articlelist_get_group()!=NULL);
 
 	/* set up search function struct */
 	header_data.author = author_tmp;
 	header_data.subject = subject_tmp;
+	header_data.message_id = message_id_tmp;
 	header_data.unread_only = unread_only;
 	header_data.case_sensitive = case_sensitive;
 
@@ -141,7 +165,7 @@ find_article (const gchar* author,
 		/* next node */
 		tmp = reverse
 			? articlelist_node_prev (tmp, FALSE)
-			: articlelist_node_next (tmp, FALSE);
+			: articlelist_node_next (tmp);
 		if (!tmp) { /* wrap */
 			int n = reverse
 				? GTK_CLIST(tree)->rows-1
@@ -151,7 +175,7 @@ find_article (const gchar* author,
 				break;
 			tmp = reverse
 				? articlelist_node_prev (tmp, FALSE)
-				: articlelist_node_next (tmp, FALSE);
+				: articlelist_node_next (tmp);
 		}
 		if (tmp==sel)
 			break;
@@ -165,7 +189,7 @@ find_article (const gchar* author,
 
 	/* select the node.. */
 	if (tmp!=sel)
-		articlelist_set_selected_nodes (&tmp, 1);
+		articlelist_set_selected_nodes_nolock (&tmp, 1);
 
 	/* cleanup */
 	g_free (author_tmp);
@@ -174,33 +198,29 @@ find_article (const gchar* author,
 
 static gchar * find_article__author = NULL;
 static gchar * find_article__subject = NULL;
+static gchar * find_article__message_id = NULL;
 static gboolean find_article__case_sensitive = FALSE;
 static gboolean find_article__unread_only = FALSE;
 static gboolean find_article__reverse = FALSE;
 
 
 static void
-find_dialog_clicked_cb (GnomeDialog    * dialog,
-                        int              button,
-                        GtkWidget      * table)
+find_dialog_response_cb (GtkDialog     * dialog,
+                         int             response,
+                         GtkWidget     * table)
 {
- 	g_return_if_fail (button==0 || button==1 || button==2);
 	g_return_if_fail (table!=NULL);
 
-	if (button==2) /* "close" button */
-	{
-		pan_lock();
-		gtk_widget_unmap (GTK_WIDGET(find_dialog));
-		pan_unlock();
-	}
-	else if (button==0 || button==1) /* "find next" button */
+	if (response==PAN_RESPONSE_FIND_PREV || response==PAN_RESPONSE_FIND_NEXT)
 	{
 		GtkWidget * author_entry;
 		GtkWidget * subject_entry;
+		GtkWidget * message_id_entry;
 		GtkWidget * case_sensitive_checkbox;
 		GtkWidget * unread_only_checkbox;
 		gchar * author;
 		gchar * subject;
+		gchar * message_id;
 		gboolean case_sensitive;
 		gboolean unread_only;
 
@@ -220,6 +240,14 @@ find_dialog_clicked_cb (GnomeDialog    * dialog,
 		if (!*subject)
 			replace_gstr (&subject, NULL);
 
+		/* get message-id to search for, or NULL if none...*/
+		message_id_entry = (GtkWidget*) gtk_object_get_data ( GTK_OBJECT(find_dialog), "message-id" );
+		g_return_if_fail ( message_id_entry != NULL );
+		message_id = g_strdup ( gtk_entry_get_text ( GTK_ENTRY(message_id_entry) ) );
+		g_strstrip (message_id);
+		if (!*message_id)
+			replace_gstr (&message_id, NULL);
+
 		/* get case sensitivity... */
 		case_sensitive_checkbox = (GtkWidget*) gtk_object_get_data ( GTK_OBJECT(find_dialog), "case" );
 		g_return_if_fail ( case_sensitive_checkbox!=NULL );
@@ -233,12 +261,17 @@ find_dialog_clicked_cb (GnomeDialog    * dialog,
 		/* update the find_article fields with these widget values */
 		replace_gstr (&find_article__author, author);
 		replace_gstr (&find_article__subject, subject);
+		replace_gstr (&find_article__message_id, message_id);
 		find_article__case_sensitive = case_sensitive;
 		find_article__unread_only = unread_only;
 
-		find_article__reverse = button==0;
+		find_article__reverse = response == PAN_RESPONSE_FIND_PREV;
 		articlelist_find_next_cb ( );
 	}
+	else
+	{
+		gtk_widget_unmap (GTK_WIDGET(find_dialog));
+	}
 }
 
 void
@@ -249,7 +282,7 @@ articlelist_find_next_cb (void)
 		return;
 
 	/* user may have hit "find again" without hitting "find" first */
-	if (!find_article__subject && !find_article__author) {
+	if (!find_article__subject && !find_article__author && !find_article__message_id) {
 		articlelist_find_text_cb();
 		return;
 	}
@@ -257,6 +290,7 @@ articlelist_find_next_cb (void)
 	/* got a subject and/or an author.. let's go find 'em */
 	find_article (find_article__author,
 	              find_article__subject,
+	              find_article__message_id,
 	              find_article__case_sensitive,
 	              find_article__unread_only,
 	              find_article__reverse);
@@ -273,65 +307,91 @@ articlelist_find_text_cb (void)
 	GtkWidget * author_entry;
 	GtkWidget * subject_label;
 	GtkWidget * subject_entry;
+	GtkWidget * message_id_label;
+	GtkWidget * message_id_entry;
 	GtkWidget * case_sensitive_checkbutton;
 	GtkWidget * unread_only_checkbutton;
+	guint       row;
 
 	/* There can be only one! (find window) */
 	if (find_dialog != NULL)
 	{
-		gnome_dialog_set_parent (GNOME_DIALOG(find_dialog),
-		                         GTK_WINDOW(Pan.window));
 		gtk_widget_show_all (find_dialog);
-		gtk_widget_map (GTK_WIDGET(find_dialog));
+		gtk_widget_map (find_dialog);
 		return;
 	}
-	
-	find_dialog = gnome_dialog_new (_("Pan: Find Message"),
-					GNOME_STOCK_PIXMAP_BACK,
-					GNOME_STOCK_PIXMAP_FORWARD,
-					GNOME_STOCK_PIXMAP_CLOSE, NULL);
-	gnome_dialog_set_default ( GNOME_DIALOG(find_dialog), 0 );
 
-	table = gtk_table_new (4, 2, FALSE);
+	find_dialog = gtk_dialog_new_with_buttons (_("Pan: Find Message"),
+	                                           GTK_WINDOW(Pan.window),
+	                                           GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                           GTK_STOCK_GO_BACK, PAN_RESPONSE_FIND_PREV,
+	                                           GTK_STOCK_GO_FORWARD, PAN_RESPONSE_FIND_NEXT,
+	                                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+	                                           NULL);
 
+	table = gtk_table_new (5, 2, FALSE);
+
+	row = 0;
 	subject_label = gtk_label_new ( _("Subject:") );
 	gtk_table_attach ( GTK_TABLE(table),
 			   subject_label,
 			   0, 1,
-			   0, 1,
+			   row, row+1,
 			   0,
 			   0,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+			   GUI_PAD_SMALL, GUI_PAD_SMALL );
 
 	subject_entry = gtk_entry_new ( );
 	gtk_object_set_data (GTK_OBJECT(find_dialog), "subject", subject_entry);
 	gtk_table_attach (GTK_TABLE(table),
 			  subject_entry,
 			  1, 2,
-			  0, 1,
+			  row, row+1,
 			  GTK_EXPAND|GTK_FILL|GTK_SHRINK,
 			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+			  GUI_PAD_SMALL, GUI_PAD_SMALL );
 
+	++row;
 	author_label = gtk_label_new (_("Author:"));
 	gtk_table_attach (GTK_TABLE(table),
 			  author_label,
 			  0, 1,
-			  1, 2,
+			  row, row+1,
 			  0,
 			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+			  GUI_PAD_SMALL, GUI_PAD_SMALL );
 
 	author_entry = gtk_entry_new ( );
 	gtk_object_set_data ( GTK_OBJECT(find_dialog), "author", author_entry );
 	gtk_table_attach (GTK_TABLE(table),
 			  author_entry,
 			  1, 2,
-			  1, 2,
+			  row, row+1,
 			  GTK_EXPAND|GTK_FILL|GTK_SHRINK,
 			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+			  GUI_PAD_SMALL, GUI_PAD_SMALL );
 
+	++row;
+	message_id_label = gtk_label_new (_("Message-ID:"));
+	gtk_table_attach (GTK_TABLE(table),
+			message_id_label,
+			0, 1,
+			row, row+1,
+			0,
+			0,
+			GUI_PAD_SMALL, GUI_PAD_SMALL );
+
+	message_id_entry = gtk_entry_new ();
+	gtk_object_set_data ( GTK_OBJECT(find_dialog), "message-id", message_id_entry );
+	gtk_table_attach (GTK_TABLE(table),
+			message_id_entry,
+			1, 2,
+			row, row+1,
+			GTK_EXPAND|GTK_FILL|GTK_SHRINK,
+			0,
+			GUI_PAD_SMALL, GUI_PAD_SMALL );
+
+	++row;
 	unread_only_checkbutton =
 		gtk_check_button_new_with_label (_("Unread Only"));
 	gtk_toggle_button_set_active (
@@ -341,34 +401,31 @@ articlelist_find_text_cb (void)
 	gtk_table_attach ( GTK_TABLE(table),
 			   unread_only_checkbutton,
 			   0, 2,
-			   2, 3,
+			   row, row+1,
 			   GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			   GTK_EXPAND | GTK_FILL,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+			   GUI_PAD_SMALL, GUI_PAD_SMALL );
 
+	++row;
 	case_sensitive_checkbutton = gtk_check_button_new_with_label ( _("Case Sensitive") );
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(case_sensitive_checkbutton), FALSE );
 	gtk_object_set_data ( GTK_OBJECT(find_dialog), "case", case_sensitive_checkbutton );
 	gtk_table_attach ( GTK_TABLE(table),
 			   case_sensitive_checkbutton,
 			   0, 2,
-			   3, 4,
+			   row, row+1,
 			   GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			   GTK_EXPAND | GTK_FILL,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
-
+			   GUI_PAD_SMALL, GUI_PAD_SMALL );
 	
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (find_dialog)->vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(find_dialog)->vbox), table, FALSE, FALSE, 0);
 	
-	gtk_signal_connect (GTK_OBJECT(find_dialog), "clicked",
-			    GTK_SIGNAL_FUNC(find_dialog_clicked_cb),
-			    table);
-	gtk_signal_connect (GTK_OBJECT(find_dialog), "destroy",
-			    GTK_SIGNAL_FUNC(find_dialog_destroy_cb), NULL);
+	g_signal_connect (find_dialog, "response",
+	                  G_CALLBACK(find_dialog_response_cb), table);
+	g_signal_connect (find_dialog, "destroy",
+	                  G_CALLBACK(find_dialog_destroy_cb), NULL);
 
 	gtk_widget_grab_focus (subject_entry);
 
-	gnome_dialog_set_parent (GNOME_DIALOG(find_dialog),
-	                         GTK_WINDOW(Pan.window));
 	gtk_widget_show_all (find_dialog);
 }
