@@ -49,29 +49,32 @@ extern GdkColor watched_color;
 ***  State Info
 **/
 
-void           articlelist_set_filter                   (const Filter*, FilterShow);
 Group*         articlelist_get_group                    (void);
 void           articlelist_set_threaded                 (gboolean threaded_on);
 GtkCTreeNode*  articlelist_get_selected_node            (void);
 Article*       articlelist_get_selected_article_nolock  (void);
 GPtrArray*     articlelist_get_selected_articles_nolock (void);
+guint          articlelist_get_selected_count_nolock    (void);
 
 /**
 ***  Stepping Through the Articles
 **/
 
+gboolean articlelist_has_prev_read (void);
+
 GtkCTreeNode* articlelist_node_next (GtkCTreeNode*);
 GtkCTreeNode* articlelist_node_prev (GtkCTreeNode*, gboolean sibling_only);
 void articlelist_set_selected_nodes_nolock (GtkCTreeNode**, int qty);
 
 void articlelist_reset_style_nolock        (void);
 
-void articlelist_read_next                 (gboolean cached_only, gboolean read);
+void articlelist_read_next                 (void);
 void articlelist_read_next_unread          (void);
 void articlelist_read_next_thread          (void);
 void articlelist_read_next_unread_thread   (void);
 void articlelist_read_prev                 (void);
 void articlelist_read_prev_thread          (void);
+void articlelist_read_prev_read            (void);
 void articlelist_read_top_of_thread        (void);
 void articlelist_read_parent               (void);
 void articlelist_read_message_id           (const gchar * message_id);
@@ -80,7 +83,7 @@ void articlelist_read_message_id           (const gchar * message_id);
 ***  Contents
 **/
 
-void articlelist_set_group_nolock (Group*);
+void articlelist_set_group (Group*);
 void articlelist_refresh (void);
 void articlelist_read_selected (void);
 
@@ -90,8 +93,6 @@ void articlelist_read_selected (void);
 
 void articlelist_expand_selected_threads (void);
 void articlelist_collapse_selected_threads (void);
-void articlelist_expand_all_threads (void);
-void articlelist_collapse_all_threads (void);
 
 void articlelist_select_all_nolock (void);
 void articlelist_deselect_all_nolock (void);
@@ -125,4 +126,23 @@ PanCallback* articlelist_get_group_changed_callback (void);
  */
 extern PanCallback* articlelist_selection_changed;
 
+
+/***
+****
+***/
+
+void articlelist_selected_mark_read_nolock (void);
+void articlelist_selected_mark_unread_nolock (void);
+void articlelist_download_flagged_bodies (void);
+void articlelist_selected_flag_for_dl_nolock (void);
+void articlelist_selected_unflag_for_dl_nolock (void);
+void articlelist_selected_download_nolock (void);
+void articlelist_selected_thread_watch_nolock (void);
+void articlelist_selected_thread_ignore_nolock (void);
+void articlelist_spamize_nolock (void);
+void articlelist_selected_copy_to_folder_nolock (void);
+void articlelist_selected_cancel (void);
+void articlelist_selected_supersede (void);
+
+
 #endif /* __ARTICLE_LIST_H__ */
