@@ -1,4 +1,3 @@
-#include <pthread.h>
 #include <pan/base/argset.h>
 
 ArgSet*
@@ -37,6 +36,12 @@ argset_new5 (gpointer p1, gpointer p2, gpointer p3, gpointer p4, gpointer p5) {
 	g_ptr_array_add (argset->args, p5);
 	return argset;
 }
+ArgSet*
+argset_new6 (gpointer p1, gpointer p2, gpointer p3, gpointer p4, gpointer p5, gpointer p6) {
+	ArgSet * argset = argset_new5 (p1, p2, p3, p4, p5);
+	g_ptr_array_add (argset->args, p6);
+	return argset;
+}
 
 
 gpointer
@@ -66,7 +71,5 @@ argset_free (ArgSet * argset) {
 void
 run_in_worker_thread (WorkerFunc func, gpointer arg)
 {
-	pthread_t thread;
-	pthread_create (&thread, NULL, func, arg);
-	pthread_detach (thread);
+	g_thread_create (func, arg, FALSE, NULL);
 }
