@@ -38,10 +38,10 @@
 #include <pan/base/group.h>
 #include <pan/base/util-mime.h>
 
-const gchar * default_incoming_name_addr;
-const gchar * default_incoming_name_real;
+const char * default_incoming_name_addr;
+const char * default_incoming_name_real;
 
-static void fire_articles_changed (Group*, Article**, gint article_qty, ArticleChangeType);
+static void fire_articles_changed (Group*, Article**, int article_qty, ArticleChangeType);
 
 /***
 ****
@@ -89,7 +89,6 @@ article_constructor (Article * article, Group * group)
 	article->part = (gint16)0;
 	article->parts = (gint16)0;
 	article->score = (gint16)0;
-	article->crosspost_qty = (gint8)0;
 	article->linecount = (guint16)0;
 	article->state = (guint16)0;
 	article->unread_children = (guint16)0;
@@ -146,7 +145,6 @@ article_dup (Group * group, const Article * article)
 	retval->parts               = article->parts;
 	retval->part                = article->part;
 	retval->linecount           = article->linecount;
-	retval->crosspost_qty       = article->crosspost_qty;
 	retval->state               = article->state;
 	retval->date                = article->date;
 	retval->number              = article->number;
@@ -171,7 +169,7 @@ article_dup (Group * group, const Article * article)
 ***/
 
 static void
-articles_set_dirty (Article ** articles, gint qty)
+articles_set_dirty (Article ** articles, int qty)
 {
 	Group * group;
 	debug_enter ("articles_set_dirty");
@@ -262,8 +260,8 @@ article_is_read (const Article * a)
 static void
 articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_data)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Group * g = GROUP(key);
 	GArray * numbers = (GArray*) value;
 	gboolean read = user_data!=NULL;
@@ -351,7 +349,7 @@ articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_d
  * If no matching GArray* exists, it's created and added to the hash before returning.
  */
 static GArray*
-get_group_array_from_hash (GHashTable * hash, Group * group, gint size)
+get_group_array_from_hash (GHashTable * hash, Group * group, int size)
 {
 	GArray * arr = (GArray*) g_hash_table_lookup (hash, group);
 	if (arr == NULL)
@@ -455,11 +453,11 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 
 void
 articles_add_flag (Article  ** articles,
-                   gint        qty,
+                   int         qty,
                    gushort     flag)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Article ** changed;
 
 	/* sanity clause */
@@ -484,11 +482,11 @@ articles_add_flag (Article  ** articles,
 
 void
 articles_remove_flag (Article         ** articles,
-                      gint               qty,
+                      int                qty,
                       gushort            flag)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Article ** changed;
 
 	/* sanity clause */
@@ -518,11 +516,11 @@ articles_remove_flag (Article         ** articles,
 ****
 ***/
 
-gchar*
+char*
 article_get_thread_message_id (const Article* article)
 {
 	char * pch;
-	const gchar * refs;
+	const char * refs;
 
 	/* sanity clause */
 	g_return_val_if_fail (article_is_valid(article), NULL);
@@ -539,7 +537,7 @@ article_get_thread_message_id (const Article* article)
 	}
 	else /* top of the thread */
 	{
-		const gchar * msg_id = article_get_message_id (article);
+		const char * msg_id = article_get_message_id (article);
 		g_return_val_if_fail (is_nonempty_string(msg_id), NULL);
 		pch = g_strdup (msg_id);
 	}
@@ -614,13 +612,11 @@ article_get_unique_threads (const GPtrArray    * articles,
 	debug_enter ("articlelist_get_unique_threads");
 
 	/* sanity clause */
-	retval = g_ptr_array_new ();
+	retval = g_ptr_array_sized_new (128);
 	g_return_val_if_fail (articles_are_valid ((const Article **)articles->pdata, articles->len), retval);
 
 	/* get the thread for each article passed in */
-	thread = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (retval, 128);
-	pan_g_ptr_array_reserve (thread, 128);
+	thread = g_ptr_array_sized_new (128);
 	all = g_hash_table_new (g_str_hash, g_str_equal);
 	for (i=0; i<articles->len; ++i)
 	{
@@ -642,7 +638,7 @@ article_get_unique_threads (const GPtrArray    * articles,
 		for (j=0; j<thread->len; ++j)
 		{
 			Article * b = ARTICLE(g_ptr_array_index(thread,j));
-			const gchar * message_id = article_get_message_id (b);
+			const char * message_id = article_get_message_id (b);
 
 			if (g_hash_table_lookup (all, message_id) == NULL)
 			{
@@ -666,8 +662,28 @@ article_get_unique_threads (const GPtrArray    * articles,
 ****
 ***/
 
+int
+article_get_crosspost_qty (const Article  * article)
+{
+	int retval;
+
+	g_return_val_if_fail (article_is_valid (article), 0);
+
+	if (!is_nonempty_string (article->xref))
+		retval = 1;
+	else {
+		const char * pch;
+		retval = 0;
+		for (pch=article->xref+6; *pch; ++pch)
+			if (*pch==':')
+				++retval;
+	}
+
+	return retval;
+}
+
 void
-article_remove_header (Article * a, const gchar * key)
+article_remove_header (Article * a, const char * key)
 {
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
@@ -696,29 +712,29 @@ article_remove_header (Article * a, const gchar * key)
 	}
 }
 
-const gchar*
-article_get_extra_header (const Article * a, const gchar * key)
+const char*
+article_get_extra_header (const Article * a, const char * key)
 {
-	const gchar * retval = NULL;
+	const char * retval = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (a!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(key), NULL);
 
 	if (a->headers!=NULL)
-		retval = (const gchar*) g_hash_table_lookup (a->headers, key);
+		retval = (const char*) g_hash_table_lookup (a->headers, key);
 
 	return retval;
 }
 
 gboolean
-article_header_is_internal (const gchar * key)
+article_header_is_internal (const char * key)
 {
 	return is_nonempty_string(key) && !g_strncasecmp(key,"Pan-",4);
 }
 
 gboolean
-article_header_is_extra (const gchar * key)
+article_header_is_extra (const char * key)
 {
 	/* sanity check */
 	if (!is_nonempty_string(key)) return FALSE;
@@ -757,7 +773,7 @@ article_xref_foreach (const Article       * a,
 	if (is_nonempty_string(a->xref))
 	{
 		Run run;
-		const gchar * march;
+		const char * march;
 		Server * const server = a->group->server;
 
 		/* skip the servername; we've got the server already */
@@ -766,18 +782,15 @@ article_xref_foreach (const Article       * a,
 		/* walk through the xrefs, of format "group1:number group2:number" */
 		while (get_next_token_run (march, ' ', &march, &run.str, &run.len))
 		{
-			const gchar * delimit = run_strchr (&run, ':');
+			const char * delimit = run_strchr (&run, ':');
 			if (delimit != NULL)
 			{
 				const gulong number = strtoul (delimit+1, NULL, 10);
 				Group * g = NULL;
-				gchar * group_name;
+				char * group_name;
 
 				pan_strndup_alloca (group_name, run.str, delimit-run.str);
-
-				if (server_has_group_in_type (server, group_name, set))
-					g = server_get_named_group (server, group_name);
-
+				g = server_get_named_group_in_type (server, group_name, set);
 				if (g!=NULL && (a->group!=g || !skip_group_a))
 					(*func)(g, number, user_data);
 			}
@@ -786,11 +799,11 @@ article_xref_foreach (const Article       * a,
 }
 
 static void
-article_set_header_impl (Article         * a,
-                         const gchar     * key,
-                         const gchar     * val,
-                         HeaderAction      action,
-                         gboolean          init)
+article_set_header_impl (Article        * a,
+                         const char     * key,
+                         const char     * val,
+                         HeaderAction     action,
+                         gboolean         init)
 {
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
@@ -832,8 +845,8 @@ article_set_header_impl (Article         * a,
 
 void
 article_init_header (Article         * a,
-                     const gchar     * key,
-                     const gchar     * val,
+                     const char      * key,
+                     const char      * val,
                      HeaderAction      action)
 {
 	article_set_header_impl (a, key, val, action, TRUE);
@@ -841,8 +854,8 @@ article_init_header (Article         * a,
 
 void
 article_set_header (Article         * a,
-                    const gchar     * key,
-                    const gchar     * val,
+                    const char      * key,
+                    const char      * val,
                     HeaderAction      action)
 {
 	article_set_header_impl (a, key, val, action, FALSE);
@@ -908,78 +921,75 @@ article_has_body (const Article * a)
 	return retval;
 }
 
-gchar*
-article_get_author_str (const Article * a, gchar * buf, gint bufsize)
+char*
+article_get_author_str (const Article * a, char * buf, int bufsize)
 {
 	const gboolean have_addr = a!=NULL && is_nonempty_string (a->author_addr);
 	const gboolean have_name = a!=NULL && is_nonempty_string (a->author_real);
 
 	*buf = '\0';
 
-	if (have_addr && have_name) {
+	if (have_addr && have_name)
 		g_snprintf (buf, bufsize, "\"%s\" <%s>", a->author_real, a->author_addr);
-	}
-	else if (have_addr) {
-		strncpy (buf, a->author_addr, bufsize);
-		buf[bufsize-1] = '\0';
-	}
-	else if (have_name) {
-		strncpy (buf, a->author_real, bufsize);
-		buf[bufsize-1] = '\0';
-	}
+	else if (have_addr)
+		g_strlcpy (buf, a->author_addr, bufsize);
+	else if (have_name)
+		g_strlcpy (buf, a->author_real, bufsize);
 
 	g_strstrip (buf);
 	return buf;
 }
 
-void
-article_get_short_author_str (const Article * a, gchar * buf, gint len)
+int
+article_get_short_author_str (const Article * a, char * buf, int len)
 {
-	g_return_if_fail (article_is_valid(a));
-	g_return_if_fail (buf!=NULL);
-	g_return_if_fail (len>0);
+	int retval;
+
+	/* sanity clause */
+	g_return_val_if_fail (article_is_valid(a), 0);
+	g_return_val_if_fail (buf!=NULL, 0);
+	g_return_val_if_fail (len>0, 0);
 
 	if (is_nonempty_string (a->author_real) && strcmp(a->author_real,default_incoming_name_real))
 	{
-		strncpy (buf, a->author_real, len);
-		buf[len-1] = '\0';
+		retval = g_strlcpy (buf, a->author_real, len);
 	}
 	else if (is_nonempty_string (a->author_addr) && strcmp(a->author_addr,default_incoming_name_addr))
 	{
-		const gchar * author = a->author_addr;
-		const gchar * pch = strchr (author, '@');
-		if (pch == NULL)
-			pch = author + strlen(author);
-		strncpy (buf, author, MIN(len,pch-author));
-		buf[len-1] = '\0';
+		const char * author = a->author_addr;
+		const char * p = strchr (author, '@');
+		if (p == NULL)
+			p = author + strlen(author);
+		retval = g_strlcpy (buf, author, MIN(len,p-author));
 	}
 	else
 	{
-		strncpy (buf, default_incoming_name_real, len);
-		buf[len-1] = '\0';
+		retval = g_strlcpy (buf, default_incoming_name_real, len);
 	}
+
+	return retval;
 }
 
 
-gchar*
+char*
 article_get_message (const Article * a)
 {
-	gchar * retval = NULL;
+	char * retval = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
 
 	if (retval==NULL)
 	{
-		const gchar * message_id = article_get_message_id (a);
+		const char * message_id = article_get_message_id (a);
 		if (acache_has_message (message_id))
-			retval = acache_get_message (message_id);
+			retval = acache_get_message (message_id, TRUE);
 	}
 
 	if (retval==NULL)
 	{
 		GString * s = g_string_new (NULL);
-		gchar * pch;
+		char * pch;
 
 		pch = article_get_headers (a);
 		g_string_append (s, pch);
@@ -998,21 +1008,21 @@ article_get_message (const Article * a)
 	return retval;
 }
 
-gchar*
+char*
 article_get_headers (const Article * a)
 {
-	gchar * retval = NULL;
+	char * retval = NULL;
 
 	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
 
 	/* do we have it in acache? */
 	if (retval==NULL)
 	{
-		const gchar * message_id = article_get_message_id (a);
+		const char * message_id = article_get_message_id (a);
 		if (acache_has_message (message_id))
 		{
-			gchar * s = acache_get_message (message_id);
-			gchar * pch = pan_strstr (s, "\n\n");
+			char * s = acache_get_message (message_id, TRUE);
+			char * pch = pan_strstr (s, "\n\n");
 			if (pch != NULL) {
 				*pch = '\0';
 				retval = s;
@@ -1023,8 +1033,8 @@ article_get_headers (const Article * a)
 	/* can we build it from a? */
 	if (retval==NULL)
 	{
-		gchar * tmp;
-		const gchar * c_tmp;
+		char * tmp;
+		const char * c_tmp;
 		GString * s = g_string_new (NULL);
 
 		/* subject */
@@ -1034,7 +1044,7 @@ article_get_headers (const Article * a)
 
 		/* author */
 		if (1) {
-			gchar author[512] = { '\0' };
+			char author[512] = { '\0' };
 	       		article_get_author_str (a, author, sizeof(author));
 			if (is_nonempty_string (author))
 				g_string_sprintfa (s, "%s: %s\n", HEADER_FROM, author);
@@ -1059,7 +1069,7 @@ article_get_headers (const Article * a)
 
 		/* extra headers */
 		if (1) {
-			gint i;
+			int i;
 			GPtrArray * z = article_get_all_headers (a);
 			for (i=0; i<z->len; i+=2) {
 				const char* key = (const char*) g_ptr_array_index (z, i);
@@ -1089,7 +1099,14 @@ article_get_body (const Article * a)
 	{
 		const char * body = article_get_extra_header (a, PAN_BODY);
 		if (body != NULL)
-			retval = g_strdup (body);
+		{
+			const char * charset = article_get_extra_header (a, PAN_CHARSET);
+
+			if (is_nonempty_string (charset))
+				retval = g_convert (body, -1, charset, "UTF-8", NULL, NULL, NULL);
+			else
+				retval = g_strdup (body);
+		}
 	}
 
 	/* see if we've got it in acache... */
@@ -1098,9 +1115,9 @@ article_get_body (const Article * a)
 		const char * message_id = article_get_message_id (a);
 		if (acache_has_message (message_id))
 		{
-			char * s = acache_get_message (message_id);
+			char * s = acache_get_message (message_id, TRUE);
 			gboolean is_html = FALSE;
-			GMimeMessage * m = pan_g_mime_parser_construct_message ((const char **)&s, 1);
+			GMimeMessage * m = pan_g_mime_parser_construct_message ((const char**)&s, 1);
 			retval = g_mime_message_get_body (m, TRUE, &is_html);
 			g_mime_object_unref (GMIME_OBJECT(m));
 			g_free (s);
@@ -1155,7 +1172,7 @@ article_get_header_run_hash_from_text (const char     * article_text,
 
 		if (get_next_token_run (line, ':', NULL, &key, &key_len))
 		{
-			gint val_len = 0;
+			int val_len = 0;
 			const char * val = NULL;
 			Run * key_run = buf + buf_qty++;
 			Run * val_run = buf + buf_qty++;
@@ -1188,6 +1205,7 @@ article_set_author_from_header_impl (Article       * a,
 	char * real = real_buf;
 	char * pch;
 
+	/* sanity clause */
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (a->group!=NULL);
 	g_return_if_fail (is_nonempty_string(header_from));
@@ -1206,7 +1224,7 @@ article_set_author_from_header_impl (Article       * a,
 	/* use the real mail address, or fill in a default */
 	pch = addr;
 	if (!is_nonempty_string (pch))
-		pch = (gchar*)default_incoming_name_addr;
+		pch = (char*)default_incoming_name_addr;
 	a->author_addr = group_chunk_string (a->group, pch, TRUE);
 
 	/* use the real name, or fill in a default. */
@@ -1215,12 +1233,12 @@ article_set_author_from_header_impl (Article       * a,
 		a->author_real = group_chunk_string (a->group, real, TRUE);
 	}
 	else if (addr!=NULL && ((pch=strchr(addr,'@'))!=NULL)) {
-		gchar * tmp;
+		char * tmp;
 		pan_strndup_alloca (tmp, addr, pch-addr);
 		a->author_real = group_chunk_string (a->group, tmp, TRUE);
 	}
 	else {
-		const gchar * cpch = (gchar*)default_incoming_name_real;
+		const char * cpch = (const char*)default_incoming_name_real;
 		a->author_real = group_chunk_string (a->group, cpch, TRUE);
 	}
 
@@ -1236,13 +1254,13 @@ article_set_author_from_header_impl (Article       * a,
 
 void
 article_init_author_from_header (Article         * a,
-                                 const gchar     * header_from)
+                                 const char      * header_from)
 {
 	article_set_author_from_header_impl (a, header_from, TRUE);
 }
 void
 article_set_author_from_header (Article         * a,
-                                const gchar     * header_from)
+                                const char      * header_from)
 {
 	article_set_author_from_header_impl (a, header_from, FALSE);
 }
@@ -1266,6 +1284,7 @@ void
 article_set_from_raw_message   (Article         * a,
                                 const char      * text)
 {
+	const char *from, *reply_to, *subject, *message_id;
 	guint16 lines;
 	GMimeMessage * msg;
 	char * body;
@@ -1276,7 +1295,7 @@ article_set_from_raw_message   (Article         * a,
 	/* get the message */
 	g_return_if_fail (a != NULL);
 	g_return_if_fail (is_nonempty_string(text));
-	msg = pan_g_mime_parser_construct_message ((const char **)&text, 1);
+	msg = pan_g_mime_parser_construct_message (&text, 1);
 	g_return_if_fail (msg != NULL);
 
 	/* body */
@@ -1289,19 +1308,21 @@ article_set_from_raw_message   (Article         * a,
 	g_free (body);
 
 	/* headers */
-	if (msg->header != NULL) {
-		a->date = msg->header->date;
-		if (msg->header->from != NULL)
-			article_init_header (a, HEADER_FROM, msg->header->from, DO_CHUNK_SHARE);
-		if (msg->header->reply_to != NULL)
-			article_init_header (a, HEADER_REPLY_TO, msg->header->reply_to, DO_CHUNK_SHARE);
-		if (msg->header->subject != NULL)
-			article_init_header (a, HEADER_SUBJECT, msg->header->subject, DO_CHUNK_SHARE);
-		if (msg->header->message_id != NULL)
-			article_init_header (a, HEADER_MESSAGE_ID, msg->header->message_id, DO_CHUNK);
-		g_mime_header_foreach (msg->header->headers, set_from_raw_init_header_gmhfunc, a);
-	}
-
+	g_mime_message_get_date (msg, &a->date, NULL);
+	from = g_mime_message_get_sender (msg);
+	if (from != NULL)
+		article_init_header (a, HEADER_FROM, from, DO_CHUNK_SHARE);
+	reply_to = g_mime_message_get_reply_to (msg);
+	if (reply_to != NULL)
+		article_init_header (a, HEADER_REPLY_TO, reply_to, DO_CHUNK_SHARE);
+	subject = g_mime_message_get_subject (msg);
+	if (subject != NULL)
+		article_init_header (a, HEADER_SUBJECT, subject, DO_CHUNK_SHARE);
+	message_id = g_mime_message_get_message_id (msg);
+	if (message_id != NULL)
+		article_init_header (a, HEADER_MESSAGE_ID, message_id, DO_CHUNK);
+	g_mime_header_foreach (GMIME_OBJECT (msg)->headers, set_from_raw_init_header_gmhfunc, a);
+	
 	/* fallback if no message-id */
 	if (!is_nonempty_string(a->message_id)) {
 		char * id;
@@ -1331,27 +1352,23 @@ article_set_from_raw_message   (Article         * a,
 }
 
 
-static gboolean
-breakpoint (const Article * a)
-{
-	if (a!=NULL && a->group!=NULL)
-		g_message ("bad article's group: [%s]", a->group->name);
-	return FALSE;
-}
+/****
+*****  SANITY CHECKS
+****/
 
 gboolean
 article_is_valid (const Article * a)
 {
-	g_return_val_if_fail (a!=NULL, breakpoint(a));
-	g_return_val_if_fail (a->group!=NULL, breakpoint(a));
-	g_return_val_if_fail (group_is_valid(a->group), breakpoint(a));
-	g_return_val_if_fail (a->group->_articles_refcount>=0, breakpoint(a));
-	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, breakpoint(a));
-	g_return_val_if_fail (a->message_id!=NULL, breakpoint(a));
-	g_return_val_if_fail (*a->message_id=='<', breakpoint(a));
-	g_return_val_if_fail (is_nonempty_string(a->subject), breakpoint(a));
-	g_return_val_if_fail (a->number!=0, breakpoint(a));
-	g_return_val_if_fail (!a->references || *a->references=='<', breakpoint(a));
+	g_return_val_if_fail (a!=NULL, FALSE);
+	g_return_val_if_fail (a->group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(a->group), FALSE);
+	g_return_val_if_fail (a->group->_articles_refcount>=0, FALSE);
+	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, FALSE);
+	g_return_val_if_fail (a->message_id!=NULL, FALSE);
+	g_return_val_if_fail (*a->message_id=='<', FALSE);
+	g_return_val_if_fail (is_nonempty_string(a->subject), FALSE);
+	g_return_val_if_fail (a->number!=0, FALSE);
+	g_return_val_if_fail (!a->references || *a->references=='<', FALSE);
 
 	return TRUE;
 }
@@ -1359,7 +1376,7 @@ article_is_valid (const Article * a)
 gboolean
 articles_are_valid (const Article ** a, int qty)
 {
-	gint i;
+	int i;
 
 	/* sanity clause */
 	g_return_val_if_fail (qty>=0, FALSE);
@@ -1372,14 +1389,14 @@ articles_are_valid (const Article ** a, int qty)
 	return TRUE;
 }
 
-const gchar*
+const char*
 article_get_message_id (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
 	return a->message_id;
 }
  
-const gchar*
+const char*
 article_get_subject (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
@@ -1397,7 +1414,7 @@ article_get_combined_linecount (const Article * a)
 	retval = a->linecount;
 	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
 		GSList * l;
-		gint part = a->part + 1;
+		int part = a->part + 1;
 		for (l=a->threads; l; l=l->next) {
 			Article * child = ARTICLE(l->data);
 			if (child->part == part) {
@@ -1426,7 +1443,7 @@ article_get_articles_changed_callback (void)
 static void
 fire_articles_changed (Group              * group,
                        Article           ** articles,
-                       gint                 article_qty,
+                       int                  article_qty,
                        ArticleChangeType    type)
 {
 	ArticleChangeEvent e;
