@@ -26,8 +26,6 @@
 #include <stdio.h>
 #include <time.h>
 
-#include <pthread.h>
-
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 
@@ -56,7 +54,7 @@ debug_printf (const gchar   * filename,
 	time_t now_time_t;
 	va_list args;
 	struct tm now_tm;
-	const gint self = (int) pthread_self ();
+	const gpointer self = g_thread_self ();
 
 	if (t2d == NULL)
 		t2d = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -79,19 +77,19 @@ debug_printf (const gchar   * filename,
 	va_end (args);
 
 	time (&now_time_t);
-	gmtime_r (&now_time_t, &now_tm);
+	pan_gmtime_r (&now_time_t, &now_tm);
 	strftime (timebuf, sizeof(timebuf), "%H:%M:%S", &now_tm);
 
 	if (flag&DEBUG_TRACE)
 	{
-		const gint old_depth = GPOINTER_TO_INT (g_hash_table_lookup (t2d, GINT_TO_POINTER(self)));
+		const gint old_depth = GPOINTER_TO_INT (g_hash_table_lookup (t2d, self));
 		const gint new_depth = old_depth + (*fmt=='+' ? 1 : -1);
 
 		g_hash_table_insert (t2d, 
-		                     GINT_TO_POINTER(self),
+		                     self,
 		                     GINT_TO_POINTER(new_depth));
 
-		printf ("(%30s:%04d:%30s)(thread %4d)(time %s)(depth %3d) %s: %s\n",
+		printf ("(%30s:%04d:%30s)(thread %p)(time %s)(depth %3d) %s: %s\n",
 			filename,
 			line_num,
 			function,
@@ -106,7 +104,7 @@ debug_printf (const gchar   * filename,
 	}
 	else
 	{
-		printf ("(%30s:%04d:%30s)(thread %4d)(time %s) %s: %s\n",
+		printf ("(%30s:%04d:%30s)(thread %p)(time %s) %s: %s\n",
 			filename,
 			line_num,
 			function,
