@@ -170,135 +170,20 @@ create_filename (const gchar       * path,
 *****/
 
 static void
-write_big_article (GPtrArray * articles, FILE * fp_out)
-{
-	gint i;
-	gboolean is_uu = FALSE;
-	gboolean ignoring_uu_noise = FALSE;
-	const gchar * cut_begin = "BEGIN -- Cut Here -- cut here";
-	const size_t cut_begin_len = strlen (cut_begin);
-	const gchar * cut_end = "END -- Cut Here -- cut here";
-	const size_t cut_end_len = strlen (cut_end);
-	debug_enter ("write_big_article");
-
-	/* sanity clause */
-	g_return_if_fail (articles!=NULL);
-	g_return_if_fail (articles->len);
-
-	/* get the articles & find out how big the buffer must be */
-	for (i=0; i!=articles->len; ++i)
-	{
-		const Article * a;
-		gchar * message = NULL;
-		gchar * body;
-
-		/* get the article message */
-		a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a != NULL)
-			message = acache_get_message (article_get_message_id(a));
-		if (message == NULL) {
-			log_add_va (LOG_ERROR,
-				_("Couldn't find body for message `%s' - decoding may be incomplete or incorrect"),
-				article_get_subject(a));
-			continue;
-		}
-
-		/* find the beginning of the body */
-		body = pan_strstr (message, "\n\n");
-		if (body != NULL)
-			body += 2;
-
-		/* keep the headers for the first item */
-		if (i==0 && body!=NULL)
-			fwrite (message, sizeof(char), body-message, fp_out);
-
-		/* if we couldn't find the headers, assume they're missing.
-		   AFAIK this won't happen; this is just CYA. */
-		if (body == NULL)
-			body = message;
-
-		if (1)
-		{
-			gint line_len = 0;
-			const gchar * march = body;
-			const gchar * line_begin = NULL;
-
-			while (get_next_token_run (march, '\n', &march, &line_begin, &line_len))
-			{
-				gboolean skip_line = ignoring_uu_noise;
-
-				if (is_uu) {
-					if (cut_end_len==line_len && !strncmp (line_begin, cut_end, cut_end_len))
-						skip_line = ignoring_uu_noise = TRUE;
-					else if (cut_begin_len==line_len && !strncmp (line_begin, cut_begin, cut_begin_len))
-						ignoring_uu_noise = FALSE;
-					else if (uu_is_ending_line(line_begin))
-						is_uu = FALSE;
-					else if (!is_uu_line (line_begin, line_len))
-						skip_line = TRUE;
-				} else {
-					if (uu_is_beginning_line(line_begin))
-						is_uu = TRUE;
-				}
-
-				if (!skip_line)
-					fwrite (line_begin, sizeof(char), line_len+1, fp_out);
-			}
-		}
-
-		/* cleanup */
-		g_free (message);
-	}
-
-	debug_exit ("write_big_article");
-}
-
-static void
-get_array_of_decodable_parts_func (GMimePart * part, gpointer data)
+get_array_of_decodable_parts_func (GMimeObject *part, gpointer data)
 {
 	GPtrArray * a = (GPtrArray*) data;
 	const GMimeContentType * type;
-       
-	type = g_mime_part_get_content_type (part);
-	if ((is_nonempty_string(g_mime_part_get_filename (part))) ||
-		(!g_mime_content_type_is_type (type, "text", "*") && !g_mime_content_type_is_type (type, "multipart", "*")))
+	
+	if (GMIME_IS_MULTIPART (part))
+		return;
+
+	type = g_mime_object_get_content_type (part);
+	if ((is_nonempty_string (g_mime_part_get_filename (GMIME_PART (part)))) ||
+	    (!g_mime_content_type_is_type (type, "text", "*")))
 		g_ptr_array_add (a, part);
 }
 
-static void
-get_decodable_parts (GPtrArray       * articles,
-                     GMimeMessage   ** setme_msg,
-                     GPtrArray      * fillme_array)
-{
-	FILE * fp;
-	gchar * tmpfilename;
-	debug_enter ("get_decodable_parts");
-
-	/* sanity check */
-	g_return_if_fail (articles!=NULL);
-
-	/* get the message */
-	fp = NULL;
-	tmpfilename = pan_make_temp (&fp);
-	if (fp != NULL)
-	{
-		/* write the articles to one big file */
-		write_big_article (articles, fp);
-		fclose (fp);
-		fp = NULL;
-
-		/* parse that file */
-		fp = fopen (tmpfilename, "r");
-		*setme_msg = pan_g_mime_parser_construct_message_from_file (fp);
-
-		/* build the array of decodable parts */
-		g_mime_message_foreach_part (*setme_msg, get_array_of_decodable_parts_func, fillme_array);
-	}
-
-	remove (tmpfilename);
-	g_free (tmpfilename);
-	debug_exit ("get_decodable_parts");
-}
 
 /**
 *** Public Routines
@@ -307,21 +192,24 @@ get_decodable_parts (GPtrArray       * articles,
 gboolean
 decode_article (const decode_data * dd)
 {
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 	GPtrArray * articles;
 	GMimeMessage * mm;
 	GPtrArray * attachments;
 	GSList * p;
-	gint i = 0;
-	gboolean use_mutex;
+	int i = 0;
 	gboolean success = TRUE;
 	Article * first;
 	GString * filenames;
+	char * filename = NULL;
+	GMimeStream * stream;
+	GMimeFilterYenc * yenc_filter;
+	FILE ** fps = NULL;
+	const char ** message_ids;
 	debug_enter ("decode_article");
 
 	/* sanity clause */
 	g_return_val_if_fail (dd!=NULL, FALSE);
-	g_return_val_if_fail (dd->server!=NULL, FALSE);
+	g_return_val_if_fail (server_is_valid (dd->server), FALSE);
 	g_return_val_if_fail (dd->articles!=NULL, FALSE);
 	for (p=dd->articles; p!=NULL; p=p->next) {
 		Article * a = ARTICLE(p->data);	
@@ -331,101 +219,152 @@ decode_article (const decode_data * dd)
 
 	first = ARTICLE(dd->articles->data);
 
-	/* This is sad, really. individual GMimeMessages are safe enough;
-	   the reason we use mutexes here is that decoding a big mp3 takes
-	   up so _much_ memory that running two at once makes my system
-	   swap to its knees. */
-	use_mutex = g_slist_length (dd->articles) > 3;
-	if (use_mutex) {
-		status_item_emit_status_va (dd->item, _("Waiting to decode \"%s\""),
-			article_get_subject(first));
-		g_static_mutex_lock (&mutex);
-	}
-
-	/* let the client know what we're doing */
-	status_item_emit_status_va (dd->item, _("Processing \"%s\""), article_get_subject(first));
-
-	/* make a GPtrArray of the articles */
-	articles = g_ptr_array_new ();
+	/* make an array of Articles */
+	articles = g_ptr_array_sized_new (g_slist_length(dd->articles));
 	for (p=dd->articles; p!=NULL; p=p->next)
 		g_ptr_array_add (articles, p->data);
 
+	/* make array of message_ids */
+	message_ids = g_new (const char*, articles->len);
+	for (i=0; i<articles->len; ++i)
+		message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(articles,i)));
+
+	/* checkout the messages */
+	acache_file_checkout_fps (message_ids, articles->len, &fps);
+
 	/* get the decodable parts */
-	status_item_emit_status_va (dd->item, _("Decoding \"%s\""), article_get_subject(first));
-	mm = NULL;
 	attachments = g_ptr_array_new ();
-	get_decodable_parts (articles, &mm, attachments);
+	status_item_emit_status_va (dd->item, _("Decoding \"%s\""), article_get_subject(first));
+	mm = pan_g_mime_parser_construct_message_from_file (fps, articles->len);
+	g_mime_message_foreach_part (mm, get_array_of_decodable_parts_func, attachments);
 
 	/* decode & save the parts */
 	success = attachments->len != 0;
 	filenames = g_string_new (NULL);
+	stream = NULL;
+	yenc_filter = NULL;
 	for (i=0; success && i!=attachments->len; ++i)
 	{
-		FILE * fp = NULL;
 		const GMimePart * part = (const GMimePart*) g_ptr_array_index (attachments, i);
 		const GMimeDataWrapper * content;
-		gchar * filename = create_filename (dd->path, article_get_subject(first), dd->filename, part);
+		YencInfo * yenc = GMIME_OBJECT(part)->user_data;
+		int part_number = -1;
 
-		/* remember this filename */
-		g_string_sprintfa (filenames, "%s\n", filename);
+		if (yenc != NULL)
+			part_number = yenc->part;
 
-		/* access the path */
-		if (success) {
-			gchar * path = g_dirname (filename);
-			gboolean path_ok = directory_check (path);
-			if (!path_ok) {
-				log_add_va (LOG_ERROR, _("Decode can't access path \"%s\""), path);
-				success = FALSE;
+		if (part_number < 2)
+		{
+			g_free (filename);
+			filename = create_filename (dd->path, article_get_subject(first), dd->filename, part);
+
+			/* remember this filename */
+			g_string_sprintfa (filenames, "%s\n", filename);
+
+			/* access the path */
+			if (success) {
+				gchar * path = g_dirname (filename);
+				gboolean path_ok = directory_check (path);
+				if (!path_ok) {
+					log_add_va (LOG_ERROR, _("Decode can't access path \"%s\""), path);
+					success = FALSE;
+				}
+				g_free (path);
 			}
-			g_free (path);
-		}
 
-		/* open the output file */
-		if (success) {
-			errno = 0;
-			fp = fopen (filename, "w+");
-			if (fp == NULL) {
-				log_add_va (LOG_ERROR, _("Decode unable to create file \"%s\" %s"),
-						filename,
-						errno==0 ? "" : g_strerror(errno));
-				success = FALSE;
+			/* open the output file */
+			if (success) {
+				FILE * fp;
+				errno = 0;
+				fp = fopen (filename, "w+");
+				if (stream != NULL) {
+					g_object_unref (stream);
+					stream = NULL;
+				}
+				if (fp != NULL) {
+					stream = g_mime_stream_file_new (fp);
+					if (yenc != NULL) {
+						GMimeStream * inner_stream = stream;
+						GMimeFilter * filter;
+						stream = g_mime_stream_filter_new_with_stream (inner_stream);
+						filter = g_mime_filter_yenc_new (GMIME_FILTER_YENC_DIRECTION_DECODE);
+						g_mime_stream_filter_add (GMIME_STREAM_FILTER(stream), filter);
+						yenc_filter = (GMimeFilterYenc*) filter;
+						g_mime_stream_unref (inner_stream);
+					}
+				}
+				else {
+					log_add_va (LOG_ERROR, _("Decode unable to create file \"%s\" %s"),
+							filename,
+							errno==0 ? "" : g_strerror(errno));
+					success = FALSE;
+				}
 			}
 		}
 
-
+		/* make sure we have the necessary info to continue */          
+		if (success) {
+			if (stream == NULL || filename == NULL ||
+			    (yenc != NULL && yenc_filter == NULL)) { 
+				log_add (LOG_ERROR, _("Could not decode article - file may be corrupt/incomplete"));
+				success = FALSE;
+			}
+		}
+		
 		/* make sure there's content to write */
 		content = g_mime_part_get_content_object (part);
 		if (!content)
 			success = FALSE;
 
 		/* write the content */
-		if (success) {
-			GMimeStream *stream = g_mime_stream_file_new (fp);
-			g_mime_data_wrapper_write_to_stream ((GMimeDataWrapper*)content, (stream));
-			g_mime_stream_unref (stream);
-			fp = NULL;
+		if (success)
+		{
+			/* write the stream */
+			g_mime_data_wrapper_write_to_stream ((GMimeDataWrapper*)content, stream);
+			g_mime_stream_flush (stream);
+			
 			status_item_emit_status_va (dd->item, _("Saved \"%s\""), filename);
-		}
 
-		/* log the success */
-		if (success) {
-			gchar * msg = g_strdup_printf (
-				_("Decoded \"%s\" from group \"%s\", \"%s\" part #%d"),
-				filename,
-				(dd->group ? dd->group->name : "Unknown"),
-				article_get_subject(first),
-				i+1);
-			log_add (LOG_INFO, msg);
-			debug0 (DEBUG_DECODE, msg);
-			g_free (msg);
-		}
+			if (yenc != NULL)
+			{
+				/* log any crc errors */
+				if ((yenc->pcrc!=0) && (yenc->pcrc != g_mime_filter_yenc_get_pcrc(yenc_filter)))
+					log_add_va (LOG_ERROR,
+						    _("Checksum for `%s' part %d failed - file is corrupt"),
+						    filename, part_number);
+				if ((yenc->crc!=0) && (yenc->crc != g_mime_filter_yenc_get_crc(yenc_filter)))
+					log_add_va (LOG_ERROR,
+						    _("Checksum for `%s' failed - file may be corrupt"),
+						    filename);
 
-		/* cleanup this iteration */
-		g_free (filename);
+				/* reset the filter to get ready for the next part */
+				yenc_filter->pcrc = GMIME_YENCODE_CRC_INIT;
+				yenc_filter->state = GMIME_YENCODE_STATE_INIT;
+			}
+		}
 	}
 
+	/* log the success */
+	if (success) {
+		gchar * msg = g_strdup_printf (
+			_("Decoded \"%s\" from group \"%s\", \"%s\""),
+			filename,
+			(dd->group ? dd->group->name : "Unknown"),
+			article_get_subject(first));
+		log_add (LOG_INFO, msg);
+		debug0 (DEBUG_DECODE, msg);
+		g_free (msg);
+	}
+
+	/* cleanup */
+	if (stream)
+		g_object_unref (stream);
+	stream = NULL;
+	g_free (filename);
+	filename = NULL;
+
 	/* remember the filename */
-	if (filenames->len && !article_get_header (first, PAN_HEADER_FILENAME))
+	if (filenames->len && !article_get_extra_header (first, PAN_HEADER_FILENAME))
 		article_set_header (first, PAN_HEADER_FILENAME, filenames->str, DO_CHUNK);
 
 	/* update the node */
@@ -433,12 +372,27 @@ decode_article (const decode_data * dd)
 	articles_add_flag (&first, 1, (success?STATE_DECODED:STATE_DECODE_FAILED));
 
 	/* cleanup */
-	g_ptr_array_free (articles, TRUE);
-	g_mime_message_destroy (mm);
-	g_ptr_array_free (attachments, TRUE);
+	/* FIXME: when GMime ports to glib2, call _set_data_full() in
+	   mime-util s.t.  we don't have to the YencInfo structs here */
+	for (i=0; i!=attachments->len; ++i) {
+		GMimePart * part = (GMimePart*) g_ptr_array_index (attachments, i);
+		YencInfo * yenc = GMIME_OBJECT(part)->user_data;
+		if (yenc != NULL) {
+			g_free (yenc->filename);
+			g_free (yenc);
+			GMIME_OBJECT(part)->user_data = NULL;
+		}
+	}
+	if (mm != NULL)
+		g_mime_object_unref (GMIME_OBJECT(mm));
+	else for (i=0; i<articles->len; ++i)
+		fclose (fps[i]);
+	g_free (fps);
 	g_string_free (filenames, TRUE);
-	if (use_mutex)
-		g_static_mutex_unlock (&mutex);
+	acache_file_checkin (message_ids, articles->len);
+	g_ptr_array_free (attachments, TRUE);
+	g_ptr_array_free (articles, TRUE);
+	g_free (message_ids);
 	debug_exit ("decode_article");
 
 	return success;
