@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,13 @@
 
 #include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
 
 #include <glib.h>
 
+#include <gmime/memchunk.h>
+
 #include <pan/base/article.h>
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
@@ -50,82 +52,88 @@ static gboolean
 file_headers_load_group (Group * group, StatusItem * status)
 {
 	gboolean success = FALSE;
-        gchar * path;
-	GArray *dat, *idx;
+	char * dat = NULL;
+	char * idx = NULL;
+	size_t dat_len = 0;
+	size_t idx_len = 0;
+	char path[PATH_MAX];
 	debug_enter ("file_headers_load_group");
 
-	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string (group->name), FALSE);
-	g_return_val_if_fail (is_nonempty_string (group->server->name), FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
+	g_return_val_if_fail (group_is_valid (group), FALSE);
 
 	if (status!=NULL)
 		status_item_emit_status_va (status, _("Loading %s"), group_get_readable_name(group));
 
 	/* open the index file */
-	path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
-	idx = read_file (path);
-	g_free (path);
+	g_snprintf (path, sizeof(path), "%s%c%s%c%s.idx",
+	            get_data_dir(), G_DIR_SEPARATOR,
+	            server_get_name (group->server), G_DIR_SEPARATOR,
+	            group->name);
+	g_file_get_contents (path, &idx, &idx_len, NULL);
 
 	/* open the data file */
-	path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
-	dat = read_file (path);
-	g_free (path);
+	g_snprintf (path, sizeof(path), "%s%c%s%c%s.dat",
+	            get_data_dir(), G_DIR_SEPARATOR,
+	            server_get_name (group->server), G_DIR_SEPARATOR,
+	            group->name);
+	g_file_get_contents (path, &dat, &dat_len, NULL);
 
 	/* allocate the articles array */
-	pan_warn_if_fail (group->_articles == NULL);
+	pan_warn_if_fail (group->_articles==NULL);
 	group_get_articles (group);
 
 	if (dat!=NULL && idx!=NULL)
 	{
-		const gchar * march = idx->data;
+		const char * march = idx;
 		const glong version = get_next_token_int (march, '\n', &march);
 
-		if (version==1 || version==2 || version==3 || version==4)
+		if (version==1 || version==2 || version==3 || version==4 || version==5 || version==6 || version== 7)
 		{
 			int i;
+			int purged_article_count = 0;
 			long l;
 			const long qty = get_next_token_long (march, '\n', &march);
-			GPtrArray * addme = g_ptr_array_new ();
-			gint purged_article_count = 0;
+			GPtrArray * addme;
 
+			/* set the "one big chunk" */
 			pan_warn_if_fail (group->_one_big_chunk == NULL);
-			group->_one_big_chunk = dat->data;
+			group->_one_big_chunk = dat;
 
+			/* load the articles */
+		       	addme = g_ptr_array_sized_new (qty);
 			for (i=0; i!=qty; ++i)
 			{
-				Article * a = article_new (group);
-				int extra_header_qty;
 				int j;
+				int extra_header_qty;
+				Article * a = article_new (group);
 
 				/* message id */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->message_id = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->message_id = dat + l;
 
 				/* author */
 				if (version<2) /* version 2 split author into 2 fields */
 				{
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						article_init_author_from_header (a, dat->data+l);
+					if (0<=l && l<dat_len)
+						article_init_author_from_header (a, dat+l);
 				}
 				else
 				{
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->author_addr = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->author_addr = dat + l;
 
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->author_real = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->author_real = dat + l;
 				}
 
 				/* subject */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->subject = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->subject = dat + l;
 
 				/* date string - removed in version 3 */
 				if (version<3)
@@ -133,33 +141,50 @@ file_headers_load_group (Group * group, StatusItem * status)
 
 				/* references */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->references = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->references = dat + l;
 
 				/* xrefs added in version 4 */
 				if (version>=4) {
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->xref = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->xref = dat + l;
 				}
 
 				/* numeric fields */
 				a->part           = (gint16) get_next_token_int (march, '\n', &march);
 				a->parts          = (gint16) get_next_token_int (march, '\n', &march);
 				a->linecount      = (guint16) get_next_token_int (march, '\n', &march);
-				a->crosspost_qty  = (gint8) get_next_token_int (march, '\n', &march);
-				a->state          = (guint16) get_next_token_int (march, '\n', &march);
+
+				/* crosspost_qty - removed in version 6 */
+				if (version<6)
+					skip_next_token (march, '\n', &march);
+
+				/* state - removed in version 6, back in 7 */
+				if (version!=6)
+					a->state  = (guint16) get_next_token_int (march, '\n', &march);
+
 				a->date           = (time_t) get_next_token_ulong (march, '\n', &march);
 				a->number         = (gulong) get_next_token_ulong (march, '\n', &march);
 
+				if (1) {
+					/* an article is new if it's flagged as new AND it's unread */
+					gboolean b = FALSE;
+					if (version >= 5)
+						b = get_next_token_int (march, '\n', &march) != 0;
+					if (b)
+						b = !newsrc_is_article_read (group_get_newsrc(group), a->number);
+					a->is_new = b;
+				}
+
 				/* extra headers */
 				extra_header_qty  = get_next_token_int (march, '\n', &march);
 				for (j=0; j<extra_header_qty; ++j)
 				{
 					const glong key_idx = get_next_token_long (march, '\n', &march);
 					const glong val_idx = get_next_token_long (march, '\n', &march);
-					const gchar * key = 0<=key_idx && key_idx<dat->len ? dat->data+key_idx : NULL;
-					const gchar * val = 0<=val_idx && val_idx<dat->len ? dat->data+val_idx : NULL;
+					const char * key = 0<=key_idx && key_idx<dat_len ? dat+key_idx : NULL;
+					const char * val = 0<=val_idx && val_idx<dat_len ? dat+val_idx : NULL;
 					if (key!=NULL && val!=NULL)
 						article_init_header (a, key, val, 0);
 				}
@@ -188,7 +213,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 					group_get_readable_name(group));
 			}
 
-			group_add_articles_remove_unused (group, addme, status);
+			group_init_articles (group, addme, status);
 			group_set_article_qty (group, addme->len);
 			group->articles_dirty = purged_article_count!=0;
 			success = TRUE;
@@ -201,28 +226,28 @@ file_headers_load_group (Group * group, StatusItem * status)
 		}
 	}
 
-	if (dat != NULL) g_array_free (dat, !success);
-	if (idx != NULL) g_array_free (idx, TRUE);
+	/* cleanup */
+	g_free (idx);
+	if (!success)
+		g_free (dat);
 
 	debug_exit ("file_headers_load_group");
 	return success;
 }
 
-static gchar*
+static char*
 get_mbox_filename (const Group * folder)
 {
-	gchar * retval;
+	char * retval;
 	debug_enter ("get_mbox_filename");
 
 	g_return_val_if_fail (folder!=NULL, NULL);
-	g_return_val_if_fail (folder->server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (folder->server), NULL);
 	g_return_val_if_fail (group_is_folder(folder), NULL);
-	g_return_val_if_fail (is_nonempty_string(data_dir), NULL);
-	g_return_val_if_fail (is_nonempty_string(folder->server->name), NULL);
 
-	retval = g_strdup_printf ("%s%c%s%c%s.mbox", data_dir,
+	retval = g_strdup_printf ("%s%c%s%c%s.mbox", get_data_dir(),
 	                                             G_DIR_SEPARATOR,
-	                                             folder->server->name,
+	                                             server_get_name (folder->server),
 	                                             G_DIR_SEPARATOR,
 	                                             folder->name);
 
@@ -233,9 +258,9 @@ get_mbox_filename (const Group * folder)
 static gboolean
 file_headers_load_folder (Group * folder, StatusItem * status)
 {
-	gint i;
-	gchar * text;
-	const gchar * march;
+	int i;
+	char * text = NULL;
+	const char * march;
 	GPtrArray * articles;
 	GString * msg;
 	debug_enter ("file_headers_load_folder");
@@ -245,14 +270,11 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 
 	/* get the mbox file */
 	{
-		GArray * array;
-		gchar * fname = get_mbox_filename (folder);
-		array = read_file (fname);
+		char * fname = get_mbox_filename (folder);
+		g_file_get_contents (fname, &text, NULL, NULL);
 		g_free (fname);
-		if (array == NULL)
+		if (text == NULL)
 			return FALSE;
-		text = array->data;
-		g_array_free (array, FALSE);
 	}
 
 	/* allocate the articles array */
@@ -266,7 +288,7 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 	msg = g_string_sized_new (2048);
 	while (march != NULL)
 	{
-		gchar * from = NULL;
+		char * from = NULL;
 
 		/* parse the next mbox message */
 		march = mbox_get_next_message (march, &from, msg);
@@ -287,7 +309,7 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 
 	/* if we've got articles then add them; otherwise, clean up */
 	if (articles->len != 0)
-		group_add_articles_remove_unused (folder, articles, status);
+		group_init_articles (folder, articles, status);
 	else {
 		g_hash_table_destroy (folder->_articles);
 		folder->_articles = NULL;
@@ -304,15 +326,15 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 void
 file_headers_load (Group * group, StatusItem * status)
 {
-	gint size;
-	struct timeval start;
-	struct timeval finish;
-	double diff;
 	gboolean success = FALSE;
+	int size;
+	double diff;
+	GTimeVal start;
+	GTimeVal finish;
 	debug_enter ("file_headers_load");
 
 	/* start the stopwatch */
-	gettimeofday (&start, NULL);
+	g_get_current_time (&start);
 
 	if (group_is_folder (group))
 	{
@@ -334,9 +356,9 @@ file_headers_load (Group * group, StatusItem * status)
 	}
 
 	/* timing stats */
-	gettimeofday (&finish, NULL);
+	g_get_current_time (&finish);
 	diff = finish.tv_sec - start.tv_sec;
-	diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+	diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
 	size = g_hash_table_size(group_get_articles(group));
 	if (size != 0)
 		log_add_va (LOG_INFO, _("Loaded %d articles for group `%s' in %.1f seconds (%.0f art/sec)"),
@@ -355,8 +377,8 @@ file_headers_load (Group * group, StatusItem * status)
  */
 long
 get_string_offset (GHashTable   * hash,
-                   GMemChunk    * long_chunk,
-                   const gchar  * str,
+                   MemChunk     * long_chunk,
+                   const char   * str,
                    FILE         * fp,
                    long         * pos)
 {
@@ -381,7 +403,7 @@ get_string_offset (GHashTable   * hash,
 	}
 	else /* first time; remember where we kept it */
 	{
-		glong * plong = g_chunk_new (glong, long_chunk);
+		glong * plong = (glong*) memchunk_alloc (long_chunk);
 		retval = *plong = *pos;
 		g_hash_table_insert (hash, (gpointer)str, plong);
 		*pos += fwrite (str, 1, strlen(str)+1, fp);
@@ -395,8 +417,8 @@ static gboolean
 file_headers_save_folder (Group * folder, StatusItem * status)
 {
 	FILE * fp = NULL;
-	gchar * fname = NULL;
-	gchar * fname_tmp = NULL;
+	char * fname = NULL;
+	char * fname_tmp = NULL;
 	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_folder");
 
@@ -406,7 +428,7 @@ file_headers_save_folder (Group * folder, StatusItem * status)
 
 	/* make sure the directory exists */
 	if (ok) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, folder->server->name);
+		char * pch = g_strdup_printf ("%s%c%s", get_data_dir(), G_DIR_SEPARATOR, server_get_name (folder->server));
 		if (!directory_check (pch)) {
 			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
 			ok = FALSE;
@@ -433,7 +455,7 @@ file_headers_save_folder (Group * folder, StatusItem * status)
 
 		for (i=0; i!=array->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(array,i));
-			gchar buf[32], *pch, *mbox;
+			char buf[32], *pch, *mbox;
 
 			/* add headers */
 			pch = article_get_headers (a);
@@ -497,23 +519,17 @@ file_headers_save_group (Group * group, StatusItem * status)
 	guint i;
 	FILE * idx_fp;
 	FILE * dat_fp;
-	gchar * pch;
-	gchar * idx_path;
-	gchar * dat_path;
+	char * pch;
+	char * idx_path;
+	char * dat_path;
 	long pos = 0;
-	GHashTable * hash_name = NULL;
-	GHashTable * hash_mail = NULL;
-	GHashTable * hash_refs = NULL;
-	GHashTable * hash_misc = NULL;
-	GMemChunk * long_chunk = NULL;
+	GHashTable * hash = NULL;
+	MemChunk * long_chunk = NULL;
 	GPtrArray * articles;
 	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_group");
 
-	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
-	g_return_val_if_fail (group->_articles!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
+	g_return_val_if_fail (group_is_valid (group), FALSE);
 
 	/* if nothing to save */
 	articles = group_get_article_array (group);
@@ -528,7 +544,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	**/
 
 	if (1) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, group->server->name);
+		char * pch = g_strdup_printf ("%s%c%s", get_data_dir(), G_DIR_SEPARATOR, server_get_name (group->server));
 		if (!directory_check (pch))
 			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
 		g_free (pch);
@@ -537,8 +553,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 	/* open index file */
 	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
-		data_dir, G_DIR_SEPARATOR,
-		group->server->name, G_DIR_SEPARATOR,
+		get_data_dir(), G_DIR_SEPARATOR,
+		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	idx_fp = fopen (idx_path, "w+");
 	if (idx_fp == NULL) {
@@ -549,8 +565,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 	/* open data file */
 	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp",
-		data_dir, G_DIR_SEPARATOR,
-		group->server->name, G_DIR_SEPARATOR,
+		get_data_dir(), G_DIR_SEPARATOR,
+		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	dat_fp = fopen (dat_path, "w+");
 	if (dat_fp == NULL) {
@@ -563,28 +579,25 @@ file_headers_save_group (Group * group, StatusItem * status)
 	}
 
 	/* Write DATBASE_VERSION */
-	fprintf (idx_fp, "4\n%ld\n", (long)articles->len);
+	fprintf (idx_fp, "7\n%ld\n", (long)articles->len);
 
 	/* Write the article information... */
 	pos = 0;
-	hash_name = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_mail = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_refs = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_misc = g_hash_table_new (g_str_hash, g_str_equal);
-	long_chunk = g_mem_chunk_create (glong, 8192, G_ALLOC_ONLY);
+	hash = g_hash_table_new (g_str_hash, g_str_equal);
+	long_chunk = memchunk_new (sizeof(glong), 4096, FALSE);
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index (articles, i));
 		GPtrArray * extra_headers = article_get_all_headers (a);
-		const gchar * subject = article_get_subject (a);
-		const gchar * message_id = article_get_message_id (a);
+		const char * subject = article_get_subject (a);
+		const char * message_id = article_get_message_id (a);
 		guint j;
-		const long id_idx          = get_string_offset (NULL,      NULL,       message_id,     dat_fp, &pos);
-		const long author_addr_idx = get_string_offset (hash_mail, long_chunk, a->author_addr, dat_fp, &pos);
-		const long author_real_idx = get_string_offset (hash_name, long_chunk, a->author_real, dat_fp, &pos);
-		const long subj_idx        = get_string_offset (hash_misc, long_chunk, subject,        dat_fp, &pos);
-		const long refs_idx        = get_string_offset (hash_refs, long_chunk, a->references,  dat_fp, &pos);
-		const long xref_idx        = get_string_offset (hash_refs, long_chunk, a->xref      ,  dat_fp, &pos);
+		const long id_idx          = get_string_offset (hash, long_chunk, message_id,     dat_fp, &pos);
+		const long author_addr_idx = get_string_offset (hash, long_chunk, a->author_addr, dat_fp, &pos);
+		const long author_real_idx = get_string_offset (hash, long_chunk, a->author_real, dat_fp, &pos);
+		const long subj_idx        = get_string_offset (hash, long_chunk, subject,        dat_fp, &pos);
+		const long refs_idx        = get_string_offset (hash, long_chunk, a->references,  dat_fp, &pos);
+		const long xref_idx        = get_string_offset (hash, long_chunk, a->xref,        dat_fp, &pos);
 
 		pan_warn_if_fail (a->number != 0);
 
@@ -593,18 +606,17 @@ file_headers_save_group (Group * group, StatusItem * status)
 			"%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n"
 			"%d\n" "%d\n"
 			"%u\n"
-			"%d\n"
-			"%d\n"
+			"%u\n"
 			"%lu\n"
-			"%lu\n",
+			"%lu\n"
+			"%d\n",
 			id_idx, author_addr_idx, author_real_idx, subj_idx, refs_idx, xref_idx,
-			(int)a->part,
-			(int)a->parts,
+			(int)a->part, (int)a->parts,
 			(unsigned int)a->linecount,
-			(int)a->crosspost_qty,
 			(int)a->state,
 			(unsigned long)a->date,
-			(unsigned long)a->number);
+			(unsigned long)a->number,
+			(int)(a->is_new != 0));
 
 		/* make sure the extra headers are in pairs */
 		if ((extra_headers->len % 2) != 0) {
@@ -614,8 +626,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 		/* remove extra headers that we don't want to save */
 		for (j=0; j<extra_headers->len; ) {
-			const gchar * key = (const gchar*) g_ptr_array_index (extra_headers, j);
-			const gchar * val = (const gchar*) g_ptr_array_index (extra_headers, j+1);
+			const char * key = (const char*) g_ptr_array_index (extra_headers, j);
+			const char * val = (const char*) g_ptr_array_index (extra_headers, j+1);
 			if (is_nonempty_string(key) && is_nonempty_string(val))
 				j += 2;
 			else {
@@ -627,21 +639,19 @@ file_headers_save_group (Group * group, StatusItem * status)
 		/* write the extra headers */
 		fprintf (idx_fp, "%d\n", (int)extra_headers->len/2); /* extra header qty */
 		for (j=0; j<extra_headers->len; j+=2) {
-			const gchar * key = (const gchar*) g_ptr_array_index (extra_headers, j);
-			const gchar * val = (const gchar*) g_ptr_array_index (extra_headers, j+1);
-			const long key_index = get_string_offset (hash_misc, long_chunk, key, dat_fp, &pos);
-			const long val_index = get_string_offset (hash_misc, long_chunk, val, dat_fp, &pos);
+			const char * key = (const char*) g_ptr_array_index (extra_headers, j);
+			const char * val = (const char*) g_ptr_array_index (extra_headers, j+1);
+			const long key_index = get_string_offset (hash, long_chunk, key, dat_fp, &pos);
+			const long val_index = get_string_offset (hash, long_chunk, val, dat_fp, &pos);
 			fprintf (idx_fp, "%ld\n%ld\n", key_index, val_index);
 		}
 
 		pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
 		g_ptr_array_free (extra_headers, TRUE);
 	}
-	g_hash_table_destroy (hash_name); hash_name = NULL;
-	g_hash_table_destroy (hash_mail); hash_mail = NULL;
-	g_hash_table_destroy (hash_refs); hash_refs = NULL;
-	g_hash_table_destroy (hash_misc); hash_misc = NULL;
-	g_mem_chunk_destroy (long_chunk);
+	g_hash_table_destroy (hash);
+	hash = NULL;
+	memchunk_destroy (long_chunk);
 
 	/* close the temp files */
 	fclose (idx_fp);
@@ -652,8 +662,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 	if (ok)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.idx",
-			data_dir, G_DIR_SEPARATOR,
-			group->server->name, G_DIR_SEPARATOR,
+			get_data_dir(), G_DIR_SEPARATOR,
+			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
 		ok = pan_rename (idx_path, pch);
 		g_free (idx_path);
@@ -663,8 +673,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 	if (ok)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.dat",
-			data_dir, G_DIR_SEPARATOR,
-			group->server->name, G_DIR_SEPARATOR,
+			get_data_dir(), G_DIR_SEPARATOR,
+			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
 		ok = pan_rename (dat_path, pch);
 		g_free (dat_path);
@@ -699,13 +709,13 @@ file_headers_save_noref (Group * group, StatusItem * status)
 		}
 		else /* save the group */
 		{
-			struct timeval start;
-			struct timeval finish;
+			GTimeVal start;
+			GTimeVal finish;
 			double diff;
 			gboolean ok = TRUE;
 
 			/* start the stopwatch */
-			gettimeofday (&start, NULL);
+			g_get_current_time (&start);
 
 			/* save the group */
 			ok = group_is_folder (group)
@@ -716,9 +726,9 @@ file_headers_save_noref (Group * group, StatusItem * status)
 				group->articles_dirty = TRUE;
 			else {
 				/* end the stopwatch */
-				gettimeofday (&finish, NULL);
+				g_get_current_time (&finish);
 				diff = finish.tv_sec - start.tv_sec;
-				diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+				diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
 				log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
 					articles->len,
 					group_get_readable_name (group),
@@ -753,25 +763,29 @@ file_headers_destroy (const Group * group)
 {
 	debug_enter ("file_headers_destroy");
 
-	g_return_if_fail (group!=NULL);
-	g_return_if_fail (group->server!=NULL);
-	g_return_if_fail (is_nonempty_string(data_dir));
+	g_return_if_fail (group_is_valid (group));
 
 	if (group_is_folder(group))
 	{
-		gchar * mbox = get_mbox_filename (group);
+		char * mbox = get_mbox_filename (group);
 		remove (mbox);
 		g_free (mbox);
 	}
 	else
 	{
-		gchar * path;
+		char * path;
 
-		path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.idx",
+		                        get_data_dir(), G_DIR_SEPARATOR,
+		                        server_get_name (group->server), 
+					G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 
-		path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.dat",
+		                        get_data_dir(), G_DIR_SEPARATOR,
+		                        server_get_name (group->server), 
+					G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 	}
