@@ -25,19 +25,17 @@
 
 #include <config.h>
 
-#include <string.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/time.h>
-
 #include <ctype.h>
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <limits.h>
 #include <time.h>
 
 #include <glib.h>
 
+#include <unistd.h>
+
 #include <pan/base/debug.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/pan-i18n.h>
@@ -47,44 +45,106 @@
 **********************  BEGINNING OF SOURCE
 *********************/
 
-/*****
-******
-*****/
+#define is_code_char(A) (_code_chars[(guchar)(A)])
+#define is_tag_char(A) (_tag_chars[(guchar)(A)])
+#define is_unquoted_char(A) (_unquoted_chars[(guchar)(A)])
+#define is_quoted_char(A) (_quoted_chars[(guchar)(A)])
+#define is_paren_char(A) (_paren_chars[(guchar)(A)])
 
-static gboolean
-is_unquoted_char (int ch)
-{
-	/* ASCII printable character except !()<>@,;:\".[] */
-	return isgraph(ch) && strchr("!()<>@,;:\\\".[]", ch)==NULL;
-}
+#if 0
+#define PRINT_TABLE(A) \
+	printf ("static char " #A "[UCHAR_MAX] = {"); \
+	for (i=0; i<UCHAR_MAX; ++i) { \
+		if (!(i%40)) \
+			printf ("\n\t"); \
+		printf ("%d,", A[i]); \
+	} \
+	printf ("};\n\n");
 
-static gboolean
-is_quoted_char (int ch)
-{
-	/* ASCII printable character except "()<>\ */
-	return isgraph(ch) && strchr("\"()<>\\", ch)==NULL;
-}
+static char _unquoted_chars[UCHAR_MAX];
+static char _quoted_chars[UCHAR_MAX];
+static char _tag_chars[UCHAR_MAX];
+static char _code_chars[UCHAR_MAX];
+static char _paren_chars[UCHAR_MAX];
 
-static gboolean
-is_tag_char (int ch)
+void
+gnksa_init (void)
 {
-	/* ascii printable character except !()<>@,;:\"[]/?=> */
-	return isgraph(ch) && strchr("!()<>@,;\\\"[]/?=", ch)==NULL;
-}
+	int i;
+	unsigned char ch;
 
-static gboolean
-is_code_char (int ch)
-{
-	/* ascii printable character except ? */
-	return isgraph(ch) && ch!='?';
-}
+	for (ch=0; ch<UCHAR_MAX; ++ch) {
+		_unquoted_chars[ch] = isgraph(ch) && ch!='!' && ch!='(' && ch!=')' && ch!='<'
+		                                  && ch!='>' && ch!='@' && ch!=',' && ch!=';'
+		                                  && ch!=':' && ch!='\\' && ch!='"' && ch!='.'
+		                                  && ch!='[' && ch!=']';
+		_quoted_chars[ch] = isgraph(ch) && ch!='"' && ch!='(' && ch!=')'
+		                                && ch!='<' && ch!='>' && ch!='\\';
+		_tag_chars[ch] = isgraph(ch) && ch!='!' && ch!='(' && ch!=')' && ch!='<'
+		                             && ch!='>' && ch!='@' && ch!=',' && ch!=';'
+		                             && ch!=':' && ch!='\\' && ch!='"' && ch!='['
+		                             && ch!=']' && ch!='/' && ch!='?' && ch!='=';
+		_paren_chars[ch] = isgraph(ch) && ch!='(' && ch!=')'
+		                               && ch!='<' && ch!='>'
+		                               && ch!='\\';
+		_code_chars[ch] = isgraph(ch) && ch!='?';
+	}
 
-static gboolean
-is_paren_char (int ch)
-{
-	/* ascii printable character except ()<>\ */
-	return isgraph(ch) && strchr("()<>\\",ch)==NULL;
+	PRINT_TABLE(_unquoted_chars)
+	PRINT_TABLE(_quoted_chars)
+	PRINT_TABLE(_tag_chars)
+	PRINT_TABLE(_code_chars)
+	PRINT_TABLE(_paren_chars)
 }
+#else
+
+static char _unquoted_chars[UCHAR_MAX] = {
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
+	0,0,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
+
+static char _quoted_chars[UCHAR_MAX] = {
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,1,1,1,1,
+	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
+
+static char _tag_chars[UCHAR_MAX] = {
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
+	0,0,1,1,0,1,1,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
+
+static char _code_chars[UCHAR_MAX] = {
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
+
+static char _paren_chars[UCHAR_MAX] = {
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
+	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
+
+#endif
+
 
 /**
 ***
@@ -181,9 +241,9 @@ static gboolean
 read_plain_word (const char * pch, const char ** end)
 {
 	/* unquoted-word / quoted-word / encoded-word */
-	if (read_unquoted_word(pch,end)) return TRUE;
 	if (read_quoted_word(pch,end)) return TRUE;
 	if (read_encoded_word(pch,end)) return TRUE;
+	if (read_unquoted_word(pch,end)) return TRUE;
 	return FALSE;
 }
 
@@ -284,7 +344,7 @@ gnksa_check_domain_literal (const char* domain)
 *****/
 
 static int
-gnksa_check_domain (const char * domain)
+gnksa_check_domain (const char * domain, guint domain_len)
 {
 	int i;
 	int label_qty;
@@ -302,7 +362,7 @@ gnksa_check_domain (const char * domain)
 	/* check for empty */
 	if (!is_nonempty_string (domain)
 		|| *domain=='.'
-		|| domain[strlen(domain)-1]=='.'
+		|| domain[domain_len-1]=='.'
 		|| pan_strstr(domain,"..")!=NULL)
 		return GNKSA_ZERO_LENGTH_LABEL;
 
@@ -392,7 +452,7 @@ gnksa_check_localpart (const char * localpart)
 *****/
 
 static gint
-gnksa_check_address (const gchar * address)
+gnksa_check_address (const gchar * address, gint address_len)
 {
 	gint retval = GNKSA_OK;
 	gchar * tmp;
@@ -403,7 +463,7 @@ gnksa_check_address (const gchar * address)
 		return GNKSA_LOCALPART_MISSING;
 
 	/* check the address */
-	tmp = g_strdup (address);
+	pan_strndup_alloca (tmp, address, address_len);
 	begin = strrchr (address, '@');
 	if (begin==NULL)
 	{
@@ -418,7 +478,7 @@ gnksa_check_address (const gchar * address)
 
 		/* check the domain */
 		if (retval == GNKSA_OK)
-			retval = gnksa_check_domain (begin);
+			retval = gnksa_check_domain (begin, address_len - (begin-address));
 
 		/* check the localpart */
 		if (retval == GNKSA_OK)
@@ -428,7 +488,6 @@ gnksa_check_address (const gchar * address)
 		*--begin = '@';
 	}
 
-	g_free (tmp);
 	return retval;
 }
 
@@ -437,30 +496,28 @@ gnksa_check_address (const gchar * address)
 *****/
 
 static int
-gnksa_split_from (const char   * from,
-                  char        ** alloc_and_setme_address,
-		  char        ** alloc_and_setme_realname,
-		  int          * addrtype,
+gnksa_split_from (const gchar  * from,
+                  gchar        * addr_buf,
+                  gint           addr_max,
+                  gchar        * name_buf,
+                  gint           name_max,
+		  gint         * addrtype,
 		  gboolean       strict)
 {
-	char * begin;
-	char * end;
-	char * work;
-	char * lparen;
+	gchar * begin;
+	gchar * end;
+	gchar * work;
+	gchar * lparen;
 
-	*alloc_and_setme_address = NULL;
-	*alloc_and_setme_realname = NULL;
+	*addr_buf = '\0';
+	*name_buf = '\0';
 
-	work = g_strdup (from);
-
-	if (is_nonempty_string(work))
-		g_strstrip (work);
+	work = pan_strdup_alloca (from);
+	g_strstrip (work);
 
 	/* empty string */
-	if (!*work)
-	{
+	if (!*work) {
 		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
-		g_free (work);
 		return GNKSA_LPAREN_MISSING;
 	}
 
@@ -472,79 +529,69 @@ gnksa_split_from (const char   * from,
 
 		/* get address part */
 		begin = strrchr (work, '<');
-		if (begin == NULL) {
-			g_free (work);
+		if (begin == NULL)
 			return GNKSA_LANGLE_MISSING;
-		}
 
 		/* copy route address from inside the <> brackets */
-		*alloc_and_setme_address = g_strdup (begin+1);
-		*strchr(*alloc_and_setme_address, '>') = '\0';
+		g_strlcpy (addr_buf, begin+1, addr_max);
+		*strchr(addr_buf, '>') = '\0';
 
 		/* From: [plain-phrase space] "<" address ">" */
 		*begin = '\0';
 		if (strict) {
 			const char * tmp = work;
-			if ((*tmp) && (!read_plain_phrase(tmp,&tmp)||!read_space(tmp,&tmp))) {
-				g_free (work);
+			if ((*tmp) && (!read_plain_phrase(tmp,&tmp)||!read_space(tmp,&tmp)))
 				return GNKSA_ILLEGAL_PLAIN_PHRASE;
-			}
 		}
 
 		/* get realname part */
 		g_strstrip (work);
-		*alloc_and_setme_realname = g_strdup (work);
+		g_strlcpy (name_buf, work, name_max);
 	}
 	else if ((lparen = strchr (work,'(')) != NULL) /* charles@rebelbase.com (Charles Kerr) */
 	{
 		size_t len;
-		gchar * address;
 		gchar * real;
 		const gchar * end;
 
 		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
 
 		/* address part */
-		address = lparen==work ? NULL : g_strndup (work, lparen-work);
-		if (is_nonempty_string (address))
-			g_strstrip (address);
-		*alloc_and_setme_address = address;
+		len = MIN(addr_max, lparen-work);
+		g_strlcpy (addr_buf, work, len);
+		g_strstrip (addr_buf);
+
 		if (strict) {
-			gint val = gnksa_check_address (address);
-			if (val) {
-				g_free (work);
+			gint val = gnksa_check_address (addr_buf, strlen(addr_buf));
+			if (val)
 				return val;
-			}
 		}
 
 		/* real name part */
 		real = lparen + 1;
 		g_strstrip (real);
 		len = strlen (real);
-		if (real[len-1]!=')') {
-			g_free (work);
+		if (real[len-1]!=')')
 			return GNKSA_RPAREN_MISSING;
-		}
+
 		real[len-1] = '\0';
 		end = NULL;
-		if (strict && (!read_paren_phrase(lparen+1,&end) || end==NULL)) {
-			g_free (work);
+		if (strict && (!read_paren_phrase(lparen+1,&end) || end==NULL))
 			return GNKSA_ILLEGAL_PAREN_PHRASE;
-		}
-		*alloc_and_setme_realname = g_strdup (real);
+
+		g_strlcpy (name_buf, real, name_max);
 	}
 	else if (strchr(work,'@') != NULL) /* charles@rebelbase.com */
 	{
-		*alloc_and_setme_address = work;
-		return strict ? gnksa_check_address(work) : GNKSA_OK;
+		g_strlcpy (addr_buf, work, addr_max);
+		return strict ? gnksa_check_address(addr_buf, strlen(addr_buf)) : GNKSA_OK;
 	}
 	else /* who knows what this thing is... */
 	{
-		*alloc_and_setme_realname = work;
+		g_strlcpy (name_buf, work, name_max);
 		return GNKSA_LPAREN_MISSING;
 	}
 
-	g_free (work);
 	return GNKSA_OK;
 }
 
@@ -606,32 +653,31 @@ gnksa_split_addresses (const char * from)
 
 int
 gnksa_do_check_from (const char   * from,
-                     char        ** alloc_and_setme_address,
-                     char        ** alloc_and_setme_realname,
+                     gchar        * addr,
+                     guint          addr_max,
+                     gchar        * name,
+                     guint          name_max,
 		     gboolean       strict)
 {
 	int addrtype = 0;
 	int retval;
-	char * address;
-	char * realname;
 	debug1 (DEBUG_GNKSA, "From: [%s]", from);
 
 	/* split from */
+	*addr = *name = '\0';
 	retval = gnksa_split_from (from,
-	                           alloc_and_setme_address,
-		         	   alloc_and_setme_realname, 
+	                           addr, addr_max,
+		         	   name, name_max, 
 			           &addrtype,
 				   strict);
 
-	address = *alloc_and_setme_address;
-	realname = *alloc_and_setme_realname;
-	debug1 (DEBUG_GNKSA, "Name: [%s]", (realname?realname:"NULL"));
-	debug1 (DEBUG_GNKSA, "Address: [%s]", (address?address:"NULL"));
+	debug1 (DEBUG_GNKSA, "Name: [%s]", name);
+	debug1 (DEBUG_GNKSA, "Address: [%s]", addr);
 
 	/* check address */
-	if (address != NULL) {
+	if (*addr) {
 		if (retval == GNKSA_OK)
-			retval = gnksa_check_address (address);
+			retval = gnksa_check_address (addr, strlen(addr));
 	}
 
 	debug1 (DEBUG_GNKSA, "GNKSA [%d]", retval);
@@ -652,14 +698,14 @@ gnksa_strip_realname (gchar * realname)
 	g_strstrip (realname);
 	if (*realname=='(') {
 		size_t len = strlen (realname+1);
-		g_memmove (realname, realname+1, len+1);
+		memmove (realname, realname+1, len+1);
 		pch = realname + len - 1;
 		if (*pch == ')')
 			*pch = '\0';
 	}
 
 	while ((pch = strchr(realname,'"')) != NULL)
-		g_memmove (pch, pch+1, strlen(pch+1)+1);
+		memmove (pch, pch+1, strlen(pch+1)+1);
 }
 
 /*****
@@ -670,13 +716,13 @@ int
 gnksa_check_from (const char* from, gboolean strict)
 {
 	int retval;
-	char * address;
-	char * realname;
+	gchar addr[512];
+	gchar name[512];
 
-	address = realname = NULL;
-	retval = gnksa_do_check_from (from, &address, &realname, strict);
-	g_free (address);
-	g_free (realname);
+	retval = gnksa_do_check_from (from,
+	                              addr, sizeof(addr),
+	                              name, sizeof(name),
+	                              strict);
 
 	return retval;
 }
@@ -685,66 +731,64 @@ gnksa_check_from (const char* from, gboolean strict)
 ******
 *****/
 
-/* see son-of-1036 5.3 */
-
+/**
+ * Validates a Message-ID from a References: header.
+ *
+ * The purpose of this function is to remove severely broken,
+ * usually suntactically-invalid Message-IDs, such as those
+ * missing '<', '@', or '>'. 
+ *
+ * However we want to retain Message-IDs that might be sloppy,
+ * such as ones that have possibly-invalid domains.
+ *
+ * This balance is from wanting to adhere to GNKSA that wants us
+ * to remove `damaged' Message-IDs, but we want to be pretty
+ * forgiving because these References are required for threading
+ * to work properly.
+ */
 static int
-gnksa_check_message_id (const char* message_id)
+gnksa_check_message_id (const char* message_id, gint message_id_len)
 {
 	int val;
 	gchar * tmp;
 	gchar * pch;
 
 	/* make a temporary copy that we can poke */
-       	tmp = g_strdup (message_id);
+       	pan_strndup_alloca (tmp, message_id, message_id_len);
 	if (is_nonempty_string(tmp))
 		g_strstrip (tmp);
 
 	/* make sure it's <= 250 octets (son of gnksa 1036 5.3) */
-	if (strlen(tmp) > 250) {
-		g_free (tmp);
+	if (strlen(tmp) > 250)
 		return GNKSA_ILLEGAL_LABEL_LENGTH;
-	}
 
 	/* make sure the message-id is wrapped in < > */
-	if (*tmp!='<') {
-		g_free (tmp);
+	if (*tmp!='<')
 		return GNKSA_LANGLE_MISSING;
-	}
-	if (tmp[strlen(tmp)-1]!='>') {
-		g_free (tmp);
+	if (tmp[strlen(tmp)-1]!='>')
 		return GNKSA_RANGLE_MISSING;
-	}
 
 	/* find the '@' separator */
 	pch = strrchr (tmp, '@');
-	if (pch == NULL) {
-		g_free (tmp);
+	if (pch == NULL)
 		return GNKSA_ATSIGN_MISSING;
-	}
 
 	/* check the domain name */
 	++pch;
 	*strrchr(pch,'>') = '\0';
-	val = gnksa_check_localpart (pch); /* yes, I mean to do this */
-	if (val != GNKSA_OK) {
-		g_free (tmp);
-		return val;
-	}
+	if (!is_nonempty_string (pch))
+		return GNKSA_ILLEGAL_DOMAIN;
 
 	/* check the local-part */
 	--pch;
 	*pch = '\0';
-	if (!g_strcasecmp (tmp+1, "postmaster")) { /* son-of-1036 */
-		g_free (tmp);
+	if (!g_strcasecmp (tmp+1, "postmaster")) /* son-of-1036 */
 		return GNKSA_INVALID_LOCALPART;
-	}
-	val = gnksa_check_localpart (tmp+1);
-	if (val != GNKSA_OK) {
-		g_free (tmp);
-		return val;
-	}
 
-	g_free (tmp);
+	val = gnksa_check_localpart (tmp+1);
+	if (val != GNKSA_OK)
+		return val;
+
 	return GNKSA_OK;
 }
 
@@ -752,61 +796,52 @@ gnksa_check_message_id (const char* message_id)
 ******
 *****/
 
-static char*
-remove_broken_message_ids_from_references (const char * references)
+static void
+remove_broken_message_ids_from_references (const char * references, GString * setme)
 {
-	gchar * pch;
-	gchar * tmp;
-	GString * fixed;
+	const gchar * pch;
+	const gchar * tmp;
 
 	/* sanity checks */
-	g_return_val_if_fail (is_nonempty_string(references), NULL);
+	g_return_if_fail (setme!=NULL);
+	g_string_truncate (setme, 0);
+	g_return_if_fail (is_nonempty_string(references));
 
 	/* remove broken message-ids
 	   FIXME: this tokenizing works, but it's hard to read. */
-       	fixed = g_string_sized_new (1024);
-       	tmp = g_strdup (references);
-	pch = tmp;
+       	pch = tmp = references;
 	for (;;)
 	{
-		gchar tmp_end_val;
-		gchar * begin;
-		gchar * end;
+		const gchar * begin;
+		const gchar * end;
 
-		/* skip to the beginning of a message id */
+		/* find the beginning of the message-id */
 		while (*pch!='<')
 			++pch;
 		begin = pch;
 		++pch;
+
+		/* find the end of the message-id */
 		end = strpbrk (pch, "<> ");
 		if (end==NULL)
 			end = pch + strlen(pch);
 		else if (*end == '>')
 			++end;
 
-		/* zero-terminate the message-id to check it for validity */
-		tmp_end_val = *end;
-		*end = '\0';
-		if (gnksa_check_message_id(begin)==GNKSA_OK) {
-			g_string_append (fixed, begin);
-			g_string_append_c (fixed, ' ');
+		/* check the message-id for validity */
+		if (gnksa_check_message_id (begin, end-begin) == GNKSA_OK) {
+			g_string_append_len (setme, begin, end-begin);
+			g_string_append_c (setme, ' ');
 		}
-		*end = tmp_end_val;
-		pch = end;
 
 		/* if that was the last token, then we're done */
-		if (tmp_end_val == '\0')
+		if (*end == '\0')
 			break;
 
+		pch = end;
+
 	}
-	pan_g_string_strstrip (fixed); /* remove trailing space  */
-
-	/* cleanup */
-	pch = fixed->str;
-	g_free (tmp);
-	g_string_free (fixed, FALSE);
-
-	return pch;
+	pan_g_string_strstrip (setme); /* remove trailing space  */
 }
 
 /**
@@ -817,28 +852,31 @@ remove_broken_message_ids_from_references (const char * references)
 gchar*
 gnksa_trim_references_to_len (const gchar* refs, int cutoff)
 {
-	char * fixed = remove_broken_message_ids_from_references (refs);
 	char * retval;
-	const int fixed_len = strlen (fixed);
+	GString * fixed;
+	gboolean do_free = TRUE;
 
-	if (fixed_len < cutoff) /* this is too easy. :) */
+	fixed = g_string_sized_new (1024);
+	remove_broken_message_ids_from_references (refs, fixed);
+
+	if (fixed->len < cutoff) /* this is too easy. :) */
 	{
-		retval = g_strdup(fixed);
+		retval = fixed->str;
+		do_free = FALSE;
 	}
 	else /* plow through them */
 	{
 		int leader_len;
 		int len_left;
-		const char * str_end = fixed + fixed_len;
+		const char * str_end = fixed->str + fixed->len;
 		const char * pch;
 
 		/* isolate first id, which we must always keep */
-	       	pch = fixed;
+	       	pch = fixed->str;
 		while (*pch!='<') ++pch; /* find beginning of first id */
 		while (*pch!='>') ++pch; /* find end of first id */
 		pch += 2; /* keep the "> " */
-		leader_len = pch - fixed;
-
+		leader_len = pch - fixed->str;
 
 		/* trim out 2..n until we fit.  This implicitly lets later
 		   ids make the final cut, which is in line with gnksa's
@@ -851,12 +889,12 @@ gnksa_trim_references_to_len (const gchar* refs, int cutoff)
 
 		/* generate return string */
 		retval = g_malloc (leader_len + (str_end-pch) + 1);
-		g_memmove (retval, fixed, leader_len);
-		g_memmove (retval+leader_len, pch, (str_end-pch)+1); /*+1 for end*/
+		memcpy (retval, fixed->str, leader_len);
+		memcpy (retval+leader_len, pch, (str_end-pch)+1); /*+1 for end*/
 	}
 
 	/* return */
-	g_free (fixed);
+	g_string_free (fixed, do_free);
 	pan_warn_if_fail (strlen(retval) <= cutoff);
 	return retval;
 }
@@ -881,13 +919,13 @@ gnksa_generate_message_id (const gchar * domain_name)
 	/* add unique local part to message-id */
 	{
 		gulong pid;
-		struct timeval now;
+		GTimeVal now;
 		struct tm local_now;
 		char buf[64];
 
 		pid = (gulong) getpid ();
-		gettimeofday (&now, NULL);
-		localtime_r (&now.tv_sec, &local_now);
+		g_get_current_time (&now);
+		pan_gmtime_r (&now.tv_sec, &local_now);
 		strftime (buf, sizeof(buf), "%Y.%m.%d.%H.%M.%S", &local_now);
 
 		g_string_append (msgid, "pan.");
@@ -916,8 +954,7 @@ gchar*
 gnksa_generate_message_id_from_email_addr (const gchar * addr)
 {
 	const gchar * pch;
-	gchar * domain;
-	gchar * retval;
+	gchar * message_id;
 
 	/* find the domain in the email address */
 	pch = NULL;
@@ -934,18 +971,17 @@ gnksa_generate_message_id_from_email_addr (const gchar * addr)
 	}
 
 	/* make a copy of the rhs of the email address and trim out the '>' */
-	domain = g_strdup (pch);
-	pch = strchr (domain, '>');
-	if (pch != NULL)
-		*((gchar*)pch) = '\0';
-	g_strstrip (domain);
-
-	/* generate the domain name */
-	retval = gnksa_generate_message_id (domain);
+	if (1) {
+		gchar * domain = pan_strdup_alloca (pch);
+		pch = strchr (domain, '>');
+		if (pch != NULL)
+			*((gchar*)pch) = '\0';
+		g_strstrip (domain);
+		message_id = gnksa_generate_message_id (domain);
+	}
 
 	/* cleanup */
-	g_free (domain);
-	return retval;
+	return message_id;
 }
 
 
@@ -986,46 +1022,36 @@ gnksa_generate_references (const gchar * references,
 ****
 ***/
 
-/* converts struct tm to time_t, but does not take the local
-   timezone into account unless ``local'' is nonzero.
-   This is lifted from mutt 1.2.4, which is GPL and
-   copyrighted by Michael R. Elkins <me@cs.hmc.edu> */
 static time_t
-mutt_mktime (struct tm *t)
+tzoffset_sec (time_t * now)
 {
-	time_t g;
+	struct tm gmt, lt;
+	int off;
 
-	static int AccumDaysPerMonth[12] = {
-		0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
-	};
+	pan_gmtime_r (now, &gmt);
+	pan_localtime_r (now, &lt);
 
-	/* Compute the number of days since January 1 in the same year */
-	g = AccumDaysPerMonth [t->tm_mon % 12];
+	off = (lt.tm_hour - gmt.tm_hour) * 60 + lt.tm_min - gmt.tm_min;
 
-	/* The leap years are 1972 and every 4. year until 2096,
-	 * but this algoritm will fail after year 2099 */
-	g += t->tm_mday;
-	if ((t->tm_year % 4) || t->tm_mon < 2)
-		g--;
-	t->tm_yday = g;
+	if (lt.tm_year < gmt.tm_year)
+		off -= 24 * 60;
+	else if (lt.tm_year > gmt.tm_year)
+		off += 24 * 60;
+	else if (lt.tm_yday < gmt.tm_yday)
+		off -= 24 * 60;
+	else if (lt.tm_yday > gmt.tm_yday)
+		off += 24 * 60;
 
-	/* Compute the number of days since January 1, 1970 */
-	g += (t->tm_year - 70) * 365;
-	g += (t->tm_year - 69) / 4;
+	if (off >= 24 * 60)             /* should be impossible */
+		off = 23 * 60 + 59;     /* if not, insert silly value */
+	if (off <= -24 * 60)
+		off = -(23 * 60 + 59);
+	if (off > 12 * 60) 
+		off -= 24 * 60;
+	if (off < -12 * 60)
+		off += 24 * 60;
 
-	/* Compute the number of hours */
-	g *= 24;
-	g += t->tm_hour;
-
-	/* Compute the number of minutes */
-	g *= 60;
-	g += t->tm_min;
-
-	/* Compute the number of seconds */
-	g *= 60;
-	g += t->tm_sec;
-
-	return g;
+	return off * 60;
 }
 
 gchar*
@@ -1042,20 +1068,18 @@ rfc822_date_generate (time_t secs)
 	int diff;
 	int tz_diff_hours;
 	int tz_diff_minutes;
+	struct tm local;
 	gboolean neg;
 	gchar fmt[64] = { '\0' };
 	gchar buf[64] = { '\0' };
-	struct tm ut;
-	struct tm local;
 
 	/* if 0 passed in, use current time */
 	if (secs == 0)
 		secs = time (NULL);
 
 	/* figure out the difference between this timezone and UT */
-	localtime_r (&secs, &local);
-	gmtime_r (&secs, &ut);
-	diff = (int) difftime (mutt_mktime(&local), mutt_mktime(&ut));
+	pan_localtime_r (&secs, &local);
+	diff = (int) tzoffset_sec (&secs);
 	neg = diff<0;
 	diff = abs(diff);
 	tz_diff_hours = diff / 3600;
@@ -1080,7 +1104,7 @@ rfc822_date_generate (time_t secs)
 	 * This was causing problems in that tz, which is +0930.  I'm feeling
 	 * pretty dumb here... :)
 	 */
-	sprintf (fmt, "%s, %%d %s %%Y %%H:%%M:%%S %c%02d%02d",
+	g_snprintf (fmt, sizeof(fmt), "%s, %%d %s %%Y %%H:%%M:%%S %c%02d%02d",
 		days[local.tm_wday],
 		months[local.tm_mon],
 		(neg?'-':'+'),
@@ -1091,45 +1115,3 @@ rfc822_date_generate (time_t secs)
 	strftime (buf, sizeof(buf), fmt, &local);
 	return g_strdup (buf);
 }
-
-/***
-****
-***/
-
-#ifndef HAVE_LOCALTIME_R
-struct tm*
-localtime_r(const time_t *const timep, struct tm *p_tm)
-{
-	struct tm * retval;
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-
-	g_static_mutex_lock (&mutex);
-	retval = localtime (timep);
-	if (retval != NULL) {
-		memcpy (p_tm, retval, sizeof(struct tm));
-		retval = p_tm;
-	}
-	g_static_mutex_unlock (&mutex);
-
-	return retval;
-}
-#endif
-
-#ifndef HAVE_GMTIME_R
-struct tm*
-gmtime_r (const time_t * const timep, struct tm * p_tm)
-{
-	struct tm * retval;
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-
-	g_static_mutex_lock (&mutex);
-	retval = gmtime (timep);
-	if (retval != NULL) {
-		memcpy (p_tm, retval, sizeof(struct tm));
-		retval = p_tm;
-	}
-	g_static_mutex_unlock (&mutex);
-
-	return retval;
-}
-#endif
