@@ -85,6 +85,7 @@ group_constructor (Group                * g,
 	g->description = NULL;
 	g->readable_name = NULL;
 	g->download_dir = NULL;
+	g->default_charset = NULL;
 	g->chunk = NULL;
 
 	g->_articles_refcount = (gint)0;
@@ -116,6 +117,7 @@ group_destructor (Group * o)
 	replace_gstr (&g->description, NULL);
 	replace_gstr (&g->readable_name, NULL);
 	replace_gstr (&g->download_dir, NULL);
+	replace_gstr (&g->default_charset, NULL);
 	replace_gstr (&g->filter_name, NULL);
 	replace_gstr (&g->identity_name, NULL);
 
@@ -234,6 +236,34 @@ group_set_download_dir (Group * group, const gchar* download_dir)
 	replace_gstr (&group->download_dir, g_strdup(download_dir));
 }
 
+void
+group_set_default_charset (Group * group,
+                           const char * default_charset)
+{
+	g_return_if_fail (group_is_valid(group));
+
+	if (!is_nonempty_string (default_charset))
+		default_charset = NULL;
+
+	replace_gstr (&group->default_charset, g_strdup(default_charset));
+
+	group_set_dirty (group);
+	fire_groups_changed (&group, 1);
+}
+
+const char*
+group_get_default_charset (const Group * group)
+{
+	const char * retval = get_charset_from_locale ();
+
+	g_return_val_if_fail (group_is_valid(group), retval);
+
+	if (group->default_charset != NULL)
+		retval = group->default_charset;
+
+	return retval;
+}
+
 void
 group_set_flags (Group * group, guint flags)
 {
@@ -594,10 +624,9 @@ group_add_articles_impl (Group             * group,
 	**/
 
 	new_read_qty = 0;
-	tmp_used = g_ptr_array_new ();
+	tmp_used = g_ptr_array_sized_new (articles->len);
 	newsrc = group_get_newsrc (group);
 	purged = group_get_purged (group);
-	pan_g_ptr_array_reserve (tmp_used, articles->len);
 
 	if (lock_group) {
 		g_mutex_lock (group->article_mutex);
@@ -889,8 +918,7 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 
 	/* find old articles to remove */
 	tmp = group_get_article_array (g);
-	removeme = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (removeme, tmp->len);
+	removeme = g_ptr_array_sized_new (tmp->len);
 	for (i=0; i!=tmp->len; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(tmp,i));
 		if ((a->number<low) || (a->number>high))
@@ -920,34 +948,31 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 
 static GStaticMutex article_refcount_mutex = G_STATIC_MUTEX_INIT;
 
-static gboolean 
-group_ref_articles_if_loaded (Group * group, StatusItem * status)               
-{                                                                               
-        gboolean retval = FALSE;                                                
-        debug_enter ("group_ref_articles_if_loaded");                           
+static gboolean
+group_ref_articles_if_loaded (Group * group, StatusItem * status)
+{
+	gboolean retval = FALSE;
+	debug_enter ("group_ref_articles_if_loaded");
 
-        /* sanity clause */                                                     
-        g_return_val_if_fail (group_is_valid(group), FALSE);                    
-        g_return_val_if_fail (group->_articles_refcount>=0, FALSE);             
+	/* sanity clause */
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+	g_return_val_if_fail (group->_articles_refcount>=0, FALSE);
 
-        /* ref if reffed */                                                     
-        g_static_mutex_lock (&article_refcount_mutex);                          
+	/* ref if reffed */
+	g_static_mutex_lock (&article_refcount_mutex);
+	if (group->_articles_refcount > 1) {
+		++group->_articles_refcount;
+		retval = TRUE;
+	}
+	g_static_mutex_unlock (&article_refcount_mutex);
 
-        if (group->_articles_refcount > 1) {                                    
-                ++group->_articles_refcount;                                    
-                retval = TRUE;                                                  
-        }                                                                       
-        g_static_mutex_unlock (&article_refcount_mutex);                        
-
-        debug_exit ("group_ref_articles_if_loaded");                            
-
-        return retval;                                                          
-}                                                                               
+	debug_exit ("group_ref_articles_if_loaded");
+	return retval;
+}
 
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
-
 	debug_enter ("group_ref_articles");
 
 	/* sanity clause */
@@ -1032,7 +1057,7 @@ group_set_readable_name (Group         * group,
 }
 
 
-const gchar*
+const char*
 group_get_readable_name (const Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), _("(No Group)"));
@@ -1042,6 +1067,34 @@ group_get_readable_name (const Group * group)
 		: group->name;
 }
 
+void
+group_get_collapsed_name (const Group   * group,
+                          char          * buf,
+                          size_t          bufsize)
+{
+	int pos = 0;
+	int len = 0;
+	const char * pch = NULL;
+	const char * readable = group_get_readable_name (group);
+	const char * march = readable;
+	const char * last = "";
+
+	while (pos+2<bufsize && get_next_token_run (march, '.', &march, &pch, &len)) {
+		last = pch;
+		if (len > 0) {
+			buf[pos++] = *pch;
+			buf[pos++] = '.';
+		}
+	}
+	if (pos != 0)
+		pos -= 2;
+	buf[pos] = '\0';
+
+	if (is_nonempty_string(last))
+		g_strlcpy (buf+pos, last, bufsize-pos);
+}
+
+
 /**
 ***
 **/
@@ -1126,31 +1179,37 @@ compare_ppA_to_ppA_by_number (const void * va, const void * vb)
 {
 	const Article * a = *(const Article **)va;
 	const Article * b = *(const Article **)vb;
-	if (a->number < b->number) return -1;
-	if (a->number > b->number) return 1;
-	return 0;
+	int retval;
+
+	if (a->number < b->number)
+		retval = -1;
+	else if (a->number > b->number)
+		retval = 1;
+	else
+		retval = 0;
+
+	return retval;
 }
 
 void
 group_remove_crossposts (Group     * g,
 		         gulong    * numbers,
-                         gint        number_qty)
+                         int         number_qty)
 {
 	/* sanity clause */
 	g_return_if_fail (group_is_valid(g));
 	g_return_if_fail (numbers!=NULL);
 	g_return_if_fail (number_qty>0);
 
-	/* remove articles if they're already present */
+	/* if that group's articles are loaded, remove the corresponding Articles */
 	if (group_ref_articles_if_loaded (g, NULL))
 	{
-		gint i;
+		int i;
 		GPtrArray * articles;
 		GPtrArray * remove;
 
 		/* get a list of articles sorted by number for fast lookup */
-		remove = g_ptr_array_new ();
-		pan_g_ptr_array_reserve (remove, number_qty);
+		remove = g_ptr_array_sized_new (number_qty);
 		articles = group_get_article_array (g);
 		qsort (articles->pdata, articles->len, sizeof(Article*), compare_ppA_to_ppA_by_number);
 
@@ -1203,7 +1262,7 @@ group_alloc_new_article (Group * group)
 	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_article_chunk == NULL)
-		group->_article_chunk = memchunk_new (sizeof(Article), 200, FALSE);
+		group->_article_chunk = memchunk_new (sizeof(Article), 500, FALSE);
 
 	a = (Article*) memchunk_alloc (group->_article_chunk);
 	a->group = group;
