@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,15 @@
 #include <string.h>
 #include <time.h>
 
+#include <gmime/memchunk.h>
+
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 
 
 static GStaticMutex _log_mutex = G_STATIC_MUTEX_INIT;
 static GStringChunk * _strings = NULL;
-static GMemChunk * _entries = NULL;
+static MemChunk * _entries = NULL;
 static GPtrArray * _log_entries = NULL;
 static gboolean _inited = FALSE;
 
@@ -44,8 +46,8 @@ log_rebuild_entry_memory (void)
 	_log_entries = g_ptr_array_new ();
 
 	if (_entries != NULL)
-		g_mem_chunk_destroy (_entries);
-	_entries = g_mem_chunk_create (LogEntry, 64, G_ALLOC_ONLY);
+		memchunk_destroy (_entries);
+	_entries = memchunk_new (sizeof(LogEntry), 64, FALSE);
 
 	if (_strings != NULL)
 		g_string_chunk_free (_strings);
@@ -78,7 +80,7 @@ log_shutdown_module (void)
 		_inited = FALSE;
 
 		g_ptr_array_free (_log_entries, TRUE);
-		g_mem_chunk_destroy (_entries);
+		memchunk_destroy (_entries);
 		g_string_chunk_free (_strings);
 
 		_log_entries = NULL;
@@ -122,14 +124,14 @@ log_add_entry (LogEntry * entry)
 }
 
 static LogEntry*
-log_entry_new (LogSeverity severity, const gchar * msg)
+log_entry_new (LogSeverity severity, const char * msg)
 {
 	LogEntry * entry;
 
 	if (!_inited)
 		log_init ();
-       
-	entry = g_chunk_new (LogEntry, _entries);
+
+	entry = (LogEntry*) memchunk_alloc (_entries);
 	entry->message = g_strchomp (g_string_chunk_insert (_strings, msg));
 	entry->date = time(NULL);
 	entry->severity = severity;
@@ -137,7 +139,7 @@ log_entry_new (LogSeverity severity, const gchar * msg)
 }
 
 void
-log_add (LogSeverity severity, const gchar * message)
+log_add (LogSeverity severity, const char * message)
 {
 	LogEntry * entry;
 	g_static_mutex_lock (&_log_mutex);
@@ -148,10 +150,10 @@ log_add (LogSeverity severity, const gchar * message)
 }
 
 void
-log_add_va (LogSeverity severity, const gchar * format, ...)
+log_add_va (LogSeverity severity, const char * format, ...)
 {
 	va_list args;
-	gchar * message;
+	char * message;
 	LogEntry * entry;
 	g_return_if_fail (format != NULL);
 
