@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -165,8 +165,8 @@ init_read (Newsrc * n)
 static int
 compare_pulong_to_pRange (const void * a, const void * b)
 {
-	gulong number = *(gulong *)a;
-	const Range * range = (const Range*) b;
+	const register gulong number = *(gulong *)a;
+	const register Range * range = (const Range*) b;
 	int retval;
 
 	if (number < range->low)
@@ -296,9 +296,9 @@ mark_range_read (Newsrc * n, Range * rr)
 static gulong
 mark_range_unread (Newsrc * n, const Range * ur)
 {
-	gint i;
-	gint low_index = -1;
-	gint high_index = -1;
+	int i;
+	int low_index = -1;
+	int high_index = -1;
 	gboolean contains_low = FALSE;
 	gboolean contains_high = FALSE;
 	gulong retval = 0;
@@ -463,10 +463,11 @@ void
 newsrc_set_group_range (Newsrc * n, gulong low, gulong high)
 {
 	gboolean exact_match;
-	gint low_index;
-	gint high_index;
+	int low_index;
+	int high_index;
 
-	g_return_if_fail (n != NULL);
+	/* sanity clause */
+	g_return_if_fail (n!=NULL);
 
 	n->group_low = low;
 	n->group_high = high;
@@ -619,7 +620,7 @@ newsrc_get_read_str (const Newsrc * n)
 	{
 		int i;
 		const int max_chars_per_range = 22; /* "%lu-%lu," w/10 digits for %lu */
-		char * pch = retval = g_new0 (gchar, n->read->len*max_chars_per_range + 1);
+		char * pch = retval = g_new0 (char, n->read->len*max_chars_per_range + 1);
 
 		for (i=0; i<n->read->len; ++i)
 		{
