@@ -2,7 +2,7 @@
 /* 
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,10 +26,14 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <locale.h>
 
 #include <glib.h>
 
+#include <gmime/gmime-utils.h>
+
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
 
 
 /***
@@ -360,28 +364,6 @@ pan_g_string_strstrip (GString * string)
 	}
 }
 
-void
-pan_g_string_append_len  (GString      * str,
-                          const char   * appendme,
-                          guint          len)
-{
-	g_return_if_fail (str!=NULL);
-	g_return_if_fail (appendme!=NULL);
-
-	if (len < 1024)
-	{
-		char buf[1024];
-		memcpy (buf, appendme, len);
-		buf[len] = '\0';
-		g_string_append (str, buf);
-	}
-	else
-	{
-		char * pch = g_strndup (appendme, len);
-		g_string_append (str, pch);
-		g_free (pch);
-	}
-}
 
 /***
 ****
@@ -395,14 +377,26 @@ skip_next_token (const char   * pch,
                  const char     delimiter,
                  const char  ** setme_next_token)
 {
+	register char ch;
+
 	if (is_nonempty_string(pch))
 	{
-		while (*pch && *pch!=delimiter)
-			++pch;
-		if (*pch==delimiter)
-			++pch; /* skip past delimiter */
-		if (setme_next_token)
-			*setme_next_token = pch;
+		for (;;) {
+			ch = *pch;
+			if (ch == delimiter) {
+				++pch; /* skip past delimiter */
+				if (setme_next_token)
+					*setme_next_token = pch;
+				return;
+			}
+			if (ch)
+			       ++pch;
+			else {
+				if (setme_next_token)
+					*setme_next_token = pch;
+				return;
+			}
+		}
 	}
 }
  
@@ -561,7 +555,7 @@ get_next_token_g_str (const char   * pch,
 	if (end == pch)
 		retval = end==NULL ? FALSE : *end==delimiter;
 	else {
-		pan_g_string_append_len (setme, pch, end-pch);
+		g_string_append_len (setme, pch, end-pch);
 		retval = TRUE;
 	}
 
@@ -571,7 +565,6 @@ get_next_token_g_str (const char   * pch,
 	return retval;
 }
 
-
 /***
 ****
 ****  UNSORTED
@@ -724,120 +717,211 @@ pan_str_unescape (const char * escaped)
 }
 
 /***
-****  This is taken from Sylpheed 0.7.0.
+****
 ***/
 
-static char*
-strstr_with_skip_quote (const char *haystack, const char *needle)
+struct tm*
+pan_localtime_r(const time_t *const timep, struct tm *p_tm)
 {
-	register guint haystack_len, needle_len;
-	gboolean in_squote = FALSE, in_dquote = FALSE;
+	struct tm * retval;
+#ifdef HAVE_LOCALTIME_R
+	retval = localtime_r (timep, p_tm);
+#else
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
-	haystack_len = strlen(haystack);
-	needle_len   = strlen(needle);
+	g_static_mutex_lock (&mutex);
+	retval = localtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+#endif
+	return retval;
+}
 
-	if (haystack_len < needle_len || needle_len == 0)
+struct tm*
+pan_gmtime_r (const time_t * const timep, struct tm * p_tm)
+{
+	struct tm * retval;
+#ifdef HAVE_GMTIME_R
+	retval = gmtime_r (timep, p_tm);
+#else
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	g_static_mutex_lock (&mutex);
+	retval = gmtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+#endif
+	return retval;
+}
+
+/**
+ ** Charsets
+ **/
+
+/* The language isn't used yet, but will be useful for get the default
+ * language when we integrate a spellchecker.
+ */
+
+struct LocaleStruct {
+    const char *locale, *charset, *lang_name;
+} locales[] = {
+    {"en_US",        "ISO-8859-1",       N_("English")}, 
+    {"pt_BR",        "ISO-8859-1",       N_("Brazilian")},
+    {"ca_ES",        "ISO-8859-15",      N_("Catalan")},
+    {"zh_CN.GB2312", "gb2312",           N_("Chinese Simplified")},
+    {"zh_TW.Big5",   "big5",             N_("Chinese Traditional")},
+    {"cs_CZ",        "ISO-8859-2",       N_("Czech")},
+    {"da_DK",        "ISO-8859-1",       N_("Danish")},
+    {"de_DE",        "ISO-8859-15",      N_("German")},
+    {"nl_NL",        "ISO-8859-15",      N_("Dutch")},
+    {"et_EE",        "ISO-8859-15",      N_("Estonian")},
+    {"fi_FI",        "ISO-8859-15",      N_("Finnish")},
+    {"fr_FR",        "ISO-8859-15",      N_("French")},
+    {"el_GR",        "ISO-8859-7",       N_("Greek")},
+    {"hu_HU",        "ISO-8859-2",       N_("Hungarian")},
+    {"it_IT",        "ISO-8859-15",      N_("Italian")},
+    {"ja_JP",        "ISO-2022-jp",      N_("Japanese")},
+    {"ko_KR",        "euc-kr",           N_("Korean")},
+    {"lv_LV",        "ISO-8859-13",      N_("Latvian")},
+    {"lt_LT",        "ISO-8859-13",      N_("Lithuanian")},
+    {"no_NO",        "ISO-8859-1",       N_("Norwegian")},
+    {"pl_PL",        "ISO-8859-2",       N_("Polish")},
+    {"pt_PT",        "ISO-8859-15",      N_("Portuguese")},
+    {"ro_RO",        "ISO-8859-2",       N_("Romanian")},
+    {"ru_RU",        "KOI8-R",           N_("Russian (KOI)")},
+    {"ru_SU",        "ISO-8859-5",       N_("Russian (ISO)")},
+    {"sk_SK",        "ISO-8859-2",       N_("Slovak")},
+    {"es_ES",        "ISO-8859-15",      N_("Spanish")},
+    {"sv_SE",        "ISO-8859-1",       N_("Swedish")},
+    {"tr_TR",        "ISO-8859-9",       N_("Turkish")},
+    {"uk_UK",        "KOI8-U",           N_("Ukrainian")}
+};
+
+/* find_locale_index_by_locale:
+ * finds the longest fit so the one who has en_GB will get en_US if en_GB
+ * is not defined.
+ * This function is lifted from Balsa.
+ */
+static gint
+get_closest_locale (void)
+{
+	const char * locale = setlocale (LC_CTYPE, NULL);
+	guint i, j, maxfit = 0, maxpos = 0;
+
+	g_return_val_if_fail (locale != NULL, -1);
+
+	if (!locale || strcmp(locale, "C") == 0)
+		return 0;
+
+	for (i = 0; i < G_N_ELEMENTS(locales); i++) {
+		for (j=0; locale[j] && locales[i].locale[j] == locale[j]; j++);
+		if (j > maxfit) {
+			maxfit = j;
+			maxpos = i;
+		}
+	}
+	return maxpos;
+}
+
+#define PAN_DEFAULT_CHARSET  "ISO-8859-1"
+#define PAN_DEFAULT_LANG     "English"
+
+const char *
+get_charset_from_locale (void)
+{
+	gint loc_idx;
+
+	loc_idx = get_closest_locale ();
+
+	return loc_idx != -1 ? locales[loc_idx].charset : PAN_DEFAULT_CHARSET;
+}
+
+const char *
+get_lang_from_locale (void)
+{
+	gint loc_idx;
+
+	loc_idx = get_closest_locale ();
+
+	return loc_idx != -1 ? locales[loc_idx].lang_name : PAN_DEFAULT_LANG;
+}
+
+const char*
+pan_utf8ize (const char    * str,
+             gssize          len,
+             char         ** g_freeme)
+{
+	return pan_g_convert_to_utf8 (str, g_freeme, len, NULL, NULL, NULL);
+}
+
+const char*
+pan_g_convert_to_utf8 (const char   * str,
+                       char        ** g_freeme,
+                       gssize         len,
+                       gsize        * bytes_read,
+                       gsize        * bytes_written,
+                       GError      ** error)
+{
+	const char * retval;
+
+	g_return_val_if_fail (g_freeme!=NULL, NULL);
+
+	if (!is_nonempty_string (str))
+		retval = *g_freeme = g_strdup ("");
+	else if  (!g_utf8_validate (str, len, NULL))
+		retval = *g_freeme = g_locale_to_utf8 (str, len, bytes_read, bytes_written, error);
+	else {
+		*g_freeme = NULL;
+		retval = str;
+	}
+
+	return retval;
+}
+
+gboolean
+pan_header_is_8bit_encoded (const char * pch)
+{
+	return pch!=NULL && strstr(pch,"=?")!=NULL;
+}
+
+
+char *
+pan_header_to_utf8 (const gchar * str, size_t len, const char * charset)
+{
+	char   * retval = NULL;
+	char   * pch;
+
+	if (str == NULL)
 		return NULL;
 
-	while (haystack_len >= needle_len) {
-		if (!in_squote && !in_dquote &&
-		    !strncmp(haystack, needle, needle_len))
-			return (char *)haystack;
+	if (len == -1)
+		len = strlen (str);
+	pch = g_strndup (str, len);
 
-		/* 'foo"bar"' -> foo"bar"
-		   "foo'bar'" -> foo'bar' */
-		if (*haystack == '\'') {
-			if (in_squote)
-				in_squote = FALSE;
-			else if (!in_dquote)
-				in_squote = TRUE;
-		} else if (*haystack == '\"') {
-			if (in_dquote)
-				in_dquote = FALSE;
-			else if (!in_squote)
-				in_dquote = TRUE;
-		}
-
-		haystack++;
-		haystack_len--;
+	if (pan_header_is_8bit_encoded (pch))
+		retval = g_mime_utils_8bit_header_decode ((const guchar *) pch);
+	else
+	/* naughty boys don't properly encode their 8bit string */
+	if (g_mime_utils_text_is_8bit (pch, len))
+	{
+		if (charset == NULL)
+			retval = g_locale_to_utf8 (pch, len, NULL, NULL, NULL);
+		else
+			retval = g_convert (pch, len, "UTF-8", charset, NULL,NULL,NULL);
 	}
 
-	return NULL;
+	/* fallback */
+	if (retval == NULL)
+		retval = g_strdup (pch);
+
+	g_free (pch);
+
+	return retval;
 }
 
-/* this fuction was taken from gstrfuncs.c in glib. */
-static char **
-strsplit_with_quote (const char * str,
-                     const char * delim,
-                     int max_tokens)
-{
-	GSList *string_list = NULL, *slist;
-	char **str_array, *s;
-	guint i, n = 1;
-
-	g_return_val_if_fail(str != NULL, NULL);
-	g_return_val_if_fail(delim != NULL, NULL);
-
-	if (max_tokens < 1)
-		max_tokens = G_MAXINT;
-
-	s = strstr_with_skip_quote(str, delim);
-	if (s) {
-		guint delimiter_len = strlen(delim);
-
-		do {
-			guint len;
-			char *new_str;
-
-			len = s - str;
-			new_str = g_new(char, len + 1);
-			strncpy(new_str, str, len);
-			new_str[len] = 0;
-			string_list = g_slist_prepend(string_list, new_str);
-			n++;
-			str = s + delimiter_len;
-			s = strstr_with_skip_quote(str, delim);
-		} while (--max_tokens && s);
-	}
-
-	if (*str) {
-		n++;
-		string_list = g_slist_prepend(string_list, g_strdup(str));
-	}
-
-	str_array = g_new(char*, n);
-
-	i = n - 1;
-
-	str_array[i--] = NULL;
-	for (slist = string_list; slist; slist = slist->next)
-		str_array[i--] = slist->data;
-
-	g_slist_free(string_list);
-
-	return str_array;
-}
-
-char**
-strsplit_command_line_arguments (const char * command_line)
-{
-	int i;
-	char ** argv;
-
-	/* split the command line into arguments */
-	argv = strsplit_with_quote (command_line, " ", 0);
-
-	/* remove the quotes from quoted arguments */
-	for (i=0; argv[i]!=NULL; i++) {
-		char * str = argv[i];
-		if (str[0] == '\'' || str[0] == '\"') {
-			int len = strlen(str);
-			if (str[len - 1] == str[0]) {
-				str[len - 1] = '\0';
-				memmove(str, str + 1, len - 1);
-			}
-		}
-	}
-
-	return argv;
-}
