@@ -38,7 +38,6 @@
 #include <pan/base/server.h>
 #include <pan/base/serverlist.h>
 
-static void fire_online_status_changed (Server*, gboolean online);
 static void fire_groups_added          (Server*, Group**, gint group_qty);
 static void fire_groups_removed        (Server*, Group**, gint group_qty);
 
@@ -64,8 +63,6 @@ server_constructor (Server              * server,
 	server->posting = FALSE;
 	server->gen_msgid = FALSE;
 	server->need_auth = FALSE;
-	server->online = FALSE;
-	server->online_prompted = FALSE;
 	server->reserve_connection_for_bodies = TRUE;
 	server->address = NULL;
 	server->name = NULL;
@@ -271,22 +268,6 @@ server_save_if_dirty (Server * server, StatusItem * status)
 	debug_exit ("server_save_if_dirty");
 }
 
-/**
-***  Server Object
-**/
-
-void
-server_set_online (Server    * server,
-                   gboolean    online)
-{
-	g_return_if_fail (server_is_valid(server));
-	server->online = online ? 1 : 0;
-
-	fire_online_status_changed (server, server->online);
-}
-
-
-
 /***
 ****
 ****  GROUPLIST
@@ -319,7 +300,7 @@ compare_pgroup_ppgroup_name (const void* a, const void* b)
 }
 
 static int
-compare_ppgroup_ppgroup_name (const void * a, const void * b)
+compare_ppgroup_ppgroup_name (gconstpointer a, gconstpointer b, gpointer unused)
 {
 	return strcmp ((*(const Group**)a)->name, (*(const Group**)b)->name);
 }
@@ -391,10 +372,7 @@ server_add_groups_impl (Server       * server,
 
 		/* update the groups array */
 		pan_hash_to_ptr_array  (old, server->_groups);
-		qsort (server->_groups->pdata,
-		       server->_groups->len,
-		       sizeof(gpointer),
-		       compare_ppgroup_ppgroup_name);
+		g_ptr_array_sort_with_data (server->_groups, compare_ppgroup_ppgroup_name, NULL);
 
 		if (!initializing)
 		{
@@ -494,15 +472,15 @@ server_remove_groups (Server * server, Group ** groups, int qty)
 	debug_exit ("server_remove_groups");
 }
 
-static Group*
-server_get_named_group_impl (Server           * server,
-                             const gchar      * name,
-                             ServerGroupsType   set)
+Group*
+server_get_named_group_in_type (Server            * server,
+                                const char        * name,
+                                ServerGroupsType    set)
 {
 	int index;
 	Group * retval = NULL;
 	gboolean exact_match = FALSE;
-	debug_enter ("server_get_named_group_impl");
+	debug_enter ("server_get_named_group_in_type");
 
 	/* sanity clause */
 	g_return_val_if_fail (server_is_valid (server), NULL);
@@ -531,7 +509,7 @@ server_get_named_group_impl (Server           * server,
 	if (exact_match)
 		retval = GROUP(g_ptr_array_index(server->_groups,index));
 
-	debug_exit ("server_get_named_group_impl");
+	debug_exit ("server_get_named_group_in_type");
 	return retval;
 }
 
@@ -539,22 +517,7 @@ Group*
 server_get_named_group (Server        * server,
 	                const char    * name)
 {
-	return server_get_named_group_impl (server, name, SERVER_GROUPS_ALL);
-}
-
-gboolean
-server_has_group_in_type (const Server      * server,
-                          const char        * name,
-                          ServerGroupsType    type)
-{
-	return server_get_named_group_impl ((Server*)server, name, type)!=NULL;
-}
-
-gboolean
-server_has_group (const Server  * server,
-                  const char    * name)
-{
-	return server_has_group_in_type (server, name, SERVER_GROUPS_ALL);
+	return server_get_named_group_in_type (server, name, SERVER_GROUPS_ALL);
 }
 
 gboolean
@@ -659,25 +622,3 @@ fire_groups_added (Server * server, Group ** groups, gint group_qty)
 	pan_callback_call (server_get_groups_added_callback(), server, a);
 	g_ptr_array_free (a, TRUE);
 }
-
-/**
-***  Online Status Changed
-**/
-
-PanCallback*
-server_get_online_status_changed_callback (void)
-{
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
-}
-
-static void
-fire_online_status_changed (Server * server, gboolean online)
-{
-	g_return_if_fail (server_is_valid (server));
-
-	pan_callback_call (server_get_online_status_changed_callback(),
-	                   server,
-	                   GINT_TO_POINTER(online));
-}
