@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 
 #include <config.h>
 
+#include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 
 #include <glib.h>
@@ -183,11 +185,15 @@ void
 status_item_set_active (StatusItem       * item,
                         gboolean           active)
 {
+	debug_enter ("status_item_set_active");
+
 	g_return_if_fail (item != NULL);
  
 	pan_callback_call (status_item_get_active_callback(),
 	                   item,
 	                   GINT_TO_POINTER((int)active));
+
+	debug_exit ("status_item_set_active");
 }
 
 /*****
@@ -279,18 +285,17 @@ status_item_emit_status_va (StatusItem    * item,
                             const gchar   * format,
                             ...)
 {
-	char *pch = NULL;
+	gchar buf [1024];
        	va_list args;
 
 	g_return_if_fail (item!=NULL);
 	g_return_if_fail (format!=NULL);
 
 	va_start (args, format);
-	pch = g_strdup_vprintf (format, args);
+	g_vsnprintf (buf, sizeof(buf), format, args);
 	va_end (args);
 
-	status_item_emit_status (item, pch);
-	g_free (pch);
+	status_item_emit_status (item, buf);
 }
 
 void
@@ -311,21 +316,20 @@ status_item_emit_error (StatusItem   * item,
 
 void
 status_item_emit_error_va (StatusItem   * item,
-                           const gchar  * fmt,
+                           const gchar  * format,
                            ...)
 {
-	char *pch = NULL;
        	va_list args;
+	gchar buf [1024];
 
 	g_return_if_fail (item != NULL);
-	g_return_if_fail (fmt != NULL);
+	g_return_if_fail (format != NULL);
 
-	va_start (args, fmt);
-	pch = g_strdup_vprintf (fmt, args);
+	va_start (args, format);
+	g_vsnprintf (buf, sizeof(buf), format, args);
 	va_end (args);
 
-	status_item_emit_error (item, pch);
-	g_free (pch);
+	status_item_emit_error (item, buf);
 }
 
 gint
