@@ -48,12 +48,12 @@ struct _StatusItem
 	PanCallback * done;           /* obj arg is status int */
 
 	/*Fields*/
-	gchar       * description;    /* overrides the describe() function */
-	gchar       * status_text;    /* the last status text emitted, or NULL */
+	char       * description;     /* overrides the describe() function */
+	char       * status_text;     /* the last status text emitted, or NULL */
 	GSList      * errors;         /* the emitted error strings */
-	gint          progress_val;   /* value is [0...100] */
-	gint          steps;          /* number of steps for progress */
-	gint          step;           /* number of steps so far. [0..steps] */
+	int          progress_val;    /* value is [0...100] */
+	int          steps;           /* number of steps for progress */
+	int          step;            /* number of steps so far. [0..steps] */
 
 	/* virtual functions */
 	StatusItemDescribeFunc describe;
@@ -83,48 +83,48 @@ PanCallback*  status_item_get_active_callback  (void);
 
 StatusItem*   status_item_new                  (StatusItemDescribeFunc);
 
-StatusItem*   status_item_new_with_description (const gchar      * description);
+StatusItem*   status_item_new_with_description (const char       * description);
 
-gchar*        status_item_describe             (const StatusItem * item);
+char*         status_item_describe             (const StatusItem * item);
 
 void          status_item_emit_status          (StatusItem       * item,
-                                                const gchar      * status);
+                                                const char       * status);
 
 void          status_item_emit_status_va       (StatusItem       * item,
-                                                const gchar      * fmt,
+                                                const char       * fmt,
                                                 ...);
 
 void          status_item_emit_error           (StatusItem       * item,
-                                                const gchar      * error);
+                                                const char       * error);
 
 void          status_item_emit_error_va        (StatusItem       * item,
-                                                const gchar      * fmt,
+                                                const char       * fmt,
                                                 ...);
 
 void          status_item_emit_progress        (StatusItem       * item,
-                                                gint               out_of_100);
+                                                int                out_of_100);
 
 void          status_item_emit_init_steps      (StatusItem       * item,
-                                                gint               steps);
+                                                int                steps);
 
 void          status_item_emit_next_step       (StatusItem       * item);
 
 void          status_item_emit_inc_step        (StatusItem       * item,
-                                                gint               inc);
+                                                int                inc);
 
 void          status_item_emit_set_step        (StatusItem       * item,
-                                                gint               step);
+                                                int                step);
 
 void          status_item_emit_activity        (StatusItem       * item,
-                                                gint               n);
+                                                int                n);
 
 void          status_item_emit_done            (StatusItem       * item,
-                                                gint               status);
+                                                int                status);
 
 void          status_item_set_active           (StatusItem       * item,
                                                 gboolean           active);
 
-gint          status_item_get_progress_of_100  (const StatusItem * item);
+int           status_item_get_progress_of_100  (const StatusItem * item);
 
 
 #endif /* __PAN_STATUS_ITEM__H__ */
