@@ -22,20 +22,18 @@
 #include <config.h>
 
 #include <stdio.h>
-#include <sys/stat.h>
 #include <errno.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+
 #include <unistd.h>
+#include <sys/stat.h>
 #include <sys/types.h>
-#include <dirent.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-util.h>
 
 #include <pan/base/base-prefs.h>
 #include <pan/base/log.h>
@@ -50,52 +48,23 @@
 ***/
 
 
-gchar *
+char *
 pan_make_temp (FILE ** setme_fp)
 {
-	guint pid = (guint) getpid();
+	GError * err = NULL;
 	char * retval = NULL;
-	char * fname;
-	int fd;
+	int fd = g_file_open_tmp ("pan_XXXXXX", &retval, &err);
 
-	/* try the Pan tmp directory */
-	fname = g_strdup_printf ("%s%cpan%uXXXXXX", g_get_tmp_dir(), G_DIR_SEPARATOR, pid);
-	fd = mkstemp (fname);
-
-	/* open the file */
-	if (fd != -1) {
-		retval = g_strdup (fname);
+	if (err == NULL)
 		*setme_fp = fdopen (fd, "w+");
+	else {
+		log_add_va (LOG_ERROR, _("Could not create temporary file: %s"), err->message);
+		g_error_free (err);
 	}
 
-	g_free (fname);
 	return retval;
 }
 
-void
-pan_clean_temp_dir (void)
-{
-	const gchar * tmp_dir = g_get_tmp_dir ();
-	GString *filename = g_string_new (NULL);
-	char *prefix = g_strdup_printf ("pan%u", (unsigned int)getpid ());
-	gint prefix_l = strlen (prefix);
-	DIR *dir_p = opendir(tmp_dir);
-	struct dirent *dirent_p;
-
-	while ((dirent_p = readdir(dir_p)))
-	{
-		if (!strncmp (prefix, dirent_p->d_name, prefix_l))
-		{
-			g_string_sprintf (filename,
-				  "%s%c%s", tmp_dir, G_DIR_SEPARATOR, dirent_p->d_name);
-			unlink (filename->str);
-		}	
-	}
-	closedir (dir_p);
-	g_free (prefix);
-	g_string_free (filename, TRUE);
-}
-
 
 /***
 ****
@@ -110,6 +79,8 @@ pan_normalize_filename_inplace (gchar * filename)
 	register gchar * in;
 	register gchar * out;
 
+	g_return_val_if_fail (is_nonempty_string(filename), filename);
+
 	for (in=out=filename; *in; )
 		if (in[0]==G_DIR_SEPARATOR && in[1]==G_DIR_SEPARATOR)
 			++in;
@@ -124,46 +95,17 @@ pan_normalize_filename_inplace (gchar * filename)
 gboolean
 file_exists (const char* filename)
 {
-	return is_nonempty_string(filename) && g_file_exists (filename);
-}
-
-GArray*
-read_file (const char* filename)
-{
-	FILE * fp;
-	GArray * array;
-
-	/* open file */
-	size_t len = get_filesize(filename);
-	fp = fopen (filename, "r");
-	if (fp == NULL)
-		return NULL;
-
-	array = g_array_new (TRUE, FALSE, 1);
-
-	if (len>0) /* got the size, so alloc a buffer */
-	{
-		/* read the file... */
-		g_array_set_size (array, len);
-		array->len = fread (array->data, 1, len, fp);
-	}
-	else /* possibly from a FIFO, so read & grow */
-	{
-		gchar buf[2048];
-		size_t read;
-		while ((read = fread(buf, 1, sizeof(buf), fp)))
-			g_array_append_vals (array, buf, read);
-	}
-
-	fclose (fp);
-	return array;
+	return is_nonempty_string(filename) && g_file_test (filename, G_FILE_TEST_EXISTS);
 }
 
 size_t
 get_filesize (const char* filename)
 {
 	struct stat buf;
-	if ( !stat(filename, &buf) )
+
+	g_return_val_if_fail (is_nonempty_string(filename), -1);
+
+	if (!stat(filename, &buf))
 		return buf.st_size;
 
 	return -1;
@@ -176,10 +118,10 @@ directory_check (const char *pathname)
 
 	g_return_val_if_fail (is_nonempty_string(pathname), TRUE);
 
-	if (!g_file_test (pathname, G_FILE_TEST_ISDIR))
+	if (!g_file_test (pathname, G_FILE_TEST_IS_DIR))
 	{
 		const gchar * in = pathname;
-		gchar * buf = g_malloc (strlen(pathname) + 1);
+		gchar * buf = g_alloca (strlen(pathname)+1);
 		gchar * out = buf;
 
 		*out++ = *in++;
@@ -189,12 +131,10 @@ directory_check (const char *pathname)
 			{
        				*out = '\0';
 
-				if (!g_file_test (buf, G_FILE_TEST_ISDIR))
+				if (!g_file_test (buf, G_FILE_TEST_IS_DIR))
 				{
 					if (mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR))
 					{
-
-
 						log_add_va (LOG_ERROR, _("Couldn't create directory `%s': %s"),
 						                       buf, g_strerror (errno));
 						retval = FALSE;
@@ -210,8 +150,11 @@ directory_check (const char *pathname)
 			++out;
 			++in;
 		}
-		g_free (buf);
 	}
+
+	if (!retval)
+		log_add_va (LOG_ERROR, _("Unable to access or create directory \"%s\""), pathname);
+
 	return retval;
 }
 
@@ -222,9 +165,9 @@ directory_check (const char *pathname)
 ***/
 
 const gchar*
-mbox_get_next_message (const gchar * text,
-                       gchar ** setme_from,
-                       GString * setme_msg)
+mbox_get_next_message (const gchar   * text,
+                       gchar        ** setme_from,
+                       GString       * setme_msg)
 {
 	const gchar * march;
 	gboolean have_from = FALSE;
@@ -273,7 +216,7 @@ mbox_get_next_message (const gchar * text,
 			}
 
 			/* append the line */
-			pan_g_string_append_len (setme_msg, line_begin, line_len);
+			g_string_append_len (setme_msg, line_begin, line_len);
 			g_string_append_c (setme_msg, '\n');
 		}
 	}
