@@ -27,9 +27,7 @@
 
 #include <glib.h>
 
-gchar* pan_make_temp (FILE **);
-
-void pan_clean_temp_dir (void);
+char* pan_make_temp (FILE **);
 
 /**
  * Returns the number of bytes in the file, or 0 on error.
@@ -45,34 +43,28 @@ size_t get_filesize (const char* filename);
  *
  * @param pathname
  */
-void directory_check (const char* pathname);
+gboolean directory_check (const char* pathname);
 
 
-gint change_file_mode_rw (FILE * fp, const gchar * file);
-
-/**
- * @return a new GArray of lines in the specified file.
- */
-GArray* read_file (const char* filename);
+int change_file_mode_rw (FILE * fp, const char * file);
 
 gboolean file_exists (const char  * filename);
 
-gchar* pan_normalize_filename_inplace (gchar * filename);
+char* pan_normalize_filename_inplace (char * filename);
 
-const gchar*
-mbox_get_next_message (const gchar * buf,
-                       gchar ** setme_from,
-                       gchar ** setme_msg);
+const char*
+mbox_get_next_message (const char * buf,
+                       char ** setme_from,
+                       GString * setme_msg);
 
-
-gchar*
-mbox_format_message (const gchar * message,
-                     const gchar * author_addr,
+char*
+mbox_format_message (const char * message,
+                     const char * author_addr,
                      time_t date);
 
-void
-pan_rename (const gchar * old_filename,
-            const gchar * new_filename);
+gboolean
+pan_rename (const char * old_filename,
+            const char * new_filename);
 
 #endif /* __UTIL_H__ */
 
