@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,21 +21,18 @@
 #ifndef __DIALOGS_H__
 #define __DIALOGS_H__
 
-#include <pan/group.h>
-#include <pan/rule.h>
+#include <pan/base/group.h>
 
 void dialog_about	        (GtkWidget * window);
 
-void dialog_download_headers	(Group *);
+void dialog_download_headers	(GtkWindow * parent, Group *);
 
 void dialog_newuser	        (void);
 
-void dialog_group_props         (Group *);
+void dialog_group_props         (GtkWindow * parent, Group *);
 
 void dialog_log_viewer	        (void);
 
 void dialog_rules               (void);
 
-void dialog_edit_rule           (Rule *);
-
 #endif
