@@ -31,17 +31,30 @@ extern const gchar * FILTER_NEW_CLASS_ID;
 
 typedef struct _FilterNew   FilterNew;
 
+typedef enum
+{
+	FILTER_NEW_NEW,
+	FILTER_NEW_UNREAD,
+	FILTER_NEW_READ
+}
+FilterNewType;
+
 struct _FilterNew
 {
 	/* parent class */
 	Filter parent;
+
+	FilterNewType type;
 };
 
 /**
 ***  PUBLIC
 **/
 
-Filter*      filter_new_article_new             (void);
+Filter*      filter_new_article_new             (FilterNewType type);
+
+void         filter_new_set_state               (FilterNew * filter,
+                                                 FilterNewType type);
 
 
 #endif
