@@ -3,7 +3,7 @@
  * Author: Charles Kerr <charles@rebelbase.com>
  *
  * Pan - A Newsreader for X
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,8 +34,8 @@
 
 #include <glib.h>
 
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 
 #include <pan/base/log.h>
 #include <pan/base/pan-i18n.h>
@@ -51,7 +51,6 @@
 #include <pan/filters/filter-linecount.h> 
 #include <pan/filters/filter-new.h> 
 #include <pan/filters/filter-phrase.h> 
-#include <pan/filters/filter-read.h> 
 #include <pan/filters/filter-saved.h> 
 #include <pan/filters/filter-score.h> 
 #include <pan/filters/filter-thread.h> 
@@ -358,14 +357,6 @@ write_phrase (GString * appendme, const FilterPhrase * f, gint depth)
 	g_string_append (appendme, "/>");
 }
 static void
-write_read (GString * appendme, const FilterRead * f, gint depth)
-{
-	newline_depth (appendme, depth);
-	g_string_append (appendme, "<read");
-	add_filter_arguments (appendme, FILTER(f));
-	g_string_append (appendme, "/>");
-}
-static void
 write_thread (GString * appendme, const FilterThread * f, gint depth)
 {
 	newline_depth (appendme, depth);
@@ -440,9 +431,6 @@ write_filter (GString * appendme, const Filter * f, gint depth)
 	else if (!strcmp (f->class_id, FILTER_PHRASE_CLASS_ID))
 		write_phrase (appendme, FILTER_PHRASE(f), depth);
 
-	else if (!strcmp (f->class_id, FILTER_READ_CLASS_ID))
-		write_read (appendme, FILTER_READ(f), depth);
-
 	else if (!strcmp (f->class_id, FILTER_THREAD_CLASS_ID))
 		write_thread (appendme, FILTER_THREAD(f), depth);
 
@@ -469,7 +457,7 @@ filter_xml_to_string  (Filter  ** filters,
 			g_string_append (str, "\n\n");
 	}
 
-	g_string_prepend (str, "<?xml version=\"1.0\" ?>\n"
+	g_string_prepend (str, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
 	                       "<!DOCTYPE rules SYSTEM \"filters.dtd\">\n"
 	                       "<filters>\n");
 	g_string_append (str, "\n</filters>");
@@ -559,296 +547,296 @@ filter_xml_write (const gchar * filename,
 *****/
 
 static void
-populate_filter (Filter * f, GHashTable * attr)
+populate_filter (Filter * f, xmlNodePtr node)
 {
-	if (g_hash_table_lookup (attr, "negate") != NULL)
+	xmlChar * val;
+
+	if ((val = xmlGetProp (node, "negate")) != NULL) {
 		filter_negate (f);
+		xmlFree (val);
+	}
 }
 
 static void
-populate_score (FilterScore * f, GHashTable * attr)
+populate_score (FilterScore * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "min_score");
-	if (value != NULL)
-		f->minimum_score = atoi (value);
+	if ((val = xmlGetProp (node, "min_score")) != NULL) {
+		f->minimum_score = atoi (val);
+		xmlFree (val);
+	}
 }
 
 static void
-populate_saved (FilterSaved * f, GHashTable * attr)
+populate_saved (FilterSaved * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 	FilterSavedState state = FILTER_SAVED_SAVED;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "state");
-	if (value != NULL)
+	if ((val = xmlGetProp (node, "state")) != NULL)
 	{
-		if (!pan_strcmp(value,"saved")) state = FILTER_SAVED_SAVED;
-		else if (!pan_strcmp(value,"queued")) state = FILTER_SAVED_QUEUED;
-		else if (!pan_strcmp(value,"not_saved")) state = FILTER_SAVED_NOT_SAVED;
+		if (!pan_strcmp(val,"saved")) state = FILTER_SAVED_SAVED;
+		else if (!pan_strcmp(val,"queued")) state = FILTER_SAVED_QUEUED;
+		else if (!pan_strcmp(val,"not_saved")) state = FILTER_SAVED_NOT_SAVED;
+
+		xmlFree (val);
 	}
 
 	filter_saved_set_state (f, state);
 }
 
 static void
-populate_binary (FilterBinary * f, GHashTable * attr)
+populate_binary (FilterBinary * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 	FilterBinaryState state = FILTER_BINARY_NONBINARY;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "state");
-	if (value != NULL)
+	if ((val = xmlGetProp (node, "state")) != NULL)
 	{
-		if (!pan_strcmp(value,"text")) state = FILTER_BINARY_NONBINARY;
-		else if (!pan_strcmp(value,"complete")) state = FILTER_BINARY_COMPLETE;
-		else if (!pan_strcmp(value,"incomplete")) state = FILTER_BINARY_INCOMPLETE;
+		if (!pan_strcmp(val,"text")) state = FILTER_BINARY_NONBINARY;
+		else if (!pan_strcmp(val,"complete")) state = FILTER_BINARY_COMPLETE;
+		else if (!pan_strcmp(val,"incomplete")) state = FILTER_BINARY_INCOMPLETE;
+
+		xmlFree (val);
 	}
 
 	filter_binary_set_state (f, state);
 }
 
 static void
-populate_read (FilterRead * f, GHashTable * attr)
+populate_cached (FilterCached * f, xmlNodePtr node)
 {
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 }
 
 static void
-populate_cached (FilterCached * f, GHashTable * attr)
+populate_new (FilterNew * f, xmlNodePtr node)
 {
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 }
 
 static void
-populate_new (FilterNew * f, GHashTable * attr)
+populate_thread (FilterThread * f, xmlNodePtr node)
 {
-	populate_filter (FILTER(f), attr);
-}
-
-static void
-populate_thread (FilterThread * f, GHashTable * attr)
-{
-	const gchar * value;
+	xmlChar * val;
 	FilterThreadState state = FILTER_THREAD_WATCHED;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar*) g_hash_table_lookup (attr, "type");
-	if (value != NULL)
+	if ((val = xmlGetProp (node, "type")) != NULL)
 	{
-		if (!pan_strcmp(value,"watched")) state = FILTER_THREAD_WATCHED;
-		else if (!pan_strcmp(value,"ignored")) state = FILTER_THREAD_IGNORED;
-		else if (!pan_strcmp(value,"nothing")) state = FILTER_THREAD_NOTHING;
+		if (!pan_strcmp(val,"watched")) state = FILTER_THREAD_WATCHED;
+		else if (!pan_strcmp(val,"ignored")) state = FILTER_THREAD_IGNORED;
+		else if (!pan_strcmp(val,"nothing")) state = FILTER_THREAD_NOTHING;
 		else pan_warn_if_reached ();
+
+		xmlFree (val);
 	}
 
 	filter_thread_set_state (f, state);
 }
 
 static void
-populate_linecount (FilterLineCount * f, GHashTable * attr)
+populate_linecount (FilterLineCount * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "min_lines");
-	if (value != NULL)
-		f->minimum_line_count = atoi (value);
+	if ((val = xmlGetProp (node, "min_lines")) != NULL) {
+		f->minimum_line_count = atoi (val);
+		xmlFree (val);
+	}
 }
 
 static void
-populate_crosspost (FilterCrosspost * f, GHashTable * attr)
+populate_crosspost (FilterCrosspost * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "min_groups");
-	if (value != NULL)
-		f->minimum_crosspost_count = atoi (value);
+	if ((val = xmlGetProp (node, "min_groups")) != NULL) {
+		f->minimum_crosspost_count = atoi (val);
+		xmlFree (val);
+	}
 }
 
 static void
-populate_date (FilterDate * f, GHashTable * attr)
+populate_date (FilterDate * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
-	value = (const gchar *) g_hash_table_lookup (attr, "min_days_old");
-	if (value != NULL)
-		f->minimum_days_old = atoi (value);
+	if ((val = xmlGetProp (node, "min_days_old")) != NULL) {
+		f->minimum_days_old = atoi (val);
+		xmlFree (val);
+	}
 }
 static void
-populate_mine (FilterMine * f, GHashTable * attr)
+populate_mine (FilterMine * f, xmlNodePtr node)
 {
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 }
 
 static void
-populate_phrase (FilterPhrase * f, GHashTable * attr)
+populate_phrase (FilterPhrase * f, xmlNodePtr node)
 {
-	const gchar * phrase = NULL;
-	const gchar * value;
+	xmlChar * val;
+	char * phrase = NULL;
 	gboolean is_regex = FALSE;
 	FilterPhraseState state = FILTER_PHRASE_SUBJECT;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
 	/* type */
-	value = (const gchar*) g_hash_table_lookup (attr, "type");
-	if (value != NULL) {
-		if (!pan_strcmp (value, "subject"))
+	if ((val = xmlGetProp (node, "type")) != NULL) {
+		if (!pan_strcmp (val, "subject"))
 			state = FILTER_PHRASE_SUBJECT;
-		else if (!pan_strcmp (value, "author"))
+		else if (!pan_strcmp (val, "author"))
 			state = FILTER_PHRASE_AUTHOR;
-		else if (!pan_strcmp (value, "message-id"))
+		else if (!pan_strcmp (val, "message-id"))
 			state = FILTER_PHRASE_MESSAGE_ID;
 		else
-			log_add_va (LOG_ERROR, "Invalid filter phrase: `%s'", value);
+			log_add_va (LOG_ERROR, "Invalid filter phrase: `%s'", val);
+
+		xmlFree (val);
 	}
 
 	/* phrase */
-	phrase = (const gchar*) g_hash_table_lookup (attr, "phrase");
+	if ((val = xmlGetProp (node, "phrase")) != NULL) {
+		phrase = g_strdup (val);
+		xmlFree (val);
+	}
 
 	/* regex */
-	value = (const gchar*) g_hash_table_lookup (attr, "is_regex");
-	if (value != NULL)
-		is_regex = *value=='t';
+	if ((val = xmlGetProp (node, "is_regex")) != NULL) {
+		is_regex = *val=='t';
+		xmlFree (val);
+	}
 
 	filter_phrase_set (f, state, phrase, is_regex);
+	xmlFree (phrase);
 }
 
 static void
-populate_aggregate (FilterAggregate * f, GHashTable * attr)
+populate_aggregate (FilterAggregate * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_filter (FILTER(f), attr);
+	populate_filter (FILTER(f), node);
 
 	/* type */
-	value = (const gchar *) g_hash_table_lookup (attr, "type");
-	if (value != NULL) {
-		if (!pan_strcmp(value,"and")) filter_aggregate_set_type (f, AGGREGATE_TYPE_AND);
-		else if (!pan_strcmp(value,"or")) filter_aggregate_set_type (f, AGGREGATE_TYPE_OR);
+	if ((val = xmlGetProp (node, "type")) != NULL) {
+		if (!pan_strcmp(val,"and")) filter_aggregate_set_type (f, AGGREGATE_TYPE_AND);
+		else if (!pan_strcmp(val,"or")) filter_aggregate_set_type (f, AGGREGATE_TYPE_OR);
+		xmlFree (val);
 	}
 }
 
 static void
-populate_top (FilterTop * f, GHashTable * attr)
+populate_top (FilterTop * f, xmlNodePtr node)
 {
-	const gchar * value;
+	xmlChar * val;
 
-	populate_aggregate (FILTER_AGGREGATE(f), attr);
+	populate_aggregate (FILTER_AGGREGATE(f), node);
 
 	/* name */
-	value = (const gchar *) g_hash_table_lookup (attr, "name");
-	if (value != NULL)
-		filter_top_set_name (f, value);
+	if ((val = xmlGetProp (node, "name")) != NULL) {
+		filter_top_set_name (f, val);
+		xmlFree (val);
+	}
 
 	/* visible */
-	value = (const gchar *) g_hash_table_lookup (attr, "visible");
-	filter_top_set_visible (f, value != NULL && *value=='t');
+	val = xmlGetProp (node, "visible");
+	filter_top_set_visible (f, val!=NULL && *val=='t');
+	if (val != NULL)
+		xmlFree (val);
 }
 
 static Filter*
-create_filter (xmlDoc * doc, xmlNode * node)
+create_filter (xmlDoc * doc, xmlNodePtr node)
 {
 	Filter * f = NULL;
-	const xmlAttr * attr;
 	const gchar * tag = (const gchar*) node->name;
-	GHashTable * attributes = g_hash_table_new (g_str_hash, g_str_equal);
-	GStringChunk * strchunk = g_string_chunk_new (128);
-
-	for (attr=node->properties; attr!=NULL; attr=attr->next) {
-		const gchar * name = (const gchar*) attr->name;
-		xmlChar * value = xmlNodeListGetString (doc, attr->val, FALSE);
-		gchar * clear_value = pan_str_unescape ((const gchar*) value);
-		g_hash_table_insert (attributes,
-		                     g_string_chunk_insert(strchunk,name),
-		                     g_string_chunk_insert(strchunk,clear_value));
-		g_free (clear_value);
-		xmlFree (value);
-	}
 
 	if (!pan_strcmp(tag,"aggregate"))
 	{
 		f = filter_aggregate_new ();
-		populate_aggregate (FILTER_AGGREGATE(f), attributes);
+		populate_aggregate (FILTER_AGGREGATE(f), node);
 	}
 	else if (!pan_strcmp(tag,"filter"))
 	{
 		f = filter_top_new ();
-		populate_top (FILTER_TOP(f), attributes);
+		populate_top (FILTER_TOP(f), node);
 	}
 	else if (!pan_strcmp(tag,"phrase"))
 	{
 		f = filter_phrase_new ();
-		populate_phrase (FILTER_PHRASE(f), attributes);
+		populate_phrase (FILTER_PHRASE(f), node);
 	}
 	else if (!pan_strcmp(tag,"linecount"))
 	{
 		f = filter_line_count_new ();
-		populate_linecount (FILTER_LINE_COUNT(f), attributes);
+		populate_linecount (FILTER_LINE_COUNT(f), node);
 	}
 	else if (!pan_strcmp(tag,"crosspost"))
 	{
 		f = filter_crosspost_new ();
-		populate_crosspost (FILTER_CROSSPOST(f), attributes);
+		populate_crosspost (FILTER_CROSSPOST(f), node);
 	}
 	else if (!pan_strcmp(tag,"date"))
 	{
 		f = filter_date_new ();
-		populate_date (FILTER_DATE(f), attributes);
+		populate_date (FILTER_DATE(f), node);
 	}
 	else if (!pan_strcmp(tag,"mine"))
 	{
 		f = filter_mine_new ();
-		populate_mine (FILTER_MINE(f), attributes);
+		populate_mine (FILTER_MINE(f), node);
 	}
 	else if (!pan_strcmp(tag,"thread"))
 	{
 		f = filter_thread_new ();
-		populate_thread (FILTER_THREAD(f), attributes);
+		populate_thread (FILTER_THREAD(f), node);
 	}
 	else if (!pan_strcmp(tag,"read"))
 	{
-		f = filter_read_new ();
-		populate_read (FILTER_READ(f), attributes);
+		f = filter_new_article_new (FILTER_NEW_READ);
+		populate_new (FILTER_NEW(f), node);
 	}
 	else if (!pan_strcmp(tag,"binary"))
 	{
 		f = filter_binary_new ();
-		populate_binary (FILTER_BINARY(f), attributes);
+		populate_binary (FILTER_BINARY(f), node);
 	}
 	else if (!pan_strcmp(tag,"saved"))
 	{
 		f = filter_saved_new ();
-		populate_saved (FILTER_SAVED(f), attributes);
+		populate_saved (FILTER_SAVED(f), node);
 	}
 	else if (!pan_strcmp(tag,"score"))
 	{
 		f = filter_score_new ();
-		populate_score (FILTER_SCORE(f), attributes);
+		populate_score (FILTER_SCORE(f), node);
 	}
 	else if (!pan_strcmp(tag,"cached"))
 	{
 		f = filter_cached_new ();
-		populate_cached (FILTER_CACHED(f), attributes);
+		populate_cached (FILTER_CACHED(f), node);
 	}
 	else if (!pan_strcmp(tag,"new"))
 	{
-		f = filter_new_article_new ();
-		populate_new (FILTER_NEW(f), attributes);
+		f = filter_new_article_new (FILTER_NEW_NEW);
+		populate_new (FILTER_NEW(f), node);
 	}
 
 	/* Children */
@@ -870,8 +858,6 @@ create_filter (xmlDoc * doc, xmlNode * node)
 		g_ptr_array_free (a, TRUE);
 	}
 
-	g_hash_table_destroy (attributes);
-	g_string_chunk_free (strchunk);
 	return f;
 }
 
