@@ -20,7 +20,8 @@
  * 
  */
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/group.h>
@@ -83,33 +84,6 @@ start_task_headers (GPtrArray * a,
 ***  Public Actions
 **/
 
-void
-group_action_subscribed_download_new (void)
-{
-        GPtrArray * a;
-	debug_enter ("group_action_subscribed_download_new");
-
-	a = server_get_groups (serverlist_get_active_server(),
-	                       SERVER_GROUPS_SUBSCRIBED);
-	start_task_headers (a, HEADERS_NEW, download_bodies_too);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("group_action_subscribed_download_new");
-}
-
-void
-group_action_download_all (void)
-{
-        GPtrArray * a;
-	debug_enter ("group_action_download_all");
-		 
-	a = get_selected_groups ();
-	start_task_headers (a, HEADERS_ALL, download_bodies_too);
-	g_ptr_array_free (a, TRUE);
-					 
-	debug_exit ("group_action_download_all");
-}
-
 void
 group_action_selected_download_new (void)
 {
@@ -117,26 +91,37 @@ group_action_selected_download_new (void)
 	debug_enter ("group_action_selected_download_new");
 			 
 	a = get_selected_groups ();
-	start_task_headers (a, HEADERS_NEW, download_bodies_too);
+	start_task_headers (a, HEADERS_NEW, FALSE);
 	g_ptr_array_free (a, TRUE);
 						 
 	debug_exit ("group_action_selected_download_new");
 }
 
+void
+group_action_selected_download_new_and_bodies (void)
+{
+	GPtrArray * a = get_selected_groups ();
+	start_task_headers (a, HEADERS_NEW, TRUE);
+	g_ptr_array_free (a, TRUE);
+}
 
 void
-group_action_selected_download_sample (void)
+group_action_subscribed_download_new_and_bodies (void)
 {
-        GPtrArray * a;
-	debug_enter ("group_action_selected_download_sample");
-		 
-	a = get_selected_groups ();
-	start_task_headers (a, HEADERS_SAMPLE, download_bodies_too);
+	GPtrArray * a = server_get_groups (serverlist_get_active_server(), SERVER_GROUPS_SUBSCRIBED);
+	start_task_headers (a, HEADERS_NEW, TRUE);
 	g_ptr_array_free (a, TRUE);
-					 
-	debug_exit ("group_action_selected_download_sample");
 }
 
+void
+group_action_subscribed_download_new (void)
+{
+	GPtrArray * a = server_get_groups (serverlist_get_active_server(), SERVER_GROUPS_SUBSCRIBED);
+	start_task_headers (a, HEADERS_NEW, FALSE);
+	g_ptr_array_free (a, TRUE);
+}
+
+
 void
 group_action_selected_download_dialog (void)
 {
@@ -149,7 +134,7 @@ group_action_selected_download_dialog (void)
 	for (i=0; i!=a->len; ++i) {
 		Group * group = GROUP(g_ptr_array_index(a,i));
 		if (!group_is_folder(group))
-			dialog_download_headers (group);
+			dialog_download_headers (GTK_WINDOW(Pan.window), group);
 	}
 						 
 	/* cleanup */
@@ -224,7 +209,7 @@ group_action_selected_properties (void)
 		 
 	a = get_selected_groups ();
 	for (i=0; i<a->len; ++i)
-		dialog_group_props (GROUP(g_ptr_array_index(a,i)));
+		dialog_group_props (GTK_WINDOW(Pan.window), GROUP(g_ptr_array_index(a,i)));
 
 	g_ptr_array_free (a, TRUE);
 	debug_exit ("group_action_selected_properties");
@@ -233,12 +218,13 @@ group_action_selected_properties (void)
 void
 group_action_selected_empty (void)
 {
+	guint i;
         GPtrArray * groups;
 	debug_enter ("group_action_selected_empty");
 
 	groups = get_selected_groups ();
-	if (groups->len > 0)
-		group_empty_dialog (groups);
+	for (i=0; i!=groups->len; ++i)
+		group_empty (GROUP(g_ptr_array_index(groups,i)), FALSE);
 	g_ptr_array_free (groups, TRUE);
 
 	debug_exit ("group_action_selected_empty");
@@ -249,19 +235,21 @@ group_action_selected_empty (void)
 ***/
 
 static void
-delete_groups_dialog_cb (gint reply, gpointer data)
+delete_groups_dialog_response_cb (GtkWidget * dialog, int response, gpointer data)
 {
 	GPtrArray * a;
 	debug_enter ("delete_groups_dialog_cb");
 
 	a = (GPtrArray*) data;
 
-	if (reply == 0) {
+	if (response == GTK_RESPONSE_YES)
+	{
 		Server * s = GROUP(g_ptr_array_index(a,0))->server;
 		server_destroy_groups (s, (Group**)a->pdata, a->len);
 	}
 
 	g_ptr_array_free (a, TRUE);
+	gtk_widget_destroy (GTK_WIDGET(dialog));
 	debug_exit ("delete_groups_dialog_cb");
 }
 void
@@ -270,15 +258,17 @@ group_action_selected_destroy (void)
 	GPtrArray * a = get_selected_groups ();
 	if (a->len != 0)
 	{
-		gchar * txt = g_strdup_printf (
-			_("Are you sure you want to delete these %d groups/folders and their articles?"), (gint)a->len);
+		GtkWidget * w;
 		pan_lock ();
-		gnome_question_dialog_parented (txt,
-		                                delete_groups_dialog_cb,
-		                                a,
-		                                GTK_WINDOW(Pan.window));
+		w = gtk_message_dialog_new (GTK_WINDOW(Pan.window),
+		                            GTK_DIALOG_DESTROY_WITH_PARENT,
+		                            GTK_MESSAGE_QUESTION,
+		                            GTK_BUTTONS_YES_NO,
+		                            _("Are you sure you want to delete these %d groups/folders and their articles?"),
+		                            (int)a->len);
+		g_signal_connect (GTK_OBJECT(w), "response", G_CALLBACK(delete_groups_dialog_response_cb), a);
+		gtk_widget_show (w);
 		pan_unlock ();
-		g_free (txt);
 	}
 	else g_ptr_array_free (a, TRUE);
 }
