@@ -19,21 +19,28 @@
  */
 
 #include <config.h>
-#include <errno.h>
 
-#include <gnome.h>
+#include <errno.h>
+#include <string.h>
+
+#include <glib.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/file-grouplist.h>
 #include <pan/base/fnmatch.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/serverlist.h>
 
 #include <pan/articlelist.h>
 #include <pan/dialogs/dialogs.h>
 #include <pan/globals.h>
 #include <pan/grouplist.h>
+#include <pan/group-action.h>
 #include <pan/group-ui.h>
 #include <pan/gui.h>
 #include <pan/prefs.h>
@@ -41,21 +48,37 @@
 #include <pan/task-grouplist.h>
 #include <pan/util.h>
 
-#include "xpm/folder.xpm"
+#include <pan/xpm/pan-pixbufs.h>
+
+static GtkItemFactoryEntry group_popup_entries[] =
+{
+	/* r */ {N_("/Mark Group _Read"), NULL, group_action_selected_mark_read, 0, NULL},
+	/* d */ {N_("/_Delete Group's Articles"), NULL, group_action_selected_empty, 0, "<StockItem>", GTK_STOCK_DELETE},
+	/*   */ {N_("/---"), NULL, NULL, 0, "<Separator>"},
+	/* h */ {N_("/Get New _Headers"), NULL, group_action_selected_download_new, 0, NULL},
+	/* b */ {N_("/Get New Headers and _Bodies"), NULL, group_action_selected_download_new_and_bodies, 0, NULL},
+	/* o */ {N_("/More Download _Options..."), NULL, group_action_selected_download_dialog, 0, NULL},
+	/* c */ {N_("/Refresh Article _Counts"), NULL, group_action_selected_update_count_info, 0, "<StockItem>", GTK_STOCK_REFRESH},
+	/*   */ {N_("/---"), NULL, NULL, 0, "<Separator>"},
+	/* s */ {N_("/_Subscribe"), NULL, group_action_selected_subscribe, 0, "<ImageItem>", icon_stock_book_red},
+	/* u */ {N_("/_Unsubscribe"), NULL, group_action_selected_unsubscribe, 0, NULL},
+	/* p */ {N_("/Group _Properties..."), NULL, group_action_selected_properties, 0, "<StockItem>", GTK_STOCK_PROPERTIES},
+	/*   */ {N_("/---"), NULL, NULL, 0, "<Separator>"},
+	/* t */ {N_("/Dele_te Group"), NULL, group_action_selected_destroy, 0, "<StockItem>", GTK_STOCK_DELETE}
+};
 
 static gint grouplist_update_ui_mainthread (gpointer);
 static void group_menu_popup (GdkEventButton * event);
-static void grouplist_filter_changed_cb_nolock (void);
+static gboolean grouplist_filter_changed_cb_nolock (void);
 static void grouplist_refresh_filter_nolock (void);
 static void grouplist_update_groups (const Group ** groups, int group_qty);
 static void grouplist_set_server_nolock (Server *);
 
 extern GtkWidget   * ttips;
-static GnomePixmap * folder_pixmap           = NULL;
-static GnomePixmap * subscribed_pixmap       = NULL;
+static GtkItemFactory * popup_factory        = NULL;
+static GtkWidget * popup_menu                = NULL;
 
 static int           group_mode              = GROUP_ALL;
-static GtkWidget   * group_clist_menu        = NULL;
 static Server      * my_server               = NULL;
 static GtkWidget   * grouplist_mode_menu     = NULL;
 static GtkWidget   * groupname_filter_entry  = NULL;
@@ -71,38 +94,25 @@ PanCallback * grouplist_server_set_callback = NULL;
 PanCallback * grouplist_group_selection_changed = NULL;
 
 static void
-grouplist_get_group_readable_name (const Group * group, gchar * buf, gint buflen)
+grouplist_get_group_readable_name (const Group * group, char * buf, int buflen)
 {
-	const gchar * readable = group_get_readable_name (group);
-	gint bufleft;
+	const char * readable = group_get_readable_name (group);
+	int bufleft;
+
+	/**
+	***  fill the name
+	**/
 
 	if (!collapse_group_names)
-		strncpy (buf, readable, buflen);
+		g_strlcpy (buf, readable, buflen);
 	else
-	{
-		gint pos = 0;
-		const gchar * pch = NULL;
-		const gchar * march = readable;
-		const gchar * last = "";
-		gint len = 0;
-
-		while (pos+2<buflen && get_next_token_run (march, '.', &march, &pch, &len)) {
-			last = pch;
-			if (len > 0) {
-				buf[pos++] = *pch;
-				buf[pos++] = '.';
-			}
-		}
-		if (pos != 0)
-			pos -= 2;
-		buf[pos] = '\0';
-
-		if (is_nonempty_string(last))
-			strncpy (buf+pos, last, buflen-pos);
-	}
-	buf[buflen-1] = '\0';
+		group_get_collapsed_name (group, buf, buflen);
 	bufleft = buflen - strlen(buf);
 
+	/**
+	***  add the parenthetical status messages
+	**/
+
 	if (group_is_moderated (group)) {
 		strncat (buf, _(" (Moderated)"), bufleft);
 		buf[buflen-1] = '\0';
@@ -125,10 +135,11 @@ grouplist_get_visible_server (void)
 }
 
 static void
-group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
+grouplist_dialog_response_cb (GtkDialog * dialog, int response, gpointer user_data)
 {
-	if (num==0) /* yes, get list */
-		queue_add (TASK(task_grouplist_new (server, GROUPLIST_ALL)));
+	if (response == GTK_RESPONSE_YES)
+		queue_add (TASK(task_grouplist_new (SERVER(user_data), GROUPLIST_ALL)));
+	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
 
@@ -175,14 +186,10 @@ grouplist_selection_changed_cb (gpointer call_object,
 
 	if ((single_group!=NULL)
 		&& ((button_click_count==2) /* double click */
-			|| (button_click_count==1 /*&& Pan.viewmode==GUI_PANED*/
-					&& (  (mb==2 && grouplist_mb2_action==CLICK_ACTION_LOAD)
-					    ||(mb==1 && grouplist_mb1_action==CLICK_ACTION_LOAD)))))
+			|| (button_click_count==1 && mb==1 && !single_click_selects)))
 	{
-		gui_page_set (ARTICLELIST_PAGE);
-		pan_lock ();
-		articlelist_set_group_nolock (single_group);
-		pan_unlock ();
+		gui_page_set (HEADERS_PANE);
+		articlelist_set_group (single_group);
 	}
 
 	/* reset the click count for next time... */
@@ -192,10 +199,11 @@ grouplist_selection_changed_cb (gpointer call_object,
 	debug_exit ("grouplist_selection_changed_cb");
 }
 
-static void
+static gboolean
 grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 {
 	debug_enter ("grouplist_button_press");
+
 	switch (event->button)
 	{
 		case 1:
@@ -210,10 +218,12 @@ grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 			group_menu_popup (event);
 			break;
 	}
+
 	debug_exit ("grouplist_button_press");
+	return FALSE;
 }
 
-static void
+static gboolean
 grouplist_key_press (GtkWidget     * widget,
                      GdkEventKey   * event,
                      gpointer        user_data)
@@ -230,6 +240,7 @@ grouplist_key_press (GtkWidget     * widget,
 			button_click_count = -1;
 	} 
 	debug_exit ("grouplist_key_press");
+	return FALSE;
 }
 
 static gboolean select_callback_pending = FALSE;
@@ -242,7 +253,7 @@ select_row_cb_mainthread (gpointer data)
 
        	groups = grouplist_get_selected_groups ();
 	pan_callback_call (grouplist_group_selection_changed,
-	                   Pan.group_clist,
+	                   Pan.group_tree,
 			   groups);
 	select_callback_pending = FALSE;
 	g_ptr_array_free (groups, TRUE);
@@ -261,7 +272,7 @@ select_row_cb (GtkWidget  * widget,
 	if (!select_callback_pending)
 	{
 		select_callback_pending = TRUE;
-		run_in_main_thread_nolock (select_row_cb_mainthread, NULL);
+		gui_queue_add (select_row_cb_mainthread, NULL);
 	}
 
 	debug_exit ("select_row_cb");
@@ -281,7 +292,7 @@ grouplist_set_selected_groups_nolock (const GPtrArray * a)
 	{
 		guint i;
 
-		clist = GTK_CLIST(Pan.group_clist);
+		clist = GTK_CLIST(Pan.group_tree);
 		gtk_clist_freeze (clist);
 		gtk_clist_unselect_all (clist);
 		for (i=0; i!=a->len; ++i)
@@ -311,7 +322,7 @@ grouplist_get_selected_group (void)
 	debug_enter ("grouplist_get_selected_group");
 
 	pan_lock ();
-	clist = GTK_CLIST(Pan.group_clist);
+	clist = GTK_CLIST(Pan.group_tree);
 	l = clist->selection;
 	if (l != NULL)
 		retval = GROUP(gtk_clist_get_row_data(clist, GPOINTER_TO_INT(l->data)));
@@ -324,7 +335,7 @@ grouplist_get_selected_group (void)
 GPtrArray*
 grouplist_get_selected_groups (void)
 {
-	GtkCList * clist = GTK_CLIST(Pan.group_clist);
+	GtkCList * clist = GTK_CLIST(Pan.group_tree);
 	GPtrArray * retval = g_ptr_array_new ();
 	GList * l = NULL;
 	debug_enter ("grouplist_get_selected_groups");
@@ -357,7 +368,7 @@ grouplist_remove_row (const Group *group)
 
 	if (group->server == grouplist_get_visible_server())
 	{
-		GtkCList *list = GTK_CLIST(Pan.group_clist);
+		GtkCList *list = GTK_CLIST(Pan.group_tree);
 		int row;
 
 		g_static_mutex_lock (&grouplist_ui_mutex);
@@ -376,7 +387,6 @@ grouplist_remove_row (const Group *group)
 ******
 *****/
 
-
 void
 grouplist_get_all (void)
 {
@@ -399,22 +409,54 @@ grouplist_get_new (void)
 ******
 *****/
 
-
-
 void
-grouplist_select_all (void)
+grouplist_add_subscribed_to_selection_nolock (void)
 {
-	pan_lock();
-	gtk_clist_select_all (GTK_CLIST (Pan.group_clist));
-	pan_unlock();
+	GtkCList * clist = GTK_CLIST (Pan.group_tree);
+	const int row_qty = clist->rows;
+	if (row_qty != 0)
+	{
+		int row;
+		gtk_clist_freeze (clist);
+		for (row=0; row<row_qty; ++row)
+			if (group_is_subscribed (GROUP(gtk_clist_get_row_data(clist,row))))
+				gtk_clist_select_row (clist, row, -1);
+		gtk_clist_thaw (clist);
+	}
 }
 
+/*****
+******
+*****/
+
+static gint
+grouplist_select_all_idle (gpointer unused) {
+	pan_lock ();
+	gtk_clist_select_all (GTK_CLIST (Pan.group_tree));
+	pan_unlock ();
+	return 0;
+}
+void
+grouplist_select_all (void) {
+	gui_queue_add (grouplist_select_all_idle, NULL);
+}
+
+/*****
+******
+*****/
+
+static gint
+grouplist_deselect_all_idle (gpointer unused)
+{
+	pan_lock ();
+	gtk_clist_unselect_all (GTK_CLIST(Pan.group_tree));
+	pan_unlock ();
+	return 0;
+}
 void
 grouplist_deselect_all (void)
 {
-	pan_lock ();
-	gtk_clist_unselect_all (GTK_CLIST(Pan.group_clist));
-	pan_unlock ();
+	gui_queue_add (grouplist_deselect_all_idle, NULL);
 }
 
 /*****
@@ -428,7 +470,7 @@ grouplist_set_view_mode_nolock_impl (gint view_mode)
 	debug_enter ("grouplist_set_view_mode_nolock_impl");
 
         menu = gtk_option_menu_get_menu (GTK_OPTION_MENU(grouplist_mode_menu));
-        gtk_object_ref (GTK_OBJECT(menu));
+        g_object_ref (G_OBJECT(menu));
         gtk_option_menu_remove_menu (GTK_OPTION_MENU (grouplist_mode_menu));
 	switch (group_mode = view_mode) {
 		case GROUP_ALL:
@@ -446,7 +488,7 @@ grouplist_set_view_mode_nolock_impl (gint view_mode)
 		default: break;
 	}
         gtk_option_menu_set_menu (GTK_OPTION_MENU(grouplist_mode_menu), menu);
-        gtk_object_unref (GTK_OBJECT(menu));
+        g_object_unref (G_OBJECT(menu));
 
 	debug_exit ("grouplist_set_view_mode_nolock_impl");
 }
@@ -488,9 +530,8 @@ grouplist_mode_menu_create (void)
 		GtkWidget *item = gtk_menu_item_new_with_label (foo[i].name);
 		gtk_widget_show (item);
 		gtk_menu_append (GTK_MENU (menu), item);
-		gtk_signal_connect (GTK_OBJECT(item), "activate",
-				    (GtkSignalFunc)grouplist_set_view_mode_nolock,
-				    GINT_TO_POINTER(foo[i].mode));
+		g_signal_connect (item, "activate",
+				  G_CALLBACK(grouplist_set_view_mode_nolock), GINT_TO_POINTER(foo[i].mode));
 		if (group_mode == foo[i].mode)
 			index = i;
 	}
@@ -576,9 +617,24 @@ grouplist_shutdown_module (void)
 
 	/* remember server so that we can default to it next time */
 	if (my_server!=NULL && is_nonempty_string(my_server->name))
-		gnome_config_set_string ("/Pan/State/Server", my_server->name);
+		pan_config_set_string ("/Pan/State/Server", my_server->name);
 
-	gnome_config_sync ();
+	pan_config_sync ();
+}
+
+extern GtkAccelGroup * _main_accel_group;
+
+static gboolean
+entry_focus_in_cb (GtkWidget * w, GdkEventKey * event, gpointer user_data) {
+	g_object_ref (_main_accel_group);
+	gtk_window_remove_accel_group (GTK_WINDOW(Pan.window), _main_accel_group);
+	return FALSE;
+}
+static gboolean
+entry_focus_out_cb (GtkWidget * w, GdkEventKey * event, gpointer user_data) {
+	gtk_window_add_accel_group (GTK_WINDOW(Pan.window), _main_accel_group);
+	g_object_unref (_main_accel_group);
+	return FALSE;
 }
 
 gpointer
@@ -614,16 +670,18 @@ grouplist_create (void)
 	pan_callback_add (server_get_groups_removed_callback(),
 	                  grouplist_server_groups_removed_cb, NULL);
 
-       	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_TEXT);
+	toolbar = gtk_toolbar_new ();
+	gtk_toolbar_set_orientation (GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
+	gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
 
-	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_container_set_border_width (GTK_CONTAINER(vbox), GNOME_PAD_SMALL); 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	vbox = gtk_vbox_new (FALSE, GUI_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_SMALL); 
+	hbox = gtk_hbox_new (FALSE, GUI_PAD);
 
-	Pan.group_clist = gtk_clist_new_with_titles (5, titles);
-	clist = GTK_CLIST(Pan.group_clist);
+	Pan.group_tree = gtk_clist_new_with_titles (5, titles);
+	clist = GTK_CLIST(Pan.group_tree);
 	clist->button_actions[1] = clist->button_actions[0];
-	pan_widget_set_font (GTK_WIDGET(Pan.group_clist), grouplist_font);
+	pan_widget_set_font (GTK_WIDGET(Pan.group_tree), grouplist_font);
 	gtk_clist_set_column_justification (clist, 2, GTK_JUSTIFY_RIGHT);
 	gtk_clist_set_column_justification (clist, 3, GTK_JUSTIFY_RIGHT);
 
@@ -638,16 +696,14 @@ grouplist_create (void)
 	w = gtk_label_new (_("Find:"));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
-	groupname_filter_entry = w = gnome_entry_new ("Grouplist Toolbar Filter");
-	gnome_entry_set_max_saved (GNOME_ENTRY(w), 10);
-	gtk_signal_connect (GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(w))), "activate",
-			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb_nolock), w);
-	gtk_signal_connect (GTK_OBJECT(GTK_COMBO(w)->popwin), "button_press_event",
-			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb_nolock), w);
+	groupname_filter_entry = w = gtk_entry_new ();
+	g_signal_connect (w, "focus_in_event", G_CALLBACK(entry_focus_in_cb), w);
+	g_signal_connect (w, "focus_out_event", G_CALLBACK(entry_focus_out_cb), w);
+	g_signal_connect (w, "focus_out_event", G_CALLBACK(grouplist_filter_changed_cb_nolock), w);
+	g_signal_connect (w, "activate", G_CALLBACK(grouplist_filter_changed_cb_nolock), w);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
 		_("Type in a group search string and press ENTER.  "
 		  "Wildcards are allowed."), "");
-	gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 100);
 	gtk_widget_show_all (toolbar);
 	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -657,42 +713,37 @@ grouplist_create (void)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
-	gtk_container_add (GTK_CONTAINER(w), Pan.group_clist);
+	gtk_container_add (GTK_CONTAINER(w), Pan.group_tree);
         gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 0);
 
-	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
-			    "select_row",
-			    GTK_SIGNAL_FUNC (select_row_cb), NULL);
-
-	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
-			    "unselect_row",
-			    GTK_SIGNAL_FUNC (select_row_cb), NULL);
-
-	gtk_signal_connect (GTK_OBJECT (Pan.group_clist),
-			    "button_press_event",
-			    GTK_SIGNAL_FUNC (grouplist_button_press), NULL);
-	gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "key_press_event",
-	                    GTK_SIGNAL_FUNC (grouplist_key_press),
-	                    NULL);
+	g_signal_connect (Pan.group_tree, "select_row",
+	                  G_CALLBACK(select_row_cb), NULL);
+	g_signal_connect (Pan.group_tree, "unselect_row",
+	                  G_CALLBACK(select_row_cb), NULL);
+	g_signal_connect (Pan.group_tree, "button_press_event",
+	                  G_CALLBACK(grouplist_button_press), NULL);
+	g_signal_connect (Pan.group_tree, "key_press_event",
+	                  G_CALLBACK(grouplist_key_press), NULL);
 
 
-	gtk_clist_set_column_width     (clist, 0, 16);
-	gtk_clist_set_column_min_width (clist, 0, 16);
-	gtk_clist_set_column_max_width (clist, 0, 16);
+	gtk_clist_set_column_width     (clist, 0, 20);
+	gtk_clist_set_column_min_width (clist, 0, 20);
+	gtk_clist_set_column_max_width (clist, 0, 20);
 
-	gtk_clist_set_column_width (clist, 1, 170);
-	gtk_clist_set_column_width (clist, 2, 36);
-	gtk_clist_set_column_width (clist, 3, 33);
-	gtk_clist_set_column_width (clist, 4, 350);
+	gtk_clist_set_column_width (clist, 1, 200);
+	gtk_clist_set_column_width (clist, 2, 50);
+	gtk_clist_set_column_width (clist, 3, 50);
+	gtk_clist_set_column_width (clist, 4, 400);
 
 	gtk_clist_set_selection_mode (clist, GTK_SELECTION_EXTENDED);
 
 	/* Create the Popup Menu while we are here. */
-	group_clist_menu = gnome_popup_menu_new (group_menu);	
+	popup_menu = menu_create_items (group_popup_entries,
+	                                G_N_ELEMENTS (group_popup_entries),
+	                                "<GroupView>",
+	                                &popup_factory,
+	                                NULL);
 
-	/* Lets have the Pixmap ready too */
-	folder_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d (folder_xpm);
-	subscribed_pixmap = GNOME_PIXMAP(gnome_stock_pixmap_widget_new (Pan.window, GNOME_STOCK_MENU_BOOK_RED));
 
 	/* bootstrap */
 	grouplist_set_server_nolock (serverlist_get_active_server());
@@ -702,37 +753,73 @@ grouplist_create (void)
 	return vbox;
 }
 
-static gchar * _last_filter_str = NULL;
+static gchar * groupname_last_filter_string = NULL;
 
-static void
+static gboolean
 grouplist_filter_changed_cb_nolock (void)
 {
-	GtkWidget * e = gnome_entry_gtk_entry(GNOME_ENTRY(groupname_filter_entry));
-	gchar * pch = gtk_editable_get_chars (GTK_EDITABLE(e), 0, -1);
+	gchar * pch = gtk_editable_get_chars (GTK_EDITABLE(groupname_filter_entry), 0, -1);
 
-	if (!pan_strcmp (pch, _last_filter_str))
+	if (!pan_strcmp (pch, groupname_last_filter_string))
 		g_free (pch);
 	else {
-		replace_gstr (&_last_filter_str, pch);
+		replace_gstr (&groupname_last_filter_string, pch);
 		grouplist_refresh_filter_nolock ();
 	}
+
+	return FALSE;
 }
 
 /*****
 ******
 *****/
 
+static GdkPixmap * sub_pixmap = NULL;
+static GdkBitmap * sub_bitmap = NULL;
+static GdkPixmap * folder_pixmap = NULL;
+static GdkBitmap * folder_bitmap = NULL;
+
 static void
 grouplist_add_group_nolock (const Group * group)
 {
 	gint row;
-	GnomePixmap * pixmap;
-	gchar total_buf[16] = { '\0' };
-	gchar unread_buf[16] = { '\0' };
-	gchar name_buf[256] = { '\0' };
-	const gint total = group->article_qty;
-	const gint unread = MAX(0, total - group->article_read_qty);
-	gchar * text[5];
+	char total_buf[16] = { '\0' };
+	char unread_buf[16] = { '\0' };
+	char name_buf[256] = { '\0' };
+	char * freeme1 = NULL;
+	char * freeme2 = NULL;
+	const int total = group->article_qty;
+	const int unread = MAX(0, total - group->article_read_qty);
+	char * text[5];
+	static gboolean inited = FALSE;
+
+	/* inistialize the picture */
+	if (!inited)
+	{
+		GdkPixbuf * pixbuf;
+		GdkPixbuf * scaled;
+
+		pixbuf = gdk_pixbuf_new_from_inline (-1, icon_stock_book_red, FALSE, NULL);
+		scaled = gdk_pixbuf_scale_simple (pixbuf, 18, 18,  GDK_INTERP_HYPER);
+		gdk_pixbuf_render_pixmap_and_mask_for_colormap (scaled,
+		                                                gtk_widget_get_colormap(Pan.group_tree),
+		                                                &sub_pixmap,
+		                                                &sub_bitmap, 128);
+		g_object_unref (G_OBJECT(scaled));
+		g_object_unref (G_OBJECT(pixbuf));
+
+		/* initialize the folder icon */
+		pixbuf = gdk_pixbuf_new_from_inline (-1, icon_folder, FALSE, NULL);
+		scaled = gdk_pixbuf_scale_simple (pixbuf, 18, 18,  GDK_INTERP_HYPER);
+		gdk_pixbuf_render_pixmap_and_mask_for_colormap (scaled,
+		                                                gtk_widget_get_colormap(Pan.group_tree),
+		                                                &folder_pixmap,
+		                                                &folder_bitmap, 128);
+		g_object_unref (G_OBJECT(scaled));
+		g_object_unref (G_OBJECT(pixbuf));
+
+		inited = TRUE;
+	}
 
 	/* commatize the totals */
 	commatize_ulong (total, total_buf);
@@ -740,34 +827,33 @@ grouplist_add_group_nolock (const Group * group)
 
 	/* look for special qualities of the group */
 	grouplist_get_group_readable_name (group, name_buf, sizeof(name_buf));
-       
+     
 	text[0] = "";
-	text[1] = name_buf;
+	text[1] = freeme1 = pan_header_to_utf8 (name_buf, -1, NULL);
 	text[2] = unread_buf;
 	text[3] = total_buf;
-	text[4] = (gchar*)(group->description?group->description:"");
+	text[4] = freeme2 = pan_header_to_utf8 (group->description, -1, NULL);
 
 	/* add the row */
-	row = gtk_clist_prepend (GTK_CLIST(Pan.group_clist), (char**)text);
-	gtk_clist_set_row_data (GTK_CLIST(Pan.group_clist), row, (gpointer)group);
+	row = gtk_clist_prepend (GTK_CLIST(Pan.group_tree), (char**)text);
+	gtk_clist_set_row_data (GTK_CLIST(Pan.group_tree), row, (gpointer)group);
 
 	/* add the pixmap, if any */
-	pixmap = NULL;
 	if (group_is_folder (group))
-		pixmap = folder_pixmap;
+		gtk_clist_set_pixmap (GTK_CLIST(Pan.group_tree), row, 0, folder_pixmap, folder_bitmap);
 	else if (group_is_subscribed (group))
-		pixmap = subscribed_pixmap;
-	if (pixmap != NULL)
-		gtk_clist_set_pixmap (GTK_CLIST(Pan.group_clist), row, 0,
-		                      pixmap->pixmap,
-		                      pixmap->mask);
+		gtk_clist_set_pixmap (GTK_CLIST(Pan.group_tree), row, 0, sub_pixmap, sub_bitmap);
+
+	/* cleanup */
+	g_free (freeme1);
+	g_free (freeme2);
 }
 
 static void
 grouplist_rebuild_nolock (GPtrArray * groups, GPtrArray * sel)
 {
 	gint i;
-	GtkCList * clist = GTK_CLIST (Pan.group_clist);
+	GtkCList * clist = GTK_CLIST (Pan.group_tree);
 
 	gtk_clist_freeze (clist);
 	gtk_clist_clear (clist);
@@ -793,7 +879,7 @@ static gint
 grouplist_update_ui_mainthread (gpointer unused)
 {
 	gchar * title;
-	GtkCList * clist = GTK_CLIST(Pan.group_clist);
+	GtkCList * clist = GTK_CLIST(Pan.group_tree);
 	GPtrArray * groups;
 	GPtrArray * sel;
 	debug_enter ("grouplist_update_ui_mainthread");
@@ -863,12 +949,12 @@ grouplist_update_ui_mainthread (gpointer unused)
 	}
 
 	/* update UI */
-	pan_lock_unconditional ();
+	pan_lock ();
 	g_static_mutex_lock (&grouplist_ui_mutex);
 	gtk_clist_set_column_title (clist, 1, title);
 	grouplist_rebuild_nolock (groups, sel);
 	g_static_mutex_unlock (&grouplist_ui_mutex);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	/* cleanup */
 	g_ptr_array_free (sel, TRUE);
@@ -883,7 +969,7 @@ grouplist_refresh_nolock (void)
 	Server * server = grouplist_get_visible_server ();
 
 	if (server!=NULL && server==grouplist_get_visible_server())
-		run_in_main_thread_nolock (grouplist_update_ui_mainthread, NULL);
+		gui_queue_add (grouplist_update_ui_mainthread, NULL);
 }
 
 /* firing this callback is delegated to here because you should never fire
@@ -909,12 +995,12 @@ grouplist_set_server_nolock (Server * server)
 	if (my_server==NULL)
 	{
 		if (grouplist_get_visible_server()==old_server)
-			gtk_clist_clear (GTK_CLIST(Pan.group_clist));
+			gtk_clist_clear (GTK_CLIST(Pan.group_tree));
 	}
 	else
 	{
 		gint view_mode;
-		GPtrArray * groups;
+		GPtrArray * groups = NULL;
 		const ServerGroupsType have_type = file_grouplist_exists (my_server);
 
 		/* if we have subbed, always load those, otherwise load all */
@@ -936,30 +1022,23 @@ grouplist_set_server_nolock (Server * server)
 		}
 		else
 		{
-			gchar * s;
 			GtkWidget * w;
 
-			view_mode = GROUP_ALL;
+                        view_mode = GROUP_ALL;
 			groups = NULL;
 
 			if (grouplist_get_visible_server()==old_server)
-				gtk_clist_clear (GTK_CLIST(Pan.group_clist));
+				grouplist_rebuild_nolock (NULL, NULL);
 
-			s = g_strdup_printf (
-				_("We don't have a group list for server \"%s\".\nShall we get one?"),
-				server->name);
-			w = gnome_message_box_new (s, GNOME_MESSAGE_BOX_QUESTION,
-						      GNOME_STOCK_BUTTON_YES,
-						      GNOME_STOCK_BUTTON_NO, NULL);
-			gnome_dialog_set_default (GNOME_DIALOG(w), 0);
-			gnome_dialog_set_parent (GNOME_DIALOG(w),
-						 GTK_WINDOW(Pan.window));
-			gtk_signal_connect (GTK_OBJECT(w), "clicked",
-					    GTK_SIGNAL_FUNC(group_dialog_clicked_cb),
-					    server);
+			w = gtk_message_dialog_new (GTK_WINDOW(Pan.window),
+			                            GTK_DIALOG_DESTROY_WITH_PARENT,
+			                            GTK_MESSAGE_QUESTION,
+			                            GTK_BUTTONS_YES_NO,
+			                            _("We don't have a list of groups for \"%s\".\nShall we get download it?"),
+			                            server->name);
+			gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_YES);
+			g_signal_connect (w, "response", G_CALLBACK(grouplist_dialog_response_cb), server);
 			gtk_widget_show_all (w);
-
-			g_free (s);
 		}
 
 		/* update the UI */
@@ -996,26 +1075,27 @@ grouplist_update_group_nolock (const Group * group)
 	/* sanity check */
 	g_return_if_fail (group != NULL);
 
-	list = GTK_CLIST (Pan.group_clist);
+	list = GTK_CLIST (Pan.group_tree);
        	row = gtk_clist_find_row_from_data (list,(gpointer)group);
 	if (row != -1)
 	{
-		gchar buf[256] = { '\0' };
-		const gint total = group->article_qty;
-		const gint unread = MAX(0, total-group->article_read_qty);
+		char buf[256] = { '\0' };
+		const char * utf8_name = NULL;
+		char * freeme;
+		const int total = group->article_qty;
+		const int unread = MAX(0, total-group->article_read_qty);
 
 		/* icon column */
 		if (group_is_subscribed (group))
-			gtk_clist_set_pixmap (GTK_CLIST(Pan.group_clist), row, 0,
-			                      subscribed_pixmap->pixmap,
-			                      subscribed_pixmap->mask);
+			gtk_clist_set_pixmap (GTK_CLIST(Pan.group_tree), row, 0, sub_pixmap, sub_bitmap);
 		else if (!group_is_folder(group))
-			gtk_clist_set_text (GTK_CLIST(Pan.group_clist),
-					    row, 0, "");
+			gtk_clist_set_text (GTK_CLIST(Pan.group_tree), row, 0, "");
 
 		/* name column */
 		grouplist_get_group_readable_name (group, buf, sizeof(buf));
-		gtk_clist_set_text (list, row, 1, buf);
+		utf8_name = freeme = pan_header_to_utf8 (buf, -1, NULL);
+		gtk_clist_set_text (list, row, 1, utf8_name);
+		g_free (freeme);
 
 		/* total column */
 		commatize_ulong (total, buf);
@@ -1032,16 +1112,16 @@ grouplist_update_groups_mainthread (gpointer data)
 {
 	guint i;
 	GPtrArray * a = (GPtrArray*)data;
-	GtkCList * list = GTK_CLIST (Pan.group_clist);
+	GtkCList * list = GTK_CLIST (Pan.group_tree);
 	debug_enter ("grouplist_update_groups_mainthread");
 
 	g_static_mutex_lock (&grouplist_ui_mutex);
-	pan_lock_unconditional ();
+	pan_lock ();
 	gtk_clist_freeze (list);
 	for (i=0; i!=a->len; ++i)
 		grouplist_update_group_nolock (GROUP(g_ptr_array_index(a,i)));
 	gtk_clist_thaw (list);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 	g_static_mutex_unlock (&grouplist_ui_mutex);
 
 	/* cleanup */
@@ -1069,13 +1149,10 @@ grouplist_update_groups (const Group ** groups, int group_qty)
 			g_ptr_array_add (a, (gpointer)groups[i]);
 
 	/* if we have any that we should update, queue it */
-	if (a->len != 0) {
-		pan_lock ();
-		run_in_main_thread_nolock (grouplist_update_groups_mainthread, a);
-		pan_unlock ();
-	} else {
+	if (a->len != 0)
+		gui_queue_add (grouplist_update_groups_mainthread, a);
+	else
 		g_ptr_array_free (a, TRUE);
-	}
 
 	debug_exit ("grouplist_update_groups");
 }
@@ -1090,7 +1167,7 @@ grouplist_internal_select_row (gint      row,
                                gboolean  activate,
                                gboolean  lock)
 {
-        GtkCList* list = GTK_CLIST(Pan.group_clist);
+        GtkCList* list = GTK_CLIST(Pan.group_tree);
 
 	if (activate)
 		button_click_count = 2;
@@ -1108,7 +1185,7 @@ grouplist_select_row_if_unread (int row,
                                 gboolean activate,
                                 gboolean lock)
 {
-        GtkCList * list = GTK_CLIST(Pan.group_clist);
+        GtkCList * list = GTK_CLIST(Pan.group_tree);
 	Group * g = gtk_clist_get_row_data(list,row);
 	const gulong total = g->article_qty;
 	const gulong read = g->article_read_qty;
@@ -1128,7 +1205,7 @@ grouplist_next_unread_group (gboolean activate)
 	int row_qty, i, row=0;
 
 	pan_lock ();
-	list = GTK_CLIST(Pan.group_clist);
+	list = GTK_CLIST(Pan.group_tree);
         row_qty = list->rows;
 
 	if (row_qty != 0)
@@ -1166,7 +1243,7 @@ grouplist_next_group (gboolean activate)
 	int row = 0;
 
 	pan_lock ();
-	list = GTK_CLIST(Pan.group_clist);
+	list = GTK_CLIST(Pan.group_tree);
         row_qty = list->rows;
 
 	if (row_qty != 0)
@@ -1201,18 +1278,25 @@ grouplist_activate_next_group (void)
 static void
 group_menu_popup (GdkEventButton *event)
 {
+	GtkItemFactory * gif = popup_factory;
+	const Group * group = grouplist_get_selected_group ();
+	const gboolean have_group = group!=NULL;
 	debug_enter ("group_menu_popup");
 
-	/* We must be dealing with real groups */
-	{
-		pan_lock ();
+	menu_set_sensitive (gif, "/Mark Group Read", have_group);
+	menu_set_sensitive (gif, "/Delete Group's Articles", have_group);
+	menu_set_sensitive (gif, "/Get New Headers", have_group);
+	menu_set_sensitive (gif, "/Get New Headers and Bodies", have_group);
+	menu_set_sensitive (gif, "/More Download Options...", have_group);
+	menu_set_sensitive (gif, "/Refresh Article Counts", have_group);
+	menu_set_sensitive (gif, "/Subscribe", have_group);
+	menu_set_sensitive (gif, "/Unsubscribe", have_group);
+	menu_set_sensitive (gif, "/Group Properties...", have_group);
+	menu_set_sensitive (gif, "/Delete Group", have_group);
 
-		/* show the menu */
-		gtk_menu_popup (GTK_MENU(group_clist_menu),
-				NULL, NULL, NULL, NULL,
-				event->button, event->time);
-		pan_unlock ();
-	} 
+	gtk_menu_popup (GTK_MENU(popup_menu),
+	                NULL, NULL, NULL, NULL,
+	                event->button, event->time);
 
 	debug_exit ("group_menu_popup");
 }
@@ -1223,7 +1307,7 @@ grouplist_refresh_filter_nolock (void)
 	gchar * new_filter;
         gchar * pch;
 
-	pch  = gtk_editable_get_chars (GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(groupname_filter_entry))), 0, -1);
+	pch  = gtk_editable_get_chars (GTK_EDITABLE(groupname_filter_entry), 0, -1);
 
 	/* build the new filter string */
 	new_filter = NULL;
