@@ -24,7 +24,8 @@
 
 #include <string.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/run.h>
@@ -38,7 +39,7 @@
 #include <pan/util.h>
 
 gulong header_flags = ~0;
-const gint max_header_width = 60;
+const int max_header_width = 60;
 extern GtkTooltips * ttips;
 
 
@@ -49,27 +50,36 @@ extern GtkTooltips * ttips;
 static void
 showmore_cb (GtkButton *button, gpointer user_data)
 {
-	const gchar* showme = (const gchar*) gtk_object_get_data (GTK_OBJECT(button), "text");
-	gnome_ok_dialog_parented (showme, GTK_WINDOW(Pan.window));
+	GtkWidget * w;
+	const char* showme = (const char*) gtk_object_get_data (GTK_OBJECT(button), "text");
+
+	w = gtk_message_dialog_new (GTK_WINDOW(Pan.window), 0, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "%s", showme);
+	g_signal_connect_swapped (GTK_OBJECT(w), "response", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT(w));
+	gtk_widget_show_all (w);
 }
 
-static gint
-find_breakpoint (const gchar  * str,
-                 gchar          delimiter,
-                 gint           max_width)
+static int
+find_breakpoint (const char  * str,
+                 char          delimiter,
+                 int           max_width)
 {
-	const gint len = strlen(str);
-	gint retval;
+	int len;
+	int retval;
 	debug_enter ("find_breakpoint");
 
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(str), -1);
+	g_return_val_if_fail (max_width>0, -1);
+
+       	len = strlen (str);
 	if (!delimiter || len<=max_width)
 	{
 		retval = -1;
 	}
 	else
 	{
-		const gchar * pch = str;
-		const gchar * last_match = NULL;
+		const char * pch = str;
+		const char * last_match = NULL;
 		for (; *pch; ++pch)
 		{
 			if (*pch == delimiter)
@@ -86,19 +96,19 @@ find_breakpoint (const gchar  * str,
 }
 
 static void
-add_header_nolock (GtkWidget        * table,
-	           gint               row,
-	           const gchar      * str1,
-	           const gchar      * str2,
-	           gchar              delimiter,
-                   gint               max_width)
+add_header_nolock (GtkWidget      * table,
+	           int               row,
+	           const char      * str1,
+	           const char      * str2,
+	           char             delimiter,
+                   int              max_width)
 {
 	GtkTable * t;
 	GtkWidget * w1;
 	GtkWidget * w2 = NULL;
-	gint breakpoint;
+	int breakpoint;
 	debug_enter ("add_header_nolock");
-       
+
        	t = GTK_TABLE(table);
        	w1 = gtk_label_new (str1);
 
@@ -115,8 +125,8 @@ add_header_nolock (GtkWidget        * table,
 		GtkWidget * hbox = gtk_hbox_new (FALSE, 0);
 		GtkWidget * l = NULL;
 		GtkWidget * l2 = gtk_label_new (_("<More>"));
-		gchar * full = NULL;
-		gchar * pch = NULL;
+		char * full = NULL;
+		char * pch = NULL;
 
 		/* create the truncated label */
 		pch = pan_strdup_alloca (str2);
@@ -130,8 +140,13 @@ add_header_nolock (GtkWidget        * table,
 		/* break the line into easier-to-read pieces
 		   for the popup dialog */
 		pch = full = g_strdup(str2);
-		for (;;) {
-			int index = find_breakpoint (pch, delimiter, max_width);
+		if (delimiter != ' ') {
+			for (; *pch; pch = g_utf8_next_char (pch))
+				if (*pch == delimiter)
+					*pch = '\n';
+		}
+		else for (;;) {
+			const int index = find_breakpoint (pch, delimiter, max_width);
 			if (index == -1) break;
 			pch[index] = '\n';
 			pch += (index + 1);
@@ -144,10 +159,9 @@ add_header_nolock (GtkWidget        * table,
 		gtk_widget_show (l);
 		gtk_widget_show (l2);
 		gtk_widget_show (hbox);
-		gtk_object_set_data_full (
-			GTK_OBJECT(w2), "text", full, g_free);
-		gtk_signal_connect (
-			GTK_OBJECT(w2), "clicked", showmore_cb, NULL);
+		gtk_object_set_data_full (GTK_OBJECT(w2), "text", full, g_free);
+		g_signal_connect (GTK_OBJECT(w2), "clicked",
+		                  G_CALLBACK(showmore_cb), NULL);
 		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w2, str2, NULL);
 	}
 
@@ -164,7 +178,7 @@ add_header_nolock (GtkWidget        * table,
 }
 
 static const Run*
-get_header_run (GHashTable * hash, const gchar * header)
+get_header_run (GHashTable * hash, const char * header)
 {
 	Run key;
 	(void) run_init (&key, header, strlen(header));
@@ -172,28 +186,25 @@ get_header_run (GHashTable * hash, const gchar * header)
 }
 
 static void
-get_and_add_header_nolock (GtkWidget    * table,
-                           gint         * row,
-                           GHashTable   * run_hash,
-                           const gchar  * header,
-                           const gchar  * header_i18n,
-                           gchar          delimiter,
-                           gint           max_len)
+get_and_add_header_nolock (GtkWidget   * table,
+                           int         * row,
+                           GHashTable  * run_hash,
+			   const char  * charset,
+                           const char  * header,
+                           const char  * header_i18n,
+                           char          delimiter,
+                           int           max_len)
 {
 	const Run * val = get_header_run (run_hash, header);
 	if (val != NULL)
 	{
-		gchar * decoded = NULL;
-		gchar * pch;
+		char * value;
 
-		pan_strndup_alloca (pch, val->str, val->len);
+		value = pan_header_to_utf8 (val->str, val->len, charset);
 
-		if (pan_header_is_8bit_encoded (pch)) 
-			pch = decoded = g_mime_utils_8bit_header_decode (pch);
+		add_header_nolock (table, (*row)++, header_i18n, value, delimiter, max_header_width);
 
-		add_header_nolock (table, (*row)++, header_i18n, pch, delimiter, max_header_width);
-
-		g_free (decoded);
+		g_free (value);
 	}
 }
 
@@ -202,32 +213,38 @@ gui_headers_set_nolock (GtkWidget       * table,
 		        const Article   * article,
 		        gulong            header_fields)
 {
-	gint row = 0;
+	int row = 0;
+	char * message = NULL;
 	gpointer buf = NULL;
-	gchar * message = NULL;
 	GHashTable * hash = NULL;
+	const gchar * charset = NULL;
 	debug_enter ("gui_headers_set_nolock");
 
 	/* remove previous children */
 	g_return_if_fail (GTK_IS_TABLE(table));
 	gtk_container_foreach (GTK_CONTAINER (table),
-	                       GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
+	                       (GtkCallback)(gtk_widget_destroy), NULL);
 
 	/* if no article, that's all we need to do */
 	if (article == NULL)
 		return;
 
+	/* work out which charset to use */
+	charset = article_get_extra_header (article, PAN_CHARSET);
+	if (charset == NULL)
+		charset = group_get_default_charset (article->group);
+
 	/* get the article headers */
 	message = article_get_message (article);
 	if (article_get_header_run_hash_from_text (message, &hash, &buf))
 	{
 		const Run * val;
 		const Run * val2;
-		gchar * pch;
+		char * pch;
 
 		/* if author turned on, show it unconditionally */
 		if (header_fields & UI_HEADER_AUTHOR)
-			get_and_add_header_nolock (table, &row, hash, HEADER_FROM, _("From: "), ' ', max_header_width);
+			get_and_add_header_nolock (table, &row, hash, charset, HEADER_FROM, _("From: "), ' ', max_header_width);
 
 		/* if newsgroups turned on, and this is a crosspost, show it */
 		if (header_fields & UI_HEADER_NEWSGROUPS) {
@@ -240,15 +257,15 @@ gui_headers_set_nolock (GtkWidget       * table,
 
 		/* message-id */
 		if (header_fields & UI_HEADER_MESSAGE_ID)
-			get_and_add_header_nolock (table, &row, hash, HEADER_MESSAGE_ID, _("Message-Id: "), ' ', max_header_width);
+			get_and_add_header_nolock (table, &row, hash, charset, HEADER_MESSAGE_ID, _("Message-Id: "), ' ', max_header_width);
 
 		/* references */
 		if (header_fields & UI_HEADER_REFERENCES)
-			get_and_add_header_nolock (table, &row, hash, HEADER_REFERENCES, _("References: "), ' ', max_header_width);
+			get_and_add_header_nolock (table, &row, hash, charset, HEADER_REFERENCES, _("References: "), ' ', max_header_width);
 
 		/* date */
 		if (header_fields & UI_HEADER_DATE)
-			get_and_add_header_nolock (table, &row, hash, HEADER_DATE, _("Date: "), '\0', max_header_width);
+			get_and_add_header_nolock (table, &row, hash, charset, HEADER_DATE, _("Date: "), '\0', max_header_width);
 
 		/* reply-to */
 		if (header_fields & UI_HEADER_REPLY_TO) {
@@ -273,9 +290,9 @@ gui_headers_set_nolock (GtkWidget       * table,
 
 		/* newsreader */
 		if (header_fields & UI_HEADER_NEWSREADER) {
-			gint i;
-			static gchar * tags[] = { HEADER_USER_AGENT, HEADER_X_NEWSREADER, HEADER_X_MAILER };
-			static gint tag_count = sizeof tags / sizeof tags[0];
+			int i;
+			static char * tags[] = { HEADER_USER_AGENT, HEADER_X_NEWSREADER, HEADER_X_MAILER };
+			static int tag_count = G_N_ELEMENTS(tags);
 			for (i=0; i<tag_count; ++i) {
 				if ((val = get_header_run (hash, tags[i]))) {
 					pan_strndup_alloca (pch, val->str, val->len);
@@ -287,7 +304,7 @@ gui_headers_set_nolock (GtkWidget       * table,
 
 		/* subject */
 		if (header_fields & UI_HEADER_SUBJECT)
-			get_and_add_header_nolock (table, &row, hash, HEADER_SUBJECT, _("Subject: "), ' ', max_header_width);
+			get_and_add_header_nolock (table, &row, hash, charset, HEADER_SUBJECT, _("Subject: "), ' ', max_header_width);
 
 		g_hash_table_destroy (hash);
 		g_free (buf);
