@@ -22,9 +22,11 @@
 
 #include <ctype.h>
 
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
@@ -36,7 +38,7 @@
 #include <pan/prefs.h>
 #include <pan/util.h>
 
-/* FIXME: replace the old gnome_config keys here & in gui_shutdown() then remove uglyhack_idx */
+/* FIXME: replace the old pan_config keys here & in gui_shutdown() then remove uglyhack_idx */
 static GtkWidget*
 pack_widgets (GtkWidget * w1, GtkWidget * w2, GtkOrientation orient, gint uglyhack_idx)
 {
@@ -45,7 +47,7 @@ pack_widgets (GtkWidget * w1, GtkWidget * w2, GtkOrientation orient, gint uglyha
 
 	if (w1!=NULL && w2!=NULL) {
 		const gchar * keys[] = { "/Pan/Geometry/vpaned=302", "/Pan/Geometry/hpaned=265" };
-		const gint pos = gnome_config_get_int (keys[uglyhack_idx]);
+		const gint pos = pan_config_get_int (keys[uglyhack_idx]);
 		if (vertical) {
 			w = gtk_vpaned_new ();
 			gtk_widget_set_usize (w1, -1, 50);
@@ -84,31 +86,7 @@ gui_paned_construct (void)
 
 	pan_lock ();
 
-	/* clear out the workarea */
-	gtk_widget_unrealize (GTK_WIDGET(contents_vbox));
-	gtk_widget_hide_all (GTK_WIDGET(contents_vbox));
-	if (groups_vbox->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
-	if (articlelist_ctree->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(articlelist_ctree->parent), articlelist_ctree);
-	if (text_box->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
-	gtk_widget_unrealize (groups_vbox);
-	gtk_widget_unrealize (articlelist_ctree);
-	gtk_widget_unrealize (text_box);
-
-	/* make a new UI */
-	if (1) {
-		GList * children = gtk_container_children (GTK_CONTAINER(contents_vbox));
-		GList * l;
-		for (l=children; l!=NULL; l=l->next)
-			gtk_container_remove (GTK_CONTAINER(contents_vbox), GTK_WIDGET(l->data));
-		g_list_free (l);
-	}
-
-	gtk_widget_show (groups_vbox);
-	gtk_widget_show (articlelist_ctree);
-	gtk_widget_show (text_box);
+	gui_layout_clear_workarea ();
 
 	/**
 	***  The layout_str is a four-character string where the first
@@ -126,7 +104,7 @@ gui_paned_construct (void)
 		else if (tolower(*pch)=='t')
 			w[i] = show_header_pane ? articlelist_ctree : NULL;
 		else
-			w[i] = show_article_pane ? text_box : NULL;
+			w[i] = show_body_pane ? text_box : NULL;
 	}
 
 	/* layout part 2: build the array */
@@ -166,38 +144,38 @@ gui_paned_construct (void)
 	}
 
 	if (work != NULL)
-		gtk_box_pack_start (GTK_BOX(contents_vbox), work, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX(Pan.workarea), work, TRUE, TRUE, 0);
 
 	Pan.viewmode = GUI_PANED;
-	gui_restore_column_widths (Pan.group_clist, "group");
+	gui_restore_column_widths (Pan.group_tree, "group");
 	gui_restore_column_widths (Pan.article_ctree, "thread_pane");
-	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
+	gtk_widget_show_all (GTK_WIDGET(Pan.workarea));
 
 	pan_unlock ();
 	debug_exit ("gui_paned_construct");
 }
 
 
-/*---[ gui_paned_get_current_pane ]-----------------------------------
+/*---[ gui_paned_get_current_pane_nolock ]----------------------------
  * get the current pane, returns an integer reflecting which pane
  * the user is 'in' ... almost identical to the gtk_notebook_get_page
  * functions
  *--------------------------------------------------------------------*/
 int
-gui_paned_get_current_pane (void)
+gui_paned_get_current_pane_nolock (void)
 {
-	gint retval = GROUPS_PAGE; /* default to group pane */
+	int retval = GROUPS_PANE; /* default to group pane */
 
 	if (GTK_WIDGET_HAS_FOCUS (Pan.article_ctree))
-		retval = ARTICLELIST_PAGE;
+		retval = HEADERS_PANE;
 	else if (GTK_WIDGET_HAS_FOCUS (Pan.text))
-		retval = MESSAGE_PAGE;
+		retval = BODY_PANE;
 	
 	return retval;
 }
 
 void
-gui_paned_page_set (int page, GtkWidget * focus_item)
+gui_paned_page_set_nolock (int page, GtkWidget * focus_item)
 {
 	if (GTK_IS_WIDGET(focus_item))
 		gtk_widget_grab_focus (focus_item);
