@@ -20,8 +20,6 @@
 
 #include <glib.h>
 
-#include <gnome.h>
-
 #include <pan/prefs.h>
 
 #include <pan/base/base-prefs.h>
@@ -51,7 +49,7 @@ static gchar     * _default_mail_id = NULL;
 static gchar *
 get_identity_filename (void)
 {
-	return g_strdup_printf ("%s%cprofiles.xml", data_dir, G_DIR_SEPARATOR);
+	return g_strdup_printf ("%s%cprofiles.xml", get_data_dir(), G_DIR_SEPARATOR);
 }
 
 /*
