@@ -23,10 +23,17 @@
 
 #include <glib.h>
 #include <pan/base/message-check.h>
+#include <gtk/gtkwidget.h>
 
-void check_article_and_prompt (const Article    * article_to_post,
-                               Server           * server_to_post_article,
-                               GPtrArray        * appendme_error_strings,
-                               GoodnessLevel    * setme_goodness_level);
+typedef void (*ArticleCheckedFunc)(Server          * server,
+                                   Article         * article,
+                                   GoodnessLevel     goodness,
+                                   gpointer          user_data);
+
+void check_article_and_prompt (GtkWidget           * window,
+                               Server              * server_to_post_article,
+                               Article             * article_to_post,
+                               ArticleCheckedFunc    checked_func,
+                               gpointer              checked_func_user_data);
 
 #endif
