@@ -40,6 +40,7 @@
 
 #include <pan/message-check-ui.h>
 #include <pan/message-send.h>
+#include <pan/globals.h>
 #include <pan/nntp.h>
 #include <pan/prefs.h>
 #include <pan/queue.h>
@@ -74,8 +75,8 @@ build_nntp_header (GString        * postme,
                    const gchar    * subject_append,
 		   gboolean         force_new_message_id)
 {
-	gchar * tmp;
-	const gchar * pch;
+	char * tmp;
+	const char * pch;
 	debug_enter ("build_nntp_header");
 
 	/* From: */
@@ -87,7 +88,7 @@ build_nntp_header (GString        * postme,
 		pan_warn_if_fail (is_nonempty_string (tmp));
 		g_string_sprintfa (postme, "From: %s\r\n", tmp);
 		g_free (tmp);
-		internet_address_destroy (ia);
+		internet_address_unref (ia);
 	}
 
 	/* Subject: */
@@ -108,7 +109,7 @@ build_nntp_header (GString        * postme,
 	g_free (tmp);
 
 	/* Newsgroups: */
-	pch = article_get_header (article, HEADER_NEWSGROUPS);
+	pch = article_get_extra_header (article, HEADER_NEWSGROUPS);
 	tmp = g_mime_utils_8bit_header_encode (pch);
 	pan_warn_if_fail (is_nonempty_string (tmp));
 	g_string_sprintfa (postme, "Newsgroups: %s\r\n", tmp);
@@ -119,21 +120,21 @@ build_nntp_header (GString        * postme,
 	g_string_sprintfa (postme, "Path: not-for-mail\r\n");
 
 	/* Followup-To: */
-	pch = article_get_header (article, HEADER_FOLLOWUP_TO);
+	pch = article_get_extra_header (article, HEADER_FOLLOWUP_TO);
 	tmp = g_mime_utils_8bit_header_encode (pch);
 	if (is_nonempty_string (tmp))
 		g_string_sprintfa (postme, "Followup-To: %s\r\n", tmp);
 	g_free (tmp);
 
 	/* Reply-To: */
-	pch = article_get_header (article, HEADER_REPLY_TO);
+	pch = article_get_extra_header (article, HEADER_REPLY_TO);
 	tmp = g_mime_utils_8bit_header_encode (pch);
 	if (is_nonempty_string (tmp))
 		g_string_sprintfa (postme, "Reply-To: %s\r\n", tmp);
 	g_free (tmp);
 
 	/* Organization: */
-	pch = article_get_header (article, HEADER_ORGANIZATION);
+	pch = article_get_extra_header (article, HEADER_ORGANIZATION);
 	tmp = g_mime_utils_8bit_header_encode (pch);
 	if (is_nonempty_string (tmp))
 		g_string_sprintfa (postme, "Organization: %s\r\n", tmp);
@@ -149,16 +150,19 @@ build_nntp_header (GString        * postme,
 	}
 
 	/* References: */
-	pch = article_get_header (article, HEADER_REFERENCES);
+	pch = article->references;
 	if (is_nonempty_string (pch))
 		g_string_sprintfa (postme, "References: %s\r\n", pch);
 
 	/* User Agent: */
-	g_string_append (postme, "User-Agent: Pan/" VERSION " (Unix)\r\n");
+	g_string_sprintfa (postme, "User-Agent: Pan/%s (%s)\r\n", VERSION, VERSION_TITLE);
 
 	/* Add mime header, charset type, encoding type */
+	pch = article_get_extra_header (article, PAN_CHARSET);
+	if (!is_nonempty_string (pch))
+		pch = g_mime_charset_locale_name ();
 	g_string_sprintfa (postme, "Mime-Version: 1.0\r\n");
-	g_string_sprintfa (postme, "Content-Type: text/plain; charset=%s\r\n", g_mime_charset_locale_name());
+	g_string_sprintfa (postme, "Content-Type: text/plain; charset=%s\r\n", pch);
 	g_string_sprintfa (postme, "Content-Transfer-Encoding: 8bit\r\n");
 
 	/* Extra headers: */
@@ -198,21 +202,22 @@ static GPtrArray*
 build_nntp_message (Server * server, Article * article)
 {
 	GPtrArray * bodies;
-	gchar * tmp; /* *append; */
+	char * tmp; /* *append; */
 	GString * body;
 	GString * postme;
+	const char * charset;
 	debug_enter ("build_nntp_message");
 
 	/* sanity checks... */
 	g_return_val_if_fail (article_is_valid(article), NULL);
-	g_return_val_if_fail (article_get_header(article,HEADER_NEWSGROUPS), 0);
+	g_return_val_if_fail (article_get_extra_header(article,HEADER_NEWSGROUPS), 0);
 
        	bodies = g_ptr_array_new();
 #if 0
 	if (article_has_attachment (article))
 	{
-		const gchar * filename = article_get_header (article, PAN_ATTACH_FILE);
-		const gint lines_per_part = strtol (article_get_header (article, PAN_LINES_PER_PART), NULL, 10);
+		const gchar * filename = article_get_extra_header (article, PAN_ATTACH_FILE);
+		const gint lines_per_part = strtol (article_get_extra_header (article, PAN_LINES_PER_PART), NULL, 10);
 		GPtrArray *parts = encode_file (filename, lines_per_part); 
 		int i;
 
@@ -291,6 +296,16 @@ build_nntp_message (Server * server, Article * article)
 
 		/* body */
 		tmp = article_get_body (article);
+		charset = article_get_extra_header (article, PAN_CHARSET);
+		if (is_nonempty_string (charset))
+		{
+			/* FIXME: is this g_convert() even necessary ? */
+	
+			gchar * converted = g_convert (tmp, -1, charset, "UTF-8",  NULL, NULL, NULL);
+
+			if (is_nonempty_string (converted))
+				replace_gstr (&tmp, converted);
+		}
 		g_string_assign (body, tmp);
 		g_free (tmp);
 		pan_g_string_replace (body, "\n.", "\n.."); /* RFC977 3.10.1 ... */
@@ -380,15 +395,12 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 	{
 		GString * text = g_string_new (NULL);
 
-		/*FIXME these should give a subject, too */
-		/* figure out what worked and what didn't. */
 		if (status & SMTP_FAILED)
 			g_string_append (text, _("E-Mail send failed.\n"));
 		if (status & NNTP_FAILED)
 			g_string_append (text, _("Usenet posting failed.\n"));
-
-		/* let the user know that something didn't work. */
 		if (text->len != 0) {
+			g_string_append (text, _("Check Tools|Log Viewer for more information.\n"));
 			g_string_append (text, _("Your message was saved in the folder ``pan.sendlater''"));
 			log_add (LOG_ERROR|LOG_URGENT, text->str);
 		}
@@ -438,8 +450,8 @@ send_article_now (StatusItem           * item,
 	/* initialize variables */
        	article = data->article;
        	subject = article_get_subject (article);
-       	mail_to = data->do_mail ? article_get_header (article, PAN_MAIL_TO) : NULL;
-       	post_to = data->do_post ? article_get_header (article, HEADER_NEWSGROUPS) : NULL;
+       	mail_to = data->do_mail ? article_get_extra_header (article, PAN_MAIL_TO) : NULL;
+       	post_to = data->do_post ? article_get_extra_header (article, HEADER_NEWSGROUPS) : NULL;
 
 	/* mail the message */
 	if (is_nonempty_string(mail_to))
@@ -515,24 +527,12 @@ send_article_now (StatusItem           * item,
 *****
 ****/
 
-gint
-queue_article_for_posting (Server     * server,
-                           Article    * article,
-                           gboolean     check_article_first)
+static void
+queue_article_checked_cb (Server        * server,
+                          Article       * article,
+                          GoodnessLevel   goodness,
+                          gpointer        user_data)
 {
-	GPtrArray * errors;
-	GoodnessLevel goodness = OKAY;
-	debug_enter ("queue_article_for_posting");
-
-	/* sanity check */
-	g_return_val_if_fail (server!=NULL, 1);
-	g_return_val_if_fail (article_is_valid(article), 1);
-
-	/* if desired, make sure the article is okay. */
-       	errors = g_ptr_array_new ();
-	if (check_article_first)
-		check_article_and_prompt (article, server, errors, &goodness);
-
 	/* if the goodness is okay, queue a send task */
 	if (goodness == OKAY)
 	{
@@ -543,27 +543,37 @@ queue_article_for_posting (Server     * server,
 		   be valid when the task runs.  The accompanying unref is
 		   in task_send_dtor... */
 		group_ref_articles (article->group, NULL);
-		group_ref_articles (serverlist_get_named_folder(PAN_SENT), NULL);
+		group_ref_articles (serverlist_get_named_folder(PAN_SENT), NULL); 
 
 		data->article = article;
 		data->original_group = article->group;
-		data->do_post = article_get_header(article,HEADER_NEWSGROUPS) != NULL;
-		data->do_mail = article_get_header(article,PAN_MAIL_TO) != NULL;
+		data->do_post = article_get_extra_header(article,HEADER_NEWSGROUPS) != NULL;
+		data->do_mail = article_get_extra_header(article,PAN_MAIL_TO) != NULL;
 		task = TASK(task_func_ptr_new (server,
 		                               task_send_describe,
 		                               task_send_run,
 		                               task_send_dtor,
 		                               data, TRUE));
 		task->needs_socket = is_nonempty_string(
-			article_get_header(article,HEADER_NEWSGROUPS));
+			article_get_extra_header(article,HEADER_NEWSGROUPS));
 		queue_add (task);
 	}
+}
 
-	/* cleanup */
-	pan_g_ptr_array_foreach (errors, (GFunc)g_free, NULL);
-	g_ptr_array_free (errors, TRUE);
-	debug_exit ("queue_article_for_posting");
-	return goodness==OKAY ? 0 : 1;
+void
+queue_article_for_posting (Server     * server,
+                           Article    * article,
+                           gboolean     check_article_first)
+{
+	/* sanity check */
+	g_return_if_fail (server_is_valid (server));
+	g_return_if_fail (article_is_valid(article));
+
+	/* if desired, make sure the article is okay. */
+	if (check_article_first)
+		check_article_and_prompt (Pan.window, server, article, queue_article_checked_cb, NULL);
+	else
+		queue_article_checked_cb (server, article, OKAY, NULL);
 }
 
 
@@ -590,7 +600,7 @@ flush_sendlater_articles (void)
 		const gchar * h;
 	       
 		a = ARTICLE(g_ptr_array_index(articles,i));
-		h = article_get_header (a, PAN_SERVER);
+		h = article_get_extra_header (a, PAN_SERVER);
 
 		if (is_nonempty_string (h)) {
 			s = serverlist_get_named_server (h);
