@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,8 @@
 
 #include <config.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/pan-i18n.h>
@@ -42,19 +43,17 @@
 *********************/
 
 static void
-newsrc_import_dialog_clicked_cb (GnomeDialog * dialog, gint num, gpointer data)
+newsrc_import_dialog_response_cb (GtkDialog * dialog, int response, gpointer data)
 {
 	Server * server = SERVER(data);
 
 	g_return_if_fail (server != NULL);
 
-	if (num == 0)
+	if (response == GTK_RESPONSE_OK)
 	{
-		GnomeFileEntry * gfe = gtk_object_get_data (
-			GTK_OBJECT(dialog), "file_entry");
-		gchar * filename = gnome_file_entry_get_full_path (gfe, 0);
+		GtkEntry * entry = GTK_ENTRY (gtk_object_get_data (GTK_OBJECT(dialog), "file_entry"));
+		const char * filename = gtk_entry_get_text (entry);
 		newsrc_import (server, filename, FALSE);
-		g_free (filename);
 	}
 
         gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -67,30 +66,29 @@ newsrc_import_dialog (Server * server)
 	GtkWidget * dialog;
 	GtkWidget * w;
 	GtkWidget * file_entry;
-	gchar * s;
+	char * s;
 
 	pan_lock ();
 
-	dialog = gnome_dialog_new (_("Pan: Import .newsrc File"),
-	                           GNOME_STOCK_BUTTON_OK,
-	                           GNOME_STOCK_BUTTON_CANCEL,
-	                           NULL);
+	dialog = gtk_dialog_new_with_buttons (_("Pan: Import .newsrc File"),
+	                                      GTK_WINDOW(Pan.window),
+	                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                      GTK_STOCK_OK, GTK_RESPONSE_OK,
+	                                      NULL);
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD_SMALL);
 
 	s = _("Select .newsrc File to Read:");
 	w = gtk_label_new (s);
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
-	file_entry = gnome_file_entry_new ("newsrc", s);
-	gnome_file_entry_set_default_path (GNOME_FILE_ENTRY(file_entry),
-	                                   g_get_home_dir());
-	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
-	                    GTK_SIGNAL_FUNC (newsrc_import_dialog_clicked_cb),
-	                    server);
+	file_entry = gtk_entry_new ();
+	g_signal_connect (GTK_OBJECT(dialog), "response",
+	                  G_CALLBACK (newsrc_import_dialog_response_cb), server);
 	gtk_box_pack_start (GTK_BOX(hbox), file_entry, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, 0, 0, 0);
 	gtk_object_set_data (GTK_OBJECT(dialog), "file_entry", file_entry);
-	gnome_dialog_set_parent (GNOME_DIALOG(dialog), GTK_WINDOW(Pan.window));
+
 	gtk_widget_show_all (dialog);
 	pan_unlock();
 }
@@ -100,19 +98,17 @@ newsrc_import_dialog (Server * server)
 *****/
 
 static void
-newsrc_export_dialog_clicked_cb (GnomeDialog * dialog, int num, gpointer data)
+newsrc_export_dialog_response_cb (GtkDialog * dialog, int response, gpointer data)
 {
 	Server * server = SERVER(data);
 
 	g_return_if_fail (server != NULL);
 
-	if (num == 0)
+	if (response == GTK_RESPONSE_OK)
 	{
-		GnomeFileEntry * gfe = gtk_object_get_data (
-			GTK_OBJECT(dialog), "file_entry");
-		gchar * filename = gnome_file_entry_get_full_path (gfe, 0);
-		newsrc_export (server, filename, FALSE, TRUE);
-		g_free (filename);
+		GtkEntry * entry = GTK_ENTRY (gtk_object_get_data (GTK_OBJECT(dialog), "file_entry"));
+		const char * filename = gtk_entry_get_text (entry);
+		newsrc_export (server, filename, FALSE);
 	}
 
         gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -124,30 +120,29 @@ newsrc_export_dialog (Server * server)
 	GtkWidget * dialog;
 	GtkWidget * w;
 	GtkWidget * file_entry;
-	gchar * s;
+	char * s;
 
 	pan_lock ();
 
-	dialog = gnome_dialog_new (_("Pan: Export .newsrc File"),
-	                           GNOME_STOCK_BUTTON_OK,
-	                           GNOME_STOCK_BUTTON_CANCEL,
-	                           NULL);
+	dialog = gtk_dialog_new_with_buttons (_("Pan: Export .newsrc File"),
+	                                      GTK_WINDOW(Pan.window),
+	                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                      GTK_STOCK_OK, GTK_RESPONSE_OK,
+	                                      NULL);
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD_SMALL);
 
 	s = _("Select .newsrc File to Write:");
 	w = gtk_label_new (s);
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
-	file_entry = gnome_file_entry_new ("newsrc", s);
-	gnome_file_entry_set_default_path (GNOME_FILE_ENTRY(file_entry),
-	                                   g_get_home_dir());
-	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
-	                    GTK_SIGNAL_FUNC (newsrc_export_dialog_clicked_cb),
-	                    server);
+	file_entry = gtk_entry_new ();
+	pan_gtk_entry_set_text (file_entry, g_get_home_dir());
+	g_signal_connect (GTK_OBJECT(dialog), "response",
+	                  G_CALLBACK(newsrc_export_dialog_response_cb), server);
 	gtk_box_pack_start (GTK_BOX(hbox), file_entry, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, 0, 0, 0);
 	gtk_object_set_data (GTK_OBJECT(dialog), "file_entry", file_entry);
-	gnome_dialog_set_parent (GNOME_DIALOG(dialog), GTK_WINDOW(Pan.window));
 	gtk_widget_show_all (dialog);
 	pan_unlock();
 }
