@@ -41,8 +41,6 @@
 #include <pan/task.h>
 #include <pan/auth-spa.h>
 
-gboolean online_prompt = TRUE;
-
 /** deprecated, use nntp_command */
 extern const char * sockwrite_err_msg;
 extern const char * sockread_err_msg;
@@ -104,7 +102,6 @@ nntp_command (StatusItem           * status,
 {
 	int val;
 	size_t len;
-	debug_enter ("nntp_command");
 
 	g_return_val_if_fail (sock!=NULL, -1);
 	g_return_val_if_fail (is_nonempty_string(command), -1);
@@ -194,6 +191,14 @@ nntp_disconnect (StatusItem   * status,
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
+	/* Flush the socket. This ensures any remaining data (like TCP closing 
+	 * packets) are read between sending the NNTP command (which makes the 
+	 * server close the socket), and us closing our end of the socket.
+	 * This prevents the socket going into TIME_WAIT state (or worse, 
+	 * FIN_WAIT2)
+	 */
+	pan_socket_flush (socket);
+
 	/* disconnected successfully */
 	debug_exit ("nntp_disconnect");
 	return TASK_SUCCESS;
@@ -579,13 +584,17 @@ init_header_if_match (Article       * a,
 	if (!strncmp (token, token_key, token_key_len))
 	{
 		const gchar * val = token + token_key_len;
-		if (!pan_header_is_8bit_encoded (val))
-			article_init_header (a, header, val, action);
-		else {
-			gchar * b = g_mime_utils_8bit_header_decode ((const guchar*)val);
-			article_init_header (a, header, b, action);
-			g_free (b);
-		}
+		const gchar * charset = NULL;
+		gchar       * cvt = NULL;
+
+		charset = article_get_extra_header (a, PAN_CHARSET);
+		if (!is_nonempty_string (charset))
+			charset = group_get_default_charset (a->group);
+		
+		cvt = pan_header_to_utf8 (val, -1, charset);
+		article_init_header (a, header, cvt, action);
+		g_free (cvt);
+		
 		match = TRUE;
 	}
 
@@ -701,7 +710,7 @@ nntp_article_download_to_gstr (StatusItem      * status,
 					{ "Followup-To: ", 13, HEADER_FOLLOWUP_TO, DO_CHUNK_SHARE },
 					{ "Xref: ", 6, HEADER_XREF, DO_CHUNK },
 				};
-				static gint TOKEN_QTY = sizeof(tokens) / sizeof(tokens[0]);
+				static gint TOKEN_QTY = G_N_ELEMENTS (tokens);
 
 				/* trim linefeed */
 				g_strstrip (line->str);
@@ -721,7 +730,12 @@ nntp_article_download_to_gstr (StatusItem      * status,
 
 	/* save the message */
 	if (retval == TASK_SUCCESS) {
-        	acache_set_message (article_get_message_id(article), setme->str, setme->len+1, checkout);
+		const char * default_charset = group_get_default_charset (group);
+        	acache_set_message (article_get_message_id(article),
+		                    setme->str,
+		                    setme->len,
+		                    checkout,
+		                    default_charset);
 		articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
 	}
 
@@ -877,6 +891,7 @@ nntp_download_headers (StatusItem       * status,
 		const gchar * march = NULL;
 		const gchar * tok = NULL;
 		int tok_len = 0;
+		const gchar * charset = group_get_default_charset (group);
 
 		/* handle end-of-list */
 		if (!strncmp(buffer, ".\r\n", 3))
@@ -895,21 +910,21 @@ nntp_download_headers (StatusItem       * status,
 
 		/* get subject */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
-			if (!pan_header_is_8bit_encoded (tok))
-				article_init_header (a, HEADER_SUBJECT, tok, DO_CHUNK_SHARE);
-			else {
-				gchar * dec = g_mime_utils_8bit_header_decode ((const guchar*)tok);
-				article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
-				g_free (dec);
-			}
+			gchar * cvt;
+		      	((gchar*)tok)[tok_len] = '\0';
+			cvt = pan_header_to_utf8 (tok, tok_len, charset);
+			article_init_header (a, HEADER_SUBJECT, cvt, DO_CHUNK_SHARE);
+			g_free (cvt);
 		}
 
 		/* get author */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
-			if (is_nonempty_string(tok))
-				article_init_author_from_header (a, tok);
+			if (is_nonempty_string(tok)) {
+				gchar * cvt = pan_header_to_utf8 (tok, tok_len, charset);
+				article_init_author_from_header (a, cvt);
+				g_free (cvt);
+			}
 		}
 
 		/* get date */
@@ -944,17 +959,18 @@ nntp_download_headers (StatusItem       * status,
 
 		/* get crossref */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
-			if (tok_len==0 || strncmp(tok,"Xref: ", 6))
-				a->crosspost_qty = 1;
-			else {
-				const gchar * foo;
-				g_strstrip ((gchar*)tok); /* remove trailing linefeed */
-				a->crosspost_qty = 0;
-				for (foo=tok+6; *foo; ++foo)
-					if (*foo==':')
-						++a->crosspost_qty;
-				article_init_header (a, HEADER_XREF, tok+6, DO_CHUNK);
+			((char*)tok)[tok_len] = '\0';
+			if (tok_len>6 && !strncmp(tok,"Xref: ", 6)) {
+				const char * header = tok + 6;
+				const char * pch;
+				int crosspost_qty = 0;
+				for (pch=header; *pch; ++pch)
+					if (*pch==':')
+						++crosspost_qty;
+				if (crosspost_qty > 1) {
+					g_strstrip ((char*)header); /* remove trailing linefeed */
+					article_init_header (a, HEADER_XREF, header, DO_CHUNK);
+				}
 			}
 		}
 
@@ -1037,8 +1053,8 @@ nntp_download_headers (StatusItem       * status,
 
 		/* update the count & progress ui */
 		status_item_emit_next_step (status);
-		if (!(setme->len % 57))
-			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low));
+		if (!(setme->len % 100))
+			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low+1));
 	}
 	g_string_free (buf, TRUE);
 
@@ -1088,7 +1104,7 @@ nntp_download_bodies (StatusItem       * status,
 		if (qty > 0) {
 			articles_remove_flag ((Article**)articles->pdata+*index, qty, STATE_DOWNLOAD_FLAGGED);
 			if (checkout) {
-				const gchar ** message_ids = alloca (sizeof(gchar*) * qty);
+				const gchar ** message_ids = g_alloca (sizeof(gchar*) * qty);
 				for (i=*index; i<articles->len; ++i)
 					message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(articles,i)));
 				acache_file_checkout (message_ids, qty);
@@ -1189,7 +1205,7 @@ nntp_cancel (StatusItem     * status,
 	g_return_val_if_fail (is_nonempty_string(newsgroups), TASK_FAIL_HOPELESS);
 
 	/* let the user know what we're doing */
-	status_item_emit_status_va (status, _("Cancelling message \"%s\""), msg_id);
+	status_item_emit_status_va (status, _("Canceling message \"%s\""), msg_id);
 
 	/* build the message to post */
 	msg = g_string_new (NULL);
