@@ -19,22 +19,30 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <string.h>
+
+#include <glib.h>
+#include <gtk/gtk.h>
+
+#include <locale.h>
 
 #include <pan/base/acache.h>
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
 #include <pan/base/newsrc-port.h>
 #include <pan/base/pan-config.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/serverlist.h>
 #include <pan/base/util-file.h>
-#include <pan/filters/filter-manager.h>
+
 #include <pan/rules/rule-manager.h>
 #include <pan/identities/identity-manager.h>
 #include <pan/dialogs/dialogs.h>
 
-#include <gmime/gmime-charset.h>
+#include <gmime/gmime.h>
 
+#include <pan/group-action.h>
 #include <pan/globals.h>
 #include <pan/gui.h>
 #include <pan/pan.h>
@@ -43,206 +51,12 @@
 #include <pan/thread-watch.h>
 #include <pan/util.h>
 
-static gint save_session (GnomeClient        * client,
-			  gint                 phase,
-			  GnomeSaveStyle       save_style,
-			  gint                 is_shutdown,
-			  GnomeInteractStyle   interact_style,
-			  gint                 is_fast,
-			  gpointer             client_data);
-
-static void session_die  (GnomeClient        * client,
-                          gpointer             client_data);
-
-const gchar* sockread_err_msg = NULL;
-const gchar* sockwrite_err_msg = NULL;
-
-PanApp Pan;
-
-static gchar     *geometry = NULL;
-static gint       debug_acache = 0;
-static gint       debug_socket = 0;
-static gint       debug_socket_input = 0;
-static gint       debug_socket_output = 0;
-static gint       debug_queue = 0;
-static gint       debug_gnksa = 0;
-static gint       debug_newsrc = 0;
-static gint       debug_trace = 0;
-static gint       debug_decode = 0;
-static gint       debug_gui_locks = 0;
-static gint       debug_pan_object = 0;
-static gint       debug_defaults = 0;
-static gint       mute = 0;
-
-struct poptOption options[] = {
-	{ "geometry", '\0', POPT_ARG_STRING, &geometry, 0,
-	  N_("Specify the geometry of the window."),
-	  N_("GEOMETRY") },
-	{ "mute", '\0', 0, &mute, 0,
-	  N_("Send outgoing messages to stdout instead of smtp/nntp servers for debugging purposes."), NULL },
-	{ "debug-cache", '\0', 0, &debug_acache, 0,
-	  N_("Turn on article cache debugging messages."), NULL },
-	{ "debug-socket-input", '\0', 0, &debug_socket_input, 0,
-	  N_("Turn on socket input debugging messages."), NULL },
-	{ "debug-socket-output", '\0', 0, &debug_socket_output, 0,
-	  N_("Turn on socket output debugging messages."), NULL },
-	{ "debug-socket", '\0', 0, &debug_socket, 0,
-	  N_("Turn on socket input/output debugging messages."), NULL },
-	{ "debug-sockets", '\0', 0, &debug_socket, 0,
-	  N_("Same as --debug-socket"), NULL },
-	{ "debug-trace", '\0', 0, &debug_trace, 0,
-	  N_("Trace function calls"), NULL },
-	{ "debug-queue", '\0', 0, &debug_queue, 0,
-	  N_("Turn on task scheduler debugging messages."), NULL },
-	{ "debug-decode", '\0', 0, &debug_decode, 0,
-	  N_("Turn on attachment decoding debugging messages."), NULL },
-	{ "debug-newsrc", '\0', 0, &debug_newsrc, 0,
-	  N_("Turn on newsrc decoding debugging messages."), NULL },
-	{ "debug-gui-locks", '\0', 0, &debug_gui_locks, 0,
-	  N_("Turn on gui thread lock debugging messages."), NULL },
-	{ "debug-gnksa", '\0', 0, &debug_gnksa, 0,
-	  N_("Turn on debugging of GNKSA code."), NULL },
-	{ "debug-pan-object", '\0', 0, &debug_pan_object, 0,
-	  N_("Turn on pan object system debugging messages."), NULL },
-	{ "debug", '\0', 0, &debug_defaults, 0,
-	  N_("Turn on most of the generally-useful debug flags.  A good first choice when preparing a bug report."), NULL },
-
-	{ NULL, '\0', 0, NULL, 0 }
-};
-
-/*---[ main ]---------------------------------------------------------
- * _the_ main function!
- *--------------------------------------------------------------------*/
-int
-main (int argc, char *argv[])
-{
-	GnomeClient * client;
-	gulong debug_flags = 0;
-        poptContext pctx;
-
-	gtk_set_locale ();
-
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
-
-	sockread_err_msg = _("Error reading from socket.");
-	sockwrite_err_msg = _("Error writing to socket.");
-	PAN_SENT = _("pan.sent");
-	PAN_SENDLATER = _("pan.sendlater");
-
-	g_thread_init (NULL);
-	gnome_init_with_popt_table ("Pan", VERSION, argc, argv, options, 0, &pctx);
-	poptFreeContext (pctx);
-	g_mime_charset_init ();
-
-#if 0 /* for rpm */
-#if GNOME_LIBS_VERSION>=60
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR "/pan.png");
-#endif
-#endif
-
-	debug_flags = (debug_acache ? DEBUG_ACACHE : 0)
-		| (debug_socket||debug_socket_input ? DEBUG_SOCKET_INPUT : 0)
-		| (debug_socket||debug_socket_output ? DEBUG_SOCKET_OUTPUT : 0)
-		| (debug_queue ? DEBUG_QUEUE : 0)
-		| (debug_decode ? DEBUG_DECODE : 0)
-		| (debug_newsrc ? DEBUG_NEWSRC : 0)
-		| (debug_gnksa ? DEBUG_GNKSA : 0)
-		| (debug_trace ? DEBUG_TRACE : 0)
-		| (debug_gui_locks ? DEBUG_LOCK : 0)
-		| (debug_pan_object ? DEBUG_PAN_OBJECT : 0)
-		| (debug_defaults ? (DEBUG_TRACE|DEBUG_DECODE|DEBUG_ACACHE|DEBUG_SOCKET_INPUT|DEBUG_SOCKET_OUTPUT): 0);
-	set_debug_level (debug_flags);
-
-
-	log_add_va (LOG_INFO, _("Pan %s Started"), VERSION);
-
-	cmap = gdk_colormap_get_system ();
-	prefs_init ();
-	filter_manager_init ();
-	serverlist_init ();
-
-	/* Connect up to the session manager */
-	client = gnome_master_client ();
-	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
-			    GTK_SIGNAL_FUNC (save_session), argv[0]);
-	gtk_signal_connect (GTK_OBJECT (client), "die",
-			    GTK_SIGNAL_FUNC (session_die), NULL);
-
-	/* initialize variables in the global app structure */
-	Pan.main_t = pthread_self ();
-	pan_mute = mute;
-	if (pan_mute)
-		g_message ("Running in mute mode: messages will be sent to "
-		           "stdout instead of smtp/nntp server");
-
-	queue_init (pan_config_get_int("/Pan/General/Max_Connections_Total=4"),
-	            pan_config_get_int("/Pan/General/Max_Task_Tries_Total=4"));
-
-	/* if it's a new user, pop up the newuser dialog.
-	   Otherwise show the main window. */
-
-	if (identity_manager_count () == 0)
-	{
-		dialog_newuser ();
-	}
-	else
-	{
-		gui_construct (geometry);
-		gtk_widget_show_all (Pan.window);
-
-		if (newsrc_do_port) {
-			Server * s = serverlist_get_active_server ();
-			if (s != NULL)
-				newsrc_import (s, newsrc_filename, newsrc_port_subscribed_only);
-		}
-	}
-
-	gtk_main ();
-
-	return 0;
-}
-
-/**
- * pan_shutdown:
- *
- * Shutdown Pan, closing all TCP connections and saving all preferences
- **/
-void
-pan_shutdown (void)
-{
-
-	/* FIXME do we save tasks? */
-	queue_shutdown (FALSE);
-
-	if (newsrc_do_port) {
-		Server * s = serverlist_get_active_server();
-		if (s!=NULL) {
-			newsrc_export (s, newsrc_filename,
-			                  newsrc_port_subscribed_only,
-			                  newsrc_do_backup_before_export);
-		}
-	}
-
-	g_free (message_body_font);
-	g_free (thread_normal_font);
-	g_free (thread_new_replies_font);
-	g_free (grouplist_font);
-
-	identity_manager_shutdown_module ();
-	serverlist_shutdown ();
-	acache_shutdown ();
-	thread_shutdown_module ();
-	rule_manager_shutdown_module ();
-	log_shutdown_module ();
-
-	pan_object_bookkeeping ();
-
-	gtk_main_quit ();
-	pan_clean_temp_dir ();
-}
+extern char *
+bind_textdomain_codeset (const char * domainname,
+                         const char * codeset);
 
 
+#ifdef GNOME
 /***
 ****   Session management
 ***/
@@ -251,7 +65,6 @@ pan_shutdown (void)
 ** save_session
 ** saves the current session for later revival
 */
-
 static gint
 save_session (GnomeClient         * client,
               gint                  phase,
@@ -277,10 +90,209 @@ save_session (GnomeClient         * client,
 /**
 *** session_die: Gracefullly end the session
 **/
-
 static void
 session_die (GnomeClient *    client,
              gpointer         client_data)
 {
 	pan_shutdown ();
 }
+#endif
+
+static void
+show_help (void)
+{
+	const char * help = _(
+		"Pan " VERSION "\n"
+		"\n"
+		"  --version        Prints Pan's version number\n"
+		"  --help           Prints this message and exits\n"
+		"  --debug          Turns on the most commonly-useful options for a bug report\n"
+		"\n"
+		"  --mute           Debugging tool to send new posts to console, not the server\n"
+		"  --debug-cache    Print debugging messages for the article cache\n"
+		"  --debug-sockets  Print debugging messages when talking to the news server\n"
+		"  --debug-queue    Print debugging messages related to managing tasks\n"
+		"  --debug-decode   Print debugging messages related to decoding attachments\n"
+		"  --debug-newsrc   Print debugging messages related to .newsrc files\n"
+		"  --debug-gnksa    Print debugging messages related to gnksa correctness\n"
+		"  --debug-trace    Print debugging messages when entering/leaving functions\n"
+		"  --debug-lock     Print debugging messages related to threaded gui locks\n"
+		"  --debug-object   Print debugging messages related to refcounted objects\n"
+		"\n");
+	fprintf (stderr, "%s", help);
+}
+
+const gchar* sockread_err_msg = NULL;
+const gchar* sockwrite_err_msg = NULL;
+
+PanApp Pan;
+
+/*---[ main ]---------------------------------------------------------
+ * _the_ main function!
+ *--------------------------------------------------------------------*/
+int
+main (int argc, char *argv[])
+{
+	int i;
+	gulong debug_flags = 0;
+	gboolean mute = FALSE;
+
+	/* init the text domain & codeset,
+	   then force all of the locale segments to update from the
+	   environment.  Unless we do this they will all default to C. */
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+	setlocale (LC_ALL, "");
+
+	/* set some globals */
+	sockread_err_msg = _("Error reading from socket.");
+	sockwrite_err_msg = _("Error writing to socket.");
+	PAN_SENT = _("pan.sent");
+	PAN_SENDLATER = _("pan.sendlater");
+
+	/* init gtk */
+	g_set_prgname ("Pan");
+	gtk_set_locale ();
+	g_thread_init (NULL);
+	gdk_threads_init ();
+	gtk_init (&argc, &argv);
+	g_mime_init (GMIME_INIT_FLAG_UTF8);
+
+	/* parse arguments  */
+	mute = FALSE;
+	debug_flags = 0;
+	for (i=0; i<argc; ++i)
+	{
+		const char * arg = argv[i];
+
+		if (!strcmp(arg,"--mute"))
+			mute = TRUE;
+		else if (!strcmp (arg, "--debug-cache"))
+			debug_flags |= DEBUG_ACACHE;
+		else if (!strcmp (arg, "--debug-socket-input"))
+			debug_flags |= DEBUG_SOCKET_INPUT;
+		else if (!strcmp (arg, "--debug-socket-output"))
+			debug_flags |= DEBUG_SOCKET_OUTPUT;
+		else if (!strcmp (arg, "--debug-socket"))
+			debug_flags |= DEBUG_SOCKET_INPUT|DEBUG_SOCKET_OUTPUT;
+		else if (!strcmp (arg, "--debug-sockets"))
+			debug_flags |= DEBUG_SOCKET_INPUT|DEBUG_SOCKET_OUTPUT;
+		else if (!strcmp (arg, "--debug-queue"))
+			debug_flags |= DEBUG_QUEUE;
+		else if (!strcmp (arg, "--debug-decode"))
+			debug_flags |= DEBUG_DECODE;
+		else if (!strcmp (arg, "--debug-newsrc"))
+			debug_flags |= DEBUG_NEWSRC;
+		else if (!strcmp (arg, "--debug-gnksa"))
+			debug_flags |= DEBUG_GNKSA;
+		else if (!strcmp (arg, "--debug-trace"))
+			debug_flags |= DEBUG_TRACE;
+		else if (!strcmp (arg, "--debug-lock"))
+			debug_flags |= DEBUG_LOCK;
+		else if (!strcmp (arg, "--debug-object"))
+			debug_flags |= DEBUG_PAN_OBJECT;
+		else if (!strcmp (arg, "--debug"))
+			debug_flags |= DEBUG_TRACE|DEBUG_DECODE|DEBUG_ACACHE|DEBUG_SOCKET_INPUT|DEBUG_SOCKET_OUTPUT;
+		else if (!strcmp (arg, "--version")) {
+			printf ("Pan %s\n", VERSION);
+			return 0;
+		}
+		else if (!strcmp (arg, "--help") || !strcmp(arg, "-help")) {
+			show_help ();
+			return 0;
+		}
+	}
+	set_debug_level (debug_flags);
+	log_add_va (LOG_INFO, _("Pan %s Started"), VERSION);
+
+	cmap = gdk_colormap_get_system ();
+	prefs_init ();
+	serverlist_init ();
+
+	/* Connect up to the session manager */
+#ifdef GNOME
+	if (1) {
+		GnomeClient * client = gnome_master_client ();
+		g_signal_connect (client, "save_yourself",
+		                  G_CALLBACK(save_session), argv[0]);
+		g_signal_connect (client, "die",
+		                  G_CALLBACK(session_die), NULL);
+	}
+#endif
+
+	/* initialize variables in the global app structure */
+	Pan.main_t = g_thread_self ();
+	pan_mute = mute;
+	if (pan_mute)
+		g_message ("Running in mute mode: messages will be sent to "
+		           "stdout instead of smtp/nntp server");
+
+	queue_init (pan_config_get_int("/Pan/General/Max_Connections_Total=4"),
+	            pan_config_get_int("/Pan/General/Max_Task_Tries_Total=4"));
+
+	/* if it's a new user, pop up the newuser dialog.
+	   Otherwise show the main window. */
+
+	if (identity_manager_count () == 0)
+	{
+		dialog_newuser ();
+	}
+	else
+	{
+		gui_construct (NULL);
+		gtk_widget_show_all (Pan.window);
+
+		if (newsrc_do_port) {
+			Server * s = serverlist_get_active_server ();
+			if (s != NULL)
+				newsrc_import (s, newsrc_filename, newsrc_port_subscribed_only);
+		}
+	}
+
+	if (fetch_new_and_bodies_on_startup)
+		group_action_subscribed_download_new_and_bodies ();
+	else if (fetch_new_on_startup)
+		group_action_subscribed_download_new ();
+
+	gtk_main ();
+
+	return 0;
+}
+
+/**
+ * pan_shutdown:
+ *
+ * Shutdown Pan, closing all TCP connections and saving all preferences
+ **/
+void
+pan_shutdown (void)
+{
+
+	queue_shutdown ();
+
+	if (newsrc_do_port) {
+		Server * s = serverlist_get_active_server();
+		if (s!=NULL) {
+			newsrc_export (s, newsrc_filename,
+			                  newsrc_port_subscribed_only);
+		}
+	}
+
+	g_free (message_body_font);
+	g_free (thread_normal_font);
+	g_free (thread_new_replies_font);
+	g_free (grouplist_font);
+
+	identity_manager_shutdown_module ();
+	serverlist_shutdown ();
+	acache_shutdown ();
+	thread_shutdown_module ();
+	rule_manager_shutdown_module ();
+	log_shutdown_module ();
+
+	pan_object_bookkeeping ();
+
+	gtk_main_quit ();
+}
+
