@@ -25,42 +25,21 @@
 *** Global Variables
 **/
 
-typedef enum
-{
-	ONLINE_PROMPT	= 0,
-	ONLINE_YES	= 1,
-	ONLINE_NO	= 2
-}
-GoingOnlinePreference;
-
-typedef enum
-{
-	CLICK_ACTION_SELECT,
-	CLICK_ACTION_LOAD
-}
-ListClickAction;
-
-extern GoingOnlinePreference   going_online_preference;
-extern ListClickAction         grouplist_mb1_action;
-extern ListClickAction         grouplist_mb2_action;
-extern ListClickAction         articlelist_mb1_action;
-extern ListClickAction         articlelist_mb2_action;
+extern gboolean                remove_failed_tasks;
+extern gboolean                single_click_selects;
 extern gboolean                header_pane_is_threaded;
 extern gboolean                show_group_pane;
 extern gboolean                show_header_pane;
-extern gboolean                show_article_pane;
-extern gboolean                download_bodies_too;
-extern gboolean                navigate_cached_only;
-extern gboolean                navigate_read_on_select;
+extern gboolean                show_body_pane;
 extern gboolean                text_use_fixed_font;
 extern gboolean                text_window_smooth_scrolling;
 extern gint                    text_window_smooth_scrolling_speed;
+extern gboolean                fetch_new_on_startup;
+extern gboolean                fetch_new_and_bodies_on_startup;
 extern gboolean                fetch_new_on_group_load;
-extern gboolean                display_article_dates_in_local_time;
+extern gboolean                expand_all_threads_by_default;
 extern gboolean                hide_mpart_child_nodes;
-extern gboolean                auto_part_selection;
 extern gboolean                pan_mute;
-extern gboolean                use_gdk_fontset_load;
 extern gchar                 * thread_date_format;
 extern gchar                 * body_date_format;
 extern gchar                 * grouplist_font;
@@ -70,13 +49,10 @@ extern gchar                 * message_body_font;
 extern gchar                 * message_body_font_fixed;
 extern gchar                 * external_web_browser;
 extern gchar                 * external_editor;
-extern gint                    watched_thread_span_days;
-extern gchar                 * extra_posting_headers;
 extern gchar                 * mail_server_address;
 extern gchar                 * layout_str;
 extern gint                    mail_server_port;
 extern gint                    wrap_column;
-extern gboolean                wrap_body_on_reply;
 extern gboolean                collapse_group_names;
 extern gboolean                use_system_fg;
 extern gboolean                use_system_bg;
@@ -84,7 +60,7 @@ extern gboolean                use_system_bg;
 extern gchar *                 newsrc_filename;
 extern gboolean                newsrc_do_port;
 extern gboolean                newsrc_port_subscribed_only;
-extern gboolean                newsrc_do_backup_before_export;
+
 
 
 /** 
