@@ -23,9 +23,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/article.h>
+#include <pan/base/pan-config.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/globals.h>
@@ -39,7 +42,7 @@
 /* private structure for a print dialog */
 typedef struct
 {
-	GtkWidget  * window;           /* Gnomedialog */
+	GtkWidget  * window;           /* GtkDialog */
 	GtkWidget  * print_command;    /* entry box, should likely be 'lpr' */
 	Article    * article;
 }
@@ -49,7 +52,6 @@ PrintDialog;
 static void print_dialog_print_clicked (PrintDialog *dialog);
 static void print_dialog_destroy (PrintDialog *dialog);
 static gint print_execute (const gchar *command, const gchar *message);
-static void print_dialog_cb (GtkWidget *widget, gint num, gpointer data);
 static void print_dialog (Article * article);
 
 
@@ -79,7 +81,7 @@ print_dialog_destroy (PrintDialog *dialog)
 {
 	gtk_widget_destroy (dialog->window);
 
-	pan_object_unref (PAN_OBJECT(dialog->article));
+	group_unref_articles (dialog->article->group, NULL);
 	g_free (dialog);
 }
 
@@ -114,7 +116,7 @@ print_dialog_print_clicked (PrintDialog * dialog)
 			"Printing of message \"%s\" failed!",
 			article_get_subject(article));
 	else
-		gnome_config_set_string ("/Pan/Print/print_command", command);
+		pan_config_set_string ("/Pan/Print/print_command", command);
 
 	/* clean up */
 	print_dialog_destroy (dialog);
@@ -125,18 +127,16 @@ print_dialog_print_clicked (PrintDialog * dialog)
 
 /* The user pressed a button. Lets like, do things and stuff. Dude. */
 static void
-print_dialog_cb (GtkWidget * widget_notused,
-                 gint         num,
-		 gpointer     user_data)
+print_dialog_cb (GtkDialog * dialog, int response, gpointer user_data)
 {
 	g_return_if_fail (user_data);
 
-	switch (num)
+	switch (response)
 	{
-		case 0: /* Print */
+		case GTK_RESPONSE_OK: /* Print */
 			print_dialog_print_clicked (user_data);
 			break;
-		case 1: /* Cancel */
+		case GTK_RESPONSE_CANCEL: /* Cancel */
 			print_dialog_destroy (user_data);
 			break;
 		default:
@@ -158,20 +158,23 @@ print_dialog (Article * article)
 	g_return_if_fail (article != NULL);
 
 	/* make sure the Article stays alive during this dialog */
-	pan_object_ref (PAN_OBJECT(article));
+	group_ref_articles (article->group, NULL);
 
 	/* dialog object */
 	dialog = g_new0 (PrintDialog, 1);
 	dialog->article = article;
-	dialog->window = gnome_dialog_new (_("Pan: Print"),
-					   _("Print"),
-					   GNOME_STOCK_BUTTON_CANCEL, NULL);
+	dialog->window = gtk_dialog_new_with_buttons (_("Pan: Print"),
+	                                              GTK_WINDOW(Pan.window),
+	                                              GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                              GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                              GTK_STOCK_PRINT, GTK_RESPONSE_OK,
+						      NULL);
 
 	/* dialog table */
 	table = gtk_table_new (1, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD_SMALL);
-        gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
-        gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(table), GUI_PAD_SMALL);
+        gtk_table_set_row_spacings (GTK_TABLE(table), GUI_PAD_SMALL);
+        gtk_table_set_col_spacings (GTK_TABLE(table), GUI_PAD_SMALL);
 
 	/* print command label */
 	label = gtk_label_new (_("Print Command:"));
@@ -184,10 +187,10 @@ print_dialog (Article * article)
 
 	/* print command edit field */
 	dialog->print_command = gtk_entry_new ();
-	p = gnome_config_get_string ("/Pan/Print/print_command");
+	p = pan_config_get_string ("/Pan/Print/print_command");
 	if (p == NULL)
 		p = g_strdup ("lpr");
-	gtk_entry_set_text (GTK_ENTRY(dialog->print_command), p);
+	pan_gtk_entry_set_text (dialog->print_command, p);
 	g_free (p);
 	gtk_table_attach (GTK_TABLE(table),
 	                  dialog->print_command,
@@ -196,16 +199,11 @@ print_dialog (Article * article)
 	                  4, 0);
 
 	/* add the main dialog table to the dialog */
-	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog->window)->vbox),
+	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog->window)->vbox),
 	                   table);
-	gtk_signal_connect (GTK_OBJECT(dialog->window),
-	                    "clicked",
-	                    GTK_SIGNAL_FUNC(print_dialog_cb),
-	                    dialog);
+	g_signal_connect (GTK_OBJECT(dialog->window), "response",
+	                  G_CALLBACK(print_dialog_cb), dialog);
 
-	/* show the dialog */
-	gnome_dialog_set_parent (GNOME_DIALOG(dialog->window),
-	                         GTK_WINDOW(Pan.window));
 	gtk_widget_show_all (dialog->window);
 }
 
@@ -241,7 +239,6 @@ print_cb (GtkWidget * widget_notused,
 }
 
 #ifdef HAVE_GNOME_PRINT
-
 void
 pan_gnome_print_cb (void)
 {
@@ -255,5 +252,4 @@ pan_gnome_print_preview_cb (void)
 	/* FIXME(csk): gnome-print stuff goes here */
 	print_cb (NULL, NULL);
 }
-
 #endif
