@@ -2,7 +2,7 @@
 /*
  * Author: Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,12 @@
 
 #include <config.h>
 
-#include <pthread.h>
-#include <sys/stat.h>
-#include <sys/time.h>
 #include <errno.h>
-#include <unistd.h>
 
 #include <glib.h>
 
+#include <unistd.h>
+
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
@@ -66,7 +64,7 @@ typedef enum
 	TODO_SET_MAX_SOCKET_QTY,
 	TODO_SET_MAX_TRIES,
 	TODO_SHUTDOWN,
-	TODO_PAUSED
+	TODO_ONLINE
 }
 QueueTodoAction;
 
@@ -80,9 +78,8 @@ QueueTodoAction;
 
 typedef struct
 {
-	Task *task;
+	GSList * tasks;
 	int index_1;
-	int index_2;
 	QueueTodoAction action;
 }
 QueueTodo;
@@ -99,6 +96,10 @@ SocketInfo;
 **********************  Private Function Prototypes
 *********************/
 
+static void fire_online_status_changed (gboolean online);
+
+static void queue_run_thread (gpointer data, gpointer user_data);
+
 static void* queue_mainloop (void*);
 
 static void queue_run_what_we_can (void);
@@ -108,10 +109,9 @@ static void queue_set_task_status (Task* task,
 
 static void socket_cleanup (Task* task);
 
-static void queue_new_todo (QueueTodoAction action,
-                            Task* task,
-                            int i1,
-                            int i2);
+static void queue_new_todo (QueueTodoAction     action,
+                            GSList            * tasks,
+                            int                 i);
 
 static guint queue_get_length (void);
 
@@ -145,11 +145,11 @@ static void queue_do_todo (void);
 ************  Public
 ***********/
 
-PanCallback * queue_task_added           = NULL;
-PanCallback * queue_task_removed         = NULL;
-PanCallback * queue_task_moved           = NULL;
-PanCallback * queue_task_status_changed  = NULL;
-PanCallback * queue_max_tries_changed    = NULL;
+PanCallback * queue_tasks_added           = NULL;
+PanCallback * queue_tasks_removed         = NULL;
+PanCallback * queue_tasks_moved           = NULL;
+PanCallback * queue_task_status_changed   = NULL;
+PanCallback * queue_max_tries_changed     = NULL;
 
 /***********
 ************  Private
@@ -157,17 +157,18 @@ PanCallback * queue_max_tries_changed    = NULL;
 
 
 /* FIXME glib 2.0: for efficiency of adding to end, use a GQueue instead of GSList */
-static GSList* todo_list = NULL;
+static GQueue * todo_queue = NULL;
 static GSList* task_list = NULL;
 
-static pthread_mutex_t sock_lock;
-static pthread_mutex_t cond_mutex;
-static pthread_mutex_t todo_mutex;
-static pthread_mutex_t task_lock;
-static pthread_cond_t qcond = PTHREAD_COND_INITIALIZER;
-static gboolean work_to_do = FALSE;
+static GThreadPool * _tpool = NULL;
 
-static pthread_t queue_thread = 0;
+static GMutex * sock_lock = NULL;
+static GMutex * task_lock = NULL;
+static GMutex * cond_mutex = NULL;
+static GMutex * todo_mutex = NULL;
+
+static GCond * qcond = NULL;
+static gboolean work_to_do = FALSE;
 
 static GHashTable* task_to_status = NULL;
 
@@ -179,7 +180,7 @@ static int max_tries = 5;
 
 static guint running_tasks = 0;
 
-static gboolean paused = FALSE;
+static gboolean _online = TRUE;
 
 /*********************
 **********************  BEGINNING OF SOURCE
@@ -192,20 +193,22 @@ static gboolean paused = FALSE;
 void
 queue_wakeup (void)
 {
+	/*gboolean do_wakeup;*/
 	debug_enter ("queue_wakeup");
-        pthread_mutex_lock (&cond_mutex);
-	work_to_do = TRUE;
-        pthread_cond_signal (&qcond);
-        pthread_mutex_unlock (&cond_mutex);
-	debug_exit ("queue_wakeup");
-}
 
-static void
-server_online_status_changed_cb (gpointer obj, gpointer arg, gpointer data)
-{
-	debug_enter ("server_online_status_changed_cb");
-	queue_wakeup ();
-	debug_exit ("server_online_status_changed_cb");
+	/*g_mutex_lock (todo_mutex);
+	do_wakeup = !g_queue_is_empty (todo_queue);
+	g_mutex_unlock (todo_mutex);*/
+
+	if (TRUE /*do_wakeup*/)
+	{
+		g_mutex_lock (cond_mutex);
+		work_to_do = TRUE;
+		g_cond_signal (qcond);
+		g_mutex_unlock (cond_mutex);
+	}
+
+	debug_exit ("queue_wakeup");
 }
 
 void
@@ -215,80 +218,87 @@ queue_init (int socket_qty, int tries)
 
 	task_to_status = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	queue_task_added = pan_callback_new ();
-	queue_task_removed = pan_callback_new ();
-	queue_task_moved = pan_callback_new ();
+	todo_queue = g_queue_new ();
+
+	queue_tasks_added = pan_callback_new ();
+	queue_tasks_removed = pan_callback_new ();
+	queue_tasks_moved = pan_callback_new ();
 	queue_task_status_changed = pan_callback_new ();
 	queue_max_tries_changed = pan_callback_new ();
 
-	pthread_mutex_init (&cond_mutex, NULL);
-	pthread_mutex_init (&todo_mutex, NULL);
-	pthread_mutex_init (&sock_lock, NULL);
-	pthread_mutex_init (&task_lock, NULL);
+	sock_lock = g_mutex_new ();
+	task_lock = g_mutex_new ();
+	cond_mutex = g_mutex_new ();
+	todo_mutex = g_mutex_new ();
 
-	pan_callback_add (server_get_online_status_changed_callback(),
-	                  server_online_status_changed_cb, NULL);
+	qcond = g_cond_new ();
 
 	max_tries = tries;
 	max_socket_qty = socket_qty;
+	_tpool = g_thread_pool_new (queue_run_thread, NULL, max_socket_qty, TRUE, NULL);
 
 	/* fire up the queue thread */
-	pthread_create (&queue_thread, NULL, queue_mainloop, NULL);
-	pthread_detach (queue_thread);
+	g_thread_create (queue_mainloop, NULL, FALSE, NULL);
 
 	debug_exit ("queue_init");
 }
 
 
 gboolean
-queue_is_paused (void)
+queue_is_online (void)
 {
-	return paused;
+	return _online;
 }
 void
-queue_set_paused (gboolean paused)
+queue_set_online (gboolean online)
 {
-	queue_new_todo (TODO_PAUSED, NULL, paused, -1);
+	queue_new_todo (TODO_ONLINE, NULL, online);
 }
 static void
-real_queue_set_paused (gint p)
+real_queue_set_online (gint p, gboolean * do_wakeup)
 {
-	if ((paused = p))
+	if (_online == p)
 	{
-		GSList * l;
-
-		pthread_mutex_lock (&sock_lock);
-		for (l=sockets; l!=NULL; l=l->next)
-		{
-			SocketInfo * i = (SocketInfo*)l->data;
-			task_hint_abort (i->task);
-			queue_set_task_status (i->task, QUEUE_TASK_STATUS_ABORTING);
-		}
-		pthread_mutex_unlock (&sock_lock);
+		if (_online)
+			*do_wakeup = TRUE;
 	}
 	else
 	{
-		queue_wakeup ();
+		_online = p;
+
+		fire_online_status_changed (_online);
+
+		if (_online)
+			*do_wakeup = TRUE;
+		else
+		{
+			GSList * l;
+
+			g_mutex_lock (sock_lock);
+			for (l=sockets; l!=NULL; l=l->next)
+			{
+				SocketInfo * i = (SocketInfo*)l->data;
+				task_hint_abort (i->task);
+				queue_set_task_status (i->task, QUEUE_TASK_STATUS_ABORTING);
+			}
+			g_mutex_unlock (sock_lock);
+		}
 	}
 }
 
 
 void
-queue_shutdown (gboolean save_tasks)
+queue_shutdown (void)
 {
-	queue_new_todo (TODO_SHUTDOWN, NULL, save_tasks, -1);
+	queue_new_todo (TODO_SHUTDOWN, NULL, 0);
 }
 static void
-real_queue_shutdown (int save_tasks)
+real_queue_shutdown (void)
 {
 	gint len;
 	GSList * l;
 	GSList * sockets_tmp;
 
-	/* stop listening to events */
-	pan_callback_remove (server_get_online_status_changed_callback(),
-	                     server_online_status_changed_cb, NULL);
-
 	/* clear the status hashtable */
 	g_hash_table_destroy (task_to_status);
 	task_to_status = NULL;
@@ -296,17 +306,14 @@ real_queue_shutdown (int save_tasks)
 
 
 	/* optionally save the tasks */
-	if (save_tasks)
-	{
-	}
 
 	/* unref the tasks */
 
 	/* get the socket array */
-	pthread_mutex_lock (&sock_lock);
+	g_mutex_lock (sock_lock);
 	sockets_tmp = sockets;
 	sockets = NULL;
-	pthread_mutex_unlock (&sock_lock);
+	g_mutex_unlock (sock_lock);
 
 	/* close the sockets */
 	len = g_slist_length (sockets_tmp);
@@ -322,7 +329,7 @@ void
 queue_add (Task * task)
 {
 	g_return_if_fail (task!=NULL);
-	queue_task_insert (task, task->high_priority ? 0 : -1);
+	queue_insert_tasks (g_slist_append (NULL, task), task->high_priority ? 0 : -1);
 }
 
 GPtrArray*
@@ -331,16 +338,28 @@ queue_get_tasks (void)
 	GSList * l;
 	GPtrArray * a = g_ptr_array_new ();
 
-        pthread_mutex_lock (&task_lock);
+        g_mutex_lock (task_lock);
 	for (l=task_list; l!=NULL; l=l->next) {
 		pan_object_ref (PAN_OBJECT(l->data));
 		g_ptr_array_add (a, l->data);
 	}
-        pthread_mutex_unlock (&task_lock);
+        g_mutex_unlock (task_lock);
 
 	return a;
 }
 
+gboolean
+queue_is_empty (void)
+{
+	gboolean retval;
+
+	g_mutex_lock (task_lock);
+	retval = task_list == NULL;
+	g_mutex_unlock (task_lock);
+
+	return retval;
+}
+
 /**
 ***
 **/
@@ -353,14 +372,20 @@ queue_get_max_sockets (void)
 void
 queue_set_max_sockets (int i)
 {
-	queue_new_todo (TODO_SET_MAX_SOCKET_QTY, NULL, i, 0);
+	queue_new_todo (TODO_SET_MAX_SOCKET_QTY, NULL, i);
 	queue_wakeup ();
 }
 static void
-real_queue_set_max_sockets (int i)
+real_queue_set_max_sockets (int i, gboolean * do_wakeup)
 {
-	max_socket_qty = i;
-	/* FIXME: fire some callback? */
+	if (max_socket_qty != i)
+	{
+		if (i > max_socket_qty)
+			*do_wakeup = TRUE;
+
+		max_socket_qty = i;
+		g_thread_pool_set_max_threads (_tpool, max_socket_qty, NULL);
+	}
 }
 
 guint
@@ -369,7 +394,7 @@ queue_get_running_task_count (void)
 	gint running_count = 0;
 	GSList *l;
 
-	pthread_mutex_lock (&task_lock);
+	g_mutex_lock (task_lock);
 	for (l=task_list; l!=NULL; l=l->next)
 	{
 		QueueTaskStatus status = queue_get_task_status (l->data);
@@ -379,7 +404,7 @@ queue_get_running_task_count (void)
 		    status == QUEUE_TASK_STATUS_CONNECTING)
 			++running_count;
 	}
-	pthread_mutex_unlock (&task_lock);
+	g_mutex_unlock (task_lock);
 
 	return running_count;
 }
@@ -389,34 +414,39 @@ queue_get_running_task_count (void)
 **/
 
 void
-queue_task_abort (Task* task)
+queue_abort_tasks (GSList * tasks)
 {
-	queue_new_todo (TODO_TASK_ABORT, task, -1, -1);
+	queue_new_todo (TODO_TASK_ABORT, tasks, -1);
 }
 static void
-real_queue_task_abort (Task* task)
+real_queue_abort_tasks (GSList * tasks)
 {
-	QueueTaskStatus status = queue_get_task_status (task);
+	GSList * l;
+	for (l=tasks; l!=NULL; l=l->next)
+	{
+		Task * task = TASK(l->data);
+		QueueTaskStatus status = queue_get_task_status (task);
 
-	if (status==QUEUE_TASK_STATUS_RUNNING)
-	{
-		/* ask a running task to abort */
-		task_hint_abort (TASK(task));
-		queue_set_task_status (task, QUEUE_TASK_STATUS_ABORTING);
-	}
-	else if (task->hint_abort && paused)
-	{
-		/* the task's been stopped because the queue is paused */
-		task->tries = 0;
-		task->hint_abort = FALSE;
-		queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
-	}
-	else
-	{
-		/* the task has failed */
-		task->tries = 0;
-		task->hint_abort = FALSE;
-		queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
+		if (status==QUEUE_TASK_STATUS_RUNNING)
+		{
+			/* ask a running task to abort */
+			task_hint_abort (TASK(task));
+			queue_set_task_status (task, QUEUE_TASK_STATUS_ABORTING);
+		}
+		else if (task->hint_abort && !queue_is_online())
+		{
+			/* the task's been stopped because the queue is offline */
+			task->tries = 0;
+			task->hint_abort = FALSE;
+			queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
+		}
+		else
+		{
+			/* the task has failed */
+			task->tries = 0;
+			task->hint_abort = FALSE;
+			queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
+		}
 	}
 }
 
@@ -425,127 +455,63 @@ real_queue_task_abort (Task* task)
 **/
 
 void
-queue_task_insert (Task* task, int index)
+queue_insert_tasks (GSList * tasks, int index)
 {
-	if (task!=NULL)
-		queue_new_todo (TODO_INSERT, task, index, -1);
+	if (tasks != NULL)
+		queue_new_todo (TODO_INSERT, tasks, index);
 }
 
 static void
-real_queue_task_insert (Task* task, int index)
+real_queue_insert_tasks (GSList * new_tasks, int index, gboolean * do_wakeup)
 {
+	GSList * tmp;
+	GSList * insertme;
 	guint task_len;
-	debug2 (DEBUG_QUEUE, "queue_task_insert %p to pos %d", task, index);
-
-	g_return_if_fail (task!=NULL);
-	g_return_if_fail (index==-1 || (guint)index<=queue_get_length());
-
-	/* add to the list */
-	pthread_mutex_lock (&task_lock);
-	task_list = g_slist_insert (task_list, task, index);
-	g_hash_table_insert (task_to_status, task, GINT_TO_POINTER(QUEUE_TASK_STATUS_QUEUED));
-	task_len = g_slist_length (task_list);
-	pthread_mutex_unlock (&task_lock);
-
-	gui_set_queue_size (running_tasks, task_len);
-
-	/* call callbacks */
-	debug1 (DEBUG_QUEUE, "queue size is now %u, calling callbacks", task_len);
-	pan_callback_call (queue_task_added, task, GINT_TO_POINTER(index));
-
-	queue_wakeup ();
-	
-	debug2 (DEBUG_QUEUE, "queue_task_insert %p to pos %d done", task, index);
-}
-
-/**
-***
-**/
-
-void
-queue_task_remove (Task* task)
-{
-	queue_new_todo (TODO_REMOVE, task, -1, -1);
-}
-static void
-real_queue_task_remove (Task* task)
-{
-	debug1 (DEBUG_QUEUE, "queue_task_remove %p", task);
-
-	g_return_if_fail (task!=NULL);
-	if (task->is_running)
-	{
-		task_hint_abort (task);
-		queue_set_task_status (task, QUEUE_TASK_STATUS_ABORTING);
-	}
-	else /* not running; simple cleanup. queue_run_thread() zeroes out is_running */
-	{
-		gint old_index;
-		guint task_len;
-		const gboolean had_socket = task->sock != NULL;
-		socket_cleanup (task);
-
-		/* remove the task */
-		pthread_mutex_lock (&task_lock);
-	       	old_index = g_slist_index (task_list, task);
-		task_list = g_slist_remove (task_list, task);
-		task_len = g_slist_length (task_list);
-		pthread_mutex_unlock (&task_lock);
-
-		gui_set_queue_size (running_tasks, task_len);
-
-		/* fire callbacks */
-		pan_callback_call (queue_task_removed, task, GINT_TO_POINTER(old_index));
-		g_hash_table_remove (task_to_status, task);
-		pan_object_unref (PAN_OBJECT(task));
-
-		/* maybe another socket task can run now */
-		if (had_socket)
-			queue_wakeup ();
-	}
-
-	debug1 (DEBUG_QUEUE, "queue_task_remove %p done", task);
-}
-
-/**
-***
-**/
-
-void
-queue_task_move (Task* task, int index)
-{
-	queue_new_todo (TODO_MOVE, task, index, -1);
-}
-static void
-real_queue_task_move (Task* task, int index)
-{
-	int old_index;
-	int indices [2];
-
-	debug2 (DEBUG_QUEUE, "queue_task_move %p to %d", task, index);
+	debug_enter ("real_queue_insert_tasks");
 
 	/* sanity clause */
-	g_return_if_fail (task!=NULL);
+	g_return_if_fail (new_tasks!=NULL);
 	g_return_if_fail (index==-1 || (guint)index<=queue_get_length());
-	
-	pthread_mutex_lock (&task_lock);
-	
-	old_index = g_slist_index (task_list, task);
-	if (old_index != -1)
+
+	/* make our own GSList nodes for inserting */
+	insertme = g_slist_copy (new_tasks);
+
+	/* inside a task_lock... */
+	g_mutex_lock (task_lock);
 	{
-		task_list = g_slist_remove (task_list, task);
-		task_list = g_slist_insert (task_list, task, index);
-		
-		indices[0] = old_index;
-		indices[1] = index;
+		/**
+		***  Add to the list
+		**/
+		if (task_list == NULL) /* no previous list */
+			task_list = insertme;
+		else { /* insert into the list */
+			GSList * nth = g_slist_nth (task_list, index);
+			if (nth == NULL) { /* index out of range, append to end */
+				index = g_slist_length (task_list);
+				nth = g_slist_last (task_list);
+			}
+			g_slist_last (insertme)->next = nth->next;
+			nth->next = insertme;
+		}
+
+		/**
+		***  Mark the new tasks as queued
+		**/
+		for (tmp=new_tasks; tmp!=NULL; tmp=tmp->next) {
+			Task * task = TASK(tmp->data);
+			g_hash_table_insert (task_to_status, task, GINT_TO_POINTER(QUEUE_TASK_STATUS_QUEUED));
+		}
+
+		/* we need this for gui_set_queue_size */
+		task_len = g_slist_length (task_list);
 	}
+	g_mutex_unlock (task_lock);
 
-	pthread_mutex_unlock (&task_lock);
-
-	if (old_index != -1)
-		pan_callback_call (queue_task_moved, task, indices);
-
-	debug1 (DEBUG_QUEUE, "queue_task_move %p done", task);
+	/* cleanup */
+	gui_set_queue_size (running_tasks, task_len);
+	pan_callback_call (queue_tasks_added, new_tasks, GINT_TO_POINTER(index));
+	*do_wakeup = TRUE;
+	debug_exit ("real_queue_insert_tasks");
 }
 
 /**
@@ -553,19 +519,154 @@ real_queue_task_move (Task* task, int index)
 **/
 
 void
-queue_task_requeue_failed (Task* task, int index)
+queue_remove_tasks (GSList * tasks)
 {
-	g_return_if_fail (task != NULL);
-	g_return_if_fail (queue_get_task_status(task) == QUEUE_TASK_STATUS_FAILED);
-
-	queue_new_todo (TODO_REQUEUE, task, index, -1);
+	queue_new_todo (TODO_REMOVE, tasks, -1);
 }
 static void
-real_queue_task_requeue_failed (Task* task, int index)
+real_queue_remove_tasks (GSList * tasks, gboolean * do_wakeup)
 {
-	task->tries = 0;
-	task->hint_abort = FALSE;
-	queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
+	GSList * l;
+	GSList * removed = NULL;
+
+	for (l=tasks; l!=NULL; l=l->next)
+	{
+		Task * task = TASK(l->data);
+
+		if (task->is_running)
+		{
+			/* abort the task; after it's done running
+			 * queue_run_thread() will send the task back here
+			 * for part two  */
+			task_hint_abort (task);
+			queue_set_task_status (task, QUEUE_TASK_STATUS_REMOVING);
+		}
+		else /* not running */
+		{
+			/* not running */
+			socket_cleanup (task);
+
+			/* remove the task */
+			g_mutex_lock (task_lock);
+			task_list = g_slist_remove (task_list, task);
+			removed = g_slist_prepend (removed, task);
+			g_mutex_unlock (task_lock);
+		}
+	}
+
+	if (removed != NULL)
+	{
+		removed = g_slist_reverse (removed);
+
+		gui_set_queue_size (running_tasks, g_slist_length(task_list));
+		*do_wakeup = TRUE;
+		pan_callback_call (queue_tasks_removed, removed, NULL);
+
+		for (l=removed; l!=NULL; l=l->next) {
+			Task * task = TASK(l->data);
+			g_hash_table_remove (task_to_status, task);
+			pan_object_unref (PAN_OBJECT(task));
+		}
+
+		g_slist_free (removed);
+	}
+}
+
+/**
+***
+**/
+
+void
+queue_move_tasks (GSList * tasks, int index)
+{
+	g_return_if_fail (tasks!=NULL);
+
+	queue_new_todo (TODO_MOVE, tasks, index);
+}
+
+static void
+real_queue_move_tasks (GSList * tasks, int moveto_index)
+{
+	GSList * l;
+	GSList * ref;
+
+	/* sanity clause */
+	g_return_if_fail (tasks!=NULL);
+	g_return_if_fail (task_list!=NULL);
+
+	g_mutex_lock (task_lock);
+
+	/* find the point of reference */
+	if (moveto_index == 0)
+		ref = NULL;
+	else {
+		ref = g_slist_nth (task_list, moveto_index-1);
+		while (ref != NULL) {
+			if (g_slist_find (tasks, ref->data) == NULL)
+				break;
+			ref = ref->next;
+		}
+		if (ref == NULL)
+			ref = g_slist_last (task_list);
+	}
+
+	/* remove the tasks */
+	for (l=tasks; l!=NULL; l=l->next)
+		task_list = g_slist_remove (task_list, l->data);
+
+	/* add them back in */
+	if (ref == NULL) {
+		l = g_slist_copy (tasks);
+		g_slist_last(l)->next = task_list;
+		task_list = l;
+		moveto_index = 0;
+	} else {
+		l = g_slist_copy (tasks);
+		g_slist_last(l)->next = ref->next;
+		ref->next = l;
+		moveto_index = g_slist_index (task_list, tasks->data);
+	}
+
+	/* let everyone know */
+	g_mutex_unlock (task_lock);
+	pan_callback_call (queue_tasks_moved, tasks, GINT_TO_POINTER(moveto_index));
+	debug_exit ("real_queue_move_tasks");
+}
+
+/**
+***
+**/
+
+void
+queue_requeue_failed_tasks (GSList * tasks)
+{
+	GSList * l;
+
+	/* sanity clause */
+	g_return_if_fail (tasks != NULL);
+	for (l=tasks; l!=NULL; l=l->next) {
+		Task * task = (Task*) l->data;
+		g_return_if_fail (queue_get_task_status(task) == QUEUE_TASK_STATUS_FAILED);
+	}
+
+	queue_new_todo (TODO_REQUEUE, tasks, -1);
+}
+static void
+real_requeue_failed_tasks (GSList * tasks, gboolean * do_wakeup)
+{
+	if (tasks != NULL)
+	{
+		GSList * l;
+
+		for (l=tasks; l!=NULL; l=l->next) {
+			Task * task = TASK(l->data);
+			task->tries = 0;
+			task->hint_abort = FALSE;
+			queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
+		}
+
+		*do_wakeup = TRUE;
+	}
 }
 
 /**
@@ -581,7 +682,7 @@ queue_get_max_tries (void)
 void
 queue_set_max_tries (int i)
 {
-	queue_new_todo (TODO_SET_MAX_TRIES, NULL, i, -1);
+	queue_new_todo (TODO_SET_MAX_TRIES, NULL, i);
 }
 static void
 real_queue_set_max_tries (int i)
@@ -625,19 +726,18 @@ queue_set_task_status (Task* task, QueueTaskStatus status)
 ************/
 
 static void
-queue_new_todo (QueueTodoAction action, Task* task, int i1, int i2)
+queue_new_todo (QueueTodoAction action, GSList * tasks, int i)
 {
 	QueueTodo *todo = g_new (QueueTodo, 1);
-	todo->task = task;
-	todo->index_1 = i1;
-	todo->index_2 = i2;
+	todo->tasks = tasks;
+	todo->index_1 = i;
 	todo->action = action;
 
-	pthread_mutex_lock (&todo_mutex);
-	todo_list = g_slist_append (todo_list, todo);
-	debug3 (DEBUG_QUEUE, "todo queue now has %d tasks (new task type: %d, int 1: %d)",
-		(int)g_slist_length(todo_list), action, i1);
-	pthread_mutex_unlock (&todo_mutex);
+	g_mutex_lock (todo_mutex);
+	g_queue_push_tail (todo_queue, todo);
+	debug3 (DEBUG_QUEUE, "todo queue now has %u tasks (new task type: %d, int 1: %d)",
+		todo_queue->length, action, i);
+	g_mutex_unlock (todo_mutex);
 
 	queue_wakeup ();
 }
@@ -646,20 +746,20 @@ static guint
 queue_get_length (void)
 {
 	guint size;
-	pthread_mutex_lock (&task_lock);
+	g_mutex_lock (task_lock);
 	size = g_slist_length (task_list);
-	pthread_mutex_unlock (&task_lock);
+	g_mutex_unlock (task_lock);
 	return size;
 }
 
-static void*
-queue_run_thread (void* vp)
+static void
+queue_run_thread (gpointer data, gpointer user_data)
 {
-	/* preparing to run */
-	gint result;
-	Task* task = TASK(vp);
+	int status;
+	int result;
+	gboolean aborting;
+	Task* task = TASK(data);
 	gchar* desc = status_item_describe (STATUS_ITEM(task));
-	gboolean aborted;
 	debug2 (DEBUG_QUEUE, "queue_run_thread: task %p [%s]", task, desc);
 
 	/* run */
@@ -676,61 +776,68 @@ queue_run_thread (void* vp)
 	status_item_set_active (STATUS_ITEM(task), FALSE);
 
 	/* clean up socket.
-	   We throw away any socket of a failed or aborted task,
+	   We throw away any socket of a failed or aborting task,
 	   just in case the read buffer has junk in there. */
-	aborted = queue_get_task_status(task) == QUEUE_TASK_STATUS_ABORTING;
-	if (task->sock!=NULL && (aborted || result!=TASK_SUCCESS))
+	status = queue_get_task_status (task);
+	aborting = status==QUEUE_TASK_STATUS_ABORTING || status==QUEUE_TASK_STATUS_REMOVING;
+	if (task->sock!=NULL && (aborting || result!=TASK_SUCCESS))
 		task->sock->error = TRUE;
 	socket_cleanup (task);
 
 	/* clean up the task */
 	task->is_running = FALSE;
 
-	if (result==TASK_SUCCESS) 
+	if (result==TASK_SUCCESS)
 	{
-		queue_task_remove (task);
+		/* success, remove the task */
+		queue_remove_tasks (g_slist_append(NULL,task));
 	}
-	else if (result==TASK_FAIL_HOPELESS && remove_failed_tasks) 
+	else if (result==TASK_FAIL_HOPELESS && remove_failed_tasks)
 	{
-		queue_task_remove (task);
+		/* failed, and requeueing won't help, so remove it now */
+		queue_remove_tasks (g_slist_append(NULL,task));
 	}
-	else if (result==TASK_FAIL && task->tries<max_tries && !aborted) /* retry */
+	else if (status == QUEUE_TASK_STATUS_REMOVING)
 	{
+		/* someone is waiting until run() finishes to remove the task */
+		queue_remove_tasks (g_slist_append(NULL,task));
+	}
+	else if (status == QUEUE_TASK_STATUS_ABORTING)
+	{
+		/* someone is waiting until run() finishes to abort the task */
+		task->tries = 0;
+		queue_abort_tasks (g_slist_append(NULL,task));
+	}
+	else if (task->tries<max_tries && result != TASK_FAIL_HOPELESS)
+	{
+		/* failed, but still has tries left to requeue */
 		++task->tries;
 		queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
 		queue_wakeup ();
 	}
-	else if (aborted) /* aborted */
-	{
-		task->tries = 0;
-		queue_task_abort (task);
-	}
-	else /* failed */
+	else
 	{
+		/* failed, let the user requeue later */
 		task->tries = 0;
 		task->hint_abort = FALSE;
 		queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
 		queue_wakeup ();
 	}
 
-	debug1 (DEBUG_QUEUE, "queue_run_thread: task %p done", vp);
-	pthread_exit (NULL);
-	return NULL;
+	debug1 (DEBUG_QUEUE, "queue_run_thread: task %p done", data);
 }
 
 
 static void
 queue_run (Task* task)
 {
-	pthread_t thread;
 	g_return_if_fail (task!=NULL);
 	debug2 (DEBUG_QUEUE, "starting a thread to run task %p, sock %p", task, task->sock);
 
 	queue_set_task_status (task, QUEUE_TASK_STATUS_RUNNING);
 
 	task->is_running = TRUE;
-	pthread_create (&thread, NULL, queue_run_thread, task);
-	pthread_detach (thread);
+	g_thread_pool_push (_tpool, task, NULL);
 }
 
 
@@ -754,27 +861,7 @@ queue_get_socket_for_task (Task *task)
 	int socket_qty;
 	debug_enter ("queue_get_socket_for_task");
 
-	/* if we're not online, then try to go online, or not. */
-	if (!server->online)
-	{
-		switch (going_online_preference)
-		{
-			case ONLINE_YES:
-				server_set_online (server, TRUE);
-				break;
-
-			case ONLINE_NO:
-				return -1;
-
-			case ONLINE_PROMPT:
-			default:
-				server_prompt_to_go_online (server);
-				return -1;
-				break;
-		}
-	}
-
-	pthread_mutex_lock (&sock_lock);
+	g_mutex_lock (sock_lock);
 
 	/* if an idle socket for that server, use it. */
 	socket_qty = 0;
@@ -787,7 +874,7 @@ queue_get_socket_for_task (Task *task)
 
 		++socket_qty;
 
-		if (!server->online) /* server is offline */
+		if (!queue_is_online()) /* offline */
 			continue;
 		if (si->task != NULL) /* socket already in use */
 			continue;
@@ -810,7 +897,7 @@ queue_get_socket_for_task (Task *task)
 	}
 
 	/* if no idle sockets for that server... */
-	if (retval && server->online && socket_qty<max_socket_qty)
+	if (retval && queue_is_online() && socket_qty<max_socket_qty)
 	{
 		const int current_connections = socket_qty;
 		int slots_open = server->max_connections - current_connections;
@@ -895,7 +982,7 @@ queue_get_socket_for_task (Task *task)
 		}
 	}
 
-	pthread_mutex_unlock (&sock_lock);
+	g_mutex_unlock (sock_lock);
 
 	debug_exit ("queue_get_socket_for_item");
 	return retval;
@@ -935,7 +1022,7 @@ queue_run_what_we_can (void)
 		}
 
 		/* try to get a socket... */
-		else if (!paused && max_socket_qty>0)
+		else if (queue_is_online() && max_socket_qty>0)
 		{
 			int i;
 			queue_set_task_status (task, QUEUE_TASK_STATUS_CONNECTING);
@@ -952,21 +1039,18 @@ queue_run_what_we_can (void)
 			{
 				if (i==-2) /* connection failure */
 				{
-
-
-
 					status_item_emit_error (STATUS_ITEM(task), _("Connect Failure"));
 
 					++task->tries;
 					if (task->tries > max_tries)
-						queue_task_abort (task);
+						queue_abort_tasks (g_slist_append(NULL,task));
 				}
 
 				queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
 			}
 		}
 
-		if (todo_list != NULL)
+		if (!g_queue_is_empty (todo_queue))
 		{
 				queue_do_todo ();
 				l = task_list;
@@ -997,7 +1081,7 @@ socket_cleanup (Task* task)
 		int server_socket_qty;
 		int socket_qty;
 
-		pthread_mutex_lock (&sock_lock);
+		g_mutex_lock (sock_lock);
 
 		/* count the number of sockets this server has */
 		socket_qty = 0;
@@ -1022,7 +1106,7 @@ socket_cleanup (Task* task)
 		 *  + we don't have too many sockets overall
 		 */
 		if (!socket->error
-		    && task->server->online
+		    && queue_is_online()
 		    && socket_qty<=max_socket_qty
 		    && server_socket_qty<=task->server->max_connections)
 		{
@@ -1045,7 +1129,7 @@ socket_cleanup (Task* task)
 				socket);
 		}
 		task->sock = NULL;
-		pthread_mutex_unlock (&sock_lock);
+		g_mutex_unlock (sock_lock);
 	}
 	debug_exit ("socket_cleanup");
 }
@@ -1075,7 +1159,7 @@ sockets_upkeep (void)
 	int socket_qty;
 	debug_enter ("socket_upkeep");
 
-	pthread_mutex_lock (&sock_lock);
+	g_mutex_lock (sock_lock);
 
 	socket_qty = g_slist_length (sockets);
 
@@ -1098,7 +1182,7 @@ sockets_upkeep (void)
 		idle = info->task == NULL;
 		old = age > info->server->idle_secs_before_timeout;
 		too_many = socket_qty > max_socket_qty;
-		offline = !info->server->online;
+		offline = !queue_is_online ();
 		destroy = idle && (old||too_many||offline);
 
 		/* if it's idle and we're keeping it, send a keepalive msg.
@@ -1154,7 +1238,7 @@ sockets_upkeep (void)
 
 	pan_warn_if_fail (g_slist_length(sockets) == socket_qty);
 
-	pthread_mutex_unlock (&sock_lock);
+	g_mutex_unlock (sock_lock);
 
 	g_slist_free (cull);
 
@@ -1167,60 +1251,66 @@ sockets_upkeep (void)
 static void
 queue_do_todo (void)
 {
-	GSList * list = NULL;
-	GSList * l = NULL;
+	gboolean do_wakeup = FALSE;
+	gboolean do_shutdown = FALSE;
+	GQueue * queue;
+	debug_enter ("queue_do_todo");
 
-	pthread_mutex_lock (&todo_mutex);
-	list = todo_list;
-	todo_list = NULL;
-	pthread_mutex_unlock (&todo_mutex);
+	g_mutex_lock (todo_mutex);
+	queue = todo_queue;
+	todo_queue = g_queue_new ();
+	g_mutex_unlock (todo_mutex);
 
-	for (l=list; l!=NULL; l=l->next)
+	while (!g_queue_is_empty(queue))
 	{
-		QueueTodo* a = (QueueTodo*) l->data;
+		QueueTodo* a = (QueueTodo*) g_queue_pop_head (queue);
+
 		switch (a->action)
 		{
 			case TODO_INSERT:
-				real_queue_task_insert (a->task, a->index_1);
+				real_queue_insert_tasks (a->tasks, a->index_1, &do_wakeup);
 				break;
 			case TODO_REMOVE:
-				real_queue_task_remove (a->task);
+				real_queue_remove_tasks (a->tasks, &do_wakeup);
 				break;
 			case TODO_MOVE:
-				real_queue_task_move (a->task, a->index_1);
+				real_queue_move_tasks (a->tasks, a->index_1);
 				break;
 			case TODO_REQUEUE:
-				real_queue_task_requeue_failed (a->task, a->index_1);
+				real_requeue_failed_tasks (a->tasks, &do_wakeup);
 				break;
 			case TODO_TASK_ABORT:
-				real_queue_task_abort (a->task);
+				real_queue_abort_tasks (a->tasks);
 				break;
 			case TODO_SET_MAX_SOCKET_QTY:
-				real_queue_set_max_sockets (a->index_1);
+				real_queue_set_max_sockets (a->index_1, &do_wakeup);
 				break;
 			case TODO_SET_MAX_TRIES:
 				real_queue_set_max_tries (a->index_1);
 				break;
 			case TODO_SHUTDOWN:
-				real_queue_shutdown (a->index_1);
-				l = NULL;
+				do_shutdown = TRUE;
 				break;
-			case TODO_PAUSED:
-				real_queue_set_paused (a->index_1);
+			case TODO_ONLINE:
+				real_queue_set_online (a->index_1, &do_wakeup);
 				break;
 			default:
 				pan_warn_if_reached();
 				break;
 		}
 
-		if (l == NULL)
-			break;
+		g_slist_free (a->tasks);
+		g_free (a);
 	}
 
+	if (do_wakeup)
+		queue_wakeup ();
+	if (do_shutdown)
+		real_queue_shutdown ();
+
 	/* throw the todo list away */
-	g_slist_foreach (list, (GFunc)g_free, NULL);
-	g_slist_free (list);
-	list = NULL;
+	g_queue_free (queue);
+	debug_exit ("queue_do_todo");
 }
 
 static void*
@@ -1230,24 +1320,24 @@ queue_mainloop (void* unused)
 
 	for (;;)
 	{
-		gint retcode;
-		struct timeval now;
-		struct timespec timeout;
+		gboolean was_timeout;
+		GTimeVal sleep_until;
 
-		pthread_mutex_lock (&cond_mutex);
-		gettimeofday (&now, NULL);
-		timeout.tv_sec = now.tv_sec + TIMEOUT_SECS;
-		timeout.tv_nsec = 0;
-		retcode = 0;
-		while (!work_to_do && retcode!=ETIMEDOUT) {
-			retcode = pthread_cond_timedwait(&qcond, &cond_mutex, &timeout);
-		}
+		g_mutex_lock (cond_mutex);
+
+		/* sleep for TIMEOUT_SECS unless someone wakes us up */
+		g_get_current_time (&sleep_until);
+		g_time_val_add (&sleep_until, TIMEOUT_SECS*G_USEC_PER_SEC);
+		was_timeout = FALSE;
+		while (!work_to_do && !was_timeout)
+			was_timeout = !g_cond_timed_wait (qcond, cond_mutex, &sleep_until);
 		work_to_do = FALSE;
-                pthread_mutex_unlock (&cond_mutex);
+                g_mutex_unlock (cond_mutex);
 
-		if (retcode == ETIMEDOUT) {
+		/* do work */
+		if (was_timeout)
 			sockets_upkeep ();
-		} else {
+		else {
 			queue_do_todo ();
                 	queue_run_what_we_can ();
 		}
@@ -1255,3 +1345,19 @@ queue_mainloop (void* unused)
 
 	return NULL;
 }
+
+
+PanCallback*
+queue_get_online_status_changed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_online_status_changed (gboolean online)
+{
+	pan_callback_call (queue_get_online_status_changed_callback(),
+	                   GINT_TO_POINTER(online), NULL);
+}
