@@ -28,7 +28,6 @@
 #include <stdlib.h>
 
 #include <glib.h>
-#include <libgnome/gnome-sound.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/group.h>
@@ -108,7 +107,6 @@ rule_action_constructor (RuleAction * r)
 	debug1 (DEBUG_PAN_OBJECT, "rule_action_ctor: %p", r);
 
 	r->flags = 0;
-	r->sound_file = NULL;
 	r->alert_message = NULL;
 	r->append_file = NULL;
 	r->forward_to = NULL;
@@ -124,7 +122,6 @@ rule_action_destructor (PanObject * obj)
 	RuleAction * r = RULE_ACTION(obj);
 
 	debug1 (DEBUG_PAN_OBJECT, "rule_action_dtor: %p", r);
-	g_free (r->sound_file);
 	g_free (r->alert_message);
 	g_free (r->append_file);
 	g_free (r->forward_to);
@@ -156,7 +153,6 @@ rule_action_clear (RuleAction * a)
 	g_return_if_fail (a != NULL);
 
 	a->flags = 0;
-	replace_gstr (&a->sound_file, NULL);
 	replace_gstr (&a->alert_message, NULL);
 	replace_gstr (&a->append_file, NULL);
 	replace_gstr (&a->forward_to, NULL);
@@ -181,7 +177,6 @@ rule_action_dup (const RuleAction * src)
 
 	/* duplicate this object's bits */
 	tgt->flags          = src->flags;
-	tgt->sound_file     = g_strdup (src->sound_file);
 	tgt->alert_message  = g_strdup (src->alert_message);
 	tgt->append_file    = g_strdup (src->append_file);
 	tgt->forward_to     = g_strdup (src->forward_to);
@@ -228,12 +223,6 @@ rule_action_apply (const RuleAction * action,
 		*deleted = TRUE;
 		return;
 	}
-	if (action->flags & RULE_ACTION_PLAY_SOUND)
-	{
-		pan_warn_if_fail (is_nonempty_string(action->sound_file));
-		if (is_nonempty_string (action->sound_file))
-			gnome_sound_play (action->sound_file);
-	}
 	if (action->flags & RULE_ACTION_SHOW_ALERT)
 	{
 		const Group * g = ARTICLE(g_ptr_array_index(articles,0))->group;
@@ -299,7 +288,7 @@ rule_action_apply (const RuleAction * action,
 		guint i;
 		for (i=0; i!=articles->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(articles,i));
-			thread_set_state (a, THREAD_WATCHED);
+			pan_thread_set_state (a, THREAD_WATCHED);
 		}
 	}
 	if (action->flags & RULE_ACTION_IGNORE)
@@ -307,7 +296,7 @@ rule_action_apply (const RuleAction * action,
 		guint i;
 		for (i=0; i!=articles->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(articles,i));
-			thread_set_state (a, THREAD_IGNORED);
+			pan_thread_set_state (a, THREAD_IGNORED);
 		}
 	}
 }
