@@ -24,13 +24,14 @@
 #include <gtk/gtkwidget.h>
 #include <pan/rules/rule.h>
 
-GtkWidget*    rule_edit_dialog_create       (const Rule       * rule);
+
+/**
+ * GTK_RESPONSE_OK,
+ * GTK_RESPONSE_APPLY,
+ * GTK_RESPONSE_CANCEL,
+ */
+GtkWidget*    rule_edit_dialog_create       (GtkWindow        * parent,
+                                             const Rule       * rule);
 
 Rule*         rule_edit_dialog_get_rule     (GtkWidget        * w);
 
-enum {
-	RULE_EDIT_DIALOG_CLICKED_DONE = 0,
-	RULE_EDIT_DIALOG_CLICKED_APPLY = 1,
-	RULE_EDIT_DIALOG_CLICKED_CANCEL = 2
-};
-
