@@ -28,9 +28,8 @@
 
 #include <string.h>
 
-#include <pthread.h>
-
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/argset.h>
 #include <pan/base/debug.h>
@@ -172,8 +171,8 @@ clist_refresh (RuleListDialog * d)
 ******
 *****/
 
-static gint
-rule_dialog_close_cb (GnomeDialog * dialog, gpointer data)
+static void
+rule_dialog_response_cb (GtkDialog * dialog, gint arg1, gpointer data)
 {
 	RuleListDialog * d = (RuleListDialog*) data;
 	debug_enter ("rule_dialog_close_cb");
@@ -189,15 +188,9 @@ rule_dialog_close_cb (GnomeDialog * dialog, gpointer data)
 	}
 
 	/* cleanup dialog */
+	gtk_widget_destroy (GTK_WIDGET(dialog));
 	g_free (d);
 	debug_exit ("rule_dialog_close_cb");
-	return FALSE;
-}
-
-static void
-rule_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
-{
-	gnome_dialog_close (dialog);
 }
 
 
@@ -221,9 +214,9 @@ remove_button_clicked_cb (GtkButton * button, gpointer data)
 }
 
 static void
-add_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
+add_dialog_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
-	if (index != RULE_EDIT_DIALOG_CLICKED_CANCEL)
+	if (response != GTK_RESPONSE_CANCEL)
 	{
 		guint i;
 		GtkWidget * w = GTK_WIDGET(dialog);
@@ -252,16 +245,15 @@ g_message ("do we already have this? %d", i!=d->rules->len);
 		clist_refresh (d);
 	}
 
-	if (index != RULE_EDIT_DIALOG_CLICKED_APPLY)
-		gnome_dialog_close (dialog);
+	if (response != GTK_RESPONSE_APPLY)
+		gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 static void
 add_button_clicked_cb (GtkButton * button, gpointer data)
 {
 	RuleListDialog * f = (RuleListDialog*) data;
-	GtkWidget * dialog = rule_edit_dialog_create (NULL);
-	gtk_signal_connect (GTK_OBJECT(dialog), "clicked", GTK_SIGNAL_FUNC(add_dialog_clicked_cb), f);
-	gnome_dialog_set_parent (GNOME_DIALOG(dialog), GTK_WINDOW(f->dialog));
+	GtkWidget * dialog = rule_edit_dialog_create (GTK_WINDOW(f->dialog), NULL);
+	g_signal_connect (GTK_OBJECT(dialog), "response", G_CALLBACK(add_dialog_response_cb), f);
 	gtk_widget_show_all (dialog);
 }
 
@@ -270,9 +262,9 @@ add_button_clicked_cb (GtkButton * button, gpointer data)
 **/
 
 static void
-edit_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
+edit_dialog_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
-	if (index != RULE_EDIT_DIALOG_CLICKED_CANCEL)
+	if (response != GTK_RESPONSE_CANCEL)
 	{
 		RuleListDialog * d = (RuleListDialog*) data;
 		const gchar * e_name = (const gchar*) gtk_object_get_data (GTK_OBJECT(dialog), "edit_name");
@@ -296,10 +288,8 @@ edit_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
 		gtk_object_set_data_full (GTK_OBJECT(dialog), "edit_name", g_strdup(r_new->name), g_free);
 	}
 
-	if (index != RULE_EDIT_DIALOG_CLICKED_APPLY)
-	{
-		gnome_dialog_close (dialog);
-	}
+	if (response != GTK_RESPONSE_APPLY)
+		gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 static void
 edit_button_clicked_cb (GtkButton * button, gpointer data)
@@ -309,10 +299,9 @@ edit_button_clicked_cb (GtkButton * button, gpointer data)
 	if (sel != -1)
 	{
 		const Rule * r_old = RULE(g_ptr_array_index(d->rules,sel));
-		GtkWidget * dialog = rule_edit_dialog_create (r_old);
-		gtk_signal_connect (GTK_OBJECT(dialog), "clicked", GTK_SIGNAL_FUNC(edit_dialog_clicked_cb), d);
+		GtkWidget * dialog = rule_edit_dialog_create (GTK_WINDOW(d->dialog), r_old);
+		gtk_signal_connect (GTK_OBJECT(dialog), "response", GTK_SIGNAL_FUNC(edit_dialog_response_cb), d);
 		gtk_object_set_data_full (GTK_OBJECT(dialog), "edit_name", g_strdup(r_old->name), g_free);
-		gnome_dialog_set_parent (GNOME_DIALOG(dialog), GTK_WINDOW(d->dialog));
 		gtk_widget_show_all (dialog);
 	}
 }
@@ -465,7 +454,7 @@ apply_to_subscribed_cb (GtkButton * button, gpointer data)
 ************/
 
 GtkWidget*
-rule_dialog_new (void)
+rule_dialog_new (GtkWindow * window)
 {
 	GtkWidget * w;
 	GtkWidget * hbox;
@@ -480,14 +469,16 @@ rule_dialog_new (void)
 	rule_manager_get_rules (d->rules);
 
 	/* dialog */
-	w = d->dialog = gnome_dialog_new (_("Pan: Rules"), GNOME_STOCK_BUTTON_OK, NULL);
+	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Rules"), window,
+	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                             GTK_STOCK_OK, GTK_RESPONSE_NONE,
+	                                             NULL);
 	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT(w), "close", GTK_SIGNAL_FUNC(rule_dialog_close_cb), d);
-	gtk_signal_connect (GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(rule_dialog_clicked_cb), d);
+	gtk_signal_connect (GTK_OBJECT(w), "response", G_CALLBACK(rule_dialog_response_cb), d);
 
 	/* workarea */
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
-	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(w)->vbox), hbox, TRUE, TRUE, 0);
+	hbox = gtk_hbox_new (FALSE, GUI_PAD);
+	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(w)->vbox), hbox, TRUE, TRUE, 0);
 
 	/* clist */
 	titles[0] = _("Rules");
@@ -508,7 +499,7 @@ rule_dialog_new (void)
 	gtk_widget_set_usize (w, -2, 300);
 
 	/* button box */
-	bbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	bbox = gtk_vbox_new (FALSE, GUI_PAD_SMALL);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 0);
 
 	/* add button */
