@@ -21,7 +21,7 @@
  * 
  */
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtkwindow.h>
 
-GtkWidget* rule_dialog_new (void);
+GtkWidget* rule_dialog_new (GtkWindow * window);
 
