@@ -30,7 +30,7 @@
 #include <stdio.h>
 #include <time.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
 
 #include <pan/base/acache.h>
 #include <pan/base/article.h>
@@ -42,11 +42,10 @@
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/util-file.h>
 
+#include <pan/globals.h>
 #include <pan/nntp.h>
-#include <pan/prefs.h> /* for auto_part_selection */
 #include <pan/queue.h>
 #include <pan/task-save.h>
-#include <pan/util.h> /* for pan_lock/unlock */
 
 /*********************
 **********************  Defines / Enumerated types
@@ -68,6 +67,8 @@ static gint task_save_run (Task* task);
 
 static char* task_save_describe (const StatusItem* item);
 
+static void task_save_prune (TaskSave  * task);
+
 /*********************
 **********************  Variables
 *********************/
@@ -201,6 +202,7 @@ task_save_new (Group          * group,
 			  group->server, FALSE, need_socket);
 
 	/* create the task-save */
+	TASK(task)->type = TASK_TYPE_SAVE;
 	task->runval = TASK_FAIL;
 	task->group = group;
 	task->readable_name = g_strdup(readable_name);
@@ -217,10 +219,16 @@ task_save_new (Group          * group,
 	articles_add_flag ((Article**)a->pdata, a->len, STATE_SAVE_QUEUED);
 	g_ptr_array_free (a, TRUE);
 
+	/* prune the articles */
+	task_save_prune (task);
+
 	return PAN_OBJECT(task);
 }
 
- 
+
+extern char*
+expand_download_dir (const char * dir, const Group * group);
+
 void
 task_save_set_attachments (TaskSave      * task,
                            const gchar   * path,
@@ -239,11 +247,9 @@ task_save_set_attachments (TaskSave      * task,
 	if (!task->path_attachments)
 		task->path_attachments = g_strdup (download_dir);
 
-	/* replace "%g" with the group name, "%s" with the server name */
+	/* expand */
 	replace_gstr (&task->path_attachments,
-	              pan_substitute(task->path_attachments,"%s", task->group->server->name));
-	replace_gstr (&task->path_attachments,
-	              pan_substitute(task->path_attachments,"%g", task->group->name));
+	              expand_download_dir (task->path_attachments,  task->group));
 
 	debug_exit ("task_save_set_attachments");
 }
@@ -266,11 +272,9 @@ task_save_set_bodies (TaskSave      * task,
 	if (!task->path_bodies)
 		task->path_bodies = g_strdup (download_dir);
 
-	/* replace "%g" with the group name, "%s" with the server name */
+	/* expand */
 	replace_gstr (&task->path_bodies,
-	              pan_substitute(task->path_bodies,"%s", task->group->server->name));
-	replace_gstr (&task->path_bodies,
-	              pan_substitute(task->path_bodies,"%g", task->group->name));
+	              expand_download_dir (task->path_bodies,  task->group));
 
 	debug_exit ("task_save_set_bodies");
 }
@@ -451,51 +455,50 @@ task_save_run (Task * task)
 	{
 		gboolean bodies_ok = TRUE;
 		GSList * l;
+
 		for (l=articles; l!=NULL; l=l->next)
 		{
+			gboolean body_ok = TRUE;
 			const Article * a = ARTICLE(l->data);
 			char * txt = article_get_message (a);
 			char * fname;
-			char * full_fname;
 			char * path;
-			FILE * fp;
 
 			/* get the filename */
 			fname = g_strdup (TASK_SAVE(task)->filename_bodies);
 			if (!is_nonempty_string(fname))
-				replace_gstr (&fname, g_strdup_printf ("%s.txt", TASK_SAVE(task)->readable_name));
-
+				replace_gstr (&fname, g_strdup_printf ("%s.txt", article_get_subject(a)));
 			replace_gstr (&fname, get_unique_fname (TASK_SAVE(task)->path_bodies, fname));
 
 			/* get the directory */
-			path = g_dirname (fname);
+			path = g_path_get_dirname (fname);
 			if (!directory_check (path)) {
-				log_add_va (LOG_ERROR, _("Save bodies can't access path \"%s\""), g_dirname(fname));
-				bodies_ok = FALSE;
+				log_add_va (LOG_ERROR, _("Save Article can't access path \"%s\""), path);
+				body_ok = FALSE;
 			}
-			
-			full_fname = get_unique_fname (TASK_SAVE(task)->path_bodies, fname);
 
-			/* write the file */
-			errno = 0;
-			fp = fopen (full_fname, "w+");
-			if (fp == NULL) {
-				log_add_va (LOG_ERROR, _("Unable to open file `%s' %s"),
-				                         full_fname,
-				                         errno ? g_strerror(errno) : "");
-				bodies_ok = FALSE;
-			}
-			else {
-				fwrite (txt, sizeof(char), strlen(txt), fp);
-				fclose (fp);
-				log_add_va (LOG_INFO, _("Saved article body to `%s'"), full_fname);
+			/* open the file for writing */
+			if (body_ok) {
+				FILE * fp;
+				errno = 0;
+				fp = fopen (fname, "w+");
+				if (fp == NULL) {
+					log_add_va (LOG_ERROR, _("Can't create file `%s' %s"), fname, g_strerror(errno));
+					body_ok = FALSE;
+				} else {
+					fwrite (txt, sizeof(char), strlen(txt), fp);
+					fclose (fp);
+					log_add_va (LOG_INFO, _("Saved article body to `%s'"), fname);
+				}
 			}
 
 			/* cleanup */
 			g_free (txt);
-			g_free (fname);
 			g_free (path);
-			g_free (full_fname);
+			g_free (fname);
+
+			if (!body_ok)
+				bodies_ok = FALSE;
 		}
 
 		if (!bodies_ok)
@@ -513,58 +516,8 @@ task_save_run (Task * task)
 ******
 *****/
 
-typedef struct
-{
-	GtkWidget * clist;
-	GtkWidget * window;
-	TaskSave * task;
-	int queue_index;
-}
-ResolvePartsUI;
-
 static void
-select_articles_clicked_cb (GnomeDialog   * dialog,
-                            int             button,
-                            gpointer        user_data)
-{
-	ResolvePartsUI *ui = (ResolvePartsUI*) user_data;
-
-	if (button==0) /* "ok" */
-	{
-		GList* sel = GTK_CLIST(ui->clist)->selection;
-		const int rows = GTK_CLIST(ui->clist)->rows;
-		int row;
-
-		/* remove the non-selected articles */
-		for (row=rows-1; row>=0; --row)
-		{
-			gboolean row_selected = g_list_find (sel,GINT_TO_POINTER(row))!=NULL;
-
-			if (!row_selected)
-			{
-				GSList* n = g_slist_nth (ui->task->articles, row);
-				ui->task->articles = g_slist_remove_link(ui->task->articles,n);
-				g_slist_free_1 (n);
-			}
-		}
-
-		queue_task_insert (TASK(ui->task), ui->queue_index);
-	}
-	else
-	{
-		pan_object_unref (PAN_OBJECT(ui->task));
-	}
-
-	/* cleanup */
-	pan_lock ();
-	gtk_widget_destroy (ui->window);
-	pan_unlock ();
-	g_free (ui);
-}
-
-static void
-choose_article_parts_auto (TaskSave  * task,
-                           int           queue_index)
+weed_duplicate_parts (TaskSave  * task)
 {
 	GSList* l;
 	Article *last_article = NULL;
@@ -618,96 +571,10 @@ choose_article_parts_auto (TaskSave  * task,
 			last_article = article;
 		}
 	}
-
-	queue_task_insert (TASK(task), queue_index);
 }
 
 static void
-choose_article_parts_manual (TaskSave  * task,
-                             int           queue_index)
-{
-	GSList* l;
-	GtkWidget* w;
-	GtkWidget* list;
-	GtkWidget* vbox;
-	GtkWidget* label;
-	char* columns [4];
-	int last_part = -1;
-	ResolvePartsUI* ui = g_new0 (ResolvePartsUI, 1);
-
-	ui->queue_index = queue_index;
-	ui->task = task;
-
-	pan_lock ();
-
-	ui->window = gnome_dialog_new (_("Pan: Save Multipart Message"),
-	                               GNOME_STOCK_BUTTON_OK,
-	                               GNOME_STOCK_BUTTON_CANCEL,
-	                               NULL);
-	gtk_signal_connect (GTK_OBJECT(ui->window), "clicked",
-	                    GTK_SIGNAL_FUNC(select_articles_clicked_cb), ui);
-
-	gtk_window_set_policy (GTK_WINDOW (ui->window), TRUE, TRUE, TRUE);
-	gnome_dialog_set_default (GNOME_DIALOG(ui->window), 1);
-	vbox = GNOME_DIALOG(ui->window)->vbox;
-
-	label = gtk_label_new (
-		_("Pan found multiple copies of some of the parts of this post.\n"
-		  "Please select the parts to use for this save."));
-	gtk_box_pack_start (GTK_BOX(vbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
-
-	columns[0] = _("Subject");
-	columns[1] = _("Lines");
-	columns[2] = _("Author");
-	columns[3] = _("Date");
-	ui->clist = list = gtk_clist_new_with_titles (4, columns);
-	gtk_clist_set_selection_mode (GTK_CLIST(list), GTK_SELECTION_MULTIPLE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST(list), 0, TRUE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST(list), 1, TRUE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST(list), 2, TRUE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST(list), 3, TRUE);
-
-	for (l=task->articles; l!=NULL; l=l->next)
-	{
-		char datebuf[32];
-		char lines[8];
-		Article * article = ARTICLE (l->data);
-		const struct tm local_tm = *localtime (&article->date);
-		gchar * text[4];
-		gchar author[512];
-		int row;
-		sprintf (lines, "%d", article->linecount);
-		strftime (datebuf, sizeof(datebuf), "%Y/%m/%d %H:%M", &local_tm);
-
-	       	text[0] = (gchar*) article_get_subject (article);
-		text[1] = lines;
-		text[2] = article_get_author_str (article, author, sizeof(author));
-		text[3] = datebuf;
-
-		row = gtk_clist_append (GTK_CLIST(list), text);
-		gtk_clist_set_row_data (GTK_CLIST(list), row, article);
-		if (last_part != article->part)
-		{
-			gtk_clist_select_row (GTK_CLIST(list), row, 0);
-			last_part = article->part;
-		}
-	}
-	w = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
-		GTK_POLICY_AUTOMATIC,
-		GTK_POLICY_AUTOMATIC);
-	gtk_container_add (GTK_CONTAINER(w), ui->clist);
-	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, GNOME_PAD_SMALL);
-
-	gtk_window_set_default_size (GTK_WINDOW (ui->window), 500, 400);
-	gtk_widget_show_all (ui->window);
-
-	pan_unlock ();
-}
-
-void
-task_save_validate_and_queue_or_self_destruct (TaskSave  * task,
-                                               int          queue_index)
+task_save_prune (TaskSave  * task)
 {
 	const int length = g_slist_length(task->articles);
 
@@ -722,17 +589,6 @@ task_save_validate_and_queue_or_self_destruct (TaskSave  * task,
 		&& (length!=ARTICLE(task->articles->data)->parts)
 		&& ARTICLE(task->articles->data)->parts)
 	{
-		if (auto_part_selection)
-		{
-			choose_article_parts_auto (task, queue_index);	
-		}
-		else
-		{
-			choose_article_parts_manual (task, queue_index);
-		}
-	}
-	else
-	{
-		queue_task_insert (TASK(task), queue_index);
+		weed_duplicate_parts (task);	
 	}
 }
