@@ -40,6 +40,15 @@ enum
 	TASK_FAIL_HOPELESS		= -2  /* failed; don't retry */
 };
 
+enum
+{
+	TASK_TYPE_OTHER,
+	TASK_TYPE_BODY,
+	TASK_TYPE_BODIES,
+	TASK_TYPE_HEADERS,
+	TASK_TYPE_SAVE
+};
+
 #define TASK(a) ((Task*)a)
 typedef struct _Task Task;
 typedef int (*TaskRunFunc)(Task*);
@@ -57,6 +66,7 @@ struct _Task
 	guint high_priority      : 1;
 	guint needs_socket       : 1;
 	gint8 tries;
+	gint8 type;
 	gboolean hint_abort;
 	gboolean is_running;
 	Server * server;
