@@ -23,6 +23,7 @@
 
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
+#include <gtk/gtktextbuffer.h>
 #include <pan/base/pan-callback.h>
 
 /**
@@ -85,10 +86,10 @@ extern GdkColor text_quoted_color[3];
 GtkWidget*   text_create                     (void);
 void         text_read_more                  (void);
 void         text_select_all                 (void);
+void         text_deselect_all               (void);
 void         text_refresh                    (void);
 void         text_clear_nolock               (void);
 void         text_set_from_cached_article    (Article      * article);
-void         text_set_from_mem_article       (Article      * article, const gchar * text);
 void         text_set_from_article           (Article      * article);
 void         text_set_raw                    (const gchar  * text);
 
@@ -98,8 +99,8 @@ void         text_set_font                   (void);
 
 gchar*       text_get_message_to_reply_to    (void);
 
-void         update_body_pane                (GtkWidget    * body_widget,
-	                                      const gchar  * body,
-					      gboolean     mute_quotes);
+void         update_body_pane                (GtkTextBuffer * buffer,
+	                                      const gchar   * body,
+					      gboolean        mute_quotes);
 
 #endif /* __TEXT_H__ */
