@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,13 +34,14 @@
 #include <pan/base/util-file.h>
 #include <pan/base/pan-i18n.h>
 
-#include <pan/prefs.h>
 #include <pan/thread-watch.h>
 
 /*********************
 **********************  Defines / Enumerated types
 *********************/
 
+#define WATCHED_THREAD_SPAN_DAYS 31
+
 /*********************
 **********************  Macros
 *********************/
@@ -87,10 +88,10 @@ static guint hash_size = 0;
 ******
 *****/
 
-static gchar*
+static char*
 get_filename (void)
 {
-	return g_strdup_printf ("%s%cthreads.txt", data_dir, G_DIR_SEPARATOR);
+	return g_strdup_printf ("%s%cthreads.txt", get_data_dir(), G_DIR_SEPARATOR);
 }
 
 /*****
@@ -111,10 +112,10 @@ thread_init_module (void)
 	{
 		const time_t current_time = time(0);
 		RecordStruct * data;
-		gchar * filename;
-		GArray * file;
-		gchar ** v;
-		gint i;
+		char * filename;
+		char * file;
+		char ** v;
+		int i;
 
 		/* create the hashtable */
 		hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -123,15 +124,15 @@ thread_init_module (void)
 
 		/* load the file */
 		filename = get_filename ();
-		file = read_file (filename);
-		v = file==NULL ? NULL : g_strsplit (file->data, "\n", -1);
+		g_file_get_contents (filename, &file, NULL, NULL);
+		v = file==NULL ? NULL : g_strsplit (file, "\n", -1);
 		g_hash_table_freeze (hash);
 		for (i=0; v!=NULL && v[i]!=NULL; ++i)
 		{
 			int state;
 			time_t start_time;
-			gchar * id;
-			const gchar * march = v[i];
+			char * id;
+			const char * march = v[i];
 			time_t age_secs;
 
 			/* skip comments */
@@ -145,7 +146,7 @@ thread_init_module (void)
 
 			/* age out threads > one month old */
 			age_secs = current_time - start_time;
-			if ((age_secs/86400) > watched_thread_span_days) {
+			if ((age_secs/86400) > WATCHED_THREAD_SPAN_DAYS) {
 				dirty = TRUE;
 				g_free (id);
 				continue;
@@ -162,8 +163,7 @@ thread_init_module (void)
 
 		/* cleanup */
 		g_free (filename);
-		if (file != NULL)
-			g_array_free (file, TRUE);
+		g_free (file);
 		g_strfreev (v);
 	}
 }
@@ -176,10 +176,10 @@ thread_init_module (void)
 *************  PUBLIC
 ************/
 
-gint
+int
 pan_thread_get_state (const Article * a)
 {
-	gint retval = THREAD_NOTHING;
+	int retval = THREAD_NOTHING;
 
 	/* sanity clause */
 	g_return_val_if_fail(a, retval);
@@ -193,7 +193,7 @@ pan_thread_get_state (const Article * a)
 	{
 		if (a != NULL)
 		{
-			gchar * id = article_get_thread_message_id (a);
+			char * id = article_get_thread_message_id (a);
 
 			if (id != NULL)
 			{
@@ -216,7 +216,7 @@ pan_thread_get_state (const Article * a)
 void
 pan_thread_set_state (const Article * a, int state)
 {
-	gchar * id;
+	char * id;
 
 	/* sanity checks */
 	g_return_if_fail (article_is_valid(a));
@@ -286,7 +286,7 @@ thread_write_line (gpointer key, gpointer value, gpointer user_data)
 	const RecordStruct * data = (const RecordStruct*) value;
 	if (data->state != THREAD_NOTHING)
 	{
-		const gchar * id = (const gchar*) key;
+		const char * id = (const char*) key;
 		FILE * fp = (FILE*)user_data;
 
 		fprintf (fp, "%d,%lu,%s\n",
@@ -302,9 +302,10 @@ thread_shutdown_module (void)
 	if (dirty)
 	{
 		/* write the thread states to disk */
-		gchar * filename = get_filename ();
-		FILE * fp = fopen (filename, "w+");
-		if (fp != NULL) 
+		char * filename = get_filename ();
+		FILE * fp = fopen (filename, "w");
+
+		if (fp != NULL)
 		{
 			fprintf (fp, "# one record per line, comma-delimited fields\n");
 			fprintf (fp, "# each record has three fields:\n");
@@ -314,10 +315,10 @@ thread_shutdown_module (void)
 			g_hash_table_foreach (hash, thread_write_line, fp);
 			fclose (fp);
 		}
-		else 
+		else
 		{
-			g_warning (_("Could not open '%s' for writing: %s\n"),  
-				 filename, g_strerror(errno));                   
+			g_warning (_("Could not open '%s' for writing: %s\n"),
+				filename, g_strerror(errno));
 		}
 		g_free (filename);
 		dirty = FALSE;
