@@ -21,13 +21,17 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
 #include <time.h>
 
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/log.h>
 
 #include <pan/globals.h>
@@ -40,6 +44,22 @@
 ****
 ***/
 
+static void
+pan_dialog (const gchar * str, GtkMessageType type, GtkWindow * window)
+{
+	GtkWidget * w;
+	pan_lock();
+	w = gtk_message_dialog_new (GTK_WINDOW(window),
+	                            GTK_DIALOG_DESTROY_WITH_PARENT,
+	                            type,
+				    GTK_BUTTONS_OK,
+				    "%s", str);
+	g_signal_connect_swapped (GTK_OBJECT(w), "response",
+	                          G_CALLBACK (gtk_widget_destroy),
+	                          GTK_OBJECT(w));
+	gtk_widget_show_all (w);
+	pan_unlock();
+}
 void
 pan_info_dialog (const char *format, ...)
 {
@@ -50,9 +70,9 @@ pan_info_dialog (const char *format, ...)
 	str = g_strdup_vprintf (format, args);
 	va_end (args);
 	g_warning (str);
-	pan_lock();
-	gnome_ok_dialog_parented (str, GTK_WINDOW(Pan.window));
-	pan_unlock();
+
+	pan_dialog (str, GTK_MESSAGE_INFO, GTK_WINDOW(Pan.window));
+
 	g_free (str);
 }
 
@@ -66,9 +86,9 @@ pan_error_dialog_parented (gpointer parent, const char *format, ...)
 	str = g_strdup_vprintf (format, args);
 	va_end (args);
 	g_warning (str);
-	pan_lock();
-	gnome_error_dialog_parented (str, GTK_WINDOW(parent));
-	pan_unlock();
+
+	pan_dialog (str, GTK_MESSAGE_ERROR, GTK_WINDOW(parent));
+
 	g_free (str);
 }
 
@@ -82,9 +102,9 @@ pan_error_dialog (const char *format, ...)
 	str = g_strdup_vprintf (format, args);
 	va_end (args);
 	g_warning (str);
-	pan_lock();
-	gnome_error_dialog_parented (str, GTK_WINDOW(Pan.window));
-	pan_unlock();
+
+	pan_dialog (str, GTK_MESSAGE_ERROR, GTK_WINDOW(Pan.window));
+
 	g_free (str);
 }
 
@@ -118,7 +138,7 @@ pan_timeout_wrapper (gpointer data)
 	gint retval;
 
 	/* sanity clause */
-	pan_warn_if_fail (pthread_self() == Pan.main_t);
+	pan_warn_if_fail (g_thread_self() == Pan.main_t);
 
 	/* call the timer func */
 	main_thread_is_in_glib_callback = TRUE;
@@ -142,7 +162,7 @@ pan_timeout_add (guint32 interval, GtkFunction func, gpointer arg)
 ****
 ***/
 
-static pthread_t has_lock_thr = 0;
+static const GThread * has_lock_thr = NULL;
 
 void
 pan_lock_from (const gchar * file, const gchar * func, int line)
@@ -150,7 +170,7 @@ pan_lock_from (const gchar * file, const gchar * func, int line)
 	static const gchar * last_file = NULL;
 	static const gchar * last_func = NULL;
 	static int last_line = -1;
-	const pthread_t thr = pthread_self();
+	const GThread * thr = g_thread_self ();
 
 	/**
 	 * If pan_lock() is called from the main thread while it has a GUI lock
@@ -163,12 +183,12 @@ pan_lock_from (const gchar * file, const gchar * func, int line)
 	 */
 	if (thr==Pan.main_t && !main_thread_is_in_glib_callback)
 	{
-		debug4 (DEBUG_LOCK,"mainthread %lu attempted unnecessary lock from %s:%d (%s)", thr, file, line, func);
+		debug4 (DEBUG_LOCK,"mainthread %p attempted unnecessary lock from %s:%d (%s)", thr, file, line, func);
 	}
 	else if (thr == has_lock_thr)
 	{
-		g_error (_("thread %lu attempted double lock!\nfirst lock was in %s:%d (%s),\nnow trying for another one from %s:%d (%s)"),
-			(gulong)thr,
+		g_error (_("thread %p attempted double lock!\nfirst lock was in %s:%d (%s),\nnow trying for another one from %s:%d (%s)"),
+			thr,
 			last_file, last_line, last_func,
 			file, line, func);
 	}
@@ -182,28 +202,28 @@ pan_lock_from (const gchar * file, const gchar * func, int line)
 		last_func = func;
 		last_line = line;
 		has_lock_thr = thr;
-		debug3 (DEBUG_LOCK,"thread %lu entered gdk_threads from %s %d", thr, file, line);
+		debug3 (DEBUG_LOCK,"thread %p entered gdk_threads from %s %d", thr, file, line);
 	}
 }
 
 void
 pan_unlock_from (const gchar* file, const gchar * func, int line)
 {
-	const pthread_t thr = pthread_self();
+	const GThread* thr = g_thread_self ();
 
 	if (thr==Pan.main_t && !has_lock_thr)
 	{
-		debug4 (DEBUG_LOCK,"mainthread %lu attempted unneccessary unlock from %s:%d (%s)", (gulong)thr, file, line, func);
+		debug4 (DEBUG_LOCK,"mainthread %p attempted unneccessary unlock from %s:%d (%s)", thr, file, line, func);
 	}
 	else if (has_lock_thr != thr)
 	{
-		g_error (_("thread %lu attempted to remove a lock it didn't have from %s:%d (%s)"), (gulong)thr, file, line, func);
+		g_error (_("thread %p attempted to remove a lock it didn't have from %s:%d (%s)"), thr, file, line, func);
 	}
 	else
 	{
-		has_lock_thr = (pthread_t)0;
+		has_lock_thr = NULL;
 		gdk_threads_leave();
-		debug4 (DEBUG_LOCK,"thread %lu left gdk_threads from %s:%d (%s)", pthread_self(), file, line, func);
+		debug4 (DEBUG_LOCK,"thread %p left gdk_threads from %s:%d (%s)", g_thread_self(), file, line, func);
 	}
 }
 
@@ -303,7 +323,7 @@ linkify_text (const gchar * text)
 	{
 		gchar * url = g_strndup (pch, len);
 		if (pch != prev)
-			pan_g_string_append_len (str, prev, (guint)(pch-prev));
+			g_string_append_len (str, prev, (guint)(pch-prev));
 
 		g_string_append (str, "<a href=\"");
 		g_string_append (str, url);
@@ -340,51 +360,21 @@ get_date_display_string (time_t date, const gchar * fmt, gchar * fillme, gint le
 	*fillme = '\0';
 
 	/* get the date string */
-        if (display_article_dates_in_local_time)
-                localtime_r (&date, &tm_date);
-        else
-                gmtime_r (&date, &tm_date);
+	pan_localtime_r (&date, &tm_date);
 
 	/* try to put the date in the buffer */
 	if (!strftime (fillme, len, fmt, &tm_date))
 		*fillme = '\0';
 }
 
-void
-open_outside_file (const char *fname)
-{
-	const char * mime_type = NULL;
-	const char * mime_prog = NULL;
-	char **argv;
-	int argc;
-
-	if (!((mime_type = gnome_mime_type (fname))))
-		return;
-	if (!((mime_prog = gnome_mime_program (mime_type))))
-		return;
-	if (!strstr (mime_prog, "%f"))
-		return;
-
-	argv = g_strsplit (mime_prog, " ", -1);
-	for (argc=0; argv[argc]; argc++)
-		if (!strcmp(argv[argc], "%f"))
-			replace_gstr (argv+argc, g_strdup(fname));
-
-	gnome_execute_async (NULL, argc, argv);
-
-	g_strfreev (argv);
-}
-
-static gint
+static gboolean
 try_to_execute_url (const gchar * template, const gchar * url)
 {
-	gchar * str;
-	gchar ** argv;
-	gint argc;
-	gint retval;
+	gboolean retval;
+	char * str;
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(url), -1);
+	g_return_val_if_fail (is_nonempty_string(url), FALSE);
 	if (template == NULL)
 		template = "netscape %s";
 
@@ -395,32 +385,19 @@ try_to_execute_url (const gchar * template, const gchar * url)
 		str = g_strdup_printf ("%s %s", template, url);
 	replace_gstr (&str, pan_substitute(str, "%%%%", "%%"));
 
-	/* break the command into parts as gnome_execute_async wants */
-	argc = 0;
-	argv = NULL;
-	if (poptParseArgvString(str, &argc, &argv) != 0) {
-		g_warning ("Parse error of `%s'", template);
-		return -1;
-	}
-
-	/* try to execute it */
-	log_add_va (LOG_INFO, _("Trying to open URL `%s' with `%s'"), url, str);
-	pan_lock ();
-	retval = gnome_execute_async (NULL, argc, argv);
-	pan_unlock ();
+	/* execute it */
+	retval = g_spawn_command_line_async (str, NULL);
 
 	/* cleanup */
-	free (argv);
 	g_free (str);
 	return retval;
 }
 
 
 void
-pan_url_show (const gchar * url)
+pan_url_show (const char * url)
 {
 	gboolean is_http;
-	gchar * browser;
 
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string(url));
@@ -429,38 +406,134 @@ pan_url_show (const gchar * url)
 	is_http = !strncmp (url, "http://", 7);
 	if (is_http)
 	{
-		/* first, see if user has selected a browser in Pan. */
-		if (is_nonempty_string(external_web_browser)) {
-			gint i;
-			gint result = -1;
-			gchar ** tries = g_strsplit (external_web_browser, ":", -1);
-			for (i=0; result==-1 && tries[i]!=NULL; ++i) {
-				g_message ("trying [%s]", tries[i]);
-				result = try_to_execute_url (tries[i], url);
-			}
-			g_strfreev (tries);
-			if (result != -1)
-				return;
-		}
+		const char * cmd;
 
-		/* http://tuxedo.org/~esr/BROWSER/ */
-		browser = getenv ("BROWSER");
-		if (is_nonempty_string(browser)) {
-			gint i;
-			gint result = -1;
-			gchar ** tries = strsplit_command_line_arguments (browser);
-			for (i=0; result==-1 && tries[i]!=NULL; ++i) {
-				g_message ("trying [%s]", tries[i]);
+		if (is_nonempty_string (external_web_browser))
+			cmd = external_web_browser;
+		else
+			cmd = g_getenv ("BROWSER");
+
+		if (is_nonempty_string (cmd))
+		{
+			int i;
+			char ** tries;
+			gboolean result = FALSE;
+			
+			tries = g_strsplit (external_web_browser, ":", -1);
+			for (i=0; !result && tries[i]!=NULL; ++i)
 				result = try_to_execute_url (tries[i], url);
-			}
+
 			g_strfreev (tries);
-			if (result != -1)
-				return;
 		}
 	}
+}
 
-	/* just punt it to gnome, which is better at this */
-	pan_lock ();
-	gnome_url_show (url);
-	pan_unlock ();
+
+/***
+****  Menu Building
+***/
+
+static char*
+menu_translate(const char* path, gpointer data)
+{
+	return gettext (path);
+}
+
+GtkWidget*
+menubar_create (GtkWidget            * window,
+                GtkItemFactoryEntry  * entries,
+                guint                  entries_qty,
+                const char           * path,
+                gpointer               data)
+{
+	GtkItemFactory *factory;
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_accel_group_new ();
+	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+	g_object_unref (accel_group);
+
+	factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, path, accel_group);
+	gtk_item_factory_set_translate_func(factory, menu_translate, NULL, NULL);
+	gtk_item_factory_create_items(factory, entries_qty, entries, data);
+
+	return gtk_item_factory_get_widget (factory, path);
+}
+
+GtkWidget*
+menu_create_items (GtkItemFactoryEntry   * entries,
+                   guint                   entries_qty,
+                   const char            * path,
+                   GtkItemFactory       ** factory,
+                   gpointer                data)
+{
+	*factory = gtk_item_factory_new(GTK_TYPE_MENU, path, NULL);
+	gtk_item_factory_set_translate_func(*factory, menu_translate, NULL, NULL);
+	gtk_item_factory_create_items(*factory, entries_qty, entries, data);
+
+	return gtk_item_factory_get_widget(*factory, path);
+}
+
+void
+menu_set_sensitive (GtkItemFactory   * ifactory,
+                    const char       * path,
+                    gboolean           sensitive)
+{
+	GtkWidget *widget;
+
+	g_return_if_fail(ifactory != NULL);
+
+	widget = gtk_item_factory_get_item (ifactory, path);
+	if (widget == NULL)
+		g_error ("couldn't find menu item `%s'", path);
+
+	gtk_widget_set_sensitive (widget, sensitive);
+}
+
+void
+menu_set_checked (GtkItemFactory   * ifactory,
+                  const char       * path,
+                  gboolean           active)
+{
+	GtkCheckMenuItem * w;
+
+	g_return_if_fail (ifactory != NULL);
+
+	w = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (ifactory, path));
+	if (w == NULL)
+		g_error ("couldn't find menu item `%s'", path);
+	else if (!!w->active != !!active)
+		gtk_check_menu_item_set_active (w, active);
+}
+
+gboolean
+menu_is_checked (GtkItemFactory   * ifactory,
+                 const char       * path)
+{
+	GtkWidget * w;
+
+	g_return_val_if_fail (ifactory!=NULL, FALSE);
+	w = gtk_item_factory_get_item (ifactory, path);
+	g_return_val_if_fail (GTK_IS_CHECK_MENU_ITEM(w), FALSE);
+
+	return gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w));
+}
+
+void
+menu_set_insensitive_all(GtkMenuShell * menu_shell)
+{
+	GList * cur;
+
+	for (cur = menu_shell->children; cur != NULL; cur = cur->next)
+		gtk_widget_set_sensitive(GTK_WIDGET(cur->data), FALSE);
+}
+
+void
+pan_gtk_entry_set_text (GtkWidget            * w,
+                        const char           * text)
+{
+	char * freeme = NULL;
+	g_return_if_fail (GTK_IS_ENTRY(w));
+	gtk_entry_set_text (GTK_ENTRY(w), pan_utf8ize (text, -1, &freeme));
+	g_free (freeme);
 }
