@@ -430,6 +430,7 @@ int main (void)
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"1")) return num;
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"\t")) return num;
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"2")) return num;
+		if (pan_strcmp(get_next_token_str(march, ' ', &march),NULL)) return num;
 		g_message ("PASS get_next_token_str");
 
 		string = "Foo\t\t\t10\t2112";
@@ -439,6 +440,7 @@ int main (void)
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"")) return num;
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"10")) return num;
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"2112")) return num;
+		if (pan_strcmp(get_next_token_str(march, '\t', &march),NULL)) return num;
 		g_message ("PASS get_next_token_str 2");
 	}
 
@@ -627,37 +629,6 @@ int main (void)
 		g_message ("PASS pan_g_string_strstrip");
 	}
 
-	/* pan_g_string_append_len */
-	++num;
-	if (1)
-	{
-		GString * s = g_string_new (NULL);
-
-		/* add */
-		pan_g_string_append_len (s, "abc", 3);
-		if (pan_strcmp(s->str,"abc")) return num;
-
-		/* append */
-		pan_g_string_append_len (s, "def", 3);
-		if (pan_strcmp(s->str,"abcdef")) return num;
-
-		/* append short */
-		pan_g_string_append_len (s, "ghijklm", 3);
-		if (pan_strcmp(s->str,"abcdefghi")) return num;
-
-		/* the number's too big */
-		pan_g_string_append_len (s, "j", 3);
-		if (pan_strcmp(s->str,"abcdefghij")) return num;
-
-		/* feed it trash & don't crash */
-		pan_g_string_append_len (NULL, "abc", 2);
-		pan_g_string_append_len (NULL, NULL, 2);
-		pan_g_string_append_len (s, NULL, 2);
-
-		g_string_free (s, TRUE);
-		g_message ("PASS pan_g_string_append_len");
-	}
-
 	/* lower bound */
 	++num;
 	if (1)
