@@ -15,12 +15,17 @@
 static gboolean
 files_differ (const gchar * filename1, const gchar * filename2)
 {
-	GArray * f1 = read_file (filename1);
-	GArray * f2 = read_file (filename2);
-	gboolean diff = f1==NULL || f2==NULL || f1->len!=f2->len || memcmp(f1->data,f2->data,f1->len);
+	char *f1=NULL, *f2=NULL;
+	size_t f1_len=0, f2_len=0;
+	gboolean diff;
+
+	g_file_get_contents (filename1, &f1, &f1_len, NULL);
+	g_file_get_contents (filename2, &f2, &f2_len, NULL);
+	diff = f1==NULL || f2==NULL || f1_len!=f2_len || memcmp(f1,f2,f1_len);
+
 	g_message ("files [%s] and [%s] are %s", filename1, filename2, diff?"different":"equal");
-	if (f1!=NULL) g_array_free (f1, TRUE);
-	if (f2!=NULL) g_array_free (f2, TRUE);
+	g_free (f1);
+	g_free (f2);
 	return diff;
 }
 
@@ -47,13 +52,13 @@ import_test (void)
 	/* simple import/export */
 	++test;
 	newsrc_import (server, in, FALSE);
-	newsrc_export (server, out, FALSE, FALSE);
+	newsrc_export (server, out, FALSE);
 	if (files_differ (in, out)) return test;
 	g_message ("PASS simple newsrc import/export");
 
 	/* export + backup */
 	++test;
-	newsrc_export (server, out, FALSE, TRUE);
+	newsrc_export (server, out, FALSE);
 	if (files_differ (in, out)) return test;
 	if (files_differ (out_bak, out)) return test;
 	unlink (out);
@@ -63,7 +68,7 @@ import_test (void)
 	/* export subscribed only */
 	++test;
 	if (!file_exists (in_sub_only)) return test;
-	newsrc_export (server, out, TRUE, FALSE);
+	newsrc_export (server, out, TRUE);
 	if (files_differ (in_sub_only, out)) return test;
 	if (file_exists (out_bak)) return test;
 	unlink (out);
@@ -74,7 +79,7 @@ import_test (void)
 	server = server_new ();
 	server->name = g_strdup ("Temp");
 	newsrc_import (server, noisy, FALSE);
-	newsrc_export (server, out, FALSE, FALSE);
+	newsrc_export (server, out, FALSE);
 	if (files_differ (less_noisy, out)) return test;
 	g_message ("PASS survive importing corrupt newsrc file");
 	unlink (out);
@@ -92,17 +97,15 @@ static void
 init_pan (void)
 {
 	gchar * base;
-	gchar * data_dir;
 	gchar * download_dir;
 
 	base = getenv ("srcdir");
 	base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
-	data_dir = g_strdup_printf ("%s/data/", base);
 	download_dir = g_strdup ("/tmp/");
-	base_prefs_init (data_dir, download_dir, 100, FALSE, FALSE);
+	base_prefs_init (download_dir, ">", 100, FALSE, FALSE);
 
-	g_free (data_dir);
 	g_free (download_dir);
+	g_free (base);
 }
 
 extern int server_saving_enabled;
@@ -325,6 +328,15 @@ main (void)
 	if (strcmp(newsrc_get_read_str(n),"17577-17578,17581-17582,17589-17590")) return num;
 	g_message ("PASS newsrc_mark_range %d", num);
 
+	/* Jiri Kubicek reported this one against 0.11.2.90 */
+	num = 25;
+	newsrc_init (n, "", 1, 10);
+	newsrc_mark_range (n, 1, 4, TRUE);
+	newsrc_mark_range (n, 7, 9, TRUE);
+	newsrc_mark_range (n, 3, 4, FALSE);
+	if (strcmp(newsrc_get_read_str(n),"1-2,7-9")) return num;
+	g_message ("PASS newsrc_mark_range %d", num);
+
 	/* newsrc_set_group_range
 	 * note that after a mark all, the lower bound is 0.
 	 * this follows newsrc conventions. */
@@ -340,6 +352,13 @@ main (void)
 	if (strcmp(newsrc_get_read_str(n),"0-1000")) return num;
 	g_message ("PASS newsrc_set_group_range");
 
+	/* http://bugzilla.gnome.org/show_bug.cgi?id=77878 */
+	newsrc_init (n, "", 1, 10);
+	newsrc_mark_range (n, 1, 4, TRUE);
+	newsrc_mark_range (n, 7, 9, TRUE);
+	newsrc_mark_range (n, 3, 4, FALSE);
+	if (strcmp(newsrc_get_read_str(n),"1-2,7-9")) return num;
+	g_message ("PASS Jiri Kubicek");
 
 	/* import/export test */
 	if ((num = import_test ())) return num;
